program Diyo;
{$APPTYPE CONSOLE}

{*
Diyo Delphi Version 1.05
Editing MS932~EUC-JP.bin
Copyright 2020 Sogaya

Delphi 2009

*}

uses
  SysUtils,
  AnsiQueue in 'AnsiQueue.pas',
  Combo in 'Combo.pas',
  ConvertUTF16 in 'ConvertUTF16.pas',
  Divide in 'Divide.pas',
  Editing in 'Editing.pas',
  ErrorText in 'ErrorText.pas',
  FieldsArray in 'FieldsArray.pas',
  FieldsArrayMany in 'FieldsArrayMany.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  PathEXE in 'PathEXE.pas',
  SearchBinary in 'SearchBinary.pas',
  SparseEUC in 'SparseEUC.pas',
  Substitute in 'Substitute.pas',
  SurvivorUTF16 in 'SurvivorUTF16.pas',
  TimeName in 'TimeName.pas',
  Ucombo in 'Ucombo.pas',
  DivideSJIS in 'DivideSJIS.pas',
  PutCSV in 'PutCSV.pas';

{$R *.res}

type AnsiCharArray = array[0..4095] of AnsiChar;

const
  SIZE_OF_ANSICHAR = 1;
  ASCII_SIZE = 1;
  BUFFER_ERROR = -1;
  EXPANSION_FACTOR = 2;
  H_5F = 95; { '_' }
  H_80 = 128;
  KANJI_SIZE = 2;
  KATAKANA_SIZE = 1;
  SURRO_CSV = 'surro.csv';
  UNDERLINES_REPLACE_SURROGATE_PAIRS = True;
  VERSION_NUMBER = '1.05';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;

var
  a: AnsiCharArray;
  A_SIZE: Integer;
  A_SIZE_1: Integer;
  c: Converting;
  e: EditTable;
  EMPTY_ANSI: AnsiString;
  f: FinalEffects;
  g: Guiding;
  h: HyphenTerms;
  i: Longint;
  list_name_length: Integer;
  n: Integer;
  normal: AnsiString;
  p: PutSurroCSV;
  pascal_string: AnsiString;
  s: String;
  substitute: AnsiString;
  t: FinalEffects;
  testFile: File;

begin
  s := ParamStr(1);
  if Length(s) = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
      g.Free;

      Halt(1);
    end;

{ Checking if software can open SURRO_CSV in the current directory. }
  try
    AssignFile(testFile, SURRO_CSV);
    Reset(testFile);
    CloseFile(testFile);
  except
{ Producing SURRO_CSV }
    p := PutSurroCSV.Create;
    p.setting(SURRO_CSV);
  end;

  EMPTY_ANSI := '';

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          t := FinalEffects.Create;
          t.setting(EMPTY_ANSI);
          c := Converting.Create;
          c.setting(t);

{*
h.labelPiece extracts an ASCII character from the command line. 
h.jointPiece extracts a hankaku-katakana character from the command line. 
h.separatorPiece extracts a Shift_JIS kanji character from the command line. 
From the command line, h.quotePiece extracts a kanji character used as a substitute. The user enters a Shift_JIS kanji character; however, Diyo automatically converts it into an EUC-JP kanji character.
*}
          s := h.separatorPiece;
          i := Ord(s[1]);
          if i = H_5F then
            begin
{ Converting katakana from UTF16 to MS932 }
              s := h.jointPiece;
              c.produce_MS932_phrase( PChar(s), Length(s) );
              A_SIZE := c.MS932_phrase_size;
              A_SIZE_1 := A_SIZE - 1;
              for n := 0 to A_SIZE_1 do
              begin
                a[n] := c.copyingArrayItem(n);
              end;

              normal := EMPTY_ANSI + a[0];

              s := h.labelPiece;
              c.produce_MS932_phrase( PChar(s), Length(s) );
              A_SIZE := c.MS932_phrase_size;
              A_SIZE_1 := A_SIZE - 1;
              for n := 0 to A_SIZE_1 do
              begin
                a[n] := c.copyingArrayItem(n);
              end;

              substitute := EMPTY_ANSI + a[0];
            end
          else
            begin
{ Converting kanji from UTF16 to MS932 }
              c.produce_MS932_phrase( PChar(s), Length(s) );
              A_SIZE := c.MS932_phrase_size;
              A_SIZE_1 := A_SIZE - 1;
              for n := 0 to A_SIZE_1 do
              begin
                a[n] := c.copyingArrayItem(n);
              end;

              pascal_string := a;
              normal := Copy(pascal_string, 1, KANJI_SIZE);

              s := h.quotePiece;
              c.produce_MS932_phrase( PChar(s), Length(s) );
              A_SIZE := c.MS932_phrase_size;
              A_SIZE_1 := A_SIZE - 1;
              for n := 0 to A_SIZE_1 do
              begin
                a[n] := c.copyingArrayItem(n);
              end;

              pascal_string := a;
              substitute := Copy(pascal_string, 1, KANJI_SIZE);
            end;

          f := FinalEffects.Create;
          f.setting(h.fileName, h.listName(1), h.newName, h.pathName,
            normal, substitute, 
            h.atPoint, 
            h.noRemoving, h.noSearching, h.priority, UNDERLINES_REPLACE_SURROGATE_PAIRS, h.whichTable);

{ main function }
  e := EditTable.Create;

  list_name_length := Length(f.listName);
  if list_name_length = 0 then
    begin
      e.setting(f);
      e.editRecord(f);
    end
  else
    begin
      e.setting(f, f.listName);
      e.ifErrorOccurs(BUFFER_ERROR);
      e.editManyRecord(f);
    end;

  if f.whichTable then
    e.displayDataText;

  e.Free;
{ end of the main function }

          f.Free; c.Free; t.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
