unit Editing;

{$I+}

interface

uses
  Windows, SysUtils, SparseEUC, ErrorText, FinalEffect, PathEXE, MessagesText, Substitute;

type AnsiCharArray = array[0..4095] of AnsiChar;
type DoubleAnsiCharArray = array[0..8191] of AnsiChar;

type
  EditTable = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    ANSI_CHAR: array[0..255] of AnsiChar;
    beginning: Integer;
    buffer: array of AnsiChar;
    buffer_size: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_V_ARRAY: DoubleAnsiCharArray;
    count: Integer;
    data: array[1..906] of NormalSubstitute;
    EMPTY_ANSI: AnsiString;
    ensemble: array[0..3] of AnsiChar; { RECORD_SIZE = 4 }
    file_name: String;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_FF_C: AnsiChar;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    normal: AnsiString;
    open_close: array[1..4] of OpenCloseFunction;
    substitute: AnsiString;
    table: SparsePopulationEUC;
    v: DoubleAnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    function ANSI_CHAR_table: AnsiChar;
    function file_size(name: String): Integer;
    function kanjiCode(s: PAnsiChar; s_length: Integer): Integer;
    function reading_to_produce_normal_substitute_list: Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function displayDataText: Boolean;
    function editManyRecord(f: FinalEffects): Boolean;
    function editRecord(f: FinalEffects): Boolean;
    function renameBinaryFile(name: String; no_removing: Boolean): Integer;
    procedure ifErrorOccurs(exit_code: Integer);
    procedure setting(final_effects: FinalEffects); overload;
    procedure setting(final_effects: FinalEffects; fn: String); overload;
  end;

const
  ASCII_SIZE = 1;
  BELL = 7;
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  C_MS932_EUC_BIN = 'C:\Narfi\MS932~EUC-JP.bin';
  COMBO_ERROR = -3;
  COMMA_BYTE = 44; { 2C, ',' }
  DATA_ARRAY_LIMIT = 906;
  DATA_TEXT = 'data.txt';
  DOUBLE_KANJI = 4;
  EMPTY = '';
  EXPANSION_FACTOR = 2;
  FILE_ERROR = -2;
  FIRST_KANJI = 159;
  H_00FF = 255;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_3E = 62; { '>' }
  H_5C = 92; { '\' }
  H_7E = 126; { '~' }
  H_80 = 128;
  H_81 = 129;
  H_FC = 252;
  H_FF = 255;
  KANA_HANDAKUTEN = 223; { DFh }
  KANA_KUTEN = 161; { A1h }
  KANJI_SIZE = 2;
  KATAKANA_SIZE = 1;
  MS932_EUC_BIN = 'MS932~EUC-JP.bin';
  PLEASE_CHECK = True;
  RECORD_SIZE = 4;
  SEARCH_COUNT = 4;
  SETTING_TEXT = 'setting.txt';
  SIZE_OF_ANSICHAR = 1;
  TEMPORARY_BINARY = 'binary.tmp';
  TEXT_FILE = '.txt';

var
  x: ErrorTexts;

implementation

destructor EditTable.Destroy;
begin
  if buffer_size > 0 then
    buffer := nil;

  table.Free;

  inherited Destroy;
end;

function EditTable.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function binary_file_in_current_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, MS932_EUC_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := MS932_EUC_BIN;
  except
    s := 'Reset() error-24: ' + MS932_EUC_BIN;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    s := EMPTY;
  end;

  Result := s;
end;

function binary_file_in_installation_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, C_MS932_EUC_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := C_MS932_EUC_BIN;
  except
    s := 'Reset() error-25: ' + C_MS932_EUC_BIN;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    s := EMPTY;
  end;

  Result := s;
end;

function binary_file_in_path_directory: String;
{ directory included in Path }
var
  p: CommandPath;
  s: String;
begin
  p := CommandPath.Create;
  p.setting(True);
  s := p.pathName(MS932_EUC_BIN);
  p.Free;

  Result := s;
end;

function binary_file_in_selected_directory: String;
{ directory that may have priority }
var
  messages: String;
  s: String;
  settingFile: TextFile;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(settingFile, SETTING_TEXT);
    Reset(settingFile);
    ReadLn(settingFile, s); { Line-1 }
    CloseFile(settingFile);
  except
    messages := 'Reset() error-26: ' + SETTING_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
    s := EMPTY;
  end;

  if Length(s) > 0 then
    begin
      try
        Assign(testFile, s);
        Reset(testFile);
        CloseFile(testFile);
      except
        messages := 'Reset() error-27: ' + s;
        x := ErrorTexts.Create;
        x.setting(C_ERR, TEXT_FILE, messages);
        x.Free;
        s := EMPTY;
      end;

    end;

  Result := s;
end;

function EditTable.displayDataText: Boolean;
var
  b: Boolean;
  s: String;
  testFile: TextFile;
begin
  b := True;

  try
    AssignFile(testFile, DATA_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      WriteLn(s);
    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-28: ' + DATA_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    b := False;
  end;

  Result := b;
end;

function EditTable.editManyRecord(f: FinalEffects): Boolean;
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  found: Boolean;
  from: Integer;
  i: Integer;
  into: Integer;
  k: Integer;
  m: MessagesTexts;
  messages: String;
  newFile: File;
  r: Boolean;
  recordNumber: Integer;
  tableFile: File;
  u: Integer;
  used: Integer;
  z: AnsiString;
begin
  found := False;
  into := 33088; { 8140h } 
  r := True;
  recordNumber := 1;

  used := reading_to_produce_normal_substitute_list;

{ Finding MS932_EUC_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if f.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if f.whichTable then
          begin
            m := MessagesTexts.Create;
            messages := binaryTable + ' is primarily accessible to Diyo.';
            m.setting(DATA_TEXT, messages);
            m.Free;
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if f.noSearching then
                begin
                  messages := 'Reset() error-29: ' + C_MS932_EUC_BIN;
                  x := ErrorTexts.Create;
                  x.setting(C_ERR, TEXT_FILE, messages);
                  x.Free;
                  break;
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              messages := 'Reset() error-30: i = ' + IntToStr(SEARCH_COUNT);
              x := ErrorTexts.Create;
              x.setting(C_ERR, TEXT_FILE, messages);
              x.Free;
              break;
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  if Length(binaryTable) > 0 then
    begin
{ Reaeding MS932_EUC_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      AssignFile(newFile, TEMPORARY_BINARY);
      Rewrite(newFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        if recordNumber < FIRST_KANJI then
          begin
{ ASCII or katakana }
            b := Ord(a[0]);
            k := b;
          end
        else
          begin
{ kanji }
            b := Ord(a[0]);
            k := b shl 8;

            b := Ord(a[1]);
            k := k + b;
          end;

        ensemble[0] := a[0];
        ensemble[1] := a[1];

        for u := 1 to used do
        begin
          if data[u].pleaseCheck then
            begin
              z := data[u].normalCharacter;
              from := kanjiCode( PAnsiChar(z), Length(z) );
              if k = from then
                begin
                  z := data[u].substituteCharacter;
                  into := kanjiCode( PAnsiChar(z), Length(z) );
                  data[u].sure;
                  found := True;
                  break;
                end
              else
                begin
                  found := False;
                end;

            end
          else
            begin
              found := False;
            end;

        end;

        if found then
          begin
            ensemble[2] := table.working(into, 0);
            ensemble[3] := table.working(into, 1);
          end
        else
          begin
            ensemble[2] := a[2];
            ensemble[3] := a[3];
          end;

        BlockWrite(newFile, ensemble, block_has_been_read, block_has_been_written);
        Inc(recordNumber);
      until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

      CloseFile(newFile);
      CloseFile(tableFile);
      renameBinaryFile(f.fileName, f.noRemoving);
    end
  else
    begin
      messages := 'Length() error-31: Length(binaryTable) = 0';
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, messages);
      x.Free;
      r := False;
    end;

    Result := r;
end;

function EditTable.editRecord(f: FinalEffects): Boolean;
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  from: Integer;
  i: Integer;
  into: Integer;
  k: Integer;
  m: MessagesTexts;
  messages: String;
  newFile: File;
  r: Boolean;
  recordNumber: Integer;
  s: String;
  tableFile: File;
begin
  r := True;
  recordNumber := 1;

{ Finding MS932_EUC_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if f.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if f.whichTable then
          begin
            m := MessagesTexts.Create;
            messages := binaryTable + ' is primarily accessible to Diyo.';
            m.setting(DATA_TEXT, messages);
            m.Free;
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if f.noSearching then
                begin
                  messages := 'Reset() error-32: ' + C_MS932_EUC_BIN;
                  x := ErrorTexts.Create;
                  x.setting(C_ERR, TEXT_FILE, messages);
                  x.Free;
                  break;
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              messages := 'Reset() error-33: i = ' + IntToStr(SEARCH_COUNT);
              x := ErrorTexts.Create;
              x.setting(C_ERR, TEXT_FILE, messages);
              x.Free;
              break;
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  normal := f.normalCharacter;
  from := kanjiCode( PAnsiChar(normal), Length(normal) );

  substitute := f.substituteCharacter;
  into := kanjiCode( PAnsiChar(substitute), Length(substitute) );

  s := Format('%0:4x  %1:4x', [from, into]);
  WriteLn(s);

  if Length(binaryTable) > 0 then
    begin
{ Reaeding MS932_EUC_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      AssignFile(newFile, TEMPORARY_BINARY);
      Rewrite(newFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        if recordNumber < FIRST_KANJI then
          begin
{ ASCII or katakana }
            b := Ord(a[0]);
            k := b;
          end
        else
          begin
{ kanji }
            b := Ord(a[0]);
            k := b shl 8;

            b := Ord(a[1]);
            k := k + b;

          end;

        ensemble[0] := a[0];
        ensemble[1] := a[1];

        if k = from then
          begin
            ensemble[2] := table.working(into, 0);
            ensemble[3] := table.working(into, 1);
          end
        else
          begin
            ensemble[2] := a[2];
            ensemble[3] := a[3];
          end;

        BlockWrite(newFile, ensemble, block_has_been_read, block_has_been_written);
        Inc(recordNumber);
      until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

      CloseFile(newFile);
      CloseFile(tableFile);
      renameBinaryFile(f.fileName, f.noRemoving);
    end
  else
    begin
      messages := 'Length() error-34: Length(binaryTable) = 0';
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, messages);
      x.Free;
      r := False;
    end;

    Result := r;
end;

function EditTable.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function EditTable.kanjiCode(s: PAnsiChar; s_length: Integer): Integer;
var
  c: AnsiChar;
  firstByte: Integer;
  k: Integer;
  kanji: Boolean;
  n: Integer;
  s_length_1: Integer;
  w: Integer;
begin
  firstByte := KANA_KUTEN;
  kanji := False;
  s_length_1 := s_length -1;
  w := 33088; { 8140h }

  for n := 0 to s_length_1 do
  begin
    c := s[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;
        break;
      end
    else
      begin
        if (H_81 <= k) and (k <= H_FC) then
          begin
            if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
              begin
                w := k;
                break;
              end
            else
              begin
                firstByte := k shl 8;
                kanji := True;
              end;

          end
        else
          begin
{ ASCII }
            w := k;
            break;
          end;

      end; { if kanji then, else }

  end; { for n := 1 to s_length_1 do }

  Result := w;
end;

function EditTable.reading_to_produce_normal_substitute_list: Integer;
var
  ASCII_C: AnsiChar;
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  dough: AnsiString;
  doughLength: Integer;
  firstAnsiChar: AnsiChar;
  k: Integer;
  kanji: Boolean;
  KATAKANA_C: AnsiChar;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  q: Integer;
  s: String;
  testFile: File;
  u: Integer;
  used: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  k := 0;
  m := 0;
  u := 0; { Inc(u) before calling NormalSubstitute.Create }
  used := 0;
  v := CLEAR_V_ARRAY;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n := 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-35: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := ANSI_CHAR[COMMA_BYTE];
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    if m >= V_SIZE_1 then
      begin
        break;
      end;

    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        v[m] := firstAnsiChar;
        v[ m + 1 ] := c;
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            doughLength := m;
            pascal_string := v;
            dough := Copy(pascal_string, 1, doughLength);
            m := 0;
            Inc(u);
            data[u] := NormalSubstitute.Create;

            if doughLength < DOUBLE_KANJI then
              begin
                KATAKANA_C := dough[1];
                ASCII_C := dough[2];
                data[u].setting(EMPTY_ANSI, KATAKANA_C,  ASCII_C, PLEASE_CHECK);
              end
            else
              begin
                normal := Copy(dough, 1, KANJI_SIZE);
                q := KANJI_SIZE + 1;
                substitute := Copy(dough, q, KANJI_SIZE);
                data[u].setting(normal, substitute, PLEASE_CHECK);
              end;

            used := u;

            beginning := n + 1;
            break;
          end
        else
          begin
            if k = H_0D then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        v[m] := c;
                        Inc(m);
                        kanji := False;
                      end
                    else
                      begin
                        firstAnsiChar := c;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    v[m] := ANSI_CHAR[k];
                    Inc(m);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := ANSI_CHAR[COMMA_BYTE];
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    if m >= V_SIZE_1 then
      begin
        break;
      end;

    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        v[m] := firstAnsiChar;
        v[ m + 1 ] := c;
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            doughLength := m;
            pascal_string := v;
            dough := Copy(pascal_string, 1, doughLength);
            m := 0;
            Inc(u);
            data[u] := NormalSubstitute.Create;

            if doughLength < DOUBLE_KANJI then
              begin
                KATAKANA_C := dough[1];
                ASCII_C := dough[2];
                data[u].setting(EMPTY_ANSI, KATAKANA_C,  ASCII_C, PLEASE_CHECK);
              end
            else
              begin
                normal := Copy(dough, 1, KANJI_SIZE);
                q := KANJI_SIZE + 1;
                substitute := Copy(dough, q, KANJI_SIZE);
                data[u].setting(normal, substitute, PLEASE_CHECK);
              end;

            used := u;
          end
        else
          begin
            if k = H_0D then
              begin
                continue;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        v[m] := c;
                        Inc(m);
                        kanji := False;
                      end
                    else
                      begin
                        firstAnsiChar := c;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    v[m] := ANSI_CHAR[k];
                    Inc(m);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := used;
end;

function EditTable.renameBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  s: String;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, TEMPORARY_BINARY);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);
      BlockWrite(newFile, a, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, TEMPORARY_BINARY);
        Erase(testFile);
      end;

  except
    s := 'Reset() error-36: ' + name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function EditTable.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    s := 'SetLength() error-37: ' + IntToStr(buffer_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  Result := r;
end;

procedure EditTable.ifErrorOccurs(exit_code: Integer);
begin
  if buffer_size < 0 then
    Halt(exit_code);

end;

procedure EditTable.setting(final_effects: FinalEffects);
var
  i: Integer;
begin
  file_name := final_effects.listName;

  buffer_size := 0;

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_V_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';

  NEW_LINE := EMPTY_ANSI + H_0A_C;
  NEW_LINE_SIZE := 1;

{ Producing a conversion table in the main memory }
  table := SparsePopulationEUC.Create;
  table.setting(final_effects);
end;

procedure EditTable.setting(final_effects: FinalEffects; fn: String);
var
  i: Integer;
  s: String;
  testFile: File;
begin
  file_name := fn;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    CloseFile(testFile);

    buffer_size := file_size(file_name);
  except
    s := 'Reset() error-38: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_V_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';

  NEW_LINE := EMPTY_ANSI + H_0A_C;
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SparsePopulationEUC.Create;
  table.setting(final_effects);
end;

end.
