unit DeriveBinaryEUC;

{$I+}

interface

uses
  SysUtils, ErrorText;

type
  DeriveTable = class
  private
    no_vacancies: array[0..64588] of Boolean;
    r: array[1..4] of AnsiChar;
    w: array[1..5] of AnsiChar;
  public
    procedure setting;
  end;

const
  C_ERR = 'err';
  H_80 = 128;
  MS932_EUC = 'MS932~EUC-JP.bin';
  NEW_RECORD_SIZE = 5;
  RECORD_SIZE = 4;
  TEXT_FILE = '.txt';
  EUC_MS932 = 'EUC-JP~Shift_JIS.bin';

implementation

procedure DeriveTable.setting;
var
  ASCII_C: AnsiChar;
  b: Longint;
  binaryFile: File;
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  k: Longint;
  KANJI_C: AnsiChar;
  newFile: File;
  NON_KANJI_C: AnsiChar;
  s: String;
  x: ErrorTexts;
begin
  ASCII_C := AnsiChar(#0);
  KANJI_C := AnsiChar(#1);
  NON_KANJI_C := AnsiChar(#0);

  for k := 0 to 64588 do
  begin
    no_vacancies[k] := False;
  end;

  AssignFile(newFile, EUC_MS932);
  Rewrite(newFile, NEW_RECORD_SIZE);

  try
    AssignFile(binaryFile, MS932_EUC);
    Reset(binaryFile, RECORD_SIZE);

    repeat
      BlockRead(binaryFile, r, 1, block_has_been_read);

      w[1] := r[3];
      w[2] := r[4];

      if r[2] = NON_KANJI_C then
        begin
          b := Ord(w[1]);
          if b < H_80 then
            begin
              k := Ord(w[1]);
              if no_vacancies[k] then
                begin
                  continue;
                end
              else
                begin
                  w[3] := ASCII_C;
                  w[4] := r[1];
                  w[5] := NON_KANJI_C;
                  no_vacancies[k] := True;
                end;

            end
          else
            begin
              continue;
            end;

        end
      else
        begin
{ kanji }
          b := Ord(w[1]);
          k := b shl 8;

          b := Ord(w[2]);
          k := k + b;

          if no_vacancies[k] then
            begin
              continue;
            end
          else
            begin
              w[3] := KANJI_C;
              w[4] := r[1];
              w[5] := r[2];
              no_vacancies[k] := True;
            end;

        end;

      BlockWrite(newFile, w, block_has_been_read, block_has_been_written);
    until block_has_been_read = 0;

    CloseFile(binaryFile);
  except
    s := 'Reset() error-104: ' + MS932_EUC;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  CloseFile(newFile);
end;

end.
