program Eucsjis;
{$APPTYPE CONSOLE}

{*
Eucsjis Version 1.04
Converting the text from EUC-JP to Shift_JIS
Copyright 2020 Sogaya

Delphi 2009

*}

uses
  AnsiQueue in 'AnsiQueue.pas',
  Combo in 'Combo.pas',
  ConvertEUC in 'ConvertEUC.pas',
  ErrorText in 'ErrorText.pas',
  FieldsArrayMany in 'FieldsArrayMany.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  PathEXE in 'PathEXE.pas',
  SearchBinary in 'SearchBinary.pas',
  SurvivorEUC in 'SurvivorEUC.pas',
  TimeName in 'TimeName.pas',
  DeriveBinaryEUC in 'DeriveBinaryEUC.pas',
  DivideSJIS in 'DivideSJIS.pas';

{$R *.res}

const
  BUFFER_ERROR = -1;
  DATA_TEXT = 'data.txt';
  DO_NOT_RECORD_PATH_NAME = False;
  DO_NOT_REMOVE_THE_FILE = True;
  MAXIMUM_FIELD_COUNT = 3071;
  NOT_COUNTED = 2147483647; { 7FFFFFFFh }
  NOT_MEASURED = -1;
  PATH_NAME = 'C:\Narfi\EUC-JP~Shift_JIS.bin';
  REMOVE_ANCIENT_FILE = false;
  SEARCH_TABLE_IN_EACH_DIRECTORY = True;
  SELECTED_DIRECTORY_DOES_NOT_HAVE_PRIORITY = False;
  TEMPORARY_BINARY = 'binary.tmp';
  TEMPORARY_TEXT = 'text.tmp';
  VERSION_NUMBER = '1.04';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;

var
  c: Conversion;
  carriage_return: Boolean;
  d: DeriveTable;
  dataFile: TextFile;
  f: FinalEffects;
  file_name_length: Integer;
  g: Guiding;
  h: HyphenTerms;
  new_name: String;
  new_name_length: Integer;
  s: String;

begin
  s := ParamStr(1);
  if Length(s) = 0 then
    begin
      d := DeriveTable.Create;
      d.setting;
      Halt(0);
    end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          new_name := h.listName(1);
          new_name_length := Length(new_name);
          if new_name_length = 0 then
            begin
              new_name := h.newName;
            end;

          if h.remove0Dh then
            begin
              if h.carriageReturn then
                begin
                  carriage_return := True;
                end
              else
                begin
                  carriage_return := False;
                end;

            end
          else
            begin
              carriage_return := True;
            end;

          f := FinalEffects.Create;
          f.setting(h.fileName, new_name, h.pathName, 
            h.atPoint, 
            carriage_return, h.identicalTime, h.noRemoving, h.noSearching, h.remove0Dh, h.removeControllers, h.priority, h.produceTemporaryBinary, h.whichTable);

{ main function }
  if f.whichTable then
    begin
      try
        Assignfile(dataFile, DATA_TEXT);
        Reset(dataFile);
        CloseFile(dataFile);

        Assignfile(dataFile, DATA_TEXT);
        Erase(dataFile);
      except
        ;
      end;

    end;

  file_name_length := Length(f.fileName);
  if file_name_length = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end;

  c := Conversion.Create;
  c.setting(f, f.fileName);
  c.ifErrorOccurs(BUFFER_ERROR);

  if f.whichTable then
    c.displayDataText;

  new_name_length := Length(f.newName);
  if new_name_length = 0 then
    begin
      if f.removeControllers then
        begin
          c.presetting;
          c.standard_output_of_non_controllers;
        end
      else
        begin
          c.standard_output;
        end;

      c.Free; f.Free; h.Free;
      Halt(0);
    end;

  if f.identicalTime then
    begin
      c.produceBinaryFile(f.fileName, REMOVE_ANCIENT_FILE);
      c.renameBinaryFile(f.fileName, f.noRemoving);
    end;

  if f.carriageReturn then
    begin
      if f.produceTemporaryBinary then
        begin
          if f.removeControllers then
            begin
              c.presetting;
              c.skipping_control_to_produce_MS932_CRLF(TEMPORARY_BINARY);
            end
          else
            begin
              c.reading_to_produce_MS932_CRLF(TEMPORARY_BINARY);
            end;

          c.renameBinaryFile(f.newName, f.noRemoving);
        end
      else
        begin
          if f.removeControllers then
            begin
              c.presetting;
              c.skipping_control_to_produce_MS932_CRLF(f.newName);
            end
          else
            begin
              c.reading_to_produce_MS932_CRLF(f.newName);
            end;

        end;

    end
  else
    begin
      if f.remove0Dh then
        begin
          if f.produceTemporaryBinary then
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_MS932_LF(TEMPORARY_BINARY);
                end
              else
                begin
                  c.reading_to_produce_MS932_LF(TEMPORARY_BINARY);
                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_MS932_LF(f.newName);
                end
              else
                begin
                  c.reading_to_produce_MS932_LF(f.newName);
                end;

            end;

        end
      else
        begin
          if f.produceTemporaryBinary then
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_MS932_text(TEMPORARY_BINARY);
                end
              else
                begin
                  c.reading_to_produce_MS932_text(TEMPORARY_BINARY);
                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_MS932_text(f.newName);
                end
              else
                begin
                  c.reading_to_produce_MS932_text(f.newName);
                end;

            end;

        end;

    end;

  c.Free;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
