unit SparseEUC;

interface

uses
  Windows, SysUtils, ErrorText, PathEXE, MessagesText, FinalEffect, SearchBinary;

type OpenCloseFunction = function: String;

type AnsiCharArray = array[1..4] of AnsiChar;

type
  SparsePopulationEUC = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    CLEAR_ARRAY: AnsiCharArray;
    m: MessagesTexts;
    open_close: array[1..4] of OpenCloseFunction;
    z: array[1..193764] of AnsiChar;
{*
K
FC4Bh

FC4Ch = 64588
64588 * ( EUC_KANJI_SIZE + 1 ) = 193764

*}
  public
    function working(index: Integer; place: Integer): AnsiChar;
    procedure clear;
    procedure populated(index: Integer; place: Integer; c: AnsiChar);
    procedure setting(BINARY_TABLE: String); overload;
    procedure setting(e: SearchTable); overload;
    procedure setting(f: FinalEffects); overload;
  end;

const
  C_ERR = 'err';
  C_MS932_EUC_BIN = 'C:\Narfi\MS932~EUC-JP.bin';
  EMPTY = '';
  EUC_KANJI_SIZE = 2;
  EUC_KANJI_SIZE_1 = 3;
  FIRST_KANJI = 159;
  MS932_EUC_BIN = 'MS932~EUC-JP.bin';
  DATA_TEXT = 'data.txt';
  RECORD_SIZE = 4;
  SEARCH_COUNT = 4;
  SETTING_TEXT = 'set_eucutf16.txt';
  TEXT_FILE = '.txt';

var
  x: ErrorTexts;

implementation

function binary_file_in_current_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, MS932_EUC_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := MS932_EUC_BIN;
  except
    s := 'Reset() error-171: ' + MS932_EUC_BIN;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    s := EMPTY;
  end;

  Result := s;
end;

function binary_file_in_installation_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, C_MS932_EUC_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := C_MS932_EUC_BIN;
  except
    s := 'Reset() error-172: ' + C_MS932_EUC_BIN;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    s := EMPTY;
  end;

  Result := s;
end;

function binary_file_in_path_directory: String;
{ directory included in Path }
var
  p: CommandPath;
  s: String;
begin
  p := CommandPath.Create;
  p.setting(True);
  s := p.pathName(MS932_EUC_BIN);
  p.Free;

  Result := s;
end;

function binary_file_in_selected_directory: String;
{ directory that may have priority }
var
  messages: String;
  s: String;
  settingFile: TextFile;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(settingFile, SETTING_TEXT);
    Reset(settingFile);
    ReadLn(settingFile, s); { Line-1 }
    CloseFile(settingFile);
  except
    messages := 'Reset() error-173: ' + SETTING_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
    s := EMPTY;
  end;

  if Length(s) > 0 then
    begin
      try
        Assign(testFile, s);
        Reset(testFile);
        CloseFile(testFile);
      except
        messages := 'Reset() error-174: ' + s;
        x := ErrorTexts.Create;
        x.setting(C_ERR, TEXT_FILE, messages);
        x.Free;
        s := EMPTY;
      end;

    end;

  Result := s;
end;

function SparsePopulationEUC.working(index: Integer; place: Integer): AnsiChar;
var
  k: Integer;
begin
  k := EUC_KANJI_SIZE_1 * index + place;
  Result := z[k];
end;

procedure SparsePopulationEUC.populated(index: Integer; place: Integer; c: AnsiChar);
var
  k: Integer;
begin
  k := EUC_KANJI_SIZE_1 * index + place;
  z[k] := c;
end;

procedure SparsePopulationEUC.clear;
var
  k: Integer;
begin
  for k := 1 to 193764 do
  begin
    z[k] := #0;
  end;

end;

procedure SparsePopulationEUC.setting(BINARY_TABLE: String);
var
  b: Longint;
  block_has_been_read: Integer;
  count: Integer;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  count := 0;

  for i := 1 to A_SIZE do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Reaeding BINARY_TABLE to produce a table in the main memory }
  try
    AssignFile(tableFile, BINARY_TABLE);
    Reset(tableFile, RECORD_SIZE);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(tableFile, a, 1, block_has_been_read);

      Inc(count);
      if count < FIRST_KANJI then
        begin
{ ASCII }
          k := Ord(a[1]);
          populated(k, 0, a[3]);
          populated(k, 1, a[4]);
        end
      else
        begin
{ kanji }
          b := Ord(a[1]);
          k := b shl 8;

          b := Ord(a[2]);
          k := k + b;

          populated(k, 0, a[3]);
          populated(k, 1, a[4]);
        end;

    until block_has_been_read = 0;

    CloseFile(tableFile);
  except
    messages := 'Reset() error-175: ' + BINARY_TABLE;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
  end;

end;

procedure SparsePopulationEUC.setting(e: SearchTable);
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  count: Integer;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  count := 0;

  for i := 1 to A_SIZE do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Finding MS932_EUC_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if e.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if e.whichTable then
          begin
            m := MessagesTexts.Create;
            messages := binaryTable + ' is primarily accessible to Eucutf16.';
            m.setting(DATA_TEXT, messages);
            m.Free;
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if e.noSearching then
                begin
                  messages := 'Reset() error-176: ' + C_MS932_EUC_BIN;
                  x := ErrorTexts.Create;
                  x.setting(C_ERR, TEXT_FILE, messages);
                  x.Free;
                  Halt(1);
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              messages := 'Reset() error-177: i = ' + IntToStr(SEARCH_COUNT);
              x := ErrorTexts.Create;
              x.setting(C_ERR, TEXT_FILE, messages);
              x.Free;
              Halt(1);
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  if Length(binaryTable) > 0 then
    begin
{ Reaeding MS932_EUC_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        Inc(count);
        if count < FIRST_KANJI then
          begin
{ ASCII }
            k := Ord(a[1]);
            populated(k, 0, a[3]);
            populated(k, 1, a[4]);
          end
        else
          begin
{ kanji }
            b := Ord(a[1]);
            k := b shl 8;

            b := Ord(a[2]);
            k := k + b;

            populated(k, 0, a[3]);
            populated(k, 1, a[4]);
          end;

      until block_has_been_read = 0;

      CloseFile(tableFile);
    end
  else
    begin
      messages := 'Length() error-178: Length(binaryTable) = 0';
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, messages);
      x.Free;
      Halt(1);
    end;

end;

procedure SparsePopulationEUC.setting(f: FinalEffects);
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  count: Integer;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  count := 0;

  for i := 1 to A_SIZE do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Finding MS932_EUC_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if f.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if f.whichTable then
          begin
            m := MessagesTexts.Create;
            messages := binaryTable + ' is primarily accessible to Eucutf16.';
            m.setting(DATA_TEXT, messages);
            m.Free;
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if f.noSearching then
                begin
                  messages := 'Reset() error-179: ' + C_MS932_EUC_BIN;
                  x := ErrorTexts.Create;
                  x.setting(C_ERR, TEXT_FILE, messages);
                  x.Free;
                  Halt(1);
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              messages := 'Reset() error-180: i = ' + IntToStr(SEARCH_COUNT);
              x := ErrorTexts.Create;
              x.setting(C_ERR, TEXT_FILE, messages);
              x.Free;
              Halt(1);
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  if Length(binaryTable) > 0 then
    begin
{ Reaeding MS932_EUC_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        Inc(count);
        if count < FIRST_KANJI then
          begin
{ ASCII }
            k := Ord(a[1]);
            populated(k, 0, a[3]);
            populated(k, 1, a[4]);
          end
        else
          begin
{ kanji }
            b := Ord(a[1]);
            k := b shl 8;

            b := Ord(a[2]);
            k := k + b;

            populated(k, 0, a[3]);
            populated(k, 1, a[4]);
          end;

      until block_has_been_read = 0;

      CloseFile(tableFile);
    end
  else
    begin
      messages := 'Length() error-181: Length(binaryTable) = 0';
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, messages);
      x.Free;
      Halt(1);
    end;

end;

end.
