unit TimeName;

{$I+}

interface

uses
  SysUtils, MessagesText;

type
  TimeNames = class
  private
    base: String;
    extension: String;
    messages: String;
    name: String;
    m: MessagesTexts;
    timeName: String;
  public
    function readSerialNumber: Integer;
    function sequence_to_integer(s: String): Integer;
    function temporaryName(serial_number: Boolean): String;
    function zeroSerialNumber: Integer;
    procedure incrementSerialNumber;
    procedure setting(b: String; e: String);
  end;

const
  COLON = ':';
  COLON_UNDERLINE = ':_';
  ERROR_TEXT = 'error.txt';
  HYPHEN = '-';
  NOT_CONVERTED = -1;
  SERIAL_TEXT = 'serial.txt';

var
  sn: Integer;

implementation

function TimeNames.readSerialNumber: Integer;
var
  r: Integer;
  s: String;
  serialFile: TextFile;
begin
  try
    AssignFile(serialFile, SERIAL_TEXT);
    Reset(serialFile);
    ReadLn(serialFile, s);
    sn := sequence_to_integer(s);
    r := sn;
    CloseFile(serialFile);
  except
    r := zeroSerialNumber;
  end;

  Result := r;
end;

function TimeNames.sequence_to_integer(s: String): Integer;
var
  k: Integer;
begin
  k := NOT_CONVERTED;

  try
    k := StrToInt(s);
  except
{ EConvertError }
    m := MessagesTexts.Create;
    messages := 'sequence_to_integer() error-191: ' + s;
    m.setting(ERROR_TEXT, messages);
    m.Free;
  end;

  Result := k;
end;

function TimeNames.temporaryName(serial_number: Boolean): String;
var
  combo: String;
  minutes: String;
  minutes_seconds: String;
  q: Integer;
  s: String;
  seconds: String;
  w: String;
begin
  s := DateTimeToStr(Now);
{*
2019/08/21 14:20:25
2019/08/21 14:2:5
2019/08/21 14:2:5:_ 
*}
  q := AnsiPos(COLON, s);
  if q >= 1 then
    begin
      q := q + 1;
      minutes := Copy(s, q, 2);
      q := AnsiPos(COLON, minutes);
      if q >= 1 then
        begin
          q := q - 1;
          minutes := Copy(minutes, 1, q);
        end;

    end;

  w := Concat(s, COLON_UNDERLINE);
  q := AnsiPos(COLON, w);
  if q >= 1 then
    begin
      q := q + 1;
      minutes_seconds := Copy(w, q, 5);
      q := AnsiPos(COLON, minutes_seconds);
      if q >= 1 then
        begin
          q := q + 1;
          seconds := Copy(minutes_seconds, q, 2);
          q := AnsiPos(COLON, seconds);
          if q >= 1 then
            begin
              q := q - 1;
              seconds := Copy(seconds, 1, q);
            end;

        end;

    end;

  if serial_number then
    incrementSerialNumber;

  combo := Concat(name, minutes, HYPHEN, seconds, extension);
  if CompareStr(combo, timeName) = 0 then
    begin
      incrementSerialNumber;
      combo := Concat(name, minutes, HYPHEN, seconds, extension);
    end;

  timeName := combo;
  Result := timeName;
end;

function TimeNames.zeroSerialNumber: Integer;
var
  s: String;
  serialFile: TextFile;
begin
  try
    AssignFile(serialFile, SERIAL_TEXT);
    Rewrite(serialFile);
    sn := 0;
    s := IntToStr(sn);
    Write(serialFile, s);
    CloseFile(serialFile);
  except
    m := MessagesTexts.Create;
    messages := 'Rewrite() error-192: ' + SERIAL_TEXT;
    m.setting(ERROR_TEXT, messages);
    m.Free;
  end;

  Result := 0;
end;

procedure TimeNames.incrementSerialNumber;
var
  s: String;
  serialFile: TextFile;
begin
  try
    AssignFile(serialFile, SERIAL_TEXT);
    Rewrite(serialFile);
    sn := sn + 1;
    s := IntToStr(sn);
    Write(serialFile, s);
    CloseFile(serialFile);
  except
    m := MessagesTexts.Create;
    messages := 'Rewrite() error-193: ' + s;
    m.setting(ERROR_TEXT, messages);
    m.Free;
  end;

  name := Concat(base, IntToStr(sn), HYPHEN);
end;

procedure TimeNames.setting(b: String; e: String);
begin
  base := b;
  extension := e;
  readSerialNumber;
end;

end.
