unit CaseSensitiveDescendingUnique;

{$I+}

interface

uses
  Windows, SysUtils, Classes, FinalEffect, ConvertUTF16, ConvertWide, ConvertWideEUC, Utf16intoEUC, ReportID;

type AnsiCharArray = array[0..4095] of AnsiChar;
type CharArray = array[0..4095] of Char;

type
  CaseSensitiveDescendingUniqueBlock = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    ANSI_CHAR: array[0..255] of AnsiChar;
    beginning: Integer;
    buffer: array of Char;
    buffer_length: Integer;
    carriage_return: Boolean;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_Y_ARRAY: CharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    comboLength: Integer;
    count: Integer;
    file_name: String;
    GAP_C: Char;
    GAP_S: String;
    joint_byte: Byte;
    h: CheckID;
    H_07_C: Char;
    H_0A_C: Char;
    H_0D_C: Char;
    H_1F_C: Char;
    H_20_ANSI: AnsiChar;
    H_20_C: Char;
    H_21_ANSI: AnsiChar;
    H_21_C: Char;
    H_FE_C: AnsiChar;
    H_FF_C: AnsiChar;
    L: TStringList;
    new_name: String;
    NEW_LINE: String;
    NEW_LINE_LENGTH: Integer;
    report_ID: Boolean;
    shift_jis_text: Boolean;
    summary_size: Integer;
    upper_limit: Integer;
    UTF16_text: Boolean;
    v: TStringList;
    y: CharArray;
    Y_COUNT: Integer;
    Y_COUNT_1: Integer;
    function ANSI_CHAR_table: AnsiChar;
    function bufferingWriting(comboLength: Integer): Integer;
    function copyingStringToArray(s: PChar; s_length: Integer): Integer;
    function file_size(name: String): Integer;
    function read_EUC_lines_to_produce_list: Integer;
    function read_lines_to_produce_list: Integer;
    function read_UTF16_lines_to_produce_list: Integer;
    function setBufferSize: Integer;
    function summarySize(s: String): Integer; overload;
    function summarySize(s: String; k: Integer): Integer; overload;
  public
    destructor Destroy; override;
    function arranging_lines_in_descending_order: Integer;
    function eight_bit_character(b: Byte): AnsiChar;
    function newLine: String;
    function sequence_to_integer(s: String): Integer;
    function wideSpace(size: Integer): String;
    procedure setting(nn: String; cr: Boolean; ri: Boolean; ut: Boolean); overload;
    procedure setting(e: FinalEffects); overload;
  end;

{*
123456789012345678901
-922337203685477.5808
*}
const
  BOM_COUNT = 1;
  BUFFER_ERROR = -1;
  CHARACTERS_BYTES = 16; { size of 16 characters }
  COMBO_ERROR = -3;
  CURRENCY_COUNT = 21;
  EMPTY = '';
  FINAL_INTEGER = 2147483647; { 7FFFFFFFh }
  GAP_SIZE = 1;
  H_000000FF = 255;
  H_07 = 7; { BEL }
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_1F = 31;
  H_20 = 32;
  H_21 = 33;
  H_FE = 254;
  H_FF = 255;
  JOINT_SIZE = 1;
  NOT_CONVERTED = -1;
  NOT_FOUND = -1;
  OFFSET = 930; { SEGMENT_SIZE * ( 20 + 1 ) * 2 + TAG_AREA_SIZE + 1 }
  OFFSET_1 = 929; { OFFSET - 1 }
  OFFSET_2 = 928; { OFFSET - 2 }
  SEGMENT_SIZE = 22; { CURRENCY_COUNT + JOINT_SIZE }
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 2;
  SUTAI_TEXT = 'sutai.txt';
  TAG_AREA_SIZE = 5; { Length( ' 462 '), Length( ' 483 '), for example }
  TAG_AREA_SIZE_2 = 3; { TAG_AREA_SIZE - 2 }

implementation

destructor CaseSensitiveDescendingUniqueBlock.Destroy;
begin
  buffer := nil;
  combo := nil;
  h.Free;
  L.Free;
  v.Free;

  inherited Destroy;
end;

function CaseSensitiveDescendingUniqueBlock.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function CaseSensitiveDescendingUniqueBlock.arranging_lines_in_descending_order: Integer;
var
  binaryFile: File;
  block_has_been_written: Integer;
  c: ConvertUtf16intoEUC;
  d: Char;
  details: String;
  details_bytes: Integer;
  details_size: Integer;
  e: AnsiChar;
  exit_code: Integer;
  from: Integer;
  g: Converting;
  i: Longint;
  k: Integer;
  line_number: Integer;
  m: Integer;
  n: Integer;
  new_name_length: Integer;
  newFile: TextFile;
  pascal_string: String;
  previous: String;
  s: String;
  s_length: Integer;
  sum: Integer;
  sum_1: Integer;
  summary: String;
  t: FinalEffects;
  v_count_1: Integer;
begin
  combo[0] := H_FF_C;
  combo[1] := H_FE_C;
  comboLength := 2;
  exit_code := 0;
  sum := 0;

  if UTF16_text then
    begin
      read_UTF16_lines_to_produce_list;
    end
  else
    begin
      if shift_jis_text then
        begin
          read_lines_to_produce_list;
        end
      else
        begin
          read_EUC_lines_to_produce_list;
        end;

    end;

  v.Sort;
  v_count_1 := v.Count - 1;

{ Producing a new list }
  previous := EMPTY + H_07_C;
  for k := v_count_1 downto 0 do
  begin
    s := v[k];
    s_length := Length(s); { summary_size + TAG_AREA_SIZE + details_size = s_length }
    summary_size := summarySize(s, OFFSET_2);
    summary := Copy(s, 1, summary_size);

    if CompareStr(summary, previous) = 0 then
      begin
        line_number := k + 1;
        h.setting(line_number);
        continue;
      end
    else
      begin
        details_size := s_length - summary_size - TAG_AREA_SIZE;
        details := Copy(s, OFFSET, details_size);
        L.Add(details);
        Inc(sum);
      end;

    previous := summary;
  end;

  sum_1 := sum - 1;

  new_name_length := Length(new_name);
  if new_name_length = 0 then
    begin
      Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
      Y_COUNT_1 := Y_COUNT - 1;
      for n := 0 to Y_COUNT_1 do
      begin
        CLEAR_Y_ARRAY[n] := #0;
      end;

      for k := 0 to sum_1 do
      begin
        s := L[k];
        s_length := Length(s);

        y := CLEAR_Y_ARRAY;
        m := 0;
        for n := 1 to s_length do
        begin
          if m > Y_COUNT_1 then
            begin
              break;
            end;

          d := s[n];
          i := Ord(d);

          if i = H_20 then
            begin
              y[m] := H_21_C;
              Inc(m);
            end
          else
            begin
              if i = H_21 then
                begin
                  y[m] := H_20_C;
                  Inc(m);
                end
              else
                begin
                  y[m] := d;
                  Inc(m);
                end;

            end;

        end;

        details_size := m;
        pascal_string := y;
        details := Copy(pascal_string, 1, details_size);
        WriteLn(details);
      end;

    end
  else
    begin
      if UTF16_text then
        begin
          for k := 0 to sum_1 do
          begin
            details := L[k];
            details := details + H_0D_C;
            details := details + H_0A_C;

            details_bytes := copyingStringToArray( PChar(details), Length(details) );
            comboLength := comboLength + details_bytes;
          end;

          bufferingWriting(comboLength);
        end
      else
        begin
          if shift_jis_text then
            begin
              if carriage_return then
                begin
                  Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
                  Y_COUNT_1 := Y_COUNT - 1;
                  for n := 0 to Y_COUNT_1 do
                  begin
                    CLEAR_Y_ARRAY[n] := #0;
                  end;

                  AssignFile(newFile, new_name);
                  Rewrite(newFile);

                  for k := 0 to sum_1 do
                  begin
                    s := L[k];
                    s_length := Length(s);

                    y := CLEAR_Y_ARRAY;
                    m := 0;
                    for n := 1 to s_length do
                    begin
                      if m > Y_COUNT_1 then
                        begin
                          break;
                        end;

                      d := s[n];
                      i := Ord(d);

                      if i = H_20 then
                        begin
                          y[m] := H_21_C;
                          Inc(m);
                        end
                      else
                        begin
                          if i = H_21 then
                            begin
                              y[m] := H_20_C;
                              Inc(m);
                            end
                          else
                            begin
                              y[m] := d;
                              Inc(m);
                            end;

                        end;

                    end;

                    details_size := m;
                    pascal_string := y;
                    details := Copy(pascal_string, 1, details_size);
                    WriteLn(newFile, details);
                  end;

                  CloseFile(newFile);
                end
              else
                begin
                  t := FinalEffects.Create;
                  t.setting;
                  g := Converting.Create;
                  g.setting(t);

                  AssignFile(binaryFile, new_name);
                  Rewrite(binaryFile, SIZE_OF_ANSICHAR);

                  for k := 0 to sum_1 do
                  begin
                    s := L[k];
                    s := s + H_0A_C;
                    g.produce_MS932_phrase( PChar(s), Length(s) );

                    A_SIZE := g.MS932_phrase_size;
                    A_SIZE_1 := A_SIZE - 1;
                    for n := 0 to A_SIZE_1 do
                    begin
                      e := g.copyingArrayItem(n);
                      i := Ord(e);

                      if i = H_20 then
                        begin
                          a[n] := H_21_ANSI;
                        end
                      else
                        begin
                          if i = H_21 then
                            begin
                              a[n] := H_20_ANSI;
                            end
                          else
                            begin
                              a[n] := e;
                            end;

                        end;

                    end;

                    BlockWrite(binaryFile, a, A_SIZE, block_has_been_written);
                  end;

                  CloseFile(binaryFile);
                  g.Free; t.Free;
                end;

            end
          else
            begin
              if carriage_return then
                begin
                  NEW_LINE := EMPTY + H_0D_C;
                  NEW_LINE := NEW_LINE + H_0A_C;
                end
              else
                begin
                  NEW_LINE := EMPTY + H_0A_C;
                end;

              t := FinalEffects.Create;
              t.setting;
              c := ConvertUtf16intoEUC.Create;
              c.setting(t);

              AssignFile(binaryFile, new_name);
              Rewrite(binaryFile, SIZE_OF_ANSICHAR);

              for k := 0 to sum_1 do
              begin
                s := L[k];
                s := s + NEW_LINE;
                c.produce_EUC_phrase( PChar(s), Length(s) );

                A_SIZE := c.EUC_phrase_size;
                A_SIZE_1 := A_SIZE - 1;
                for n := 0 to A_SIZE_1 do
                begin
                  e := c.copyingArrayItem(n);
                  i := Ord(e);

                  if i = H_20 then
                    begin
                      a[n] := H_21_ANSI;
                    end
                  else
                    begin
                      if i = H_21 then
                        begin
                          a[n] := H_20_ANSI;
                        end
                      else
                        begin
                          a[n] := e;
                        end;

                    end;

                end;

                BlockWrite(binaryFile, a, A_SIZE, block_has_been_written);
              end;

              CloseFile(binaryFile);
              c.Free; t.Free;
            end;

        end;

    end;

  if report_ID then
    h.displayID;

  Result := exit_code;
end;

function CaseSensitiveDescendingUniqueBlock.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  c: AnsiChar;
  exit_code: Integer;
  i: Longint;
  k: Integer;
  n: Integer;
  newFile: File;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;

      for k := 0 to A_SIZE_1 do
      begin
        c := combo[n];
        i := Ord(c);

        if i = H_20 then
          begin
            a[k] := H_21_ANSI;
          end
        else
          begin
            if i = H_21 then
              begin
                a[k] := H_20_ANSI;
              end
            else
              begin
                a[k] := c;
              end;

          end;

        Inc(n);
        if n >= comboLength then
          begin
            A_SIZE := k + 1;
            break;
          end;

      end;

      BlockWrite(newFile, a, A_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    A_SIZE := SizeOf(a);
    A_SIZE_1 := A_SIZE - 1;
  except
    Halt(1603);
  end;

  Result := exit_code;
end;

function CaseSensitiveDescendingUniqueBlock.copyingStringToArray(s: PChar; s_length: Integer): Integer;
var
  b: Byte;
  i: Longint;
  k: Integer;
  m: Integer;
  s_length_1: Integer;
begin
  m := comboLength;
  s_length_1 := s_length - 1;

  for k := 0 to s_length_1 do
  begin
    i := Ord(s[k]);
    b := H_000000FF and i; { little-endian architecture }
    combo[m] := ANSI_CHAR[b];

    i := i shr 8;
    b := H_000000FF and i;
    combo[ m + 1 ] := ANSI_CHAR[b];

    Inc(m, SIZE_OF_CHAR);
  end;

  Result := m - comboLength;
end;

function CaseSensitiveDescendingUniqueBlock.eight_bit_character(b: Byte): AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
begin
  beginning_of_ASCII := 0;

  if b >= 1 then
    begin
      end_1 := b - 1;
      h := AnsiChar( Chr(beginning_of_ASCII) );

      for k := beginning_of_ASCII to end_1 do
      begin
        Inc(h);
      end;

    end
  else
    begin
      h := AnsiChar(#0);
    end;

  Result := h;
end;

function CaseSensitiveDescendingUniqueBlock.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function CaseSensitiveDescendingUniqueBlock.newLine: String;
begin
  Result := NEW_LINE;
end;

function CaseSensitiveDescendingUniqueBlock.read_EUC_lines_to_produce_list: Integer;
var
  block_has_been_read: Integer;
  combo_size_1: Integer;
  c: AnsiChar;
  characters_bytes_summary_size: Integer;
  details: String;
  details_size: Integer;
  from: Integer;
  i: Longint;
  j: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  pascal_unicode_string: String;
  s: String;
  summary: String;
  t: FinalEffects;
  testFile: File;
  u: ConvertEUCintoUTF16;
  z: String;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  carriage_return_appears := False;
  combo_size_1 := combo_size - 1;
  k := 0;
  m := 0;

  for n := 0 to combo_size_1 do
  begin
    combo[n] := #0;
  end;

  for n := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n := 0 to A_SIZE_1 do
      begin
        combo[k] := a[n];
        Inc(k);
        if k > combo_size_1 then
          break;

      end;
    until k > combo_size_1;

    CloseFile(testFile);
  except
    Halt(1605);
  end;

  t := FinalEffects.Create;
  t.setting;
  u := ConvertEUCintoUTF16.Create;
  u.setting(t);

{ Finding the first NEW_LINE }
  for n := beginning to combo_size_1 do
  begin
    if m >= A_SIZE_1 then
      begin
        break;
      end;

    c := combo[n];
    i := Ord(c);

    if i = H_0A then
      begin
        count := 1;

        pascal_string := a;
        u.produce_UTF16_phrase( PAnsiChar(pascal_string), m);
        Y_COUNT := u.UTF16_phrase_size;
        Y_COUNT_1 := Y_COUNT - 1;
        for k := 0 to Y_COUNT_1 do
        begin
          y[k] := u.copyingArrayItem(k);
        end;

        pascal_unicode_string := y;
        s := Copy(pascal_unicode_string, 1, Y_COUNT);

        summary_size := summarySize(s);
        characters_bytes_summary_size := CHARACTERS_BYTES + summary_size;
        details_size := Y_COUNT - characters_bytes_summary_size;
        from := characters_bytes_summary_size + 1;
        details := Copy(s, from, details_size);

        from := CHARACTERS_BYTES + 1;
        summary := Copy(s, from, summary_size);
        z := summary;

        j := OFFSET_1 - summary_size - TAG_AREA_SIZE;
        z := z + wideSpace(j);

        z := z + GAP_C;
        z := z + IntToStr(summary_size);
        z := z + GAP_C;

        z := z + details;

        v.Add(z);
        m := 0;

        beginning := n + 1;
        break;
      end
    else
      begin
        if i = H_0D then
          begin
            carriage_return_appears := True;
            NEW_LINE_LENGTH := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            if i = H_20 then
              begin
                GAP_C := H_21_C;
                GAP_S := '!';
                a[m] := H_21_ANSI;
                Inc(m);
              end
            else
              begin
                if i = H_21 then
                  begin
                    a[m] := H_20_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if i = joint_byte then
                      begin
                        a[m] := H_20_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        a[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end;

      end;

  end; { for n := beginning to combo_size_1 do }

{ Counting lines }
  for n := beginning to combo_size_1 do
  begin
    if m >= A_SIZE_1 then
      begin
        break;
      end;

    c := combo[n];
    i := Ord(c);

    if i = H_0A then
      begin
        Inc(count);
        if count > upper_limit then
          begin
            break;
          end;

        pascal_string := a;
        u.produce_UTF16_phrase( PAnsiChar(pascal_string), m);
        Y_COUNT := u.UTF16_phrase_size;
        Y_COUNT_1 := Y_COUNT - 1;
        for k := 0 to Y_COUNT_1 do
        begin
          y[k] := u.copyingArrayItem(k);
        end;

        pascal_unicode_string := y;
        s := Copy(pascal_unicode_string, 1, Y_COUNT);

        summary_size := summarySize(s);
        characters_bytes_summary_size := CHARACTERS_BYTES + summary_size;
        details_size := Y_COUNT - characters_bytes_summary_size;
        from := characters_bytes_summary_size + 1;
        details := Copy(s, from, details_size);

        from := CHARACTERS_BYTES + 1;
        summary := Copy(s, from, summary_size);
        z := summary;

        j := OFFSET_1 - summary_size - TAG_AREA_SIZE;
        z := z + wideSpace(j);

        z := z + GAP_C;
        z := z + IntToStr(summary_size);
        z := z + GAP_C;

        z := z + details;

        v.Add(z);
        m := 0;
      end
    else
      begin
        if i = H_0D then
          begin
            continue;
          end
        else
          begin
            if i = H_20 then
              begin
                a[m] := H_21_ANSI;
                Inc(m);
              end
            else
              begin
                if i = H_21 then
                  begin
                    a[m] := H_20_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if i = joint_byte then
                      begin
                        a[m] := H_20_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        a[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end;

      end;

  end; { for n := beginning to combo_size_1 do }

  u.Free; t.Free;
  Result := count;
end;

function CaseSensitiveDescendingUniqueBlock.read_lines_to_produce_list: Integer;
var
  block_has_been_read: Integer;
  combo_size_1: Integer;
  c: AnsiChar;
  characters_bytes_summary_size: Integer;
  details: String;
  details_size: Integer;
  from: Integer;
  i: Longint;
  j: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  pascal_unicode_string: String;
  s: String;
  summary: String;
  t: FinalEffects;
  testFile: File;
  u: ConvertWiden;
  z: String;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  carriage_return_appears := False;
  combo_size_1 := combo_size - 1;
  k := 0;
  m := 0;

  for n := 0 to combo_size_1 do
  begin
    combo[n] := #0;
  end;

  for n := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n := 0 to A_SIZE_1 do
      begin
        combo[k] := a[n];
        Inc(k);
        if k > combo_size_1 then
          break;

      end;
    until k > combo_size_1;

    CloseFile(testFile);
  except
    Halt(1606);
  end;

  t := FinalEffects.Create;
  t.setting;
  u := ConvertWiden.Create;
  u.setting(t);

{ Finding the first NEW_LINE }
  for n := beginning to combo_size_1 do
  begin
    if m >= A_SIZE_1 then
      begin
        break;
      end;

    c := combo[n];
    i := Ord(c);

    if i = H_0A then
      begin
        count := 1;

        pascal_string := a;
        u.produce_UTF16_phrase( PAnsiChar(pascal_string), m);
        Y_COUNT := u.UTF16_phrase_size;
        Y_COUNT_1 := Y_COUNT - 1;
        for k := 0 to Y_COUNT_1 do
        begin
          y[k] := u.copyingArrayItem(k);
        end;

        pascal_unicode_string := y;
        s := Copy(pascal_unicode_string, 1, Y_COUNT);

        summary_size := summarySize(s);
        characters_bytes_summary_size := CHARACTERS_BYTES + summary_size;
        details_size := Y_COUNT - characters_bytes_summary_size;
        from := characters_bytes_summary_size + 1;
        details := Copy(s, from, details_size);

        from := CHARACTERS_BYTES + 1;
        summary := Copy(s, from, summary_size);
        z := summary;

        j := OFFSET_1 - summary_size - TAG_AREA_SIZE;
        z := z + wideSpace(j);

        z := z + GAP_C;
        z := z + IntToStr(summary_size);
        z := z + GAP_C;

        z := z + details;

        v.Add(z);
        m := 0;

        beginning := n + 1;
        break;
      end
    else
      begin
        if i = H_0D then
          begin
            carriage_return_appears := True;
            NEW_LINE_LENGTH := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            if i = H_20 then
              begin
                GAP_C := H_21_C;
                GAP_S := '!';
                a[m] := H_21_ANSI;
                Inc(m);
              end
            else
              begin
                if i = H_21 then
                  begin
                    a[m] := H_20_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if i = joint_byte then
                      begin
                        a[m] := H_20_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        a[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end;

      end;

  end; { for n := beginning to combo_size_1 do }

{ Counting lines }
  for n := beginning to combo_size_1 do
  begin
    if m >= A_SIZE_1 then
      begin
        break;
      end;

    c := combo[n];
    i := Ord(c);

    if i = H_0A then
      begin
        Inc(count);
        if count > upper_limit then
          begin
            break;
          end;

        pascal_string := a;
        u.produce_UTF16_phrase( PAnsiChar(pascal_string), m);
        Y_COUNT := u.UTF16_phrase_size;
        Y_COUNT_1 := Y_COUNT - 1;
        for k := 0 to Y_COUNT_1 do
        begin
          y[k] := u.copyingArrayItem(k);
        end;

        pascal_unicode_string := y;
        s := Copy(pascal_unicode_string, 1, Y_COUNT);

        summary_size := summarySize(s);
        characters_bytes_summary_size := CHARACTERS_BYTES + summary_size;
        details_size := Y_COUNT - characters_bytes_summary_size;
        from := characters_bytes_summary_size + 1;
        details := Copy(s, from, details_size);

        from := CHARACTERS_BYTES + 1;
        summary := Copy(s, from, summary_size);
        z := summary;

        j := OFFSET_1 - summary_size - TAG_AREA_SIZE;
        z := z + wideSpace(j);

        z := z + GAP_C;
        z := z + IntToStr(summary_size);
        z := z + GAP_C;

        z := z + details;

        v.Add(z);
        m := 0;
      end
    else
      begin
        if i = H_0D then
          begin
            continue;
          end
        else
          begin
            if i = H_20 then
              begin
                a[m] := H_21_ANSI;
                Inc(m);
              end
            else
              begin
                if i = H_21 then
                  begin
                    a[m] := H_20_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if i = joint_byte then
                      begin
                        a[m] := H_20_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        a[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end;

      end;

  end; { for n := beginning to combo_size_1 do }

  u.Free; t.Free;
  Result := count;
end;

function CaseSensitiveDescendingUniqueBlock.read_UTF16_lines_to_produce_list: Integer;
var
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: Char;
  characters_bytes_summary_size: Integer;
  details: String;
  details_size: Integer;
  from: Integer;
  i: Longint;
  j: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  pascal_string: String;
  s: String;
  summary: String;
  testFile: File;
  z: String;
begin
  Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
  Y_COUNT_1 := Y_COUNT - 1;
  carriage_return_appears := False;
  buffer_length_1 := buffer_length - 1;
  k := 0;
  m := 0;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  for n := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_CHAR);

    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n := 0 to Y_COUNT_1 do
      begin
        buffer[k] := y[n];
        Inc(k);
        if k > buffer_length_1 then
          break;

      end;
    until k > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1607);
  end;

{ Finding the first NEW_LINE }
  for n := beginning to buffer_length_1 do
  begin
    if m >= Y_COUNT_1 then
      begin
        break;
      end;

    c := buffer[n];
    i := Ord(c);

    if i = H_0A then
      begin
        count := 1;

        Y_COUNT := m;
        pascal_string := y;
        s := Copy(pascal_string, 1, Y_COUNT);

        summary_size := summarySize(s);
        characters_bytes_summary_size := CHARACTERS_BYTES + summary_size;
        details_size := Y_COUNT - characters_bytes_summary_size;
        from := characters_bytes_summary_size + 1;
        details := Copy(s, from, details_size);

        from := CHARACTERS_BYTES + 1;
        summary := Copy(s, from, summary_size);
        z := summary;

        j := OFFSET_1 - summary_size - TAG_AREA_SIZE;
        z := z + wideSpace(j);

        z := z + GAP_C;
        z := z + IntToStr(summary_size);
        z := z + GAP_C;

        z := z + details;

        v.Add(z);
        m := 0;

        beginning := n + 1;
        break;
      end
    else
      begin
        if i = H_0D then
          begin
            carriage_return_appears := True;
            NEW_LINE_LENGTH := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            if i = H_20 then
              begin
                GAP_C := H_21_C;
                GAP_S := '!';
                y[m] := H_21_C;
                Inc(m);
              end
            else
              begin
                if i = H_21 then
                  begin
                    y[m] := H_20_C;
                    Inc(m);
                  end
                else
                  begin
                    if i = joint_byte then
                      begin
                        y[m] := H_20_C;
                        Inc(m);
                      end
                    else
                      begin
                        y[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end;

      end;

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  for n := beginning to buffer_length_1 do
  begin
    if m >= Y_COUNT_1 then
      begin
        break;
      end;

    c := buffer[n];
    i := Ord(c);

    if i = H_0A then
      begin
        Inc(count);
        if count > upper_limit then
          begin
            break;
          end;

        Y_COUNT := m;
        pascal_string := y;
        s := Copy(pascal_string, 1, Y_COUNT);

        summary_size := summarySize(s);
        characters_bytes_summary_size := CHARACTERS_BYTES + summary_size;
        details_size := Y_COUNT - characters_bytes_summary_size;
        from := characters_bytes_summary_size + 1;
        details := Copy(s, from, details_size);

        from := CHARACTERS_BYTES + 1;
        summary := Copy(s, from, summary_size);
        z := summary;

        j := OFFSET_1 - summary_size - TAG_AREA_SIZE;
        z := z + wideSpace(j);

        z := z + GAP_C;
        z := z + IntToStr(summary_size);
        z := z + GAP_C;

        z := z + details;

        v.Add(z);
        m := 0;
      end
    else
      begin
        if i = H_0D then
          begin
            continue;
          end
        else
          begin
            if i = H_20 then
              begin
                y[m] := H_21_C;
                Inc(m);
              end
            else
              begin
                if i = H_21 then
                  begin
                    y[m] := H_20_C;
                    Inc(m);
                  end
                else
                  begin
                    if i = joint_byte then
                      begin
                        y[m] := H_20_C;
                        Inc(m);
                      end
                    else
                      begin
                        y[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end;

      end;

  end; { for n := beginning to buffer_length_1 do }

  Result := count;
end;

function CaseSensitiveDescendingUniqueBlock.sequence_to_integer(s: String): Integer;
var
  r: Integer;
begin
  r := NOT_CONVERTED;

  try
    r := StrToInt(s);
  except
    Halt(1608);
  end;

  Result := r;
end;

function CaseSensitiveDescendingUniqueBlock.setBufferSize: Integer;
var
  r: Integer;
begin
  r := buffer_length;

  try
    SetLength(buffer, buffer_length);
  except
    Halt(1609);
  end;

  try
    SetLength(combo, combo_size);
  except
    Halt(1610);
  end;

  Result := r;
end;

function CaseSensitiveDescendingUniqueBlock.summarySize(s: String): Integer;
var
  c: Char;
  head: String;
  n: Integer;
  r: Integer;
begin
  r := 0;

  for n := 1 to CHARACTERS_BYTES do
  begin
    c := s[n];
    if c = GAP_C then
      begin
        continue;
      end
    else
      begin
        r := n;
        break;
      end;
  
  end;

  if r = 1 then
    begin
      for n := r to CHARACTERS_BYTES do
      begin
        c := s[n];
        if c = GAP_C then
          begin
            r := n - 1;
            break;
          end
        else
          begin
            continue;
          end;

      end;

      head := Copy(s, 1, r);
    end
  else
    begin
      Halt(1604);
    end;

  Result := sequence_to_integer(head);
end;

function CaseSensitiveDescendingUniqueBlock.summarySize(s: String; k: Integer): Integer;
var
  c: Char;
  head: String;
  n: Integer;
  r: Integer;
begin
  r := 0;

  for n := k downto 1 do
  begin
    c := s[n];
    if c = GAP_C then
      begin
        r := n;
        break;
      end
    else
      begin
        continue;
      end;
  
  end;

  if r = 0 then
    begin
      Halt(1776);
    end
  else
    begin
      r := r + GAP_SIZE;
      head := Copy(s, r, TAG_AREA_SIZE_2);
    end;

  Result := sequence_to_integer(head);
end;

function CaseSensitiveDescendingUniqueBlock.wideSpace(size: Integer): String;
var
  z: String;
begin
  z := StringOfChar(GAP_C, size);
  Result := z;
end;

procedure CaseSensitiveDescendingUniqueBlock.setting(nn: String; cr: Boolean; ri: Boolean; ut: Boolean);
var
  b: array[0..1] of Byte;
  block_has_been_read: Integer;
  bytes: Integer;
  i: Integer;
  testFile: File;
begin
  carriage_return := cr;
  joint_byte := H_1F;
  new_name := nn;
  report_ID := ri;
  shift_jis_text := True;
  upper_limit := FINAL_INTEGER;
  UTF16_text := ut;

  file_name := SUTAI_TEXT;
  GAP_C := Chr(H_20);
  GAP_S := ' ';
  H_07_C := Chr(H_07);
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  H_1F_C := Chr(H_1F);
  H_20_C := Chr(H_20);
  H_21_C := Chr(H_21);
  H_FF_C := ANSI_CHAR_table;
  H_20_ANSI := ANSI_CHAR[H_20];
  H_21_ANSI := ANSI_CHAR[H_21];
  H_FE_C := ANSI_CHAR[H_FE];

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, a, A_SIZE, block_has_been_read);
    b[0] := Ord(a[0]);
    b[1] := Ord(a[1]);
    if (b[0] = H_FF) and (b[1] = H_FE) then
      beginning := BOM_COUNT
    else
      beginning := 0;

    CloseFile(testFile);

    bytes := file_size(file_name);
    buffer_length := bytes div SIZE_OF_CHAR;
    combo_size := bytes;
  except
    Halt(1611);
  end;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  setBufferSize;
  h := CheckID.Create;
  h.presetting;
  v := TStringList.Create;
  v.CaseSensitive := True;
  v.Sorted := False; 
  L := TStringList.Create;
  L.CaseSensitive := True;
  L.Sorted := False; 
end;

procedure CaseSensitiveDescendingUniqueBlock.setting(e: FinalEffects);
var
  b: array[0..1] of Byte;
  block_has_been_read: Integer;
  bytes: Integer;
  i: Integer;
  testFile: File;
begin
  carriage_return := e.carriageReturn;
  joint_byte := e.jointByte;
  new_name := e.newName;
  report_ID := e.reportLineNumbers;
  shift_jis_text := e.Shift_JIS_file;
  upper_limit := e.maximumLineCount;
  UTF16_text := e.UTF16_file;

  file_name := SUTAI_TEXT;
  GAP_C := Chr(H_20);
  GAP_S := ' ';
  H_07_C := Chr(H_07);
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  H_1F_C := Chr(H_1F);
  H_20_C := Chr(H_20);
  H_21_C := Chr(H_21);
  H_FF_C := ANSI_CHAR_table;
  H_20_ANSI := ANSI_CHAR[H_20];
  H_21_ANSI := ANSI_CHAR[H_21];
  H_FE_C := ANSI_CHAR[H_FE];

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, a, A_SIZE, block_has_been_read);
    b[0] := Ord(a[0]);
    b[1] := Ord(a[1]);
    if (b[0] = H_FF) and (b[1] = H_FE) then
      beginning := BOM_COUNT
    else
      beginning := 0;

    CloseFile(testFile);

    bytes := file_size(file_name);
    buffer_length := bytes div SIZE_OF_CHAR;
    combo_size := bytes;
  except
    Halt(1612);
  end;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  setBufferSize;
  h := CheckID.Create;
  h.presetting;
  v := TStringList.Create;
  v.CaseSensitive := True;
  v.Sorted := False; 
  L := TStringList.Create;
  L.CaseSensitive := True;
  L.Sorted := False; 
end;

end.
