unit FinalEffect;

interface

uses
  SysUtils;

type
  FinalEffects = class
  private
    carriage_return: Boolean;
    case_insensitive: Boolean;
    descending: Boolean;
    error_level: Integer;
    euc_jp: Boolean;
    file_name: String;
    joint_byte: Byte;
    new_name: String;
    maximum_number_of_lines: Integer;
    no_removing: Boolean;
    no_searching: Boolean;
    path_name: String;
    produce_file: Boolean;
    report_ID: Boolean;
    selected_directory_has_priority: Boolean;
    shift_jis_text: Boolean;
    surrogate_pair_to_underline: Boolean;
    unique_summary: Boolean;
    UTF16_text: Boolean;
    which_table: Boolean;
  public
    function carriageReturn: Boolean;
    function notCaseSensitive: Boolean;
    function descendingOrder: Boolean;
    function errorLevel: Integer;
    function EUC_JP_file: Boolean;
    function fileName: String;
    function jointByte: Byte;
    function maximumLineCount: Integer;
    function newName: String;
    function noRemoving: Boolean;
    function noSearching: Boolean;
    function pathName: String;
    function priority: Boolean;
    function produceNewFile: Boolean;
    function reportLineNumbers: Boolean;
    function Shift_JIS_file: Boolean;
    function surrogatePairToUnderline: Boolean;
    function uniqueSummary: Boolean;
    function UTF16_file: Boolean;
    function whichTable: Boolean;
    procedure setting; overload;
    procedure setting(fn: String; nn: String; pn: String;
      ML: Integer; EL: Integer; 
      jb: Byte; 
      cr: Boolean; cs: Boolean; d: Boolean; ej: Boolean; nr: Boolean; ns: Boolean; pf: Boolean; ri: Boolean; sd: Boolean; sj: Boolean; us: Boolean; sp: Boolean; ut: Boolean; wt: Boolean); overload;

  end;

const
  FINAL_INTEGER = 2147483647; { 7FFFFFFFh }

implementation

function FinalEffects.carriageReturn: Boolean;
begin
  Result := carriage_return;
end;

function FinalEffects.notCaseSensitive: Boolean;
begin
  Result := case_insensitive;
end;

function FinalEffects.descendingOrder: Boolean;
begin
  Result := descending;
end;

function FinalEffects.errorLevel: Integer;
begin
  Result := error_level;
end;

function FinalEffects.EUC_JP_file: Boolean;
begin
  Result := euc_jp;
end;

function FinalEffects.fileName: String;
begin
  Result := file_name;
end;

function FinalEffects.jointByte: Byte;
begin
  Result := joint_byte;
end;

function FinalEffects.maximumLineCount: Integer;
begin
  Result := maximum_number_of_lines;
end;

function FinalEffects.newName: String;
begin
  Result := new_name;
end;

function FinalEffects.noRemoving: Boolean;
begin
  Result := no_removing;
end;

function FinalEffects.noSearching: Boolean;
begin
  Result := no_searching;
end;

function FinalEffects.pathName: String;
begin
  Result := path_name;
end;

function FinalEffects.priority: Boolean;
begin
  Result := selected_directory_has_priority;
end;

function FinalEffects.produceNewFile: Boolean;
begin
  Result := produce_file;
end;

function FinalEffects.reportLineNumbers: Boolean;
begin
  Result := report_ID;
end;

function FinalEffects.Shift_JIS_file: Boolean;
begin
  Result := shift_jis_text;
end;

function FinalEffects.surrogatePairToUnderline: Boolean;
begin
  Result := surrogate_pair_to_underline;
end;

function FinalEffects.uniqueSummary: Boolean;
begin
  Result := unique_summary;
end;

function FinalEffects.UTF16_file: Boolean;
begin
  Result := UTF16_text;
end;

function FinalEffects.whichTable: Boolean;
begin
  Result := which_table;
end;

procedure FinalEffects.setting;
begin
{ String }
  file_name := '';
  new_name := '';
  path_name := '';

{ Integer }
  maximum_number_of_lines := FINAL_INTEGER;
  error_level := 0;

{ Byte }
  joint_byte := 31;

{ Boolean }
  carriage_return := False;
  case_insensitive := False;
  descending := False;
  euc_jp := False;
  no_removing := False;
  no_searching := False;
  produce_file := False;
  report_ID := False;
  selected_directory_has_priority := False;
  shift_jis_text := False;
  surrogate_pair_to_underline := True;
  unique_summary := False;
  UTF16_text := False;
  which_table := False;
end;

procedure FinalEffects.setting(fn: String; nn: String; pn: String; 
  ML: Integer; EL: Integer; 
  jb: Byte; 
  cr: Boolean; cs: Boolean; d: Boolean; ej: Boolean; nr: Boolean; ns: Boolean; pf: Boolean; ri: Boolean; sd: Boolean; sj: Boolean; us: Boolean; sp: Boolean; ut: Boolean; wt: Boolean);

begin
{ String }
  file_name := fn;
  new_name := nn;
  path_name := pn;

{ Integer }
  maximum_number_of_lines := ML;
  error_level := EL;

{ Byte }
  joint_byte := jb;

{ Boolean }
  carriage_return := cr;
  case_insensitive := cs;
  descending := d;
  euc_jp := ej;
  no_removing := nr;
  no_searching := ns;
  produce_file := pf;
  report_ID := ri;
  selected_directory_has_priority := sd;
  shift_jis_text := sj;
  surrogate_pair_to_underline := sp;
  unique_summary := us;
  UTF16_text := ut;
  which_table := wt;
end;

end.
