program Hetn;
{$APPTYPE CONSOLE}

{*
Hetn Delphi Version 1.08
Reading sutai.txt to arrange lines in descending order, for example
Copyright 2020 Sogaya

Delphi 2009

*}

uses
  Windows,
  SysUtils,
  AnsiQueue in 'AnsiQueue.pas',
  Block in 'Block.pas',
  Combo in 'Combo.pas',
  ConvertUTF16 in 'ConvertUTF16.pas',
  Descending in 'Descending.pas',
  DescendingUnique in 'DescendingUnique.pas',
  Divide in 'Divide.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  PathEXE in 'PathEXE.pas',
  ReportID in 'ReportID.pas',
  SearchBinary in 'SearchBinary.pas',
  SurvivorUTF16 in 'SurvivorUTF16.pas',
  Ucombo in 'Ucombo.pas',
  Unique in 'Unique.pas',
  CaseSensitiveDescendingUnique in 'CaseSensitiveDescendingUnique.pas',
  CaseSensitiveUnique in 'CaseSensitiveUnique.pas',
  DivideSJIS in 'DivideSJIS.pas',
  ConvertWide in 'ConvertWide.pas',
  ConvertWideEUC in 'ConvertWideEUC.pas',
  RuralUTF16 in 'RuralUTF16.pas',
  PastoralUTF16 in 'PastoralUTF16.pas',
  Utf16intoEUC in 'Utf16intoEUC.pas',
  FieldsArray in 'FieldsArray.pas',
  FieldsArrayHalt in 'FieldsArrayHalt.pas',
  PutCSV in 'PutCSV.pas',
  ErrorMessage in 'ErrorMessage.pas';

{$R *.res}

const
  CRLF = True;
  DATA_TEXT = 'data.txt';
  DUMMY_FILE = 'dummy';
  EXCLAMATION_MARK = '!';
  SETTING_TEXT = 'manager.txt';
  STANDARD_OUTPUT = '';
  SURRO_CSV = 'surro.csv';
  UTF16_ENCODING_FOR_HETN = '-t';
  VERSION_NUMBER = '1.08';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;

var
  b: CurrencyBlock;
  c: DescendingUniqueBlock;
  e: DescendingBlock;
  EUC_text: Boolean;
  f: FinalEffects;
  file_name: String;
  g: Guiding;
  h: HyphenTerms;
  i: CaseSensitiveUniqueBlock;
  k: Integer;
  n: UniqueBlock;
  new_name: String;
  new_name_length: Integer;
  p: PutSurroCSV;
  produce_file: Boolean;
  q: CaseSensitiveDescendingUniqueBlock;
  r: ErrorMessages;
  s: String;
  shift_jis_text: Boolean;
  testFile: TextFile;
  UTF16_text: Boolean;
  v: Boolean;
begin
  v := False;

{ Checking if software can open SURRO_CSV in the current directory. }
  try
    AssignFile(testFile, SURRO_CSV);
    Reset(testFile);
    CloseFile(testFile);
  except
{ Producing SURRO_CSV }
    p := PutSurroCSV.Create;
    p.setting(SURRO_CSV);
  end;

  s := ParamStr(1);
  if Length(s) = 0 then
    begin
      try
        AssignFile(testFile, SETTING_TEXT);
        Reset(testFile);
        v := False;
        while not Eof(testFile) do
        begin
          ReadLn(testFile, s);
          k := AnsiPos(UTF16_ENCODING_FOR_HETN, s);
          if k = 1 then
            begin
              v := True;
              break;
            end
          else
            begin
              v := False;
            end;

        end;

        CloseFile(testFile);
      except
        WriteLn( 'File error: ' + SETTING_TEXT );
        Halt(1709);
      end;

      b := CurrencyBlock.Create;
      b.setting(STANDARD_OUTPUT, CRLF, v);
      b.arranging_lines_in_ascending_order;
      b.Free;
      Halt(0);
    end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          file_name := h.fileName;
          h.joints; { joint_byte }

          if h.UTF16_file then
            begin
              UTF16_text := True;
              EUC_text := False;
              shift_jis_text := False;
            end
          else
            begin
              UTF16_text := False;
              if h.EUC_JP_file then
                begin
                  if h.Shift_JIS_file then
                    begin
                      EUC_text := False;
                      shift_jis_text := True;
                    end
                  else
                    begin
                      EUC_text := true;
                      shift_jis_text := False;
                    end;

                end
              else
                begin
                  EUC_text := False;
                  shift_jis_text := True;
                end;

            end;

          f := FinalEffects.Create;
          new_name :=  h.listName(1);
          new_name_length := Length(new_name);
          if new_name_length > 0 then
            begin
              produce_file := True;
            end
          else
            begin
              produce_file := False;
            end;

          f.setting(h.fileName, new_name, h.pathName,
            h.maximumLineCount, h.errorLevel, 
            h.jointByte, 
            h.carriageReturn, h.notCaseSensitive, h.descendingOrder, EUC_text, h.noRemoving, h.noSearching, produce_file, h.reportLineNumbers, h.priority, shift_jis_text, h.uniqueSummary, h.surrogatePairToUnderline, UTF16_text, h.whichTable);

{ main function }
  if f.errorLevel > 1 then
    begin
      r := ErrorMessages.Create;
      r.guideNumber(VERSION_NUMBER, f.errorLevel);

      if Length(file_name) > 0 then
        begin
          if CompareStr(file_name, DUMMY_FILE) = 0 then
            begin
              ;
            end
          else
            begin
              if CompareStr(file_name, EXCLAMATION_MARK) = 0 then
                begin
                  ;
                end
              else
                begin
                  WriteLn('file_name = ' + f.fileName);
                end;

            end;

        end;

      if Length(new_name) > 0 then
        begin
          WriteLn('new_name = ' + f.newName);
        end;

      r.Free; h.Free;
      Halt(0);
    end;

  if f.descendingOrder then
    begin
      if f.uniqueSummary then
        begin
          if f.notCaseSensitive then
            begin
              c := DescendingUniqueBlock.Create;
              c.setting(f);
              c.arranging_lines_in_descending_order;
              c.Free;
            end
          else
            begin
              q := CaseSensitiveDescendingUniqueBlock.Create;
              q.setting(f);
              q.arranging_lines_in_descending_order;
              q.Free;
            end;

        end
      else
        begin
          e := DescendingBlock.Create;
          e.setting(f);
          e.arranging_lines_in_descending_order;
          e.Free;
        end;

    end
  else
    begin
      if f.uniqueSummary then
        begin
          if f.notCaseSensitive then
            begin
              n := UniqueBlock.Create;
              n.setting(f);
              n.arranging_lines_in_ascending_order;
              n.Free;
            end
          else
            begin
              i := CaseSensitiveUniqueBlock.Create;
              i.setting(f);
              i.arranging_lines_in_ascending_order;
              i.Free;
            end;

        end
      else
        begin
          b := CurrencyBlock.Create;
          b.setting(f);
          b.arranging_lines_in_ascending_order;
          b.Free;
        end;

    end;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
