unit Numbered;

interface

uses
  SysUtils;

type
  NumberedTerm = class
  private
    a_length: Integer;
    f: String;
    final: Integer;
    ft: String;
    non_number_length: Integer;
    pt: String;
    s: String;
    span: Integer;
    st: String;
    trimmed: Boolean;
    trimmed_sequence_length: Integer;
  public
    function number: Integer;
    function parameter: String;
    function sequence_to_integer(s: String): Integer;
    function subnumber: Integer;
    function trailingNumbers(full: String): Integer;
    function trimmedSequence: Boolean;
    procedure setting(a: String); overload;
    procedure setting(a: String; final_1: Integer); overload;
  end;

const
  EMPTY = '';
  FINAL_INTEGER = 2147483647; { 7FFFFFFFh }
  FINAL_TERM = '2147483647';
  NOT_CONVERTED = -1;
  NOT_FOUND = -1;
  N_0TO9 = '0123456789';
  TRIMMING = 1;

implementation

function NumberedTerm.trailingNumbers(full: String): Integer;
var
  c: String;
  fullLength: Integer;
  k: Integer;
  n: Integer;
  q: Integer;
begin
  fullLength := Length(full);
  n := 0;

  for k := fullLength downto 1 do
  begin
    c := Copy(full, k, 1);
    q := AnsiPos(c, N_0TO9);
    if q >= 1 then
      begin
        Inc(n);
        continue;
      end
    else
      begin
        break;
      end;

  end;

  Result := n;
end;

function NumberedTerm.number: Integer;
var
  k: Integer;
begin
  k := sequence_to_integer(s);
  Result := k;
end;

function NumberedTerm.parameter: String;
var
  s: String;
begin
  s := f;
  if trimmed then
    s := ft;

  Result := s;
end;

function NumberedTerm.sequence_to_integer(s: String): Integer;
var
  k: Integer;
begin
  k := NOT_CONVERTED;

  try
    k := StrToInt(s);
  except
{ EConvertError }
    Halt(1715);
  end;

  Result := k;
end;

function NumberedTerm.subnumber: Integer;
var
  k: Integer;
begin
  k := NOT_FOUND;

  if trimmed then
    k := sequence_to_integer(st);

  Result := k;
end;

function NumberedTerm.trimmedSequence: Boolean;
var
  from: Integer;
  span: Integer;
{*
a: -i1-37
first part of a: -i1-
second part of a: 37
trimmed sequence: -i1
first part of trimmed sequence: -i
second part of trimmed sequence: 1
*}
begin
{ Trimming the first part of a }
  trimmed_sequence_length := Length(f) - TRIMMING;
  pt := Copy(f, 1, trimmed_sequence_length);

  span := trailingNumbers(pt);
  if span = 0 then
    begin
      if final = FINAL_INTEGER then
        begin
          trimmed := False;
        end
      else
{ interval of 1 to 2147483646, for example }
        begin
          st := s;
          ft := f;
          s := IntToStr(final);
          trimmed := True;
        end;

    end
  else
    begin
      non_number_length := trimmed_sequence_length - span;

{ Copying the second part of trimmed sequence }
      from := non_number_length + 1;
      st := Copy(pt, from, span);

{ Copying the first part of trimmed sequence }
      ft := Copy(pt, 1, non_number_length);

      trimmed := True;
    end;

    Result := trimmed;
end;

procedure NumberedTerm.setting(a: String);
var
  from: Integer;
begin
  trimmed := False;
  final := FINAL_INTEGER;
  a_length := Length(a);

  span := trailingNumbers(a);
  non_number_length := a_length - span;

{ Copying the second part of a }
  from := non_number_length + 1;
  s := Copy(a, from, span);

{ Copying the first part of a }
  f := Copy(a, 1, non_number_length);
end;

procedure NumberedTerm.setting(a: String; final_1: Integer);
var
  from: Integer;
begin
  trimmed := False;
  final := final_1;
  a_length := Length(a);

  span := trailingNumbers(a);
  non_number_length := a_length - span;

{ Copying the second part of a }
  from := non_number_length + 1;
  s := Copy(a, from, span);

{ Copying the first part of a }
  f := Copy(a, 1, non_number_length);
end;

end.
