unit SurvivorUTF16;

interface

uses
  Windows, SysUtils, PathEXE, FinalEffect, SearchBinary, Divide, FieldsArrayHalt;

type OpenCloseFunction = function: String;

type AnsiCharArray = array[0..127] of AnsiChar;
type CharArray = array[0..4095] of Char;

type
  SurvivorsUTF16 = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    ANSI_ASCII: AnsiString;
    ASCII_C: AnsiChar;
    B_110110_S: Integer;
    B_110111_S: Integer;
    beginning: Integer;
    buffer: array of Char;
    buffer_length: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_Y_ARRAY: CharArray;
    count: Integer;
    d: Dividing;
    dough: array[0..128] of Char;
    doughLength: Integer;
    E: array[0..255] of Integer;
    f: FieldsListHalt;
    field_count: Integer;
    H: array[0..7] of Integer;
    H_0A_C: Char;
    H_0D_C: Char;
    KANJI_C: AnsiChar;
    KATAKANA_C: AnsiChar;
    LOWER_PART_MASK: Integer;
    NEW_LINE: String;
    NEW_LINE_SIZE: Integer;
    NON_KANJI_C: AnsiChar;
    open_close: array[1..4] of OpenCloseFunction;
    SET_BIT_C: Char;
    SURROGATE_PAIR_C: AnsiChar;
    SURRO_PATTERN: array[1..7] of Char;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    w: array[0..127, 0..1023] of Char;
    y: CharArray;
    Y_COUNT: Integer;
    Y_COUNT_1: Integer;
    z: array[1..262040] of AnsiChar;
    function E_table: Integer;
    function file_size(name: String): Integer;
    function hexadecimalCharacter(c: Char): Boolean;
    function setBufferSize: Integer;
{*

UTF16 encoding: 65509 (FFE5h)
( 65509 + 1 ) * ( FLAG_SIZE + UTF16_KANJI_SIZE + 1 ) = 262040
*}
  public
    function hexadecimal_sequence_to_integer(s: PChar; s_length: Integer): Integer;
    function newLine: String;
    function sequence_to_integer(s: String): Integer;
    function surrogatePairSetting: Integer;
    function surrogatePairUnderline: Integer;
    function working(index: Integer; place: Integer): AnsiChar;
    function workingPair(high_part_of_kanji: Integer; low_part_of_kanji: Integer; place: Integer): AnsiChar;
    procedure clear;
    procedure populated(index: Integer; place: Integer; c: AnsiChar);
    procedure setting(BINARY_TABLE: String; ans: AnsiString); overload;
    procedure setting(e: SearchTable; ans: AnsiString); overload;
    procedure setting(f: FinalEffects; ans: AnsiString); overload;
  end;

const
  B_110110 = 54;
  B_110111 = 55;
  BOM_COUNT = 1;
  BUFFER_ERROR = -1;
  C_UTF16_MS932_BIN = 'C:\Narfi\UTF16~MS932.bin';
  DATA_TEXT = 'data.txt';
  DO_NOT_REMOVE_SEPARATORS = False;
  EMPTY = '';
  FIRST_PART_OF_KANJI = 1; { Field-2 }
  FLAG_SIZE = 1;
  H_00010000 = 65536;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_30 = 48; { '0' }
  H_31 = 49; { '1' }
  H_39 = 57; { '9' }
  H_41 = 65; { 'A' }
  H_42 = 66; { 'B' }
  H_46 = 70; { 'F' }
  H_D840 = 55360;
  H_DC02 = 56322;
  H_FE = 254;
  H_FF = 255;
  HEXADECIMAL_CHARACTERS = '0123456789ABCDEF';
  NOT_CONVERTED = -1;
  RECORD_SIZE = 5;
  SEARCH_COUNT = 4;
  SET_BIT = 49; { '1' }
  SETTING_TEXT = 'manager.txt';
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 2;
  SUBSTITUTE_FOR_SURROGATE_PAIR = 6; { Field-7 }
  SUBSTITUTE_FOR_SURROGATE_PAIR_DATA = 'Q'; { Shift_JIS underline }
  SURRO_COUNT = 7;
  SURRO_CSV = 'surro.csv';
  SURRO_QUOTE = 34; { 22h, '"' }
  SURRO_SEPARATOR = 44; { 2Ch, ',' }
  SURRO_TEXT = 'surro.txt';
  TABLE_ARRAY_INDEX = 5; { Field-6 }
  TABLE_NUMBER = 4; { Field-5 }
  TEXT_FILE = '.txt';
  UTF16_KANJI_SIZE = 2;
  UTF16_KANJI_SIZE_2 = 4; { FLAG_SIZE + UTF16_KANJI_SIZE + 1 }
  UTF16_MS932_BIN = 'UTF16~MS932.bin';

implementation

function binary_file_in_current_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, UTF16_MS932_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := UTF16_MS932_BIN;
  except
    ;
  end;

  Result := s;
end;

function binary_file_in_installation_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, C_UTF16_MS932_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := C_UTF16_MS932_BIN;
  except
    ;
  end;

  Result := s;
end;

function binary_file_in_path_directory: String;
{ directory included in Path }
var
  p: CommandPath;
  s: String;
begin
  p := CommandPath.Create;
  p.setting;
  s := p.pathName(UTF16_MS932_BIN);
  p.Free;

  Result := s;
end;

function binary_file_in_selected_directory: String;
{ directory that may have priority }
var
  s: String;
  settingFile: TextFile;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(settingFile, SETTING_TEXT);
    Reset(settingFile);
    ReadLn(settingFile, s); { Line-1 }
    CloseFile(settingFile);
  except
    WriteLn( 'File error: ' + SETTING_TEXT );
    Halt(1633);
  end;

  if Length(s) > 0 then
    begin
      try
        Assign(testFile, s);
        Reset(testFile);
        CloseFile(testFile);
      except
        s := EMPTY;
      end;

    end;

  Result := s;
end;

function SurvivorsUTF16.E_table: Integer;
var
  beginning_of_E: Integer;
  end_1: Integer;
  h: Integer;
  i: Integer;
  k: Integer;
  m: Integer;
begin
  for i := 0 to H_FF do
  begin
    E[i] := NOT_CONVERTED;
  end;

{ 0 to 9 }
  beginning_of_E := H_30;

  end_1 := H_39 - 1;
  E[H_30] := 0;
  h := E[H_30];
  m := H_31;

  for k := beginning_of_E to end_1 do
  begin
    Inc(h);
    E[m] := h;
    Inc(m);
  end;

{ A to F }
  beginning_of_E := H_41;

  end_1 := H_46 - 1;
  E[H_41] := 10; { 'A' }
  h := E[H_41];
  m := H_42;

  for k := beginning_of_E to end_1 do
  begin
    Inc(h);
    E[m] := h;
    Inc(m);
  end;

  Result := h; { 'F' }
end;

function SurvivorsUTF16.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function SurvivorsUTF16.setBufferSize: Integer;
var
  r: Integer;
begin
  r := buffer_length;

  try
    SetLength(buffer, buffer_length);
  except
    Halt(1634);
  end;

  Result := r;
end;

function SurvivorsUTF16.hexadecimal_sequence_to_integer(s: PChar; s_length: Integer): Integer;
var
  c: Char;
  g: String;
  g_length: Integer;
  i: Longint;
  k: Integer;
  m: Integer;
  n: Integer;
  s_length_1: Integer;
  term: Integer;
begin
  g := EMPTY;
  m := 0;
  n := 0;
  s_length_1 := s_length - 1;

  for k := 0 to s_length_1 do
  begin
    c := s[k];
    if hexadecimalCharacter(c) then
      begin
        g := g + c;
      end
    else
      begin
        break;
      end;

  end;

  g_length := Length(g);
  try
    for k := g_length downto 1 do
    begin
      i := Ord(g[k]);
      term := E[i] * H[m];
      if term < 0 then
        raise Exception.Create('hexadecimal_sequence_to_integer() error');

      n := n + term;
      Inc(m);
    end;

  except
    Halt(1635);
  end;

  Result := n;
end;

function SurvivorsUTF16.hexadecimalCharacter(c: Char): Boolean;
var
  q: Integer;
  r: Boolean;
begin
  q := AnsiPos(c, HEXADECIMAL_CHARACTERS);
  if q > 0 then
    r := True
  else
    r := False;

  Result := r;
end;

function SurvivorsUTF16.newLine: String;
begin
  Result := NEW_LINE;
end;

function SurvivorsUTF16.sequence_to_integer(s: String): Integer;
var
  k: Integer;
begin
  k := NOT_CONVERTED;

  try
    k := StrToInt(s);
  except
{ EConvertError }
    Halt(1636);
  end;

  Result := k;
end;

function SurvivorsUTF16.surrogatePairSetting: Integer;
var
  b: Integer;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  exit_code: Integer;
  g: String;
  i: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  q: array[0..1] of Byte;
  s: String;
  testFile: File;
  trace: Integer;
  u: Integer;
begin
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
  Y_COUNT_1 := Y_COUNT - 1;

  for n := 1 to SURRO_COUNT do
  begin
    SURRO_PATTERN[n] := SET_BIT_C;
  end;

  try
    AssignFile(testFile, SURRO_CSV);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, v, V_SIZE, block_has_been_read);
    q[0] := Ord(v[0]);
    q[1] := Ord(v[1]);
    if (q[0] = H_FF) and (q[1] = H_FE) then
      beginning := BOM_COUNT
    else
      beginning := 0;

    CloseFile(testFile);

    buffer_length := file_size(SURRO_CSV) div 2;
  except
    WriteLn( 'File error: ' + SURRO_CSV );
    Halt(1637);
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

  b := 0;
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  exit_code := 0;
  m := 0;

  setBufferSize;
  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, SURRO_CSV);
    Reset(testFile, SIZE_OF_CHAR);

    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    Halt(1638);
  end;

{ Finding the first NEW_LINE }
  for n := beginning to buffer_length_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        count := 1;

{ Reading a line }
        d.Free;
        f.Free;
        f := FieldsListHalt.Create;
        d := Dividing.Create;
        d.presetting(SURRO_QUOTE, SURRO_SEPARATOR, DO_NOT_REMOVE_SEPARATORS);
        d.setting(f, SURRO_COUNT, SURRO_PATTERN);

        doughLength := b;
        b := 0;
        field_count := d.mapping_fields_list(dough, doughLength);
        i := sequence_to_integer( f.item(TABLE_NUMBER) );
        k := sequence_to_integer( f.item(TABLE_ARRAY_INDEX) );
        g := f.item(SUBSTITUTE_FOR_SURROGATE_PAIR);
        w[i, k] := g[1];

        s := f.item(FIRST_PART_OF_KANJI);
        u := hexadecimal_sequence_to_integer( PChar(s), Length(s) );
        populated(u, 0, SURROGATE_PAIR_C);
        populated(u, 1, ANSI_ASCII[i]);
        populated(u, 2, NON_KANJI_C);

        trace := n;
        beginning := trace + 1;
        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  for n := beginning to buffer_length_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        Inc(count);

{ Reading a line }
        d.Free;
        f.Free;
        f := FieldsListHalt.Create;
        d := Dividing.Create;
        d.presetting(SURRO_QUOTE, SURRO_SEPARATOR, DO_NOT_REMOVE_SEPARATORS);
        d.setting(f, SURRO_COUNT, SURRO_PATTERN);

        doughLength := b;
        b := 0;
        field_count := d.mapping_fields_list(dough, doughLength);
        i := sequence_to_integer( f.item(TABLE_NUMBER) );
        k := sequence_to_integer( f.item(TABLE_ARRAY_INDEX) );
        g := f.item(SUBSTITUTE_FOR_SURROGATE_PAIR);
        w[i, k] := g[1];

        s := f.item(FIRST_PART_OF_KANJI);
        u := hexadecimal_sequence_to_integer( PChar(s), Length(s) );
        populated(u, 0, SURROGATE_PAIR_C);
        populated(u, 1, ANSI_ASCII[i]);
        populated(u, 2, NON_KANJI_C);
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            continue;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_length_1 do }

  Result := exit_code;
end;

function SurvivorsUTF16.surrogatePairUnderline: Integer;
var
  basic_code: Integer;
  code_point: Integer;
  exit_code: Integer;
  g: String;
  higher_part_of_kanji: Integer;
  i: Integer;
  k: Integer;
  lower_part_of_kanji: Integer;
  s: String;
  s_length: Integer;
  testFile: TextFile;
begin
  exit_code := 0;
  LOWER_PART_MASK := 1 + 2 + 4 + 8 + 16 + 32 + 64 + 128 + 256 + 512;

  try
    AssignFile(testFile, SURRO_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      s_length := Length(s);
      if s_length = 0 then
        break;

      code_point := hexadecimal_sequence_to_integer( PChar(s), Length(s) );

{ Four bytes of UTF-16 }
      basic_code := code_point - H_00010000;
      k := basic_code shr 10;
      higher_part_of_kanji := k + B_110110_S;

      k := basic_code and LOWER_PART_MASK;
      lower_part_of_kanji := k + B_110111_S;

{ Higher-part of kanji will be useful for arrays if software subtracts H_D840 from the kanji code. }
      i := higher_part_of_kanji - H_D840;

{ Lower-part of kanji will be useful for arrays if software subtracts H_DC02 from the kanji code. }
      k := lower_part_of_kanji - H_DC02;

      populated(higher_part_of_kanji, 0, SURROGATE_PAIR_C);
      populated(higher_part_of_kanji, 1, ANSI_ASCII[i]);
      populated(higher_part_of_kanji, 2, NON_KANJI_C);

      g := SUBSTITUTE_FOR_SURROGATE_PAIR_DATA;
      w[i, k] := g[1];
    end;

    CloseFile(testFile);
  except
    Halt(1674);
  end;

  Result := exit_code;
end;

function SurvivorsUTF16.working(index: Integer; place: Integer): AnsiChar;
var
  k: Integer;
begin
{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
  k := UTF16_KANJI_SIZE_2 * index + place;
  Result := z[k];
end;

function SurvivorsUTF16.workingPair(high_part_of_kanji: Integer; low_part_of_kanji: Integer; place: Integer): AnsiChar;
var
  index: Longint;
  k: Integer;
  tableNumber: Integer;
  UTF16_C: Char;
begin
{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
  tableNumber := high_part_of_kanji - H_D840;
  k := low_part_of_kanji - H_DC02;
  UTF16_C := w[tableNumber, k];

  index := Ord(UTF16_C);
  k := UTF16_KANJI_SIZE_2 * index + place;
  Result := z[k];
end;

procedure SurvivorsUTF16.populated(index: Integer; place: Integer; c: AnsiChar);
var
  k: Integer;
begin
{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
  k := UTF16_KANJI_SIZE_2 * index + place;
  z[k] := c;
end;

procedure SurvivorsUTF16.clear;
var
  k: Integer;
begin
  for k := 1 to 262040 do
  begin
    z[k] := #0;
  end;

end;

procedure SurvivorsUTF16.setting(BINARY_TABLE: String; ans: AnsiString);
var
  b: Longint;
  block_has_been_read: Integer;
  i: Integer;
  k: Integer;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  ANSI_ASCII := ans;
  ASCII_C := AnsiChar(#0);
  KANJI_C := AnsiChar(#1);
  KATAKANA_C := AnsiChar(#2);
  NON_KANJI_C := AnsiChar(#0);
  SET_BIT_C := Chr(SET_BIT);
  SURROGATE_PAIR_C := AnsiChar(#3);

  H[0] := 1;
  H[1] := 16;
  H[2] := 256;
  H[3] := 4096;
  H[4] := 65536;
  H[5] := 1048576;
  H[6] := 16777216;
  H[7] := 268435456;

  B_110110_S := B_110110 shl 10;
  B_110111_S := B_110111 shl 10;
  E_table;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Reading BINARY_TABLE to produce a table in the main memory }
  try
    AssignFile(tableFile, BINARY_TABLE);
    Reset(tableFile, RECORD_SIZE);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(tableFile, a, 1, block_has_been_read);

      if a[2] = ASCII_C then
        begin
          k := Ord(a[0]);
          populated(k, 0, ASCII_C);
          populated(k, 1, a[3]);
          populated(k, 2, NON_KANJI_C);
        end
      else
        begin
          if a[2] = KATAKANA_C then
            begin
              b := Ord(a[1]);
              k := b shl 8;

              b := Ord(a[0]); { little-endian architecture }
              k := k + b;

              populated(k, 0, KATAKANA_C);
              populated(k, 1, a[3]);
              populated(k, 2, NON_KANJI_C);
            end
          else
            begin
{ kanji }
              b := Ord(a[1]);
              k := b shl 8;

              b := Ord(a[0]); { little-endian architecture }
              k := k + b;

              populated(k, 0, KANJI_C);
              populated(k, 1, a[3]);
              populated(k, 2, a[4]);
            end;

        end;

    until block_has_been_read = 0;

    CloseFile(tableFile);
  except
    Halt(1661);
  end;

end;

procedure SurvivorsUTF16.setting(e: SearchTable; ans: AnsiString);
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  dataFile: TextFile;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  ANSI_ASCII := ans;
  ASCII_C := AnsiChar(#0);
  KANJI_C := AnsiChar(#1);
  KATAKANA_C := AnsiChar(#2);
  NON_KANJI_C := AnsiChar(#0);
  SET_BIT_C := Chr(SET_BIT);
  SURROGATE_PAIR_C := AnsiChar(#3);

  H[0] := 1;
  H[1] := 16;
  H[2] := 256;
  H[3] := 4096;
  H[4] := 65536;
  H[5] := 1048576;
  H[6] := 16777216;
  H[7] := 268435456;

  B_110110_S := B_110110 shl 10;
  B_110111_S := B_110111 shl 10;
  E_table;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Finding UTF16_MS932_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if e.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if e.whichTable then
          begin
            AssignFile(dataFile, DATA_TEXT);
            Rewrite(dataFile);
            messages := binaryTable + ' is primarily accessible to Hetn.';
            WriteLn(dataFile, messages);
            CloseFile(dataFile);
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if e.noSearching then
                begin
                  Halt(1639);
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              Halt(1640);
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  if Length(binaryTable) > 0 then
    begin
{ Reading UTF16_MS932_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        if a[2] = ASCII_C then
          begin
            k := Ord(a[0]);
            populated(k, 0, ASCII_C);
            populated(k, 1, a[3]);
            populated(k, 2, NON_KANJI_C);
          end
        else
          begin
            if a[2] = KATAKANA_C then
              begin
                b := Ord(a[1]);
                k := b shl 8;

                b := Ord(a[0]); { little-endian architecture }
                k := k + b;

                populated(k, 0, KATAKANA_C);
                populated(k, 1, a[3]);
                populated(k, 2, NON_KANJI_C);
              end
            else
              begin
{ kanji }
                b := Ord(a[1]);
                k := b shl 8;

                b := Ord(a[0]); { little-endian architecture }
                k := k + b;

                populated(k, 0, KANJI_C);
                populated(k, 1, a[3]);
                populated(k, 2, a[4]);
              end;

          end;

      until block_has_been_read = 0;

      CloseFile(tableFile);
    end
  else
    begin
      Halt(1641);
    end;

end;

procedure SurvivorsUTF16.setting(f: FinalEffects; ans: AnsiString);
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  dataFile: TextFile;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  ANSI_ASCII := ans;
  ASCII_C := AnsiChar(#0);
  KANJI_C := AnsiChar(#1);
  KATAKANA_C := AnsiChar(#2);
  NON_KANJI_C := AnsiChar(#0);
  SET_BIT_C := Chr(SET_BIT);
  SURROGATE_PAIR_C := AnsiChar(#3);

  H[0] := 1;
  H[1] := 16;
  H[2] := 256;
  H[3] := 4096;
  H[4] := 65536;
  H[5] := 1048576;
  H[6] := 16777216;
  H[7] := 268435456;

  B_110110_S := B_110110 shl 10;
  B_110111_S := B_110111 shl 10;
  E_table;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Finding UTF16_MS932_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if f.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if f.whichTable then
          begin
            AssignFile(dataFile, DATA_TEXT);
            Rewrite(dataFile);
            messages := binaryTable + ' is primarily accessible to Hetn.';
            WriteLn(dataFile, messages);
            CloseFile(dataFile);
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if f.noSearching then
                begin
                  Halt(1642);
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              Halt(1643);
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  if Length(binaryTable) > 0 then
    begin
{ Reading UTF16_MS932_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        if a[2] = ASCII_C then
          begin
            k := Ord(a[0]);
            populated(k, 0, ASCII_C);
            populated(k, 1, a[3]);
            populated(k, 2, NON_KANJI_C);
          end
        else
          begin
            if a[2] = KATAKANA_C then
              begin
                b := Ord(a[1]);
                k := b shl 8;

                b := Ord(a[0]); { little-endian architecture }
                k := k + b;

                populated(k, 0, KATAKANA_C);
                populated(k, 1, a[3]);
                populated(k, 2, NON_KANJI_C);
              end
            else
              begin
{ kanji }
                b := Ord(a[1]);
                k := b shl 8;

                b := Ord(a[0]); { little-endian architecture }
                k := k + b;

                populated(k, 0, KANJI_C);
                populated(k, 1, a[3]);
                populated(k, 2, a[4]);
              end;

          end;

      until block_has_been_read = 0;

      CloseFile(tableFile);
    end
  else
    begin
      Halt(1644);
    end;

end;

end.
