unit Around;

interface

{$I+}

uses
  SysUtils, RingQueueCSV;

type CharArray = array[0..4095] of Char;
type LineNumberFunction = function(b: String): String;

type
  AroundPoint = class
  private
    after: Integer;
    BELL: String;
    CLEAR_ARRAY: CharArray;
    file_name: String;
    joint_byte: Byte;
    lineBegins: LineNumberFunction;
    new_name: String;
    number: Boolean;
    focal_point: Integer;
    ring: RingCSV;
    ring_count: Integer;
    y: CharArray;
    Y_SIZE: Integer;
    Y_SIZE_1: Integer;
  public
    destructor Destroy; override;
    function columnRow(i: Integer; j: Integer): AnsiString;
    function openClose(name: String): Integer;
    function produceBinaryFile(name: String; no_removing: Boolean): Integer;
    function reading: Boolean;
    function reading_CSV: Boolean;
    function renameBinaryFile(name: String; no_removing: Boolean): Integer;
    function standardOutput: Boolean;
    procedure prepareRing;
    procedure presetting(j: Byte; q: Byte; s: Byte; e: Boolean);
    procedure setting(fn: String; a: Integer; before: Integer; p: Integer; n: Boolean); overload;
    procedure setting(fn: String; nn: String; a: Integer; before: Integer; p: Integer; n: Boolean); overload;
  end;

const
  AROUND_TEXT = 'around.txt';
  DATA_TEXT = 'data.txt';
  EMPTY = '';
  FILE_ERROR = -2;
  GAP = ' ';
  H_07 = 7; { BELL }
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  PROHIBITED = False;
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 2;
  TEMPORARY_BINARY = 'binary.tmp';

var
  JOINT: String;
  line_number: Integer;

implementation

function lineBeginsWithNumber(b: String): String;
var
  combo: String;
begin
  line_number := line_number + 1;
  combo := Format('%10d', [line_number]);
  combo := combo + JOINT;
  combo := combo + b;
  Result := combo;
end;

function nonnumber(b: String): String;
begin
  Result := b;
end;

destructor AroundPoint.Destroy;
begin
  ring.Free;

  inherited Destroy;
end;

function AroundPoint.columnRow(i: Integer; j: Integer): AnsiString;
begin
  Result := ring.columnRow(i, j);
end;

function AroundPoint.openClose(name: String): Integer;
var
  r: Integer;
  testFile: File;
begin
  try
    AssignFile(testFile, name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    CloseFile(testFile);
    r := 0;
  except
    r := FILE_ERROR;
    Halt(1590);
  end;

  Result := r;
end;

function AroundPoint.produceBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, TEMPORARY_BINARY);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);
      BlockWrite(newFile, y, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, name);
        Erase(testFile);
      end;

  except
    Halt(1377);
  end;

  Result := sum;
end;

function AroundPoint.reading: Boolean;
var
  combo: String;
  count: Integer;
  testFile: TextFile;
  g: String;
  lineNumber: Integer;
  neighborhood: Boolean;
  newFile: TextFile;
  s: String;
  sum: Integer;
  written: Boolean;
begin
  combo := EMPTY;
  lineBegins := nonnumber;
  lineNumber := 0;
  neighborhood := False;
  ring.setting(BELL, ring_count);
  sum := 0;
  written := False;

  if number then
    lineBegins := lineBeginsWithNumber;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);
    AssignFile(newFile, new_name);
    Rewrite(newFile);

    if after > 0 then
      begin
        count := 0;

        while not Eof(testFile) do
        begin
          ReadLn(testFile, s);
          g := lineBegins(s);
          ring.pop_front;
          ring.push_back(g);
          if neighborhood then
            begin
              Inc(count);
              if count >= after then
                begin
                  s := ring.display(PROHIBITED);
                  combo := combo + s;
                  written := True;
                  neighborhood := False;
                  break;
                end;

            end
          else
            begin
              Inc(lineNumber);
              if lineNumber = focal_point then
                begin
                  neighborhood := True;
                  Inc(sum);
                end;

            end;

        end;

      end
    else
      begin
        while not Eof(testFile) do
        begin
          ReadLn(testFile, s);
          g := lineBegins(s);
          ring.pop_front;
          ring.push_back(g);

          Inc(lineNumber);
          if lineNumber = focal_point then
            begin
              Inc(sum);
              s := ring.display(PROHIBITED);
              combo := combo + s;
              written := True;
              neighborhood := False;
              break;
            end;

        end;

      end;

    CloseFile(testFile);

    if neighborhood then
      begin
        if not written then
          begin
            s := ring.display(PROHIBITED);
            combo := combo + s;
          end;

      end
    else
      begin
        if sum > 0 then
          begin
            neighborhood := True;
          end
        else
          begin
            neighborhood := False;
          end;

      end;

    Write(newFile, combo);
    CloseFile(newFile);
  except
    Halt(1378);
  end;

  Result := neighborhood;
end;

function AroundPoint.reading_CSV: Boolean;
var
  count: Integer;
  testFile: TextFile;
  g: String;
  lineNumber: Integer;
  neighborhood: Boolean;
  s: String;
  sum: Integer;
  written: Boolean;
begin
  lineBegins := nonnumber;
  lineNumber := 0;
  neighborhood := False;
  ring.setting(BELL, ring_count);
  sum := 0;
  written := False;

  if number then
    lineBegins := lineBeginsWithNumber;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);

    if after > 0 then
      begin
        count := 0;

        while not Eof(testFile) do
        begin
          ReadLn(testFile, s);
          g := lineBegins(s);
          ring.pop_front;
          ring.push_back(g);
          if neighborhood then
            begin
              Inc(count);
              if count >= after then
                begin
                  ring.tabulating;
                  prepareRing;
                  written := True;
                  count := 0;
                  neighborhood := False;
                end;

            end
          else
            begin
              Inc(lineNumber);
              if lineNumber = focal_point then
                begin
                  neighborhood := True;
                  Inc(sum);
                end;

            end;

        end;

      end
    else
      begin
        while not Eof(testFile) do
        begin
          ReadLn(testFile, s);
          g := lineBegins(s);
          ring.pop_front;
          ring.push_back(g);

          Inc(lineNumber);
          if lineNumber = focal_point then
            begin
              Inc(sum);
              ring.tabulating;
              prepareRing;
              written := True;
              neighborhood := False;
            end;

        end;

      end;

    CloseFile(testFile);

    if neighborhood then
      begin
        if not written then
          begin
             ring.tabulating;
          end;

      end
    else
      begin
        if sum > 0 then
          begin
            neighborhood := True;
          end
        else
          begin
            neighborhood := False;
          end;

      end;

  except
    Halt(1379);
  end;

  Result := neighborhood;
end;

function AroundPoint.renameBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, TEMPORARY_BINARY);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);
      BlockWrite(newFile, y, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, TEMPORARY_BINARY);
        Erase(testFile);
      end;

  except
    Halt(1380);
  end;

  Result := sum;
end;

function AroundPoint.standardOutput: Boolean;
var
  count: Integer;
  testFile: TextFile;
  g: String;
  lineNumber: Integer;
  neighborhood: Boolean;
  s: String;
  sum: Integer;
  written: Boolean;
begin
  lineBegins := nonnumber;
  lineNumber := 0;
  neighborhood := False;
  ring.setting(BELL, ring_count);
  sum := 0;
  written := False;

  if number then
    lineBegins := lineBeginsWithNumber;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);

    if after > 0 then
      begin
        count := 0;

        while not Eof(testFile) do
        begin
          ReadLn(testFile, s);
          g := lineBegins(s);
          ring.pop_front;
          ring.push_back(g);
          if neighborhood then
            begin
              Inc(count);
              if count >= after then
                begin
                  ring.display;
                  written := True;
                  neighborhood := False;
                  break;
                end;

            end
          else
            begin
              Inc(lineNumber);
              if lineNumber = focal_point then
                begin
                  neighborhood := True;
                  Inc(sum);
                end;

            end;

        end;

      end
    else
      begin
        while not Eof(testFile) do
        begin
          ReadLn(testFile, s);
          g := lineBegins(s);
          ring.pop_front;
          ring.push_back(g);

          Inc(lineNumber);
          if lineNumber = focal_point then
            begin
              Inc(sum);
              ring.display;
              written := True;
              neighborhood := False;
              break;
            end;

        end;

      end;

    CloseFile(testFile);

    if neighborhood then
      begin
        if not written then
          begin
            ring.display;
          end;

      end
    else
      begin
        if sum > 0 then
          begin
            neighborhood := True;
          end
        else
          begin
            neighborhood := False;
          end;

      end;

  except
    Halt(1381);
  end;

  Result := neighborhood;
end;

procedure AroundPoint.prepareRing;
begin
  ring.clearRing;
  ring.setting(BELL, ring_count);
end;

procedure AroundPoint.presetting(j: Byte; q: Byte; s: Byte; e: Boolean);
var
  i: Integer;
begin
  joint_byte := j;
  JOINT := EMPTY + Chr(joint_byte);
  Y_SIZE := SizeOf(y) div SIZE_OF_CHAR;
  Y_SIZE_1 := Y_SIZE - 1;

  for i := 0 to Y_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  ring := RingCSV.Create;
  ring.presetting(q, s, e);
end;

procedure AroundPoint.setting(fn: String; a: Integer; before: Integer; p: Integer; n: Boolean);
begin
  file_name := fn;
  new_name := AROUND_TEXT;
  after := a;
  ring_count := before + 1 + after;
  focal_point := p;
  number := n;

  line_number := 0;
  BELL := EMPTY + Chr(H_07);
end;

procedure AroundPoint.setting(fn: String; nn: String; a: Integer; before: Integer; p: Integer; n: Boolean);
begin
  file_name := fn;
  new_name := nn;
  after := a;
  ring_count := before + 1 + after;
  focal_point := p;
  number := n;

  line_number := 0;
  BELL := EMPTY + Chr(H_07);
end;

end.
