unit DivideSJIS;

interface

uses
  SysUtils, FieldsArray, Combo, AnsiQueue;

type EditFunction = function(s: AnsiString): AnsiString;
type PettyArray = array[0..255] of AnsiChar;

type
  DividingSJIS = class
  protected
    a: AnsiStringQueue;
    ANSI_ASCII: AnsiString;
    ascii_byte: Byte;
    ASCII_C: AnsiChar;
    BLANK_C: AnsiChar;
    CLEAR_BIT_C: AnsiChar;
    combo: AnsiString;
    dough: array[0..98303] of AnsiChar;
    doughLength: Integer;
    f: FieldsList;
    field_count: Integer;
    manyFields: array[0..3071] of Boolean;
    pattern: array[0..3071] of AnsiChar;
    quote_byte: Byte;
    remove_separators: Boolean;
    separator: AnsiString;
    separator_byte: Byte;
    SET_BIT_C: AnsiChar;
    size_of_united_items: Integer;
    procedure vector_to_array_arranging_in_reverse_order;
    procedure vector_to_array_complementing(extra: AnsiString; bond: Byte);
    procedure vector_to_array_editing(editing: EditFunction; field_number: Integer);
    procedure vector_to_array_mapping;
  public
    constructor Create;
    destructor Destroy; override;
    function complementing_fields_list(buffer: array of AnsiChar; bufferLength: Integer; extra: AnsiString; bond: Byte): Integer;
    function editing_field(buffer: array of AnsiChar; bufferLength: Integer; field_number: Integer): Integer; virtual;
    function fieldCount: Integer;
    function extracting_decimal_from_field(buffer: array of AnsiChar; bufferLength: Integer; field_number: Integer): Integer;
    function extracting_integer_from_field(buffer: array of AnsiChar; bufferLength: Integer; field_number: Integer): Integer;
    function mapping_fields_list(buffer: array of AnsiChar; bufferLength: Integer): Integer;
    function quoteCode: Byte;
    function reverse_unite(buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
    function reversing_fields_list(buffer: array of AnsiChar; bufferLength: Integer): Integer;
    function search_separator(buffer: array of AnsiChar; bufferLength: Integer; beginning: Integer): Integer;
    function separatorCode: Byte;
    function smashed_to_pieces(buffer: array of AnsiChar; bufferLength: Integer; selected: array of Boolean): Integer;
    function supplying_bits(c: Integer): AnsiString;
    function surge_of_set_bits(c: Integer; interval_0: Integer; interval_1: Integer): AnsiString;
    function unite(a: AnsiStringQueue; c: ComboAppend; buffer: array of AnsiChar; bufferLength: Integer): AnsiString; overload;
    function unite(buffer: array of AnsiChar; bufferLength: Integer): AnsiString; overload;
    function unite(buffer: array of AnsiChar; bufferLength: Integer; listed: Boolean): AnsiString; overload;
    function unitedSize: Integer;
    procedure presetting(ans: AnsiString; s: Byte); overload;
    procedure presetting(ans: AnsiString; q: Byte; s: Byte; e: Boolean); overload; virtual;
    procedure presetting(ans: AnsiString; q: Byte; s: Byte; e: Boolean; a: Byte); overload; virtual;
    procedure setting(fields_list: FieldsList; c: Integer); overload;
    procedure setting(fields_list: FieldsList; c: Integer; pn: array of AnsiChar); overload;
    procedure setting_for_extracting;
  end;

const
  ANSI_N_0TO9_SIZE = 10;
  ANSI_N_0TO9_MINUS_SIZE = 11;
  CLEAR_BIT = 48; { 30h, '0' }
  COMMA_BYTE = 44; { 2Ch, ',' }
  COMMAS_SHALL_DISAPPEAR_IF_PLACED_IN_QUOTES = True;
  EMPTY_S = '';
  GAP_BYTE = 32; { SP, ' ' }
  H_2E = 46; { dot '.'  }
  H_30 = 48; { '0' }
  H_39 = 57; { '9' }
  H_81 = 129;
  H_FC = 252;
  KANA_HANDAKUTEN = 223; { DFh }
  KANA_KUTEN = 161; { A1h }
  KANJI_SIZE = 2;
  MINUS_BYTE = 45; { minus '-'  }
  NOT_RECOGNIZED = -1;
  SET_BIT = 49; { 31h, '1'}
  STANDARD_QUOTE_BYTE = 34; { 22h, '"' }
  UNDERLINE_BYTE = 95; { 5Fh, '_' }
  UPPER_LIMIT_TO_FIELD_NUMBER = 3071;
  ZERO = '0';

var
  ANSI_N_0TO9: AnsiString;
  ANSI_N_0TO9_MINUS: AnsiString;
  COMMA_C_S: AnsiChar;
  DOT_C_S: AnsiChar;
  EMPTY_ANSI_S: AnsiString;
  MINUS_C_S: AnsiChar;

implementation

function leadingNumbers(full: AnsiString): Integer;
var
  c: AnsiChar;
  count: Integer;
  found: Boolean;
  fullLength: Integer;
  k: Integer;
  m: Integer;
begin
  count := 0;
  fullLength := Length(full);

  for k := 1 to fullLength do
  begin
    c := full[k];

{ Checking if a number appears. }
    found := False;
    for m := 1 to ANSI_N_0TO9_SIZE do
    begin
      if ANSI_N_0TO9[m] = c then
        begin
          found := True;
          break;
        end
      else
        begin
          continue
        end;

    end;

    if found then
      begin
{ Counting numbers }
        Inc(count);
        continue
      end
    else
      begin
        break;
      end;

  end;

  Result := count;
end;

function positiveOrNegativeNumbers(full: AnsiString): Integer;
var
  c: AnsiChar;
  count: Integer;
  found: Boolean;
  fullLength: Integer;
  k: Integer;
  m: Integer;
begin
  count := 0;
  fullLength := Length(full);

  k := 1;
  c := full[k];

{ Checking if a number or minus appears. }
  found := False;
  for m := 1 to ANSI_N_0TO9_MINUS_SIZE do
  begin
    if ANSI_N_0TO9_MINUS[m] = c then
      begin
        found := True;
        break;
      end
    else
      begin
        continue
      end;

  end;

  if found then
    begin
{ Counting numbers }
      Inc(count);

      for k := 2 to fullLength do
      begin
        c := full[k];

{ Checking if a number appears. }
        found := False;
        for m := 1 to ANSI_N_0TO9_SIZE do
        begin
          if ANSI_N_0TO9[m] = c then
            begin
              found := True;
              break;
            end
          else
            begin
              continue
            end;

        end;

        if found then
          begin
{ Counting numbers }
            Inc(count);
            continue
          end
        else
          begin
            break;
          end; { if found then, else }

      end; { for k := 2 to fullLength do }

    end; { if found then }

  Result := count;
end;

function countingNonnumbers(full: AnsiString): Integer;
var
  c: AnsiChar;
  count: Integer;
  found: Boolean;
  fullLength: Integer;
  k: Integer;
  m: Integer;
  trace: AnsiChar;
begin
  c := COMMA_C_S;
  count := 0;
  fullLength := Length(full);
  trace := c;

  for k := 1 to fullLength do
  begin
    trace := c;
    c := full[k];

{ Checking if a number appears. }
    found := False;
    for m := 1 to ANSI_N_0TO9_SIZE do
    begin
      if ANSI_N_0TO9[m] = c then
        begin
          found := True;
          break;
        end
      else
        begin
          continue
        end;

    end;

    if found then
      begin
        break;
      end
    else
      begin
{ Counting nonnumbers }
        Inc(count);
        continue
      end;

  end;

  if trace = MINUS_C_S then
    begin
      count := count - 1;
    end;

  Result := count;
end;

function trailingNumbers(full: AnsiString): Integer;
var
  c: AnsiString;
  found: Boolean;
  full_length: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
begin
  full_length := Length(full);
  n := 0;
  for k := full_length downto 1 do
  begin
    c := full[k];

{ Checking if a number appears. }
    found := False;
    for m := 1 to ANSI_N_0TO9_SIZE do
    begin
      if ANSI_N_0TO9[m] = c then
        begin
          found := True;
          break;
        end
      else
        begin
          continue
        end;

    end;

    if found then
      begin
        Inc(n);
        continue;
      end
    else
      begin
        break;
      end;

  end;

{ Checking if a number or minus appears. }
  if ANSI_N_0TO9_MINUS[ANSI_N_0TO9_MINUS_SIZE] = c then
    begin
      n := n + 1;
    end;

  Result := n;
end;

function extract_decimal(field: AnsiString): AnsiString;
var
  count: Integer;
  head: AnsiString;
  fieldLength: Integer;
  found: Boolean;
  p: Integer;
  r: AnsiString;
  tail: AnsiString;
  tailLength: Integer;
  trace: Integer;
begin
  fieldLength := Length(field);

{ Checking if a dot appears. }
  found := False;
  for p := 1 to fieldLength do
  begin
    if field[p] = DOT_C_S then
      begin
        found := True;
        break;
      end
    else
      begin
        continue
      end;

  end;

  if found then
    begin
      trace := p;
      p := p - 1;
      head := Copy(field, 1, p);

      tailLength := fieldLength - trace;
      p := trace + 1;
      tail := Copy(field, p, tailLength);
      count := leadingNumbers(tail);
      if count = 0 then
        begin
          tail := ZERO;
        end
      else
        begin
          tail := Copy(tail, 1, count);
        end;

      count := trailingNumbers(head);
      if count = 0 then
        begin
          head := ZERO;
        end
      else
        begin
          p := trace - count;
          head := Copy(head, p, count);
        end;

      r := head;
      r := r + DOT_C_S;
      r := r + tail;
    end
  else
    begin
      r := '0.0';
    end;

  Result := r;
end;

function extract_integer(field: AnsiString): AnsiString;
var
  r: AnsiString;
  count: Integer;
  fieldLength: Integer;
  from: Integer;
  p: Integer;
  tail: AnsiString;
  tailLength: Integer;
begin
  r := EMPTY_ANSI_S;
  fieldLength := Length(field);
  p := countingNonnumbers(field);
  if p = fieldLength then { countingNonnumbers() cannot find any number. }
    begin
      r := ZERO;
    end
  else
    begin
      from := p + 1;
      tailLength := fieldLength - p;
      tail := Copy(field, from, tailLength);
      count := positiveOrNegativeNumbers(tail);
      r := Copy(tail, 1, count);
      if count = 1 then
        begin
          if tail[1] = MINUS_C_S then
            begin
              r := ZERO;
            end;

        end;

    end;

  Result := r;
end;

function transparent(field: AnsiString): AnsiString;
begin
  Result := field;
end;

constructor DividingSJIS.Create;
begin
  inherited Create;

  quote_byte := 34; { 22h }
  separator_byte := 44; { 2Ch }
end;

destructor DividingSJIS.Destroy;
begin
  a.Free;

  inherited Destroy;
end;

function DividingSJIS.complementing_fields_list(buffer: array of AnsiChar; bufferLength: Integer; extra: AnsiString; bond: Byte): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_complementing(extra, bond);
  Result := field_count;
end;

function DividingSJIS.editing_field(buffer: array of AnsiChar; bufferLength: Integer; field_number: Integer): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_editing(transparent, field_number);
  Result := field_count;
end;

function DividingSJIS.fieldCount: Integer;
begin
  Result := field_count;
end;

function DividingSJIS.extracting_decimal_from_field(buffer: array of AnsiChar; bufferLength: Integer; field_number: Integer): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_editing(extract_decimal, field_number);
  Result := field_count;
end;

function DividingSJIS.extracting_integer_from_field(buffer: array of AnsiChar; bufferLength: Integer; field_number: Integer): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_editing(extract_integer, field_number);
  Result := field_count;
end;

function DividingSJIS.mapping_fields_list(buffer: array of AnsiChar; bufferLength: Integer): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_mapping;
  Result := field_count;
end;

function DividingSJIS.quoteCode: Byte;
begin
  Result := quote_byte;
end;

function DividingSJIS.reversing_fields_list(buffer: array of AnsiChar; bufferLength: Integer): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_arranging_in_reverse_order;
  Result := field_count;
end;

function DividingSJIS.reverse_unite(buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
var
  item_count_1: Integer;
  n: Integer;
begin
  combo := EMPTY_ANSI_S;
  reversing_fields_list(buffer, bufferLength);

  item_count_1 := f.itemCount - 1;
  for n := 0 to item_count_1 do
  begin
    combo := combo + f.item(n);
    combo := combo + separator;
  end;

  Result := combo;
end;

function DividingSJIS.search_separator(buffer: array of AnsiChar; bufferLength: Integer; beginning: Integer): Integer;
var
  i: Longint;
  k: Integer;
  n: Byte;
  no_splitting: Boolean;
  bufferLength_1: Integer;
  r: Integer;
begin
  no_splitting := False;
  bufferLength_1 := bufferLength - 1;
  r := NOT_RECOGNIZED;

  for k := 0 to bufferLength_1 do
  begin
    i := Ord(buffer[k]);
    n := Byte(i);

    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
          end;

        continue; { Commas disappear if placed in quotes. }
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
      end
    else
      begin
        if n = separator_byte then
          begin
            r := k;
            break;
          end;

      end;

  end;

  Result := r;
end;

function DividingSJIS.separatorCode: Byte;
begin
  Result := separator_byte;
end;

function DividingSJIS.smashed_to_pieces(buffer: array of AnsiChar; bufferLength: Integer; selected: array of Boolean): Integer;
var
  b: Integer;
  bufferLength_1: Integer;
  c: AnsiChar;
  field_number: Integer;
  firstByte: AnsiChar;
  i: Longint;
  k: Integer;
  kanji: Boolean;
  n: Byte;
  no_splitting: Boolean;
  pascal_string: AnsiString;
  z: AnsiString;
begin
  b := 0;
  field_number := 1;
  firstByte := ANSI_ASCII[COMMA_BYTE];
  kanji := False;
  no_splitting := False;
  bufferLength_1 := bufferLength - 1;

  for k := 0 to bufferLength_1 do
  begin
    c := buffer[k];
    i := Ord(c);
    n := Byte(i);

    if kanji then
      begin
        if selected[field_number] then
          begin
            dough[b] := firstByte;
            dough[ b + 1 ] := c;
            Inc(b, KANJI_SIZE);
          end;

        kanji := False;
        continue;
      end;

    if (H_81 <= n) and (n <= H_FC) then
      begin
        if (KANA_KUTEN <= n) and (n <= KANA_HANDAKUTEN) then
          begin
            if selected[field_number] then
              begin
                dough[b] := c;
                Inc(b);
              end;

            kanji := False;
          end
        else
          begin
            firstByte := c;
            kanji := True;
          end;

        continue;
      end;

{ ASCII }
    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
            if selected[field_number] then
              begin
                dough[b] := c;
                Inc(b);
              end;

          end
        else
          begin
            if n = separator_byte then
              begin
                if remove_separators then
                  begin
                    continue; { Commas disappear if placed in quotes. }
                  end
                else
                  begin
                    if selected[field_number] then
                      begin
                        dough[b] := c;
                        Inc(b);
                      end;

                  end;

              end
            else
              begin
                if selected[field_number] then
                  begin
                    dough[b] := c;
                    Inc(b);
                  end;

              end;

          end;

        continue;
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
        if selected[field_number] then
          begin
            dough[b] := c;
            Inc(b);
          end;

      end
    else
      begin
        if n = separator_byte then
          begin
            if selected[field_number] then
              begin
                pascal_string := dough;
                doughLength := b;
                z := Copy(pascal_string, 1, doughLength);
                a.push_back(z);
              end;

            b := 0;
            field_number := field_number + 1;
            if field_number > field_count then
              begin
                break;
              end;

          end
        else
          begin
            if selected[field_number] then
              begin
                dough[b] := c;
                Inc(b);
              end;

          end;

      end;

  end;

  if b > 0 then
    begin
      pascal_string := dough;
      doughLength := b;
      z := Copy(pascal_string, 1, doughLength);
      a.push_back(z);
    end
  else
    begin
      field_number := field_number - 1;
    end;

  field_count := field_number;
  Result := b;
end;

function DividingSJIS.supplying_bits(c: Integer): AnsiString;
begin
  field_count := c;
  if field_count > UPPER_LIMIT_TO_FIELD_NUMBER then
    field_count := UPPER_LIMIT_TO_FIELD_NUMBER;

  Result := StringOfChar(SET_BIT_C, field_count);
end; 

function DividingSJIS.surge_of_set_bits(c: Integer; interval_0: Integer; interval_1: Integer): AnsiString;
var
  body: AnsiString;
  bodyLength: Integer;
  head: AnsiString;
  headLength: Integer;
  tail: AnsiString;
  tailLength: Integer;
  z: AnsiString;
begin
{*
  surge_of_set_bits
0011111000000000000, for example
Setting bits from interval_0 to interval_1
*}

  field_count := c;
  if interval_1 > field_count then
    interval_1 := field_count;

  bodyLength := interval_1 - interval_0 + 1;
  body := StringOfChar(SET_BIT_C, bodyLength);

  headLength := interval_0 - 1;
  head := StringOfChar(CLEAR_BIT_C, headLength);

  tailLength := field_count - interval_1;
  tail := StringOfChar(CLEAR_BIT_C, tailLength);

  z := head;
  z := z + body;
  z := z + tail;
  Result := z;
end;

function DividingSJIS.unite(a: AnsiStringQueue; c: ComboAppend; buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
var
  item_count_1: Integer;
  n: Integer;
  g: AnsiString;
begin
  if a.count = 0 then
    mapping_fields_list(buffer, bufferLength);

  item_count_1 := f.itemCount - 1;
  for n := 0 to item_count_1 do
  begin
    g := f.item(n);
    c.append(g);
    c.append(separator);
  end;

  size_of_united_items := Length(c.built);
  Result := c.built;
end;

function DividingSJIS.unite(buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
var
  a_count_1: Integer;
  i: Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  a_count_1 := a.count - 1;
  combo := EMPTY_ANSI_S;
  for i := 0 to a_count_1 do
  begin
    combo := combo + a.q(i);
    combo := combo + separator;
  end;

  size_of_united_items := Length(combo);
  Result := combo;
end;

function DividingSJIS.unite(buffer: array of AnsiChar; bufferLength: Integer; listed: Boolean): AnsiString;
var
  item_count_1: Integer;
  n: Integer;
begin
  combo := EMPTY_ANSI_S;

  if not listed then
    mapping_fields_list(buffer, bufferLength);

  item_count_1 := f.itemCount - 1;
  for n := 0 to item_count_1 do
  begin
    combo := combo + f.item(n);
    combo := combo + separator;
  end;

  size_of_united_items := Length(combo);
  Result := combo;
end;

function DividingSJIS.unitedSize: Integer;
begin
  Result := size_of_united_items;
end;

procedure DividingSJIS.presetting(ans: AnsiString; s: Byte);
begin
  ANSI_ASCII := ans;
  separator_byte := s;

  ASCII_C := ANSI_ASCII[UNDERLINE_BYTE];
  EMPTY_ANSI_S := '';
  quote_byte := STANDARD_QUOTE_BYTE;
  remove_separators := COMMAS_SHALL_DISAPPEAR_IF_PLACED_IN_QUOTES;
  separator := EMPTY_ANSI_S + ANSI_ASCII[separator_byte];

  BLANK_C := ANSI_ASCII[GAP_BYTE];
  CLEAR_BIT_C := ANSI_ASCII[CLEAR_BIT];
  COMMA_C_S := ANSI_ASCII[COMMA_BYTE];
  DOT_C_S := ANSI_ASCII[H_2E];
  MINUS_C_S := ANSI_ASCII[MINUS_BYTE];
  SET_BIT_C := ANSI_ASCII[SET_BIT];
end;

procedure DividingSJIS.presetting(ans: AnsiString; q: Byte; s: Byte; e: Boolean);
begin
  ANSI_ASCII := ans;
  quote_byte := q;
  separator_byte := s;
  remove_separators := e;

  ASCII_C := ANSI_ASCII[UNDERLINE_BYTE];
  EMPTY_ANSI_S := '';
  separator := EMPTY_ANSI_S + ANSI_ASCII[separator_byte];

  BLANK_C := ANSI_ASCII[GAP_BYTE];
  CLEAR_BIT_C := ANSI_ASCII[CLEAR_BIT];
  COMMA_C_S := ANSI_ASCII[COMMA_BYTE];
  DOT_C_S := ANSI_ASCII[H_2E];
  MINUS_C_S := ANSI_ASCII[MINUS_BYTE];
  SET_BIT_C := ANSI_ASCII[SET_BIT];
end;

procedure DividingSJIS.presetting(ans: AnsiString; q: Byte; s: Byte; e: Boolean; a: Byte);
begin
  ANSI_ASCII := ans;
  quote_byte := q;
  separator_byte := s;
  remove_separators := e;

  ASCII_C := ANSI_ASCII[a];
  EMPTY_ANSI_S := '';
  separator := EMPTY_ANSI_S + ANSI_ASCII[separator_byte];

  BLANK_C := ANSI_ASCII[GAP_BYTE];
  CLEAR_BIT_C := ANSI_ASCII[CLEAR_BIT];
  COMMA_C_S := ANSI_ASCII[COMMA_BYTE];
  DOT_C_S := ANSI_ASCII[H_2E];
  MINUS_C_S := ANSI_ASCII[MINUS_BYTE];
  SET_BIT_C := ANSI_ASCII[SET_BIT];
end;

procedure DividingSJIS.vector_to_array_arranging_in_reverse_order;
var
  a_count_1: Integer;
  i: Integer;
  k: Integer;
begin
  a_count_1 := a.count - 1;
  k := a_count_1;
  for i := 0 to a_count_1 do
  begin
    f.setting( a.q(i), k);
    k := k - 1;
  end;

end;

procedure DividingSJIS.vector_to_array_complementing(extra: AnsiString; bond: Byte);
var
  a_count_1: Integer;
  s: AnsiString;
  i: Integer;
begin
  a_count_1 := a.count - 1;
  for i := 0 to a_count_1 do
  begin
    s := a.q(i);
    s := s + ANSI_ASCII[bond];
    s := s + extra;
    f.setting(s, i);
  end;

end;

procedure DividingSJIS.vector_to_array_editing(editing: EditFunction; field_number: Integer);
var
  a_count_1: Integer;
  field_number_1: Integer;
  i: Integer;
  s: AnsiString;
begin
  a_count_1 := a.count - 1;
  field_number_1 := field_number - 1;
  for i := 0 to a_count_1 do
  begin
    if i = field_number_1 then
      begin
        s := editing( a.q(i) );
        f.setting(s, i);
      end
    else
      begin
        f.setting( a.q(i), i);
      end;

  end;

end;

procedure DividingSJIS.vector_to_array_mapping;
var
  a_count_1: Integer;
  i: Integer;
begin
  a_count_1 := a.count - 1;
  for i := 0 to a_count_1 do
  begin
    f.setting( a.q(i), i);
  end;

end;

procedure DividingSJIS.setting(fields_list: FieldsList; c: Integer);
var
  m: Integer;
  z: AnsiString;
begin
  f := fields_list;
  field_count := c;

  manyFields[0] := True;

  for m:= 1 to field_count do
  begin
    manyFields[m] := True;
  end;

  a := AnsiStringQueue.Create;
  z := EMPTY_ANSI_S + BLANK_C;
  a.setting(ANSI_ASCII, z, 1);
end;

procedure DividingSJIS.setting(fields_list: FieldsList; c: Integer; pn: array of AnsiChar);
var
  b: Byte;
  k: Integer;
  m: Integer;
  i: Integer;
  z: AnsiString;
begin
  f := fields_list;
  field_count := c;

  manyFields[0] := True;

  for m:= 1 to field_count do
  begin
    k := m - 1;
    pattern[k] := pn[k];
    i := Ord( pattern[k] );
    b := Byte(i);

    if b = SET_BIT then
      manyFields[m] := True
    else
      manyFields[m] := False;

  end;

  a := AnsiStringQueue.Create;
  z := EMPTY_ANSI_S + BLANK_C;
  a.setting(ANSI_ASCII, z, 1);
end;

procedure DividingSJIS.setting_for_extracting;
var
  CLEAR_PETTY_ARRAY: PettyArray;
  k: Integer;
  m: Integer;
  p: PettyArray;
  pascal_string: AnsiString;
begin
  for m := 0 to 255 do
  begin
    CLEAR_PETTY_ARRAY[m] := #0;
    p[m] := #0;
  end;

  p := CLEAR_PETTY_ARRAY;
  m := 0;
  for k := H_30 to H_39 do
  begin
    p[m] := ANSI_ASCII[k];
    Inc(m);
  end;

  pascal_string := p;
  ANSI_N_0TO9 := Copy(pascal_string, 1, ANSI_N_0TO9_SIZE);

  p[m] := ANSI_ASCII[MINUS_BYTE];
  pascal_string := p;
  ANSI_N_0TO9_MINUS := Copy(pascal_string, 1, ANSI_N_0TO9_MINUS_SIZE);
end;

end.
