unit FinalEffect;

interface

uses
  SysUtils;

type
  FinalEffects = class
  private
    after: Integer;
    before: Integer;
    bottom_line_number: Integer;
    byte_count: Integer;
    carriage_return: Boolean;
    comb: Boolean;
    error_level: Integer;
    euc_jp: Boolean;
    field_count: Integer;
    field_number: Integer;
    file_name: String;
    interval_from: Integer;
    interval_to: Integer;
    joint_byte: Byte;
    label_byte: Byte;
    line_feed: Boolean;
    manage: Integer;
    maximumFieldCount: Integer;
    new_name: String;
    no_checking: Boolean;
    no_combining: Boolean;
    no_removing: Boolean;
    no_searching: Boolean;
    number_of_field_measured: Integer;
    pattern: String;
    focal_point: Integer;
    path_name: String;
    quote_byte: Byte;
    remove_separators: Boolean;
    reverse: Boolean;
    selected_directory_has_priority: Boolean;
    separator_byte: Byte;
    shared_sum: Boolean;
    shift_jis_text: Boolean;
    substituted: Boolean;
    surrogate_pair_to_underline: Boolean;
    temporary_binary: Boolean;
    top_line_number: Integer;
    which_table: Boolean;
  public
    function afterPoint: Integer;
    function atPoint: Integer;
    function beforePoint: Integer;
    function bottomLine: Integer;
    function carriageReturn: Boolean;
    function comber: Boolean;
    function equalSize: Integer;
    function errorLevel: Integer;
    function EUC_JP_file: Boolean;
    function fieldCount: Integer;
    function fieldMeasured: Integer;
    function fieldNumber: Integer;
    function fileName: String;
    function intervalFrom: Integer;
    function intervalTo: Integer;
    function jointByte: Byte;
    function labelByte: Byte;
    function lineFeed: Boolean;
    function manager: Integer;
    function multipleBitPattern: String;
    function newName: String;
    function noRemoving: Boolean;
    function noSearching: Boolean;
    function pathName: String;
    function priority: Boolean;
    function produceTemporaryBinary: Boolean;
    function quoteByte: Byte;
    function removeSeparators: Boolean;
    function reverseOrder: Boolean;
    function separatorByte: Byte;
    function sharedSum: Boolean;
    function Shift_JIS_file: Boolean;
    function substituteForDakuten: Boolean;
    function surrogatePairToUnderline: Boolean;
    function topLine: Integer;
    function upperLimit: Integer;
    function whichTable: Boolean;
    procedure tripletParameters(a: Integer; b: Integer; fp: Integer);
    procedure setting; overload;
    procedure setting(fn: String; nn: String; pn: String; p: String;
      bc: Integer; EL: Integer; fc: Integer; fr: Integer; im: Integer; LT: Integer; m: Integer; mf: Integer; nf: Integer; fp: Integer; 
      jb: Byte; LB: Byte; qb: Byte; sb: Byte; 
      cr: Boolean; c: Boolean; ej: Boolean; LD: Boolean; nc: Boolean; ng: Boolean; nr: Boolean; ns: Boolean; rs: Boolean; r: Boolean; sd: Boolean; ss: Boolean; st: Boolean; SUB: Boolean; sp: Boolean; tb: Boolean; wt: Boolean); overload;
  end;

implementation

function FinalEffects.afterPoint: Integer;
begin
  Result := after;
end;

function FinalEffects.atPoint: Integer;
begin
  Result := focal_point;
end;

function FinalEffects.beforePoint: Integer;
begin
  Result := before;
end;

function FinalEffects.bottomLine: Integer;
begin
  Result := bottom_line_number;
end;

function FinalEffects.carriageReturn: Boolean;
begin
  Result := carriage_return;
end;

function FinalEffects.comber: Boolean;
begin
  Result := comb;
end;

function FinalEffects.equalSize: Integer;
begin
  Result := byte_count;
end;

function FinalEffects.errorLevel: Integer;
begin
  Result := error_level;
end;

function FinalEffects.EUC_JP_file: Boolean;
begin
  Result := euc_jp;
end;

function FinalEffects.fieldCount: Integer;
begin
  Result := field_count;
end;

function FinalEffects.fieldMeasured: Integer;
begin
  Result := number_of_field_measured;
end;

function FinalEffects.fieldNumber: Integer;
begin
  Result := field_number;
end;

function FinalEffects.fileName: String;
begin
  Result := file_name;
end;

function FinalEffects.intervalFrom: Integer;
begin
  Result := interval_from;
end;

function FinalEffects.intervalTo: Integer;
begin
  Result := interval_to;
end;

function FinalEffects.jointByte: Byte;
begin
  Result := joint_byte;
end;

function FinalEffects.labelByte: Byte;
begin
  Result := label_byte;
end;

function FinalEffects.lineFeed: Boolean;
begin
  Result := line_feed;
end;

function FinalEffects.manager: Integer;
begin
  Result := manage;
end;

function FinalEffects.multipleBitPattern: String;
begin
  Result := pattern;
end;

function FinalEffects.newName: String;
begin
  Result := new_name;
end;

function FinalEffects.noRemoving: Boolean;
begin
  Result := no_removing;
end;

function FinalEffects.noSearching: Boolean;
begin
  Result := no_searching;
end;

function FinalEffects.pathName: String;
begin
  Result := path_name;
end;

function FinalEffects.priority: Boolean;
begin
  Result := selected_directory_has_priority;
end;

function FinalEffects.produceTemporaryBinary: Boolean;
begin
  Result := temporary_binary;
end;

function FinalEffects.quoteByte: Byte;
begin
  Result := quote_byte;
end;

function FinalEffects.removeSeparators: Boolean;
begin
  Result := remove_separators;
end;

function FinalEffects.reverseOrder: Boolean;
begin
  Result := reverse;
end;

function FinalEffects.separatorByte: Byte;
begin
  Result := separator_byte;
end;

function FinalEffects.sharedSum: Boolean;
begin
  Result := shared_sum;
end;

function FinalEffects.Shift_JIS_file: Boolean;
begin
  Result := shift_jis_text;
end;

function FinalEffects.substituteForDakuten: Boolean;
begin
  Result := substituted;
end;

function FinalEffects.surrogatePairToUnderline: Boolean;
begin
  Result := surrogate_pair_to_underline;
end;

function FinalEffects.topLine: Integer;
begin
  Result := top_line_number;
end;

function FinalEffects.upperLimit: Integer;
begin
  Result := maximumFieldCount;
end;

function FinalEffects.whichTable: Boolean;
begin
  Result := which_table;
end;

procedure FinalEffects.setting;
begin
{ String }
  file_name := '';
  new_name := '';
  path_name := '';
  pattern := '';

{ Integer }
  byte_count := 6;
  field_count := 16;
  field_number := 1;
  interval_from := 1;
  interval_to := 16;
  manage := 0;
  maximumFieldCount := 255;
  number_of_field_measured := 1;
  focal_point := 1;

{ Byte }
  joint_byte := 124; { | }
  label_byte := 60; { < }
  quote_byte := 34; { " }
  separator_byte := 44; { , }

{ Boolean }
  carriage_return := True;
  comb := False;
  error_level := 0;
  euc_jp := False;
  line_feed := False;
  no_checking := False;
  no_combining := False;
  no_removing := False;
  no_searching := False;
  remove_separators := False;
  reverse := False;
  selected_directory_has_priority := False;
  shared_sum := False;
  shift_jis_text := True;
  substituted := False;
  surrogate_pair_to_underline := True;
  temporary_binary := False;
  which_table := False;
end;

procedure FinalEffects.setting(fn: String; nn: String; pn: String; p: String; 
  bc: Integer; EL: Integer; fc: Integer; fr: Integer; im: Integer; LT: Integer; m: Integer; mf: Integer; nf: Integer; fp: Integer; 
  jb: Byte; LB: Byte; qb: Byte; sb: Byte; 
  cr: Boolean; c: Boolean; ej: Boolean; LD: Boolean; nc: Boolean; ng: Boolean; nr: Boolean; ns: Boolean; rs: Boolean; r: Boolean; sd: Boolean; ss: Boolean; st: Boolean; SUB: Boolean; sp: Boolean; tb: Boolean; wt: Boolean); 

begin
{ String }
  file_name := fn;
  new_name := nn;
  path_name := pn;
  pattern := p;

{ Integer }
  byte_count := bc;
  error_level := EL;
  field_count := fc;
  field_number := fr;
  interval_from := im;
  interval_to := LT;
  manage := m;
  maximumFieldCount := mf;
  number_of_field_measured := nf;
  focal_point := fp;

{ Byte }
  joint_byte := jb;
  label_byte := LB;
  quote_byte := qb;
  separator_byte := sb;

{ Boolean }
  carriage_return := cr;
  comb := c;
  euc_jp := ej;
  line_feed := LD;
  no_checking := nc;
  no_combining := ng;
  no_removing := nr;
  no_searching := ns;
  remove_separators := rs;
  reverse := r;
  selected_directory_has_priority := sd;
  shared_sum := ss;
  shift_jis_text := st;
  substituted := SUB;
  surrogate_pair_to_underline := sp;
  temporary_binary := tb;
  which_table := wt;
end;

procedure FinalEffects.tripletParameters(a: Integer; b: Integer; fp: Integer);
begin
  after := a;
  before := b;
  focal_point := fp;
end;

end.
