unit Guide;

{$I+}

interface

uses
  SysUtils;

type
  Guiding = class
  public
    procedure versionGuide(v: String; comments: Boolean);
  end;

const
  VERSION_TEXT = 'version.txt';

implementation

procedure Guiding.versionGuide(v: String; comments: Boolean);
var
  dataFile: TextFile;
begin
  AssignFile(dataFile, VERSION_TEXT);
  Rewrite(dataFile);
  WriteLn(dataFile, 'Narfi Delphi Version ' + v );
  WriteLn(dataFile, '');
  if comments then
    begin
      WriteLn(dataFile, 'narfi -c4 wide             # Produce narfi.txt that has 4 fields. ');
      WriteLn(dataFile, 'narfi -c4 wide narrow      # Produce a narrow file that has 4 fields. ');
      WriteLn(dataFile, 'narfi -c4 -F narrow wide   # Produce a narrow file that has 4 fields. ');
      WriteLn(dataFile, 'narfi -c4 -b1011 wide      # narfi.txt has Field-1, Field-3 and Field-4. ');
      WriteLn(dataFile, 'narfi -c4 -d2 wide         # Remove combining characters. ');
      WriteLn(dataFile, 'narfi -c4 -d2 -p wide      # H and I are omitted from ASCII characters. ');
      WriteLn(dataFile, 'narfi -c4 -E wide          # You can recognize Narfi Delphi from the title bar.');
      WriteLn(dataFile, 'narfi -c4 -e2 wide         # Extract a decimal from Field-2. ');
      WriteLn(dataFile, 'narfi -c4 -g1 wide         # Move the Narfi window to Corner-1. ');
      WriteLn(dataFile, 'narfi -c4 -k2 -a_ wide     # In Field-2, Unarfi replaces katakana with ascii.');
      WriteLn(dataFile, 'narfi -c4 -k2 -A\x5F wide  # In Field-2, Unarfi replaces katakana with ascii.');
      WriteLn(dataFile, 'narfi -c4 -n2 wide         # Extract an integer from Field-2. ');
      WriteLn(dataFile, 'narfi -c4 -o2 wide         # Remove control characters from Field-2. ');
      WriteLn(dataFile, 'narfi -c4 -q wide          # Commas disappear if placed in quotes. ');
      WriteLn(dataFile, 'narfi -c4 -r wide          # Arrange fields in reverse order. ');
      WriteLn(dataFile, 'narfi -c4 -s, wide         # CSV ');
      WriteLn(dataFile, 'narfi -c4 -S\l wide        # Wiki table ');
      WriteLn(dataFile, 'narfi -c4 -S\l -x- csv     # Do not check if the file name includes .csv ');
      WriteLn(dataFile, 'narfi -c4 -S\l -$ wide     # Wiki table written in Shift_JIS ');
      WriteLn(dataFile, 'narfi -c4 -t5-9 wide       # Display Line-5, Line-6, Line-7, Line-8 and Line-9.');
      WriteLn(dataFile, 'narfi -c4 --u wide         # EUC-JP ');
      WriteLn(dataFile, 'narfi -c4 --u -D wide      # Do not search a table. ');
      WriteLn(dataFile, 'narfi -c4 --u -K wide      # Acquire a table name from sets.txt ');
      WriteLn(dataFile, 'narfi -c4 --u -LF wide     # 0Dh is removed from every line. ');
      WriteLn(dataFile, 'narfi -c4 --u -M  wide     # -M shows which table narfi uses. ');
      WriteLn(dataFile, 'narfi -c4 --u -y wide      # Do not remove temporary files. ');
      WriteLn(dataFile, 'narfi -c4 -w~ wide         # ~ is used as quote. ');
      WriteLn(dataFile, 'narfi -c4 -W\x26 wide      # & is used as quote. ');
      WriteLn(dataFile, 'narfi -@1587 dummy         # Produce error.txt from the %ERRORLEVEL% value. ');
      WriteLn(dataFile, 'narfi -f wide              # Check if all lines share the field count. ');
      WriteLn(dataFile, 'narfi -i2-3 wide           # narfi.txt has Field-2 and Field-3. ');
      WriteLn(dataFile, 'narfi -j2 wide             # narfi.txt has Field-2 only. ');
      WriteLn(dataFile, 'narfi -G                   # Guide ');
      WriteLn(dataFile, 'narfi -h                   # Adobe Reader reads Narfi.pdf ');
      WriteLn(dataFile, 'narfi -V                   # Version ');
      WriteLn(dataFile, 'narfi -Z wide              # The number of fields may exceed 255. ');
      WriteLn(dataFile, 'narfi -z1 -# wide          # Narrow blanks so that mesh can resemble a comb. ');
      WriteLn(dataFile, 'narfi -z5 wide             # Measure Field-5. Record the max. ');
      WriteLn(dataFile, 'narfi -z5 -P182 wide       # Measure Field-5. Diplay Line-182. ');
    end
  else
    begin
      WriteLn(dataFile, 'narfi --help               ');
    end;

  CloseFile(dataFile);
end;

end.
