unit NarfiForm;

{*
Narfi Delphi Version 1.05
Selecting fields from CSV to produce a narrow file
Copyright 2020 Sogaya

Delphi 2009

*}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, Menus, ComCtrls, Grids, ExtCtrls, ShellAPI, 
  Around, ConvertEUC, ConvertWide, ErrorMessage, FinalEffect, Guide, HyphenTerm, MaxSpan, MessagesText, PutCSV, RebuildEUC, RebuildSJIS, SurvivorUTF16;

type CharArray = array[0..4095] of Char;
type PettyArray = array[0..255] of Char;

type
  TForm1 = class(TForm)
    HeaderControl1: THeaderControl;
    StatusBar1: TStatusBar;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Window1: TMenuItem;
    Center1: TMenuItem;
    Comb1: TMenuItem;
    Corner1: TMenuItem;
    Diagonal1: TMenuItem;
    Right1: TMenuItem;
    ight1: TMenuItem;
    Under1: TMenuItem;
    Help1: TMenuItem;
    Version1: TMenuItem;
    StringGrid1: TStringGrid;
    Timer1: TTimer;
    Timer2: TTimer;
    Font1: TMenuItem;
    FontDialog1: TFontDialog;
    procedure Version1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
    procedure HeaderControl1SectionTrack(HeaderControl: THeaderControl;
      Section: THeaderSection; Width: Integer; State: TSectionTrackState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Exit1Click(Sender: TObject);
    procedure Under1Click(Sender: TObject);
    procedure Right1Click(Sender: TObject);
    procedure Center1Click(Sender: TObject);
    procedure Comb1Click(Sender: TObject);
    procedure Corner1Click(Sender: TObject);
    procedure Diagonal1Click(Sender: TObject);
    procedure ight1Click(Sender: TObject);
    procedure Font1Click(Sender: TObject);
  private
    after: Integer;
    before: Integer;
    carriage_return: Boolean;
    corner: Integer;
    field_count: Integer;
    field_number: Integer;
    final_effects: FinalEffects;
    final_separator: Byte;
    focal_point: Integer;
    line_count: Integer;
    measure_span: MeasureSpan;
    number_of_field_measured: Integer;
    pattern: String;
    shift_jis_text: Boolean;
    statusBarCode: String;
    statusBarText: String;
    function statusBarEncoding(euc: Boolean): Boolean;
    procedure moveToCorner;
    procedure seePDF;
    procedure setting;
    procedure statusBarMessage(s: String);
  public
    around_point: AroundPoint;
    CLEAR_V_ARRAY: CharArray;
    combo: array[0..4095] of Char;
    comboLength: Integer;
    v: CharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    widest_item_in_field_f: Integer;
    procedure meshResemblesComb;
    procedure widenFields(row_number: Boolean);
  end;

const
  AROUND_TEXT = 'around.txt';
  ASCII_1 = ' !"#$%&()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~';
  BORDER = 28;
  COMMA_BYTE = 44; { ',' }
  COMMAS_DISAPPEAR_IF_PLACED_IN_QUOTES = True;
  DATA_TEXT = 'data.txt';
  DIAGONAL = 3;
  DO_NOT_REMOVE_ANCIENT_FILE = True;
  DO_NOT_REMOVE_NEW_FILE = True;
  DUMMY_FILE = 'dummy';
  EMPTY = '';
  EQUAL_SIZE = 6; { Each field has a width of 6 bytes. }
  EUC_JP_DATA = False;
  EXCLAMATION_MARK = '!';
  EXPANSION_FACTOR = 2;
  EXTRACT_DECIMAL_FROM_FIELD = 1;
  EXTRACT_INTEGER_FROM_FIELD = 2;
  FINAL_INTEGER = 2147483647; { 7FFFFFFFh }
  MAIN_HEIGHT = 494;
  MAIN_WIDTH = 1053;
  MAXIMUM_COLUMN_COUNT = 16;
  MAXIMUM_COLUMN_COUNT_1 = 15;
  MAXIMUM_FIELD_COUNT = 255;
  N_0TO9 = '0123456789';
  NARFI_DELPHI = 'Narfi Delphi';
  NARFI_NAME = 'Narfi';
  NARFI_TEXT = 'narfi.txt';
  NARROW_SECTION = 42;
  NEITHER_EXTRACT_NOR_REMOVE = 0;
  NO_NUMBERING = False;
  NON_NUMBER_MAY_APPEAR = False;
  NOT_MEASURED = -1;
  PATH_TEXT = 'path.txt';
  PDF_FILE_NAME = 'Narfi.pdf';
  PIXEL_TO_BYTE_RATIO = 12;
  RECORD_MAXIMUM = True;
  REMOVE_CONTROL_CHARACTERS_FROM_FIELD = 3;
  REMOVE_DAKUTEN_FROM_FIELD = 5;
  REPLACE_KATAKANA_WITH_ASCII = 4;
  ROW_NUMBER_APPEARS = True;
  SERIAL_NUMBER = True;
  SETTING_PHASE = 70; { milliseconds }
  SHIFT_JIS_TEXT_FILE = True;
  SIZE_OF_CHAR = 2;
  SLIT_WIDTH = 3;
  SOLID_LINE = 1;
  SOLID_LINE_BORDER = 2;
  SURRO_CSV = 'surro.csv';
  TASK_BAR_HEIGHT = 43;
  TEMPORARY_BINARY = 'binary.tmp';
  TRY_TO_TABULATE = True;
  UNDERLINES_REPLACE_SURROGATE_PAIRS = True;
  UPPER_LIMIT_TO_FIELD_NUMBER = 3071;
  VERSION_NUMBER = '1.05';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;
  WIDE_SECTION = 130;

var
  Form1: TForm1;

implementation

uses VersionForm;

{$R *.dfm}

function TForm1.statusBarEncoding(euc: Boolean): Boolean;
var
  b: Boolean;
begin
  if euc then
    begin
      statusBarCode := 'EUC-JP';
      b := False;
    end
  else
    begin
      statusBarCode := 'Shift_JIS';
      b := True;
    end;

  StatusBar1.Panels[0].Text := statusBarCode;

  Result := b;
end;

procedure TForm1.Center1Click(Sender: TObject);
begin
  corner := 0;
  moveToCorner;
end;

procedure TForm1.Comb1Click(Sender: TObject);
begin
  meshResemblesComb;
  moveToCorner;
end;

procedure TForm1.Corner1Click(Sender: TObject);
begin
  corner := 1;
  moveToCorner;
end;

procedure TForm1.Diagonal1Click(Sender: TObject);
begin
  corner := DIAGONAL;
  moveToCorner;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.Font1Click(Sender: TObject);
begin
  if FontDialog1.Execute then
    StringGrid1.Font := FontDialog1.Font;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  KeyPreview := True;
  BorderIcons := [biSystemMenu,biMinimize];
  Height := MAIN_HEIGHT;
  Width := MAIN_WIDTH;
  HeaderControl1.Height := 17;
  HeaderControl1.Width := 1045;
  StringGrid1.Height := 404;
  StringGrid1.Width := 1045;
  StringGrid1.ColCount := MAXIMUM_COLUMN_COUNT;
  StringGrid1.RowCount := 16;
  StatusBar1.Panels[0].Width := 100;
  statusBarMessage( GetCurrentDir );

  Timer2.Interval := SETTING_PHASE;
  Timer2.Enabled := False;
  Timer1.Interval := SETTING_PHASE;
  Timer1.Enabled := True;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  around_point.Free;
  final_effects.Free;
  measure_span.Free;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_F3 then
    Close;

end;

procedure TForm1.HeaderControl1SectionTrack(HeaderControl: THeaderControl;
  Section: THeaderSection; Width: Integer; State: TSectionTrackState);
var
  i: Integer;
begin
  if  HeaderControl1.Sections[0] = Section then
    begin
      StringGrid1.ColWidths[0] := Width - 3;
    end
  else
    begin
      for i := 1 to MAXIMUM_COLUMN_COUNT_1 do
      begin
        if  HeaderControl1.Sections[i] = Section then
          StringGrid1.ColWidths[i] := Width - 1;

      end;
    end;

end;

procedure TForm1.ight1Click(Sender: TObject);
var
  i: Integer;
  sum: Integer;
begin
  sum := 0;
  for i := 0 to MAXIMUM_COLUMN_COUNT_1 do
  begin
    sum := sum + StringGrid1.ColWidths[i];
  end;

  Width := sum + BORDER;
end;

procedure TForm1.meshResemblesComb;
var
  available: Integer;
  f: FinalEffects;
  field_width: Integer;
  i: Integer;
  m: MeasureSpan;
  occupied: Integer;
  shared_sum: Integer;
  shared_sum_1: Integer;
  space_width: Integer;
  whole: Integer;
begin
  occupied := 0;

  f := final_effects;
  m := measure_span;
  m.setting(f);

  if f.Shift_JIS_file then
    begin
      m.countFields;
    end
  else
    begin
      m.countFieldsEUC;
    end;

  shared_sum := m.fieldCount;
  shared_sum_1 := shared_sum - 1;
  if shared_sum_1 > 0 then
    begin
{ width of the whole mesh }
      whole := 0;
      for i := 0 to MAXIMUM_COLUMN_COUNT_1 do
      begin
        whole := whole + StringGrid1.ColWidths[i];
      end;

{ comb area }
      field_width := SLIT_WIDTH;
      space_width := field_width - SOLID_LINE;
      for i := shared_sum to MAXIMUM_COLUMN_COUNT_1 do
      begin
        HeaderControl1.Sections[i].Width := field_width;
        StringGrid1.ColWidths[i] := space_width;
        occupied := occupied + field_width;
      end;

{ Field-1 }
      space_width := StringGrid1.ColWidths[0];
      field_width := space_width + SOLID_LINE_BORDER;
      HeaderControl1.Sections[0].Width := field_width;
      occupied := occupied + field_width;

{ Field-2, Field-3, Field-4, ... }
      available := whole - occupied;
      field_width := available div shared_sum_1; { if a field is not Field-1 }
      if field_width < NARROW_SECTION then
        field_width := NARROW_SECTION;

      space_width := field_width - SOLID_LINE;
      for i := 1 to shared_sum_1 do
      begin
        HeaderControl1.Sections[i].Width := field_width;
        StringGrid1.ColWidths[i] := space_width;
      end;

    end;

end;

procedure TForm1.moveToCorner;
begin
  case corner of
    0: begin
         Left := ( Screen.Width - Width ) div 2;
         Top := ( Screen.Height - Height ) div 2;
       end;
    1: begin
         Left := 0;
         Top := 0;
       end;
    2: begin
         Left := Screen.Width - Width;
         Top := 0;
       end;
    3: begin
         Left := Screen.Width - Width;
         Top := Screen.Height - MAIN_HEIGHT - TASK_BAR_HEIGHT;
       end;
    4: begin
         Left := 0;
         Top := Screen.Height - MAIN_HEIGHT - TASK_BAR_HEIGHT;
       end;

  else { Corner-3 }
    Left := Screen.Width - Width;
    Top := Screen.Height - MAIN_HEIGHT - TASK_BAR_HEIGHT;
  end;

end;

procedure TForm1.Right1Click(Sender: TObject);
begin
  corner := 2;
  moveToCorner;
end;

procedure TForm1.seePDF;
begin
  ShellExecute(Handle, 'open', PDF_FILE_NAME, nil, nil, SW_NORMAL);
end;

procedure TForm1.setting;
var
  e: ErrorMessages;
  f: FinalEffects;
  file_name: String;
  flood_ebb: String;
  g: Guiding;
  h: HyphenTerms;
  k: Integer;
  m: MeasureSpan;
  maximumFieldCount: Integer;
  multipattern: String;
  new_name: String;
  new_name_length: Integer;
  p: PutSurroCSV;
  s: String;
  testFile: File;
  w: MessagesTexts;
begin
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;
  maximumFieldCount := h.upperLimit;

  if h.fullName then
    Caption := NARFI_DELPHI
  else
    Caption := NARFI_NAME;

  k := h.cornerNumber;
  if k >= 0 then
    corner := k
  else
    corner := DIAGONAL;

  moveToCorner;

{ Checking if software can open SURRO_CSV in the current directory. }
  try
    AssignFile(testFile, SURRO_CSV);
    Reset(testFile);
    CloseFile(testFile);
  except
{ Producing SURRO_CSV }
    p := PutSurroCSV.Create;
    p.setting(SURRO_CSV);
  end;

  if h.PDFhelp then
    begin
      seePDF;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.guideFile then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          if h.versionNumber then
            begin
              g := Guiding.Create;
              g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
              g.Free;

              h.Free;
              Halt(1);
            end
          else
            begin
              field_number := h.fieldNumber;
              number_of_field_measured := h.fieldMeasured;

              if h.lineFeed then
                begin
                  if h.carriageReturn then
                    begin
                      carriage_return := True;
                    end
                  else
                    begin
                      carriage_return := False;
                    end;

                end
              else
                begin
                  carriage_return := True;
                end;

              statusBarEncoding(h.EUC_JP_file);
              if h.EUC_JP_file then
                begin
                  if h.Shift_JIS_file then
                    begin
                      shift_jis_text := True;
                    end
                  else
                    begin
                      shift_jis_text := False;
                    end;

                end
              else
                begin
                  shift_jis_text := True;
                end;

              field_count := h.fieldCount;
              if field_count = maximumFieldCount then
                begin
                  pattern := h.singleField;
                  if not h.binaryRepresented then
                    begin
                      pattern := h.multipleBitPattern;

                      if not h.binaryRepresented then
                        begin
                          pattern := h.negativePattern;
                          if not h.binaryRepresented then
                            begin
                              pattern := h.singleSurge;
                            end;

                        end;

                    end;

                end
              else
                begin
    { -c5, for example }
                  flood_ebb := h.fieldCountLessThanMax;

                  multipattern := h.multipleBitPattern;
                  if not h.binaryRepresented then
                    begin
                      multipattern := h.negativePattern;
                      if not h.binaryRepresented then
                        begin
                          pattern := flood_ebb;
                        end
                      else
                        begin
                          pattern := h.gating(flood_ebb, multipattern);
                        end;

                    end
                  else
                    begin
                      pattern := h.gating(flood_ebb, multipattern);
                    end;

                end;

              file_name := h.fileName;
              h.joints; { joint_byte }
              h.labels; { label_byte }
              h.quotes; { quote_byte }
              h.separators; { separator_byte }
              if h.noChecking then
                begin
                  final_separator := h.separatorByte;
                end
              else
                begin
                  if h.extensionEqualsCSV(file_name) then
                    begin
                      final_separator := COMMA_BYTE;
                    end
                  else
                    begin
                      final_separator := h.separatorByte;
                    end;

                end;

              new_name := h.listName(1);
              new_name_length := Length(new_name);
              if new_name_length = 0 then
                begin
                  new_name := h.newName;
                end;

              if h.errorLevel > 1 then
                begin
                  e := ErrorMessages.Create;
                  e.guideNumber(VERSION_NUMBER, h.errorLevel);

                  if Length(file_name) > 0 then
                    begin
                      if CompareStr(file_name, DUMMY_FILE) = 0 then
                        begin
                          ;
                        end
                      else
                        begin
                          if CompareStr(file_name, EXCLAMATION_MARK) = 0 then
                            begin
                              ;
                            end
                          else
                            begin
                              s := 'file_name = ' + file_name;
                              w := MessagesTexts.Create;
                              w.setting(DATA_TEXT, s);
                              w.Free;
                            end;

                        end;

                    end;

                  if Length(new_name) > 0 then
                    begin
                      s := 'new_name = ' + new_name;
                      w := MessagesTexts.Create;
                      w.setting(DATA_TEXT, s);
                      w.Free;
                    end;

                  e.Free; h.Free;
                  Halt(0);
                end;

              final_effects := FinalEffects.Create;
              measure_span := MeasureSpan.Create;
              around_point := AroundPoint.Create;
              around_point.openClose(file_name);

              f := final_effects;
              f.setting(file_name, new_name, h.pathName, pattern, 
                h.equalSize, h.errorLevel, field_count, field_number, h.intervalFrom, h.intervalTo, h.manager, maximumFieldCount, number_of_field_measured, h.atPoint, 
                h.jointByte, h.labelByte, h.quoteByte, final_separator, 
                carriage_return, h.comber, h.EUC_JP_file, h.lineFeed, h.noChecking, h.noCombining, h.noRemoving, h.noSearching, h.removeSeparators, h.reverseOrder, h.priority, h.sharedSum, shift_jis_text, h.substituteForDakuten, UNDERLINES_REPLACE_SURROGATE_PAIRS, h.produceTemporaryBinary, h.whichTable);

{ Counting lines by LF_to_LF_span }
              m := measure_span;
              m.setting(f.fileName, f.atPoint, f.quoteByte, f.separatorByte, f.Shift_JIS_file, f.removeSeparators);
              m.ifErrorOccurs(BUFFER_ERROR);
              line_count := m.LF_to_LF_span;

              h.tailEnd; { NOT_COUNTED }
              focal_point := h.tripletPoint(line_count);
              if focal_point < 1 then
                begin
                  h.tailEnd(line_count);
                end;

              after := h.afterPoint;
              before := h.beforePoint;
              focal_point := h.atPoint(line_count);

              if (after = 0) and (before = 0) then
                begin
                  after := h.tripletAfter;
                  before := h.tripletBefore;
                  focal_point := h.tripletPoint;
                end;

              f.tripletParameters(after, before, focal_point);
              h.Free;
            end;

        end;

    end;

end;

procedure TForm1.statusBarMessage(s: String);
begin
  statusBarText := s;
  StatusBar1.Panels[1].Text := statusBarText;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  setting;
  Timer1.Enabled := False;
  Timer2.Enabled := True;
end;

procedure TForm1.Timer2Timer(Sender: TObject);
var
  a: AroundPoint;
  c: Conversion;
  column_count_1: Integer;
  f: FinalEffects;
  i: Integer;
  j: Integer;
  m: MeasureSpan;
  n: Integer;
  pascal_string: String;
  r: DestroyRebuildSJIS;
  row_count_1: Integer;
  s: String;
  t: FinalEffects;
  temporaryFile: File;
  u: ConvertWiden;
  w: MessagesTexts;
  y: DestroyRebuildEUC;
  z: AnsiString;
begin
  V_SIZE := SizeOf(v) div SIZE_OF_CHAR;
  V_SIZE_1 := V_SIZE - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_V_ARRAY[i] := #0;
  end;


  f := final_effects;
  m := measure_span;

  if Length(f.fileName) = 0 then
    begin
      Halt(1589);
    end;

  m.setting_beginning(f.fileName);

  if f.Shift_JIS_file then
    begin
      m.countFields;

      if f.sharedSum then
        begin
          Halt(0);
        end;

      r := DestroyRebuildSJIS.Create;
      s := f.multipleBitPattern;
      r.copyingPatternFromStringToArray( PChar(s), Length(s), field_count);
      r.setting(f.fileName, f.quoteByte, f.separatorByte, field_count, f.fieldNumber, f.labelByte, f.carriageReturn, f.removeSeparators, f.substituteForDakuten, f.reverseOrder);

      case f.manager of
        0: r.selecting(f.newName);
        1: r.selecting_extracting_decimal(f.newName);
        2: r.selecting_extracting_integer(f.newName);
        3: r.non_controller_selecting(f.newName);
        4: r.non_kana_selecting(f.newName);
        5: r.non_dakuten_selecting(f.newName);
        6: r.reversing(f.newName);
      else
        ;
      end;

      r.produceBinaryFile(f.newName, DO_NOT_REMOVE_NEW_FILE);

      r.Free;
    end
  else
    begin
      m.countFieldsEUC;

      if f.sharedSum then
        begin
          Halt(0);
        end;

      y := DestroyRebuildEUC.Create;
      s := f.multipleBitPattern;
      y.copyingPatternFromStringToArray( PChar(s), Length(s), field_count);
      y.setting(f.fileName, f.quoteByte, f.separatorByte, field_count, f.fieldNumber, f.labelByte, f.carriageReturn, f.removeSeparators, f.substituteForDakuten, f.reverseOrder);

      case f.manager of
        0: y.selecting(f.newName);
        1: y.selecting_extracting_decimal(f.newName);
        2: y.selecting_extracting_integer(f.newName);
        3: y.non_controller_selecting(f.newName);
        5: y.non_dakuten_selecting(f.newName);
        6: y.reversing(f.newName);
      else
        ;
      end;

      c := Conversion.Create;
      c.setting(f, f.newName);
      c.reading_to_produce_MS932_text(TEMPORARY_BINARY);

      c.Free; y.Free;
    end;

  if f.equalSize = NOT_MEASURED then { byte_count, HyphenTerms.fieldMeasured, COUNT_BYTES, '-z' }
    begin
{ Examining the file (not a new file) and finding the widest item in Field-f }
      if f.Shift_JIS_file then
        begin
          m.setting_beginning(f.fileName);
          m.reading;

          m.setting_beginning(f.fileName);
          widest_item_in_field_f := m.widestItem(number_of_field_measured);

          s := m.focusedApproachToItem(number_of_field_measured);
          w := MessagesTexts.Create;
          w.setting(DATA_TEXT, s);
          w.Free;
        end
      else
        begin
          m.setting_beginning(f.fileName);
          m.readingEUC;

          m.setting_beginning(f.fileName);
          widest_item_in_field_f := m.widestItem(number_of_field_measured);

          s := m.focusedApproachToItem(number_of_field_measured);
          w := MessagesTexts.Create;
          w.setting(DATA_TEXT, s, EUC_JP_DATA);
          w.Free;
        end;

    end
  else
    begin
      widest_item_in_field_f := NOT_MEASURED;
    end;

{ Reading lines at and around the specified point to produce TEMPORARY_BINARY }
  a := around_point;
  a.presetting(f.jointByte, f.quoteByte, f.separatorByte, f.removeSeparators);
  a.setting(TEMPORARY_BINARY, f.afterPoint, f.beforePoint, f.atPoint, NO_NUMBERING);

{ TEMPORARY_BINARY is a Shift_JIS CSV. }
  a.reading_CSV;

  if widest_item_in_field_f = NOT_MEASURED then
    begin
      ;
    end
  else
    begin
      m.reading;
      s := m.focusedApproachToItem(number_of_field_measured);
      w := MessagesTexts.Create;
      w.setting(DATA_TEXT, s);
      w.Free;

      widenFields(NON_NUMBER_MAY_APPEAR);
    end;

  if f.comber then
    meshResemblesComb;

  with StringGrid1 do
  begin
    t := FinalEffects.Create;
    t.setting;
    u := ConvertWiden.Create;
    u.setting(t);

    column_count_1 := ColCount - 1;
    row_count_1 := RowCount - 1;
    for i := 0 to column_count_1 do
    begin
      for j:= 0 to row_count_1 do
      begin
        z := a.columnRow(i, j);

        u.produce_UTF16_phrase( PAnsiChar(z), Length(z) );
        V_SIZE := u.UTF16_phrase_size;
        V_SIZE_1 := V_SIZE - 1;
        for n := 0 to V_SIZE_1 do
        begin
          v[n] := u.copyingArrayItem(n);
        end;

        pascal_string := v;
        s := Copy(pascal_string, 1, V_SIZE);

        Cells[i, j] := s;
      end;

    end;

    u.Free; t.Free;
  end; { with StringGrid1 do }

  if not f.noRemoving then
    begin
      AssignFile(temporaryFile, TEMPORARY_BINARY);
      Erase(temporaryFile);
    end;

  Timer2.Enabled := False;
end;

procedure TForm1.Under1Click(Sender: TObject);
begin
  corner := 4;
  moveToCorner;
end;

procedure TForm1.Version1Click(Sender: TObject);
begin
  Form2.setVersion(PDF_FILE_NAME, VERSION_NUMBER);
  Form2.Visible := True;
end;

procedure TForm1.widenFields(row_number: Boolean);
var
  available: Integer;
  f_1: Integer;
  field_width: Integer;
  i: Integer;
  occupied: Integer;
  space_width: Integer;
  width_of_Field_f: Integer;
  whole: Integer;
begin
  occupied := 0;

{ width of the whole mesh }
  whole := 0;
  for i := 0 to MAXIMUM_COLUMN_COUNT_1 do
  begin
    whole := whole + StringGrid1.ColWidths[i];
  end;

{ width of Field-f }
  f_1 := number_of_field_measured - 1;
  field_width := PIXEL_TO_BYTE_RATIO * widest_item_in_field_f;
  if field_width < NARROW_SECTION then
    field_width := NARROW_SECTION;

  width_of_Field_f := field_width;
  occupied := occupied + width_of_Field_f;

{ Field-1 }
  if row_number then
    begin
      field_width := WIDE_SECTION;
    end
  else
    begin
      field_width := NARROW_SECTION;
    end;

  space_width := field_width - SOLID_LINE;
  HeaderControl1.Sections[0].Width := field_width;
  StringGrid1.ColWidths[0] := space_width;
  occupied := occupied + field_width;

{ Field-2, Field-3, Field-4, ... }
  available := whole - occupied;
  field_width := available div (MAXIMUM_COLUMN_COUNT - 2 ); { if a field is neither Field-1 nor Field-f }
  if field_width < NARROW_SECTION then
    field_width := NARROW_SECTION;

  space_width := field_width - SOLID_LINE;
  for i := 1 to MAXIMUM_COLUMN_COUNT_1 do
  begin
    HeaderControl1.Sections[i].Width := field_width;
    StringGrid1.ColWidths[i] := space_width;
  end;

{ Field-f }
  space_width := width_of_Field_f - SOLID_LINE;
  HeaderControl1.Sections[f_1].Width := width_of_Field_f;
  StringGrid1.ColWidths[f_1] := space_width;
end;

end.
