unit RebuildSJIS;

{$I+}

interface

uses
  Windows, SysUtils, FinalEffect, DivideSJIS, FieldsArray, NoncontrollerSJIS, NondakutenSJIS, NonkanaSJIS;

type AnsiCharArray = array[0..4095] of AnsiChar;

type
  DestroyRebuildSJIS = class
  private
    ANSI_ASCII: AnsiString;
    ANSI_CHAR: array[0..255] of AnsiChar;
    ascii_byte: Byte;
    beginning: Integer;
    buffer: array of AnsiChar;
    buffer_size: Integer;
    carriage_return: Boolean;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_BIT_C: AnsiChar;
    count: Integer;
    counted: Boolean;
    d: DividingSJIS;
    dakuten_byte: Byte;
    dough: array[0..98303] of AnsiChar;
    doughLength: Integer;
    EMPTY_ANSI: AnsiString;
    f: FieldsList;
    field_count: Integer;
    field_number: Integer;
    file_name: String;
    full_house: Boolean;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_FF_C: AnsiChar;
    handakuten_byte: Byte;
    joint_byte: Byte;
    k: NonkanaDividingSJIS;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    new_name: String;
    pattern: array[0..3071] of AnsiChar;
    quote_byte: Byte;
    r: NoncontrollerDividingSJIS;
    remove_separators: Boolean;
    separator_byte: Byte;
    SET_BIT_C: AnsiChar;
    substituted: Boolean;
    u: NondakutenDividingSJIS;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    way_of_arranging: Boolean;
    y: AnsiCharArray;
    Y_COUNT: Integer;
    Y_COUNT_1: Integer;
    function ANSI_CHAR_table: AnsiChar;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    function bufferingWriting(s: PAnsiChar; s_length: Integer): Integer;
    function copyingPatternFromStringToArray(s: PChar; s_length: Integer; number_of_fields: Integer): Integer;
    function eight_bit_character(b: Byte): AnsiChar;
    function fieldCount: Integer;
    function leadingCharacters(s: PAnsiChar; s_length: Integer; character: AnsiChar): Integer;
    function newLine: AnsiString;
    function non_controller_selecting(nn: String): Integer;
    function non_dakuten_selecting(nn: String): Integer;
    function non_kana_selecting(nn: String): Integer;
    function produceBinaryFile(name: String; no_removing: Boolean): Integer;
    function reversing(nn: String): Integer;
    function selecting(nn: String): Integer;
    function selecting_extracting_decimal(nn: String): Integer;
    function selecting_extracting_integer(nn: String): Integer;
    procedure setting(f: FinalEffects; fc: Integer); overload;
    procedure setting(fn: String; qb: Byte; sb: Byte; fc: Integer; fu: Integer; ab: Byte; cr: Boolean; rs: Boolean; SUB: Boolean; wo: Boolean); overload;
  end;

const
  BUFFER_ERROR = -1;
  CLEAR_BIT = 48; { 30h, '0' }
  ERROR_CODE = -1;
  FILE_ERROR = -2;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_3E = 62; { '>' }
  H_7E = 126; { '~' }
  H_FF = 255;
  KANA_DAKUTEN = 222; { DEh }
  KANA_HANDAKUTEN = 223; { DFh }
  LISTED = True;
  SET_BIT = 49; { 31h, '1'}
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 2;
  TEMPORARY_BINARY = 'binary.tmp';
  UNDERLINE_BYTE = 95; { 5Fh, '_' }

implementation

constructor DestroyRebuildSJIS.Create;
begin
  inherited Create;

end;

destructor DestroyRebuildSJIS.Destroy;
begin
{ NoncontrollerDividingSJIS }
  r.Free;
  r := nil;

{ DividingSJIS }
  d.Free;
  d := nil;

{ FieldsList }
  f.Free;
  f := nil;

{ NondakutenDividingSJIS }
  u.Free;
  u := nil;

{ array of AnsiChar }
  buffer := nil;

  inherited Destroy;
end;

function DestroyRebuildSJIS.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function DestroyRebuildSJIS.bufferingWriting(s: PAnsiChar; s_length: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;

      for k := 0 to Y_COUNT_1 do
      begin
        y[k] := s[n];
        Inc(n);
        if n >= s_length then
          begin
            Y_COUNT := k + 1;
            Y_COUNT_1 := Y_COUNT - 1;
            break;
          end;

      end;

      BlockWrite(newFile, y, Y_COUNT, block_has_been_written);
    until n >= s_length;

    CloseFile(newFile);
    Y_COUNT := SizeOf(y);
    Y_COUNT_1 := Y_COUNT - 1;
  except
    Halt(1570);
  end;

  Result := exit_code;
end;

function DestroyRebuildSJIS.copyingPatternFromStringToArray(s: PChar; s_length: Integer; number_of_fields: Integer): Integer;
var
  c: Char;
  k: Integer;
  m: Integer;
  s_length_1: Integer;
begin
  CLEAR_BIT_C := eight_bit_character(CLEAR_BIT);
  m := 0;
  s_length_1 := s_length - 1;
  SET_BIT_C := eight_bit_character(SET_BIT);

  for k := 0 to s_length_1 do
  begin
    c := s[k];

    if c = '1' then
      begin
        pattern[m] := SET_BIT_C;
        Inc(m);
      end
    else
      begin
        if c = '0' then
          begin
            pattern[m] := CLEAR_BIT_C;
            Inc(m);
          end
        else
          begin
            continue;
          end;

      end;

  end;

  while m < number_of_fields do
  begin
    pattern[m] := SET_BIT_C;
    Inc(m);
  end;

  Result := m;
end;

function DestroyRebuildSJIS.eight_bit_character(b: Byte): AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
begin
  beginning_of_ASCII := 0;

  if b >= 1 then
    begin
      end_1 := b - 1;
      h := AnsiChar( Chr(beginning_of_ASCII) );

      for k := beginning_of_ASCII to end_1 do
      begin
        Inc(h);
      end;

    end
  else
    begin
      h := AnsiChar(#0);
    end;

  Result := h;
end;

function DestroyRebuildSJIS.fieldCount: Integer;
begin
  Result := field_count;
end;

function DestroyRebuildSJIS.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function DestroyRebuildSJIS.leadingCharacters(s: PAnsiChar; s_length: Integer; character: AnsiChar): Integer;
var
  c: AnsiChar;
  k: Integer;
  s_length_1: Integer;
begin
  s_length_1 := s_length - 1;
  for k := 0 to s_length_1 do
  begin
    c := s[k];
    if c = character then
      begin
        continue;
      end
    else
      begin
        break;
      end;

  end;

  Result := k;
end;

function DestroyRebuildSJIS.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function DestroyRebuildSJIS.non_controller_selecting(nn: String): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  testFile: File;
  z: AnsiString;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    Halt(1571);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsList.Create;
            r := NoncontrollerDividingSJIS.Create;
            r.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength, field_number);
            g := r.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsList.Create;
            r := NoncontrollerDividingSJIS.Create;
            r.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength, field_number);
            g := r.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsList.Create;
            r := NoncontrollerDividingSJIS.Create;
            r.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength, field_number);
            g := r.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsList.Create;
            r := NoncontrollerDividingSJIS.Create;
            r.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength, field_number);
            g := r.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function DestroyRebuildSJIS.non_dakuten_selecting(nn: String): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  testFile: File;
  z: AnsiString;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  dakuten_byte := KANA_DAKUTEN;
  handakuten_byte := KANA_HANDAKUTEN;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  if substituted then
    begin { Please see sortk.txt }
      dakuten_byte := Ord('H');
      handakuten_byte := Ord('I');
    end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    Halt(1572);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsList.Create;
            u := NondakutenDividingSJIS.Create;
            u.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength, field_number);
            g := u.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsList.Create;
            u := NondakutenDividingSJIS.Create;
            u.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength, field_number);
            g := u.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsList.Create;
            u := NondakutenDividingSJIS.Create;
            u.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength, field_number);
            g := u.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsList.Create;
            u := NondakutenDividingSJIS.Create;
            u.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength, field_number);
            g := u.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function DestroyRebuildSJIS.non_kana_selecting(nn: String): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  testFile: File;
  z: AnsiString;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  dakuten_byte := KANA_DAKUTEN;
  handakuten_byte := KANA_HANDAKUTEN;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  if substituted then
    begin { Please see sortk.txt }
      dakuten_byte := Ord('H');
      handakuten_byte := Ord('I');
    end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    Halt(1573);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            k.Free;
            f.Free;
            f := FieldsList.Create;
            k := NonkanaDividingSJIS.Create;
            k.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators, ascii_byte);
            k.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := k.editing_field(dough, doughLength, field_number);
            g := k.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            k.Free;
            f.Free;
            f := FieldsList.Create;
            k := NonkanaDividingSJIS.Create;
            k.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators, ascii_byte);
            k.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := k.editing_field(dough, doughLength, field_number);
            g := k.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            k.Free;
            f.Free;
            f := FieldsList.Create;
            k := NonkanaDividingSJIS.Create;
            k.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators, ascii_byte);
            k.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := k.editing_field(dough, doughLength, field_number);
            g := k.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            k.Free;
            f.Free;
            f := FieldsList.Create;
            k := NonkanaDividingSJIS.Create;
            k.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators, ascii_byte);
            k.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := k.editing_field(dough, doughLength, field_number);
            g := k.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function DestroyRebuildSJIS.produceBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, TEMPORARY_BINARY);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;
      BlockRead(testFile, v, V_SIZE, block_has_been_read);
      BlockWrite(newFile, v, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, name);
        Erase(testFile);
      end;

  except
    Halt(1574);
  end;

  Result := sum;
end;

function DestroyRebuildSJIS.reversing(nn: String): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  testFile: File;
  z: AnsiString;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    Halt(1575);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.reverse_unite(dough, doughLength);
            field_count := d.fieldCount;

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.reverse_unite(dough, doughLength);
            field_count := d.fieldCount;

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.reverse_unite(dough, doughLength);
            field_count := d.fieldCount;

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.reverse_unite(dough, doughLength);
            field_count := d.fieldCount;

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function DestroyRebuildSJIS.selecting(nn: String): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  testFile: File;
  z: AnsiString;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    Halt(1576);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function DestroyRebuildSJIS.selecting_extracting_decimal(nn: String): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  testFile: File;
  z: AnsiString;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    Halt(1577);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting;
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength, field_number);
            g := d.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting;
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength, field_number);
            g := d.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting;
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength, field_number);
            g := d.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting;
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength, field_number);
            g := d.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function DestroyRebuildSJIS.selecting_extracting_integer(nn: String): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  testFile: File;
  z: AnsiString;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    Halt(1578);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting;
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength, field_number);
            g := d.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting;
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength, field_number);
            g := d.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        beginning := n + 1;
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting;
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength, field_number);
            g := d.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;

            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsList.Create;
            d := DividingSJIS.Create;
            d.presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting;
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength, field_number);
            g := d.unite(dough, doughLength, LISTED);

            z := z + g;
            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function DestroyRebuildSJIS.setBufferSize: Integer;
var
  r: Integer;
begin
  r := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    Halt(1579);
  end;

  Result := r;
end;

procedure DestroyRebuildSJIS.setting(f: FinalEffects; fc: Integer);
var
  m: Integer;
  n: Longint;
  pascal_string: AnsiString;
  testFile: File;
begin
{ String }
  file_name := f.fileName;

{ Integer }
  field_count := fc;
  field_number := f.fieldNumber;

{ Byte }
  ascii_byte := f.labelByte;
  joint_byte := f.jointByte;
  quote_byte := f.quoteByte;
  separator_byte := f.separatorByte;

{ Boolean }
  carriage_return := f.carriageReturn;
  remove_separators := f.removeSeparators;
  substituted := f.substituteForDakuten;
  way_of_arranging := f.reverseOrder;

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y);
  Y_COUNT_1 := Y_COUNT - 1;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);
    beginning := 0;

    CloseFile(testFile);

    buffer_size := file_size(file_name);
  except
    Halt(1580);
  end;

  full_house := True;
  for m := 1 to field_count do
  begin
    n := Ord( pattern[ m - 1 ] );

    if n = CLEAR_BIT then
      full_house := False;

  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_ARRAY[m] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  if carriage_return then
    begin
      NEW_LINE_SIZE := 2;
      NEW_LINE := EMPTY_ANSI + H_0D_C;
      NEW_LINE := NEW_LINE + H_0A_C;
    end
  else
    begin
      NEW_LINE_SIZE := 1;
      NEW_LINE := EMPTY_ANSI + H_0A_C;
    end;

  setBufferSize;
end;

procedure DestroyRebuildSJIS.setting(fn: String; qb: Byte; sb: Byte; fc: Integer; fu: Integer; ab: Byte; cr: Boolean; rs: Boolean; SUB: Boolean; wo: Boolean);
var
  m: Integer;
  n: Longint;
  pascal_string: AnsiString;
  testFile: File;
begin
{ String }
  file_name := fn;

{ Integer }
  field_count := fc;
  field_number := fu;

{ Byte }
  ascii_byte := ab;
  joint_byte := UNDERLINE_BYTE;
  quote_byte := qb;
  separator_byte := sb;

{ Boolean }
  carriage_return := cr;
  remove_separators := rs;
  substituted := SUB;
  way_of_arranging := wo;

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y);
  Y_COUNT_1 := Y_COUNT - 1;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);
    beginning := 0;

    CloseFile(testFile);

    buffer_size := file_size(file_name);
  except
    Halt(1581);
  end;

  full_house := True;
  for m := 1 to field_count do
  begin
    n := Ord( pattern[ m - 1 ] );

    if n = CLEAR_BIT then
      full_house := False;

  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_ARRAY[m] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  if carriage_return then
    begin
      NEW_LINE_SIZE := 2;
      NEW_LINE := EMPTY_ANSI + H_0D_C;
      NEW_LINE := NEW_LINE + H_0A_C;
    end
  else
    begin
      NEW_LINE_SIZE := 1;
      NEW_LINE := EMPTY_ANSI + H_0A_C;
    end;

  setBufferSize;
end;

end.
