unit RingQueue;

interface

uses
  SysUtils;

type
  RingBuffer = class
  protected
    NEW_LINE: String;
    NEW_LINE_SIZE: Integer;
    queue: array[0..47] of String; { queue_size <= 48 - 1 }
    queue_begin: Integer;
    queue_end: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    function display: Integer; overload;
    function display(standard_out: Boolean): String; overload;
    function emptyRing: Boolean;
    function following(index: Integer): Integer;
    function pop_front: Integer;
    function produce_ring_buffer(size: Integer): Integer;
    function push_back(p: String): Integer;
    function queue_size: Integer;
    procedure clearRing;
    procedure setting(sequence: String; size: Integer);
  end;

const
  BLANK = ' ';
  EMPTY = '';
  EMPTY_ERROR = 2;
  FULL_HOUSE_ERROR = 3;
  H_0D = 13;
  H_0A = 10;
  QUEUE_ARRAY_SIZE = 47;
  QUEUE_ERROR_COUNT = 5;
  QUEUE_GROWN = 0;
  QUEUE_SHRUNK = 1;
  SET_LENGTH_ERROR = 4;

implementation

constructor RingBuffer.Create;
begin
  inherited Create;

end;

destructor RingBuffer.Destroy;
begin

  inherited Destroy;
end;

function RingBuffer.display: Integer;
var
  index: Integer;
  k: Integer;
  size: Integer;
begin
  size := queue_size;
  index := queue_begin;

  for k := 1 to size do
  begin
    WriteLn(queue[index]);
    index := following(index);
  end;

  Result := size;
end;

function RingBuffer.display(standard_out: Boolean): String;
var
  combo: String;
  index: Integer;
  k: Integer;
  size: Integer;
begin
  combo := EMPTY;
  size := queue_size;
  index := queue_begin;

  for k := 1 to size do
  begin
    combo := combo + queue[index];
    combo := combo + NEW_LINE;
    index := following(index);
  end;

  if standard_out then
    WriteLn(combo);

  Result := combo;
end;

function RingBuffer.emptyRing: Boolean;
var
  b: Boolean;
begin
  b := False;
  if queue_begin = queue_end then
    b := True;

  Result := b;
end;

function RingBuffer.following(index: Integer): Integer;
var
  k: Integer;
begin
  k := (index + 1) mod QUEUE_ARRAY_SIZE; 
  Result := k; 
end;

function RingBuffer.pop_front: Integer;
var
  r: Integer;
begin
  if emptyRing then
    begin
      r := EMPTY_ERROR;
    end
  else
    begin
      queue_begin := following(queue_begin);
      r := QUEUE_SHRUNK;
    end;

  Result := r;
end;

function RingBuffer.produce_ring_buffer(size: Integer): Integer;
var
  k: Integer;
  r: Integer;
begin
  clearRing;

  for k := 1 to size do
  begin
    push_back(BLANK);
  end;

  r := queue_size;
  Result := r;
end;

function RingBuffer.push_back(p: String): Integer;
var
  index: Integer;
  r: Integer;
begin
  index := following(queue_end);
  if index = queue_begin then
    begin
      r := FULL_HOUSE_ERROR;
    end
  else
    begin
      queue[queue_end] := p;
      queue_end := index;
      r := QUEUE_GROWN;
    end;

  Result := r;
end;

function RingBuffer.queue_size: Integer;
var
  size: Integer;
begin
  if queue_begin < queue_end then
    begin
      size := queue_end - queue_begin;
    end
  else
    begin
      if queue_begin = queue_end then
        begin
          size := 0;
        end
      else
        begin
          size := queue_end + QUEUE_ARRAY_SIZE - queue_begin;
        end;

    end;

  Result := size;
end;

procedure RingBuffer.clearRing;
var
  k: Integer;
  repetition: Integer;
begin
  repetition := queue_size;

  for k := 1 to repetition do
  begin
    pop_front;
  end;

end;

procedure RingBuffer.setting(sequence: String; size: Integer);
var
  k: Integer;
begin
  NEW_LINE := EMPTY + Chr(H_0D);
  NEW_LINE := NEW_LINE + Chr(H_0A);
  NEW_LINE_SIZE := 2;

  for k := 1 to size do
  begin
    push_back(sequence);
  end;

end;

end.
