unit RingQueueCSV;

interface

uses
  SysUtils, RingQueue, DivideSJIS, FieldsArray, FinalEffect, ConvertUTF16;

type AnsiCharArray = array[0..4095] of AnsiChar;

type
  RingCSV = class(RingBuffer)
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    ANSI_ASCII: AnsiString;
    ANSI_CHAR: array[0..255] of AnsiChar;
    ANSI_WIDE_SPACE: AnsiString;
    d: array[0..47] of DividingSJIS;
    EMPTY_ANSI: AnsiString;
    f: array[0..47] of FieldsList;
    GAP_C: AnsiChar;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_FF_C: AnsiChar;
    mesh: array[0..15, 0..15] of AnsiString;
    quote_byte: Byte;
    remove_separators: Boolean;
    separator_byte: Byte;
    WIDE_SPACE: String;
    function ANSI_CHAR_table: AnsiChar;
  public
    destructor Destroy; override;
    function columnRow(i: Integer; j: Integer): AnsiString;
    procedure presetting(q: Byte; s: Byte; e: Boolean);
    procedure tabulating;
  end;

const
  EXPANSION_FACTOR = 2;
  GAP_BYTE = 32; { SP }
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_3E = 62; { '>' }
  H_7E = 126; { '~' }
  H_FF = 255;
  MAXIMUM_COLUMN_COUNT = 16;
  MAXIMUM_COLUMN_COUNT_1 = 15;
  MAXIMUM_ROW_COUNT = 16;
  MAXIMUM_ROW_COUNT_1 = 15;
  UPPER_LIMIT_TO_ITEM_SIZE = 6;

implementation

destructor RingCSV.Destroy;
var
  k: Integer;
  size: Integer;
  size_1: Integer;
begin
  size := queue_size;
  size_1 := size - 1;

  for k := 0 to size_1 do
  begin
    d[k].Free;
    f[k].Free;
  end;

  inherited Destroy;
end;

function RingCSV.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function RingCSV.columnRow(i: Integer; j: Integer): AnsiString;
var
  r: AnsiString;
begin
  if i < 0 then
    begin
      r := ANSI_WIDE_SPACE;
    end
  else
    begin
      if i > MAXIMUM_COLUMN_COUNT_1 then
        begin
          r := ANSI_WIDE_SPACE;
        end
      else
        begin
          if j < 0 then
            begin
              r := ANSI_WIDE_SPACE;
            end
          else
            begin
              if j > MAXIMUM_ROW_COUNT_1 then
                begin
                  r := ANSI_WIDE_SPACE;
                end
              else
                begin
                  r := mesh[i, j];
                end;

            end;

        end;

    end;

  Result := r;
end;

procedure RingCSV.presetting(q: Byte; s: Byte; e: Boolean);
var
  k: Integer;
  n: Integer;
  pascal_string: AnsiString;
begin
  quote_byte := q;
  remove_separators := e;
  separator_byte := s;

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  WIDE_SPACE := StringOfChar(' ', UPPER_LIMIT_TO_ITEM_SIZE);

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  GAP_C := ANSI_CHAR[GAP_BYTE];
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

{ wide space of AnsiString type }
  ANSI_WIDE_SPACE := EMPTY_ANSI;
  for k := 1 to UPPER_LIMIT_TO_ITEM_SIZE do
  begin
    ANSI_WIDE_SPACE := ANSI_WIDE_SPACE + GAP_C;
  end;

  for k := 0 to MAXIMUM_ROW_COUNT_1 do
  begin
    for n := 0 to MAXIMUM_COLUMN_COUNT_1 do
    begin
      mesh[n, k] := ANSI_WIDE_SPACE;
    end;

  end;

end;

procedure RingCSV.tabulating;
var
  c: Converting;
  field_count: Integer;
  field_count_1: Integer;
  index: Integer;
  k: Integer;
  many: Byte;
  n: Integer;
  s: String;
  size: Integer;
  size_1: Integer;
  t: FinalEffects;
begin
  index := queue_begin;
  many := MAXIMUM_COLUMN_COUNT;
  size := queue_size;
  size_1 := size - 1;

  t := FinalEffects.Create;
  t.setting;
  c := Converting.Create;
  c.setting(t);

  for k := 0 to size_1 do
  begin
    s := queue[index] + WIDE_SPACE;

    c.produce_MS932_phrase( PChar(s), Length(s) );
    A_SIZE := c.MS932_phrase_size;
    A_SIZE_1 := A_SIZE - 1;
    for n := 0 to A_SIZE_1 do
    begin
      a[n] := c.copyingArrayItem(n);
    end;

    f[k] := FieldsList.Create;
    d[k] := DividingSJIS.Create;
    d[k].presetting(ANSI_ASCII, quote_byte, separator_byte, remove_separators);
    d[k].setting(f[k], many);

    field_count := d[k].mapping_fields_list(a, A_SIZE);
    field_count_1 := field_count - 1;
    for n := 0 to field_count_1 do
    begin
      mesh[n, k] := f[k].item(n);
    end;

    index := following(index);
  end;

  c.Free; t.Free;
end;

end.
