unit RuralUTF16;

interface

uses
  Windows, SysUtils, PathEXE, FinalEffect, SearchBinary;

type OpenCloseFunction = function: String;

type AnsiCharArray = array[0..127] of AnsiChar;

type
  RuralAreaUTF16 = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    ASCII_C: AnsiChar;
    CLEAR_ARRAY: AnsiCharArray;
    KANJI_C: AnsiChar;
    NON_KANJI_C: AnsiChar;
    open_close: array[1..4] of OpenCloseFunction;
    z: array[1..261072] of AnsiChar;
{*

EUC-JP encoding: 65267 (FEF3h)
( 65267 + 1 ) * ( FLAG_SIZE + UTF16_KANJI_SIZE + 1 ) = 261072
*}
  public
    function working(index: Integer; place: Integer): AnsiChar;
    procedure clear;
    procedure populated(index: Integer; place: Integer; c: AnsiChar);
    procedure setting(BINARY_TABLE: String); overload;
    procedure setting(e: SearchTable); overload;
    procedure setting(f: FinalEffects); overload;
  end;

const
  C_EUC_UTF16_BIN = 'C:\Narfi\EUC-JP~UTF16.bin';
  DATA_TEXT = 'data.txt';
  EMPTY = '';
  EUC_UTF16_BIN = 'EUC-JP~UTF16.bin';
  FLAG_SIZE = 1;
  RECORD_SIZE = 5;
  SEARCH_COUNT = 4;
  SETTING_TEXT = 'sets.txt';
  UTF16_KANJI_SIZE = 2;
  UTF16_KANJI_SIZE_2 = 4; { FLAG_SIZE + UTF16_KANJI_SIZE + 1 }

implementation

function binary_file_in_current_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, EUC_UTF16_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := EUC_UTF16_BIN;
  except
    ;
  end;

  Result := s;
end;

function binary_file_in_installation_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, C_EUC_UTF16_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := C_EUC_UTF16_BIN;
  except
    ;
  end;

  Result := s;
end;

function binary_file_in_path_directory: String;
{ directory included in Path }
var
  p: CommandPath;
  s: String;
begin
  p := CommandPath.Create;
  p.setting;
  s := p.pathName(EUC_UTF16_BIN);
  p.Free;

  Result := s;
end;

function binary_file_in_selected_directory: String;
{ directory that may have priority }
var
  s: String;
  settingFile: TextFile;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(settingFile, SETTING_TEXT);
    Reset(settingFile);
    ReadLn(settingFile, s); { Line-1 }
    CloseFile(settingFile);
  except
    WriteLn('File error: ' + SETTING_TEXT);
    Halt(1436);
  end;

  if Length(s) > 0 then
    begin
      try
        Assign(testFile, s);
        Reset(testFile);
        CloseFile(testFile);
      except
        s := EMPTY;
      end;

    end;

  Result := s;
end;

function RuralAreaUTF16.working(index: Integer; place: Integer): AnsiChar;
var
  k: Integer;
begin
{*
place = 0: KANJI_C or ASCII_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
  k := UTF16_KANJI_SIZE_2 * index + place;
  Result := z[k];
end;

procedure RuralAreaUTF16.populated(index: Integer; place: Integer; c: AnsiChar);
var
  k: Integer;
begin
{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
  k := UTF16_KANJI_SIZE_2 * index + place;
  z[k] := c;
end;

procedure RuralAreaUTF16.clear;
var
  k: Integer;
begin
  for k := 1 to 261072 do
  begin
    z[k] := #0;
  end;

end;

procedure RuralAreaUTF16.setting(BINARY_TABLE: String);
var
  b: Longint;
  block_has_been_read: Integer;
  i: Integer;
  k: Integer;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  ASCII_C := AnsiChar(#0);
  KANJI_C := AnsiChar(#1);
  NON_KANJI_C := AnsiChar(#0);

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Reaeding BINARY_TABLE to produce a table in the main memory }
  try
    AssignFile(tableFile, BINARY_TABLE);
    Reset(tableFile, RECORD_SIZE);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(tableFile, a, 1, block_has_been_read);

      if a[2] = ASCII_C then
        begin
          k := Ord(a[0]);
          populated(k, 0, ASCII_C);
          populated(k, 1, a[3]);
          populated(k, 2, NON_KANJI_C);
        end
      else
        begin
{ kanji }
          b := Ord(a[1]);
          k := b shl 8;

          b := Ord(a[0]); { little-endian architecture }
          k := k + b;

          populated(k, 0, KANJI_C);
          populated(k, 1, a[3]);
          populated(k, 2, a[4]);
        end;

    until block_has_been_read = 0;

    CloseFile(tableFile);
  except
    Halt(1437);
  end;

end;

procedure RuralAreaUTF16.setting(e: SearchTable);
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  dataFile: TextFile;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  ASCII_C := AnsiChar(#0);
  KANJI_C := AnsiChar(#1);
  NON_KANJI_C := AnsiChar(#0);

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Finding EUC_UTF16_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if e.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if e.whichTable then
          begin
            AssignFile(dataFile, DATA_TEXT);
            Rewrite(dataFile);
            messages := binaryTable + ' is primarily accessible to Narfi.';
            WriteLn(dataFile, messages);
            CloseFile(dataFile);
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if e.noSearching then
                begin
                  Halt(1438);
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              Halt(1439);
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  if Length(binaryTable) > 0 then
    begin
{ Reaeding EUC_UTF16_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        if a[2] = ASCII_C then
          begin
            k := Ord(a[0]);
            populated(k, 0, ASCII_C);
            populated(k, 1, a[3]);
            populated(k, 2, NON_KANJI_C);
          end
        else
          begin
{ kanji }
            b := Ord(a[1]);
            k := b shl 8;

            b := Ord(a[0]); { little-endian architecture }
            k := k + b;

            populated(k, 0, KANJI_C);
            populated(k, 1, a[3]);
            populated(k, 2, a[4]);
          end;

      until block_has_been_read = 0;

      CloseFile(tableFile);
    end
  else
    begin
      Halt(1440);
    end;

end;

procedure RuralAreaUTF16.setting(f: FinalEffects);
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  dataFile: TextFile;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  ASCII_C := AnsiChar(#0);
  KANJI_C := AnsiChar(#1);
  NON_KANJI_C := AnsiChar(#0);

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Finding EUC_UTF16_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if f.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if f.whichTable then
          begin
            AssignFile(dataFile, DATA_TEXT);
            Rewrite(dataFile);
            messages := binaryTable + ' is primarily accessible to Narfi.';
            WriteLn(dataFile, messages);
            CloseFile(dataFile);
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if f.noSearching then
                begin
                  Halt(1441);
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              Halt(1442);
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  if Length(binaryTable) > 0 then
    begin
{ Reaeding EUC_UTF16_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        if a[2] = ASCII_C then
          begin
            k := Ord(a[0]);
            populated(k, 0, ASCII_C);
            populated(k, 1, a[3]);
            populated(k, 2, NON_KANJI_C);
          end
        else
          begin
{ kanji }
            b := Ord(a[1]);
            k := b shl 8;

            b := Ord(a[0]); { little-endian architecture }
            k := k + b;

            populated(k, 0, KANJI_C);
            populated(k, 1, a[3]);
            populated(k, 2, a[4]);
          end;

      until block_has_been_read = 0;

      CloseFile(tableFile);
    end
  else
    begin
      Halt(1443);
    end;

end;

end.
