unit Hexadecimal;

{$I+}

interface

uses
  Windows, SysUtils, ErrorText, FinalEffect;

type AnsiCharArray = array[0..4095] of AnsiChar;

type
  Hexadecimals = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    ANSI_ASCII: AnsiString;
    ANSI_CHAR: array[0..255] of AnsiChar;
    BYTE_GAP: array[0..256] of String;
    BYTE_SEPARATOR: array[0..256] of String;
    buffer: array of AnsiChar;
    buffer_size: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    count: Integer;
    EMPTY_ANSI: AnsiString;
    file_name: String;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_0A_SEPARATOR: String;
    H_0D_SEPARATOR: String;
    H_FF_C: AnsiChar;
    joint_byte: Byte;
    limited_to_value_bytes: Integer;
    limited_to_value_fields: Integer;
    limited_to_value_lines: Integer;
    limited_to_value_rows: Integer;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    row: Integer;
    separator_byte: Byte;
    sum: Integer;
    x: ErrorTexts;
    wall_value: Integer;
    function ANSI_CHAR_table: AnsiChar;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function displayDataText: Boolean;
    function lineCount: Integer;
    function produceTemporaryTextFile: Integer;
    function limiting: Integer;
    function limiting_to_produce_data_file: Integer;
    function newLine: AnsiString;
    procedure presetting(jb: Byte); overload;
    procedure presetting(jb: Byte; sb: Byte); overload;
    procedure setting(fn: String; vb: Integer; vf: Integer; VL: Integer; vr: Integer; wv: Integer); overload;
    procedure setting(f: FinalEffects); overload;
  end;

const
  A_BYTE = 65; { 41h, 'A' }
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  D_BYTE = 68; { 44h, 'D' }
  DATA_TEXT = 'data.txt';
  EMPTY = '';
  GAP_BYTE = 32; { 20h, SP, 'D' }
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_3E = 62; { '>' }
  H_5C = 92; { '\' }
  H_7E = 126; { '~' }
  H_FF = 255;
  SIZE_OF_ANSICHAR = 1;
  TEMPORARY_TEXT = 'text.tmp';
  TEXT_FILE = '.txt';
  WALL = 256;

implementation

destructor Hexadecimals.Destroy;
begin
  buffer := nil;

  inherited Destroy;
end;

function Hexadecimals.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function Hexadecimals.displayDataText: Boolean;
var
  b: Boolean;
  s: String;
  testFile: TextFile;
begin
  b := True;

  try
    AssignFile(testFile, DATA_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      WriteLn(s);
    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-392: ' + DATA_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    b := False;
  end;

  Result := b;
end;

function Hexadecimals.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function Hexadecimals.lineCount: Integer;
begin
  Result := count;
end;

function Hexadecimals.produceTemporaryTextFile: Integer;
const
  NEW_LINE_SIZE = 2; { CR, LF }
var
  newFile: TextFile;
  s: String;
  sum: Integer;
begin
  sum := 0;

  try
    AssignFile(newFile, TEMPORARY_TEXT);
    Rewrite(newFile);

    while not Eof do
    begin
      ReadLn(s);
      WriteLn(newFile, s);
      sum := sum + Length(s);
      sum := sum + NEW_LINE_SIZE;
    end;

    CloseFile(newFile);
  except
    s := 'Reset() error-393: ' + TEMPORARY_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function Hexadecimals.limiting: Integer;
var
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  c: AnsiChar;
  k: Integer;
  n: Integer;
  s: String;
  testFile: File;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  k := 0;
  NEW_LINE_SIZE := 1; { 0Ah }
  NEW_LINE := EMPTY_ANSI + H_0A_C;
  row := 0;
  sum := 0;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n:= 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-394: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  n := 0;
  while n < limited_to_value_bytes do
  begin
    c := buffer[n];
    k := Ord(c);

    if k = H_0A then
      begin
        Write('0A ');
        Inc(sum);
        if sum >= limited_to_value_fields then
          begin
            WriteLn('');
            sum := 0;
            Inc(row);
            if row >= limited_to_value_rows then
              begin
                break;
              end;

          end;

        count := 1;
        if count >= limited_to_value_lines then
          begin
            break;
          end;

        break;
      end
    else
      begin
        if k = H_0D then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;

            Write('0D ');
          end
        else
          begin
            if k < wall_value then
              begin
                Write(BYTE_GAP[k]);
              end
            else
              begin
                Write(BYTE_GAP[WALL]);
              end;

          end;

        Inc(sum);
        if sum >= limited_to_value_fields then
          begin
            WriteLn('');
            sum := 0;
            Inc(row);
            if row >= limited_to_value_rows then
              begin
                break;
              end;

          end;

      end;

    Inc(n);
  end;

{ Counting lines }
  Inc(n);
  while n < limited_to_value_bytes do
  begin
    c := buffer[n];
    k := Ord(c);

    if k = H_0A then
      begin
        Write('0A ');
        Inc(sum);
        if sum >= limited_to_value_fields then
          begin
            WriteLn('');
            sum := 0;
            Inc(row);
            if row >= limited_to_value_rows then
              begin
                break;
              end;

          end;

        Inc(count);
        if count >= limited_to_value_lines then
          begin
            break;
          end;

      end
    else
      begin
        if k = H_0D then
          begin
            Write('0D ');
          end
        else
          begin
            if k < wall_value then
              begin
                Write(BYTE_GAP[k]);
              end
            else
              begin
                Write(BYTE_GAP[WALL]);
              end;

          end;

        Inc(sum);
        if sum >= limited_to_value_fields then
          begin
            WriteLn('');
            sum := 0;
            Inc(row);
            if row >= limited_to_value_rows then
              begin
                break;
              end;

          end;

      end;

    Inc(n);
  end;

  Result := count;
end;

function Hexadecimals.limiting_to_produce_data_file: Integer;
var
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  c: AnsiChar;
  dataFile: TextFile;
  k: Integer;
  n: Integer;
  s: String;
  testFile: File;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  k := 0;
  NEW_LINE_SIZE := 1; { 0Ah }
  NEW_LINE := EMPTY_ANSI + H_0A_C;
  row := 0;
  sum := 0;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n:= 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-395: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  AssignFile(dataFile, DATA_TEXT);
  Rewrite(dataFile);

{ Finding the first NEW_LINE }
  n := 0;
  while n < limited_to_value_bytes do
  begin
    c := buffer[n];
    k := Ord(c);

    if k = H_0A then
      begin
        Write(dataFile, H_0A_C); { Copying }
        Write(H_0A_SEPARATOR);
        Inc(sum);
        if sum >= limited_to_value_fields then
          begin
            WriteLn('');
            sum := 0;
            Inc(row);
            if row >= limited_to_value_rows then
              begin
                break;
              end;

          end;

        count := 1;
        if count >= limited_to_value_lines then
          begin
            break;
          end;

        break;
      end
    else
      begin
        if k = H_0D then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;

            Write(dataFile, H_0D_C); { Copying }
            Write(H_0D_SEPARATOR);
          end
        else
          begin
            Write(dataFile, c); { Copying }

            if k < wall_value then
              begin
                Write(BYTE_SEPARATOR[k]);
              end
            else
              begin
                Write(BYTE_SEPARATOR[WALL]);
              end;

          end;

        Inc(sum);
        if sum >= limited_to_value_fields then
          begin
            WriteLn('');
            sum := 0;
            Inc(row);
            if row >= limited_to_value_rows then
              begin
                break;
              end;

          end;

      end;

    Inc(n);
  end;

{ Counting lines }
  Inc(n);
  while n < limited_to_value_bytes do
  begin
    c := buffer[n];
    k := Ord(c);

    if k = H_0A then
      begin
        Write(dataFile, H_0A_C); { Copying }
        Write(H_0A_SEPARATOR);
        Inc(sum);
        if sum >= limited_to_value_fields then
          begin
            WriteLn('');
            sum := 0;
            Inc(row);
            if row >= limited_to_value_rows then
              begin
                break;
              end;

          end;

        Inc(count);
        if count >= limited_to_value_lines then
          begin
            break;
          end;

      end
    else
      begin
        if k = H_0D then
          begin
            Write(dataFile, H_0D_C); { Copying }
            Write(H_0D_SEPARATOR);
          end
        else
          begin
            Write(dataFile, c); { Copying }

            if k < wall_value then
              begin
                Write(BYTE_SEPARATOR[k]);
              end
            else
              begin
                Write(BYTE_SEPARATOR[WALL]);
              end;

          end;

        Inc(sum);
        if sum >= limited_to_value_fields then
          begin
            WriteLn('');
            sum := 0;
            Inc(row);
            if row >= limited_to_value_rows then
              begin
                break;
              end;

          end;

      end;

    Inc(n);
  end;

  CloseFile(dataFile);
  Result := count;
end;

function Hexadecimals.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function Hexadecimals.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    s := 'SetLength() error-396: ' + IntToStr(buffer_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  Result := r;
end;

procedure Hexadecimals.presetting(jb: Byte);
begin
  joint_byte := jb;

  BYTE_GAP[0] := '00 '; BYTE_GAP[1] := '01 '; BYTE_GAP[2] := '02 '; BYTE_GAP[3] := '03 '; BYTE_GAP[4] := '04 '; BYTE_GAP[5] := '05 '; BYTE_GAP[6] := '06 '; BYTE_GAP[7] := '07 '; BYTE_GAP[8] := '08 '; BYTE_GAP[9] := '09 '; BYTE_GAP[10] := '0A '; BYTE_GAP[11] := '0B '; BYTE_GAP[12] := '0C '; BYTE_GAP[13] := '0D '; BYTE_GAP[14] := '0E '; BYTE_GAP[15] := '0F '; BYTE_GAP[16] := '10 '; BYTE_GAP[17] := '11 '; BYTE_GAP[18] := '12 '; BYTE_GAP[19] := '13 '; BYTE_GAP[20] := '14 '; BYTE_GAP[21] := '15 '; BYTE_GAP[22] := '16 '; BYTE_GAP[23] := '17 '; BYTE_GAP[24] := '18 '; BYTE_GAP[25] := '19 '; BYTE_GAP[26] := '1A '; BYTE_GAP[27] := '1B '; BYTE_GAP[28] := '1C '; BYTE_GAP[29] := '1D ';
  BYTE_GAP[30] := '1E '; BYTE_GAP[31] := '1F '; BYTE_GAP[32] := '20 '; BYTE_GAP[33] := '21 '; BYTE_GAP[34] := '22 '; BYTE_GAP[35] := '23 '; BYTE_GAP[36] := '24 '; BYTE_GAP[37] := '25 '; BYTE_GAP[38] := '26 '; BYTE_GAP[39] := '27 '; BYTE_GAP[40] := '28 '; BYTE_GAP[41] := '29 '; BYTE_GAP[42] := '2A '; BYTE_GAP[43] := '2B '; BYTE_GAP[44] := '2C '; BYTE_GAP[45] := '2D '; BYTE_GAP[46] := '2E '; BYTE_GAP[47] := '2F '; BYTE_GAP[48] := '30 '; BYTE_GAP[49] := '31 '; BYTE_GAP[50] := '32 '; BYTE_GAP[51] := '33 '; BYTE_GAP[52] := '34 '; BYTE_GAP[53] := '35 '; BYTE_GAP[54] := '36 '; BYTE_GAP[55] := '37 '; BYTE_GAP[56] := '38 '; BYTE_GAP[57] := '39 '; BYTE_GAP[58] := '3A '; BYTE_GAP[59] := '3B ';
  BYTE_GAP[60] := '3C '; BYTE_GAP[61] := '3D '; BYTE_GAP[62] := '3E '; BYTE_GAP[63] := '3F '; BYTE_GAP[64] := '40 '; BYTE_GAP[65] := '41 '; BYTE_GAP[66] := '42 '; BYTE_GAP[67] := '43 '; BYTE_GAP[68] := '44 '; BYTE_GAP[69] := '45 '; BYTE_GAP[70] := '46 '; BYTE_GAP[71] := '47 '; BYTE_GAP[72] := '48 '; BYTE_GAP[73] := '49 '; BYTE_GAP[74] := '4A '; BYTE_GAP[75] := '4B '; BYTE_GAP[76] := '4C '; BYTE_GAP[77] := '4D '; BYTE_GAP[78] := '4E '; BYTE_GAP[79] := '4F '; BYTE_GAP[80] := '50 '; BYTE_GAP[81] := '51 '; BYTE_GAP[82] := '52 '; BYTE_GAP[83] := '53 '; BYTE_GAP[84] := '54 '; BYTE_GAP[85] := '55 '; BYTE_GAP[86] := '56 '; BYTE_GAP[87] := '57 '; BYTE_GAP[88] := '58 '; BYTE_GAP[89] := '59 ';
  BYTE_GAP[90] := '5A '; BYTE_GAP[91] := '5B '; BYTE_GAP[92] := '5C '; BYTE_GAP[93] := '5D '; BYTE_GAP[94] := '5E '; BYTE_GAP[95] := '5F '; BYTE_GAP[96] := '60 '; BYTE_GAP[97] := '61 '; BYTE_GAP[98] := '62 '; BYTE_GAP[99] := '63 '; BYTE_GAP[100] := '64 '; BYTE_GAP[101] := '65 '; BYTE_GAP[102] := '66 '; BYTE_GAP[103] := '67 '; BYTE_GAP[104] := '68 '; BYTE_GAP[105] := '69 '; BYTE_GAP[106] := '6A '; BYTE_GAP[107] := '6B '; BYTE_GAP[108] := '6C '; BYTE_GAP[109] := '6D '; BYTE_GAP[110] := '6E '; BYTE_GAP[111] := '6F '; BYTE_GAP[112] := '70 '; BYTE_GAP[113] := '71 '; BYTE_GAP[114] := '72 '; BYTE_GAP[115] := '73 '; BYTE_GAP[116] := '74 '; BYTE_GAP[117] := '75 '; BYTE_GAP[118] := '76 '; BYTE_GAP[119] := '77 ';
  BYTE_GAP[120] := '78 '; BYTE_GAP[121] := '79 '; BYTE_GAP[122] := '7A '; BYTE_GAP[123] := '7B '; BYTE_GAP[124] := '7C '; BYTE_GAP[125] := '7D '; BYTE_GAP[126] := '7E '; BYTE_GAP[127] := '7F '; BYTE_GAP[128] := '80 '; BYTE_GAP[129] := '81 '; BYTE_GAP[130] := '82 '; BYTE_GAP[131] := '83 '; BYTE_GAP[132] := '84 '; BYTE_GAP[133] := '85 '; BYTE_GAP[134] := '86 '; BYTE_GAP[135] := '87 '; BYTE_GAP[136] := '88 '; BYTE_GAP[137] := '89 '; BYTE_GAP[138] := '8A '; BYTE_GAP[139] := '8B '; BYTE_GAP[140] := '8C '; BYTE_GAP[141] := '8D '; BYTE_GAP[142] := '8E '; BYTE_GAP[143] := '8F '; BYTE_GAP[144] := '90 '; BYTE_GAP[145] := '91 '; BYTE_GAP[146] := '92 '; BYTE_GAP[147] := '93 '; BYTE_GAP[148] := '94 '; BYTE_GAP[149] := '95 ';
  BYTE_GAP[150] := '96 '; BYTE_GAP[151] := '97 '; BYTE_GAP[152] := '98 '; BYTE_GAP[153] := '99 '; BYTE_GAP[154] := '9A '; BYTE_GAP[155] := '9B '; BYTE_GAP[156] := '9C '; BYTE_GAP[157] := '9D '; BYTE_GAP[158] := '9E '; BYTE_GAP[159] := '9F '; BYTE_GAP[160] := 'A0 '; BYTE_GAP[161] := 'A1 '; BYTE_GAP[162] := 'A2 '; BYTE_GAP[163] := 'A3 '; BYTE_GAP[164] := 'A4 '; BYTE_GAP[165] := 'A5 '; BYTE_GAP[166] := 'A6 '; BYTE_GAP[167] := 'A7 '; BYTE_GAP[168] := 'A8 '; BYTE_GAP[169] := 'A9 '; BYTE_GAP[170] := 'AA '; BYTE_GAP[171] := 'AB '; BYTE_GAP[172] := 'AC '; BYTE_GAP[173] := 'AD '; BYTE_GAP[174] := 'AE '; BYTE_GAP[175] := 'AF '; BYTE_GAP[176] := 'B0 '; BYTE_GAP[177] := 'B1 '; BYTE_GAP[178] := 'B2 '; BYTE_GAP[179] := 'B3 ';
  BYTE_GAP[180] := 'B4 '; BYTE_GAP[181] := 'B5 '; BYTE_GAP[182] := 'B6 '; BYTE_GAP[183] := 'B7 '; BYTE_GAP[184] := 'B8 '; BYTE_GAP[185] := 'B9 '; BYTE_GAP[186] := 'BA '; BYTE_GAP[187] := 'BB '; BYTE_GAP[188] := 'BC '; BYTE_GAP[189] := 'BD '; BYTE_GAP[190] := 'BE '; BYTE_GAP[191] := 'BF '; BYTE_GAP[192] := 'C0 '; BYTE_GAP[193] := 'C1 '; BYTE_GAP[194] := 'C2 '; BYTE_GAP[195] := 'C3 '; BYTE_GAP[196] := 'C4 '; BYTE_GAP[197] := 'C5 '; BYTE_GAP[198] := 'C6 '; BYTE_GAP[199] := 'C7 '; BYTE_GAP[200] := 'C8 '; BYTE_GAP[201] := 'C9 '; BYTE_GAP[202] := 'CA '; BYTE_GAP[203] := 'CB '; BYTE_GAP[204] := 'CC '; BYTE_GAP[205] := 'CD '; BYTE_GAP[206] := 'CE '; BYTE_GAP[207] := 'CF '; BYTE_GAP[208] := 'D0 '; BYTE_GAP[209] := 'D1 ';
  BYTE_GAP[210] := 'D2 '; BYTE_GAP[211] := 'D3 '; BYTE_GAP[212] := 'D4 '; BYTE_GAP[213] := 'D5 '; BYTE_GAP[214] := 'D6 '; BYTE_GAP[215] := 'D7 '; BYTE_GAP[216] := 'D8 '; BYTE_GAP[217] := 'D9 '; BYTE_GAP[218] := 'DA '; BYTE_GAP[219] := 'DB '; BYTE_GAP[220] := 'DC '; BYTE_GAP[221] := 'DD '; BYTE_GAP[222] := 'DE '; BYTE_GAP[223] := 'DF '; BYTE_GAP[224] := 'E0 '; BYTE_GAP[225] := 'E1 '; BYTE_GAP[226] := 'E2 '; BYTE_GAP[227] := 'E3 '; BYTE_GAP[228] := 'E4 '; BYTE_GAP[229] := 'E5 '; BYTE_GAP[230] := 'E6 '; BYTE_GAP[231] := 'E7 '; BYTE_GAP[232] := 'E8 '; BYTE_GAP[233] := 'E9 '; BYTE_GAP[234] := 'EA '; BYTE_GAP[235] := 'EB '; BYTE_GAP[236] := 'EC '; BYTE_GAP[237] := 'ED '; BYTE_GAP[238] := 'EE '; BYTE_GAP[239] := 'EF ';
  BYTE_GAP[240] := 'F0 '; BYTE_GAP[241] := 'F1 '; BYTE_GAP[242] := 'F2 '; BYTE_GAP[243] := 'F3 '; BYTE_GAP[244] := 'F4 '; BYTE_GAP[245] := 'F5 '; BYTE_GAP[246] := 'F6 '; BYTE_GAP[247] := 'F7 '; BYTE_GAP[248] := 'F8 '; BYTE_GAP[249] := 'F9 '; BYTE_GAP[250] := 'FA '; BYTE_GAP[251] := 'FB '; BYTE_GAP[252] := 'FC '; BYTE_GAP[253] := 'FD '; BYTE_GAP[254] := 'FE '; BYTE_GAP[255] := 'FF ';
  BYTE_GAP[WALL] := '';
  BYTE_GAP[WALL] := BYTE_GAP[WALL] + Chr(joint_byte);
  BYTE_GAP[WALL] := BYTE_GAP[WALL] + Chr(joint_byte);
  BYTE_GAP[WALL] := BYTE_GAP[WALL] + ' ';
end;

procedure Hexadecimals.presetting(jb: Byte; sb: Byte);
var
  s: String;
begin
  joint_byte := jb;
  separator_byte := sb;
  s := EMPTY + Chr(separator_byte);

  BYTE_SEPARATOR[0] := '00' + s; BYTE_SEPARATOR[1] := '01' + s; BYTE_SEPARATOR[2] := '02' + s; BYTE_SEPARATOR[3] := '03' + s; BYTE_SEPARATOR[4] := '04' + s; BYTE_SEPARATOR[5] := '05' + s; BYTE_SEPARATOR[6] := '06' + s; BYTE_SEPARATOR[7] := '07' + s; BYTE_SEPARATOR[8] := '08' + s; BYTE_SEPARATOR[9] := '09' + s; BYTE_SEPARATOR[10] := '0A' + s; BYTE_SEPARATOR[11] := '0B' + s; BYTE_SEPARATOR[12] := '0C' + s; BYTE_SEPARATOR[13] := '0D' + s; BYTE_SEPARATOR[14] := '0E' + s; BYTE_SEPARATOR[15] := '0F' + s; BYTE_SEPARATOR[16] := '10' + s; BYTE_SEPARATOR[17] := '11' + s; BYTE_SEPARATOR[18] := '12' + s; BYTE_SEPARATOR[19] := '13' + s; BYTE_SEPARATOR[20] := '14' + s; BYTE_SEPARATOR[21] := '15' + s; BYTE_SEPARATOR[22] := '16' + s; BYTE_SEPARATOR[23] := '17' + s; BYTE_SEPARATOR[24] := '18' + s; BYTE_SEPARATOR[25] := '19' + s; BYTE_SEPARATOR[26] := '1A' + s; BYTE_SEPARATOR[27] := '1B' + s; BYTE_SEPARATOR[28] := '1C' + s; BYTE_SEPARATOR[29] := '1D' + s;
  BYTE_SEPARATOR[30] := '1E' + s; BYTE_SEPARATOR[31] := '1F' + s; BYTE_SEPARATOR[32] := '20' + s; BYTE_SEPARATOR[33] := '21' + s; BYTE_SEPARATOR[34] := '22' + s; BYTE_SEPARATOR[35] := '23' + s; BYTE_SEPARATOR[36] := '24' + s; BYTE_SEPARATOR[37] := '25' + s; BYTE_SEPARATOR[38] := '26' + s; BYTE_SEPARATOR[39] := '27' + s; BYTE_SEPARATOR[40] := '28' + s; BYTE_SEPARATOR[41] := '29' + s; BYTE_SEPARATOR[42] := '2A' + s; BYTE_SEPARATOR[43] := '2B' + s; BYTE_SEPARATOR[44] := '2C' + s; BYTE_SEPARATOR[45] := '2D' + s; BYTE_SEPARATOR[46] := '2E' + s; BYTE_SEPARATOR[47] := '2F' + s; BYTE_SEPARATOR[48] := '30' + s; BYTE_SEPARATOR[49] := '31' + s; BYTE_SEPARATOR[50] := '32' + s; BYTE_SEPARATOR[51] := '33' + s; BYTE_SEPARATOR[52] := '34' + s; BYTE_SEPARATOR[53] := '35' + s; BYTE_SEPARATOR[54] := '36' + s; BYTE_SEPARATOR[55] := '37' + s; BYTE_SEPARATOR[56] := '38' + s; BYTE_SEPARATOR[57] := '39' + s; BYTE_SEPARATOR[58] := '3A' + s; BYTE_SEPARATOR[59] := '3B' + s;
  BYTE_SEPARATOR[60] := '3C' + s; BYTE_SEPARATOR[61] := '3D' + s; BYTE_SEPARATOR[62] := '3E' + s; BYTE_SEPARATOR[63] := '3F' + s; BYTE_SEPARATOR[64] := '40' + s; BYTE_SEPARATOR[65] := '41' + s; BYTE_SEPARATOR[66] := '42' + s; BYTE_SEPARATOR[67] := '43' + s; BYTE_SEPARATOR[68] := '44' + s; BYTE_SEPARATOR[69] := '45' + s; BYTE_SEPARATOR[70] := '46' + s; BYTE_SEPARATOR[71] := '47' + s; BYTE_SEPARATOR[72] := '48' + s; BYTE_SEPARATOR[73] := '49' + s; BYTE_SEPARATOR[74] := '4A' + s; BYTE_SEPARATOR[75] := '4B' + s; BYTE_SEPARATOR[76] := '4C' + s; BYTE_SEPARATOR[77] := '4D' + s; BYTE_SEPARATOR[78] := '4E' + s; BYTE_SEPARATOR[79] := '4F' + s; BYTE_SEPARATOR[80] := '50' + s; BYTE_SEPARATOR[81] := '51' + s; BYTE_SEPARATOR[82] := '52' + s; BYTE_SEPARATOR[83] := '53' + s; BYTE_SEPARATOR[84] := '54' + s; BYTE_SEPARATOR[85] := '55' + s; BYTE_SEPARATOR[86] := '56' + s; BYTE_SEPARATOR[87] := '57' + s; BYTE_SEPARATOR[88] := '58' + s; BYTE_SEPARATOR[89] := '59' + s;
  BYTE_SEPARATOR[90] := '5A' + s; BYTE_SEPARATOR[91] := '5B' + s; BYTE_SEPARATOR[92] := '5C' + s; BYTE_SEPARATOR[93] := '5D' + s; BYTE_SEPARATOR[94] := '5E' + s; BYTE_SEPARATOR[95] := '5F' + s; BYTE_SEPARATOR[96] := '60' + s; BYTE_SEPARATOR[97] := '61' + s; BYTE_SEPARATOR[98] := '62' + s; BYTE_SEPARATOR[99] := '63' + s; BYTE_SEPARATOR[100] := '64' + s; BYTE_SEPARATOR[101] := '65' + s; BYTE_SEPARATOR[102] := '66' + s; BYTE_SEPARATOR[103] := '67' + s; BYTE_SEPARATOR[104] := '68' + s; BYTE_SEPARATOR[105] := '69' + s; BYTE_SEPARATOR[106] := '6A' + s; BYTE_SEPARATOR[107] := '6B' + s; BYTE_SEPARATOR[108] := '6C' + s; BYTE_SEPARATOR[109] := '6D' + s; BYTE_SEPARATOR[110] := '6E' + s; BYTE_SEPARATOR[111] := '6F' + s; BYTE_SEPARATOR[112] := '70' + s; BYTE_SEPARATOR[113] := '71' + s; BYTE_SEPARATOR[114] := '72' + s; BYTE_SEPARATOR[115] := '73' + s; BYTE_SEPARATOR[116] := '74' + s; BYTE_SEPARATOR[117] := '75' + s; BYTE_SEPARATOR[118] := '76' + s; BYTE_SEPARATOR[119] := '77' + s;
  BYTE_SEPARATOR[120] := '78' + s; BYTE_SEPARATOR[121] := '79' + s; BYTE_SEPARATOR[122] := '7A' + s; BYTE_SEPARATOR[123] := '7B' + s; BYTE_SEPARATOR[124] := '7C' + s; BYTE_SEPARATOR[125] := '7D' + s; BYTE_SEPARATOR[126] := '7E' + s; BYTE_SEPARATOR[127] := '7F' + s; BYTE_SEPARATOR[128] := '80' + s; BYTE_SEPARATOR[129] := '81' + s; BYTE_SEPARATOR[130] := '82' + s; BYTE_SEPARATOR[131] := '83' + s; BYTE_SEPARATOR[132] := '84' + s; BYTE_SEPARATOR[133] := '85' + s; BYTE_SEPARATOR[134] := '86' + s; BYTE_SEPARATOR[135] := '87' + s; BYTE_SEPARATOR[136] := '88' + s; BYTE_SEPARATOR[137] := '89' + s; BYTE_SEPARATOR[138] := '8A' + s; BYTE_SEPARATOR[139] := '8B' + s; BYTE_SEPARATOR[140] := '8C' + s; BYTE_SEPARATOR[141] := '8D' + s; BYTE_SEPARATOR[142] := '8E' + s; BYTE_SEPARATOR[143] := '8F' + s; BYTE_SEPARATOR[144] := '90' + s; BYTE_SEPARATOR[145] := '91' + s; BYTE_SEPARATOR[146] := '92' + s; BYTE_SEPARATOR[147] := '93' + s; BYTE_SEPARATOR[148] := '94' + s; BYTE_SEPARATOR[149] := '95' + s;
  BYTE_SEPARATOR[150] := '96' + s; BYTE_SEPARATOR[151] := '97' + s; BYTE_SEPARATOR[152] := '98' + s; BYTE_SEPARATOR[153] := '99' + s; BYTE_SEPARATOR[154] := '9A' + s; BYTE_SEPARATOR[155] := '9B' + s; BYTE_SEPARATOR[156] := '9C' + s; BYTE_SEPARATOR[157] := '9D' + s; BYTE_SEPARATOR[158] := '9E' + s; BYTE_SEPARATOR[159] := '9F' + s; BYTE_SEPARATOR[160] := 'A0' + s; BYTE_SEPARATOR[161] := 'A1' + s; BYTE_SEPARATOR[162] := 'A2' + s; BYTE_SEPARATOR[163] := 'A3' + s; BYTE_SEPARATOR[164] := 'A4' + s; BYTE_SEPARATOR[165] := 'A5' + s; BYTE_SEPARATOR[166] := 'A6' + s; BYTE_SEPARATOR[167] := 'A7' + s; BYTE_SEPARATOR[168] := 'A8' + s; BYTE_SEPARATOR[169] := 'A9' + s; BYTE_SEPARATOR[170] := 'AA' + s; BYTE_SEPARATOR[171] := 'AB' + s; BYTE_SEPARATOR[172] := 'AC' + s; BYTE_SEPARATOR[173] := 'AD' + s; BYTE_SEPARATOR[174] := 'AE' + s; BYTE_SEPARATOR[175] := 'AF' + s; BYTE_SEPARATOR[176] := 'B0' + s; BYTE_SEPARATOR[177] := 'B1' + s; BYTE_SEPARATOR[178] := 'B2' + s; BYTE_SEPARATOR[179] := 'B3' + s;
  BYTE_SEPARATOR[180] := 'B4' + s; BYTE_SEPARATOR[181] := 'B5' + s; BYTE_SEPARATOR[182] := 'B6' + s; BYTE_SEPARATOR[183] := 'B7' + s; BYTE_SEPARATOR[184] := 'B8' + s; BYTE_SEPARATOR[185] := 'B9' + s; BYTE_SEPARATOR[186] := 'BA' + s; BYTE_SEPARATOR[187] := 'BB' + s; BYTE_SEPARATOR[188] := 'BC' + s; BYTE_SEPARATOR[189] := 'BD' + s; BYTE_SEPARATOR[190] := 'BE' + s; BYTE_SEPARATOR[191] := 'BF' + s; BYTE_SEPARATOR[192] := 'C0' + s; BYTE_SEPARATOR[193] := 'C1' + s; BYTE_SEPARATOR[194] := 'C2' + s; BYTE_SEPARATOR[195] := 'C3' + s; BYTE_SEPARATOR[196] := 'C4' + s; BYTE_SEPARATOR[197] := 'C5' + s; BYTE_SEPARATOR[198] := 'C6' + s; BYTE_SEPARATOR[199] := 'C7' + s; BYTE_SEPARATOR[200] := 'C8' + s; BYTE_SEPARATOR[201] := 'C9' + s; BYTE_SEPARATOR[202] := 'CA' + s; BYTE_SEPARATOR[203] := 'CB' + s; BYTE_SEPARATOR[204] := 'CC' + s; BYTE_SEPARATOR[205] := 'CD' + s; BYTE_SEPARATOR[206] := 'CE' + s; BYTE_SEPARATOR[207] := 'CF' + s; BYTE_SEPARATOR[208] := 'D0' + s; BYTE_SEPARATOR[209] := 'D1' + s;
  BYTE_SEPARATOR[210] := 'D2' + s; BYTE_SEPARATOR[211] := 'D3' + s; BYTE_SEPARATOR[212] := 'D4' + s; BYTE_SEPARATOR[213] := 'D5' + s; BYTE_SEPARATOR[214] := 'D6' + s; BYTE_SEPARATOR[215] := 'D7' + s; BYTE_SEPARATOR[216] := 'D8' + s; BYTE_SEPARATOR[217] := 'D9' + s; BYTE_SEPARATOR[218] := 'DA' + s; BYTE_SEPARATOR[219] := 'DB' + s; BYTE_SEPARATOR[220] := 'DC' + s; BYTE_SEPARATOR[221] := 'DD' + s; BYTE_SEPARATOR[222] := 'DE' + s; BYTE_SEPARATOR[223] := 'DF' + s; BYTE_SEPARATOR[224] := 'E0' + s; BYTE_SEPARATOR[225] := 'E1' + s; BYTE_SEPARATOR[226] := 'E2' + s; BYTE_SEPARATOR[227] := 'E3' + s; BYTE_SEPARATOR[228] := 'E4' + s; BYTE_SEPARATOR[229] := 'E5' + s; BYTE_SEPARATOR[230] := 'E6' + s; BYTE_SEPARATOR[231] := 'E7' + s; BYTE_SEPARATOR[232] := 'E8' + s; BYTE_SEPARATOR[233] := 'E9' + s; BYTE_SEPARATOR[234] := 'EA' + s; BYTE_SEPARATOR[235] := 'EB' + s; BYTE_SEPARATOR[236] := 'EC' + s; BYTE_SEPARATOR[237] := 'ED' + s; BYTE_SEPARATOR[238] := 'EE' + s; BYTE_SEPARATOR[239] := 'EF' + s;
  BYTE_SEPARATOR[240] := 'F0' + s; BYTE_SEPARATOR[241] := 'F1' + s; BYTE_SEPARATOR[242] := 'F2' + s; BYTE_SEPARATOR[243] := 'F3' + s; BYTE_SEPARATOR[244] := 'F4' + s; BYTE_SEPARATOR[245] := 'F5' + s; BYTE_SEPARATOR[246] := 'F6' + s; BYTE_SEPARATOR[247] := 'F7' + s; BYTE_SEPARATOR[248] := 'F8' + s; BYTE_SEPARATOR[249] := 'F9' + s; BYTE_SEPARATOR[250] := 'FA' + s; BYTE_SEPARATOR[251] := 'FB' + s; BYTE_SEPARATOR[252] := 'FC' + s; BYTE_SEPARATOR[253] := 'FD' + s; BYTE_SEPARATOR[254] := 'FE' + s; BYTE_SEPARATOR[255] := 'FF' + s;
  BYTE_SEPARATOR[WALL] := '';
  BYTE_SEPARATOR[WALL] := BYTE_SEPARATOR[WALL] + Chr(joint_byte);
  BYTE_SEPARATOR[WALL] := BYTE_SEPARATOR[WALL] + Chr(joint_byte);
  BYTE_SEPARATOR[WALL] := BYTE_SEPARATOR[WALL] + s;
  H_0A_SEPARATOR := '0A' + s;
  H_0D_SEPARATOR := '0D' + s;
end;

procedure Hexadecimals.setting(fn: String; vb: Integer; vf: Integer; VL: Integer; vr: Integer; wv: Integer);
var
  i: Integer;
  pascal_string: AnsiString;
  s: String;
  testFile: File;
begin
  buffer_size := -1;
  count := -1;
  file_name := fn;
  limited_to_value_bytes := vb;
  limited_to_value_fields := vf;
  limited_to_value_lines := VL;
  limited_to_value_rows := vr;
  wall_value := wv;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);
    CloseFile(testFile);

    buffer_size := file_size(file_name);
  except
    s := 'Reset() error-397: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  NEW_LINE := EMPTY_ANSI + ANSI_CHAR[H_0A];
  NEW_LINE_SIZE := 1;

  setBufferSize;
end;

procedure Hexadecimals.setting(f: FinalEffects);
var
  i: Integer;
  pascal_string: AnsiString;
  s: String;
  testFile: File;
begin
  buffer_size := -1;
  count := -1;
  file_name := f.fileName;
  limited_to_value_bytes := f.valueBytes;
  limited_to_value_fields := f.valueFields;
  limited_to_value_lines := f.valueLines;
  limited_to_value_rows := f.valueRows;
  wall_value := f.behindWall;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);
    CloseFile(testFile);

    buffer_size := file_size(file_name);
    if buffer_size <= 0 then
      raise Exception.Create('file_size() error');

  except
    s := 'Reset() error-398: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  NEW_LINE := EMPTY_ANSI + ANSI_CHAR[H_0A];
  NEW_LINE_SIZE := 1;

  setBufferSize;
end;

end.
