unit MessagesText;

{$I+}

interface

uses
  SysUtils;

type AnsiCharArray = array[0..4095] of AnsiChar;

type
  MessagesTexts = class
  private
    ANSI_CHAR: array[0..255] of AnsiChar;
    CLEAR_V_ARRAY: AnsiCharArray;
    EOL: array[0..3] of AnsiChar;
    H_FE_C: AnsiChar;
    H_FF_C: AnsiChar;
    new_name: String;
    messages: String;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    function ANSI_CHAR_table: AnsiChar;
    function bufferingWriting(s: PChar; s_length: Integer): Integer;
  public
    function eight_bit_character(b: Byte): AnsiChar;
    procedure setting(nn: String; s: String);
  end;

const
  BOM_SIZE = 2;
  EOL_SIZE = 4;
  FILE_ERROR = -2;
  H_000000FF = 255;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_FE = 254;
  H_FF = 255;
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 2;

implementation

function MessagesTexts.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function MessagesTexts.bufferingWriting(s: PChar; s_length: Integer): Integer;
var
  b: Byte;
  bom: array[0..1] of AnsiChar;
  block_has_been_written: Integer;
  exit_code: Integer;
  i: Longint;
  k: Integer;
  n: Integer;
  newFile: File;
begin
  exit_code := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    bom[0] := H_FF_C;
    bom[1] := H_FE_C;
    BlockWrite(newFile, bom, BOM_SIZE, block_has_been_written);

    n := 0;
    repeat
      v := CLEAR_V_ARRAY;
      k := 0;

      repeat
        i := Ord(s[n]);
        Inc(n);

        b := H_000000FF and i; { little-endian architecture }
        v[k] := ANSI_CHAR[b];

        i := i shr 8;
        b := H_000000FF and i;
        v[ k + 1 ] := ANSI_CHAR[b];
        Inc(k, SIZE_OF_CHAR);

        if n >= s_length then
          begin
            V_SIZE := k;
            V_SIZE_1 := V_SIZE - 1;
            break;
          end;

      until k >= V_SIZE;

      BlockWrite(newFile, v, V_SIZE, block_has_been_written);
    until n >= s_length;

    BlockWrite(newFile, EOL, EOL_SIZE, block_has_been_written);
    CloseFile(newFile);
    V_SIZE := SizeOf(v);
    V_SIZE_1 := V_SIZE - 1;
  except
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function MessagesTexts.eight_bit_character(b: Byte): AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
begin
  beginning_of_ASCII := 0;

  if b >= 1 then
    begin
      end_1 := b - 1;
      h := AnsiChar( Chr(beginning_of_ASCII) );

      for k := beginning_of_ASCII to end_1 do
      begin
        Inc(h);
      end;

    end
  else
    begin
      h := AnsiChar(#0);
    end;

  Result := h;
end;

procedure MessagesTexts.setting(nn: String; s: String);
var
  i: Longint;
  k: Integer;
  m: Integer;
  messagesLength: Integer;
begin
  new_name := nn;
  k := Length(s);

  i := Ord(s[ k - 1 ]);
  if i = H_0D then
    begin
      messagesLength := k - 2;
      messages := Copy(s, 1, messagesLength); { Removing 0D 0A }
    end
  else
    begin
      i := Ord(s[k]);
      if i = H_0A then
        begin
          messagesLength := k - 1;
          messages := Copy(s, 1, messagesLength);
        end
      else
        begin
          messages := s;
        end;

    end;

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  for m := 0 to V_SIZE_1 do
  begin
    CLEAR_V_ARRAY[m] := #0;
  end;

  H_FF_C := ANSI_CHAR_table;
  EOL[0] := ANSI_CHAR[H_0D];
  EOL[1] := ANSI_CHAR[0];
  EOL[2] := ANSI_CHAR[H_0A];
  EOL[3] := ANSI_CHAR[0];
  H_FE_C := ANSI_CHAR[H_FE];

  bufferingWriting( PChar(s), Length(s) );
end;

end.
