program Obot;
{$APPTYPE CONSOLE}

{*
Obot Delphi Version 1.05
Reading and writing so that the first part of EXE can be represented in hexadecimal as 4D 5A, for example.
Copyright 2020 Sogaya

Delphi 2009

*}

uses
  SysUtils,
  ErrorText in 'ErrorText.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  Hexadecimal in 'Hexadecimal.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  TimeName in 'TimeName.pas';

{$R *.res}

const
  STANDARD_BUFFER_SIZE = 640;
  CERR = 'err';
  COMMA_BYTE = 44; { 2Ch, ',' }
  FEW_LINES = 3;
  FINAL_INTEGER = 2147483647; { 7FFFFFFFh }
  GAP_BYTE = 32; { 20h, SP, ' ' }
  H_2C = 44; { , }
  H_5F = 95; { _ }
  TEMPORARY_TEXT = 'text.tmp';
  TEXT_FILE = '.txt';
  UPPER_LIMIT_TO_FIELD_NUMBER = 20;
  VERSION_NUMBER = '1.05';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;
  WALL = 256;

var
  buffer_size: Integer;
  e: Hexadecimals;
  f: FinalEffects;
  file_name: String;
  file_name_length: Integer;
  final_separator: Byte;
  g: Guiding;
  h: HyphenTerms;
  s: String;
  size: Integer;
  t: Integer;
  temporaryFile: File;
  testFile: File;
  x: ErrorTexts;
begin
  s := ParamStr(1);
  if Length(s) = 0 then
    begin
{ Producing TEMPORARY_TEXT from standard input }
      e := Hexadecimals.Create;
      e.produceTemporaryTextFile;

      t := FileOpen(TEMPORARY_TEXT, fmOpenRead or fmShareDenyNone);
      size := FileSeek(t, 0, 2); { EOF }
      FileClose(t);

      e.presetting(H_5F);
      e.setting(TEMPORARY_TEXT, size, UPPER_LIMIT_TO_FIELD_NUMBER, FEW_LINES, FINAL_INTEGER, WALL);
      e.limiting;

      AssignFile(temporaryFile, TEMPORARY_TEXT);
      Erase(temporaryFile);

      e.Free;
      Halt(0);
    end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          file_name := h.fileName;
          h.joints; { joint_byte }
          h.separators; { separator_byte }
          if h.noChecking then
            begin
              final_separator := h.separatorByte;
            end
          else
            begin
              if h.extensionEqualsCSV(file_name) then
                begin
                  final_separator := COMMA_BYTE;
                end
              else
                begin
                  final_separator := h.separatorByte;
                end;

            end;

          file_name_length := Length(file_name);
          if file_name_length = 0 then
            begin
              buffer_size := STANDARD_BUFFER_SIZE;
            end
          else
            begin
              try
                AssignFile(testFile, file_name);
                Reset(testFile);
                CloseFile(testFile);
              except
                s := 'Reset() error-391: ' + file_name;
                x := ErrorTexts.Create;
                x.setting(C_ERR, TEXT_FILE, s);
                x.Free;
                Halt(1);
              end;

              t := FileOpen(file_name, fmOpenRead or fmShareDenyNone);
              size := FileSeek(t, 0, 2); { EOF }
              FileClose(t);
              if size > h.valueBytes then
                begin
                  buffer_size := h.valueBytes
                end
              else
                begin
                  buffer_size := size;
                end;

            end;

          f := FinalEffects.Create;
          f.setting(file_name,
            buffer_size, h.valueFields, h.valueLines, h.valueRows, h.behindWall,
            h.jointByte, final_separator,
            h.noRemoving);

{ main function }
  if f.separatorByte = GAP_BYTE then
    begin
      e := Hexadecimals.Create;

      file_name_length := Length(f.fileName);
      if file_name_length = 0 then
        begin
{ Producing TEMPORARY_TEXT from standard input }
          e.produceTemporaryTextFile;
          f.setFileName(TEMPORARY_TEXT);
        end;

      e.presetting(f.jointByte);
      e.setting(f);
      e.limiting;
      e.Free;
    end
  else
    begin
      e := Hexadecimals.Create;

      file_name_length := Length(f.fileName);
      if file_name_length = 0 then
        begin
{ Producing TEMPORARY_TEXT from standard input }
          e.produceTemporaryTextFile;
          f.setFileName(TEMPORARY_TEXT);
        end;

      e.presetting(f.jointByte, f.separatorByte);
      e.setting(f);
      e.limiting_to_produce_data_file;
      e.Free;
    end;

    if not f.noRemoving then
      begin
        if file_name_length = 0 then
          begin
            AssignFile(temporaryFile, TEMPORARY_TEXT);
            Erase(temporaryFile);
          end;

      end;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
