unit HyphenTerm;

interface

uses
  SysUtils, Numbered, Math, ErrorText;

type PettyArray = array[0..255] of Char;
type Triplet = array[0..2] of Integer;

type
  HyphenTerms = class
  private
    acquisition: array[1..64] of String;
    after: Integer;
    ASCII: String; { ASCII := ASCII_1 + Chr(H_27); }
    before: Integer;
    binary_represented: Boolean;
    bit_pattern_array: array[1..2] of String;
    bottom_line_number: Integer;
    buffer: array[0..3071] of Char;
    byteCode: array[1..2] of Integer;
    BYTES : String;
    carriage_return: Boolean;
    CLEAR_BIT_C: Char;
    CLEAR_PETTY_ARRAY: PettyArray;
    full_level: Integer;
    deniable: array[1..12] of Boolean;
    EBB: String;
    field_count: Integer;
    figure: array[1..6] of Integer;
    first_file_name: String;
    FLOOD: String;
    formal: Boolean;
    focal_point: Integer;
    GAP_C: Char;
    interval_from: Integer;
    interval_to: Integer;
    joint_byte: Byte;
    joint_piece: String;
    list_name: array[1..2] of String;
    maximum: array[1..2] of Integer;
    minimum: array[1..2] of Integer;
    NEW_LINE: String;
    p: PettyArray;
    P_SIZE: Integer;
    P_SIZE_1: Integer;
    parametersList: array of String;
    path_name: String;
    pattern: String;
    piece: array[1..2] of String;
    second_file_name: String;
    separator_byte: Byte;
    separator_piece: String;
    SET_BIT_C: Char;
    TAB_C: Char;
    top_line_number: Integer;
    triplet_in_context: Triplet;
    unnecessary_characters: Integer;
    x: ErrorTexts;
  public
    destructor Destroy; override;
    function afterPoint: Integer;
    function atPoint: Integer;
    function beforePoint: Integer;
    function binaryRepresented: Boolean;
    function bottomLine: Integer;
    function carriageReturn: Boolean;
    function copyingStringToArray(s: PChar; s_length: Integer): Integer;
    function count0Ah: Integer;
    function displayData: Boolean;
    function doubleSpaced: Integer;
    function doubleSpacedFormal: Boolean;
    function extensionEqualsCSV(file_name: String): Boolean;
    function fileName: String;
    function forcibleEntry: Boolean;
    function guideFile: Boolean;
    function hexadecimal_to_integer(s: String): Integer;
    function identicalTime: Boolean;
    function include0Dh: Boolean;
    function intervalFrom: Integer;
    function intervalTo: Integer;
    function jointByte: Byte;
    function jointCode: Byte;
    function jointPiece: String;
    function jointPieces(s: String): Boolean;
    function joints: String;
    function lineFeed: Boolean;
    function listName(index: Integer): String;
    function multipleBitPattern: String;
    function nearPoint: Triplet;
    function negativePattern: String;
    function newName: String;
    function nonSeparatorEnding: Boolean;
    function noRemoving: Boolean;
    function pathName: String;
    function produceTemporaryBinary: Boolean;
    function punctuation(s: String; q: Integer): Byte;
    function separatorByte: Byte;
    function separatorCode: Byte;
    function separatorPiece: String;
    function separatorPieces(s: String): Boolean;
    function separators: String;
    function singleSurge: String; { not Triplet }
    function surge_of_set_bits(interval_0: Integer; interval_1: Integer): String;
    function topLine: Integer;
    function tripletAfter: Integer;
    function tripletBefore: Integer;
    function tripletPoint: Integer;
    function unnecessaryCharacters: Integer;
    function unnecessaryWidth: Integer;
    function versionNumber: Boolean;
    procedure acquireFromText;
    procedure presetting;
    procedure setFieldCount(c: Integer);
    procedure setting;
  end;

const
  ACQUISITION_ITEM_COUNT = 64;
  AFTER_POINT = '-}';
  ASCII_1 = ' !"#$%&()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~';
  AT_POINT = '-P';
  BEFORE_POINT = '-{';
  BITS = '-e';
  C_ERR = 'err';
  CAPITAL_A = 65; { 41h }
  CAPITAL_CSV = 'CSV';
  CLEAR_BIT = 48; { 30h, '0' }
  COMMA = 44; { 2Ch }
  COMMAND_NAME = 'rufeol';
  COMMAND_NAME_FILE = 'draft.txt';
  COUNT_0AH_BEFORE_TRIGGERING_LINE_FEED = '-w';
  CRLF = '-CRLF';
  DENIED = '-';
  DISPLAY_DATA = '-d';
  DO_NOT_REMOVE_TEMPORARY_FILE = '-y';
  DOT = '.';
  DOT_SIZE = 1;
  DOUBLE_SPACED = '-x';
  EMPTY = '';
  EXPANSION_FACTOR = 64;
  FIELD_1 = 1;
  FINAL_1 = 2147483646; { 7FFFFFFEh }
  FINAL_INTEGER = 2147483647; { 7FFFFFFFh }
  FIRST_LINE = 1;
  FORCIBLE_ENTRY_OF_0DH = '-r';
  GAP = 32; { 20h }
  GREATER_THAN = 62; { 3Eh }
  GUIDE_TERM = '-G';
  H_000000FF = 255;
  H_00TO42 = 'x00x01x02x03x04x05x06x07x08x09x0Ax0Bx0Cx0Dx0Ex0Fx10x11x12x13x14x15x16x17x18x19x1Ax1Bx1Cx1Dx1Ex1Fx20x21x22x23x24x25x26x27x28x29x2Ax2Bx2Cx2Dx2Ex2Fx30x31x32x33x34x35x36x37x38x39x3Ax3Bx3Cx3Dx3Ex3Fx40x41x42';
  H_09 = 9; { TAB }
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_1A = 26; { end of MS-DOS text file }
  H_20 = 32; { SP, GAP }
  H_22 = 34; { " }
  H_27 = 39; { ' }
  H_2C = 44; { , }
  H_3C = 60; { < }
  H_43TO85 = 'x43x44x45x46x47x48x49x4Ax4Bx4Cx4Dx4Ex4Fx50x51x52x53x54x55x56x57x58x59x5Ax5Bx5Cx5Dx5Ex5Fx60x61x62x63x64x65x66x67x68x69x6Ax6Bx6Cx6Dx6Ex6Fx70x71x72x73x74x75x76x77x78x79x7Ax7Bx7Cx7Dx7Ex7Fx80x81x82x83x84x85';
  H_44 = 68; { D }
  H_5F = 95; { _ }
  H_7C = 124; { | }
  H_86TOC8 = 'x86x87x88x89x8Ax8Bx8Cx8Dx8Ex8Fx90x91x92x93x94x95x96x97x98x99x9Ax9Bx9Cx9Dx9Ex9FxA0xA1xA2xA3xA4xA5xA6xA7xA8xA9xAAxABxACxADxAExAFxB0xB1xB2xB3xB4xB5xB6xB7xB8xB9xBAxBBxBCxBDxBExBFxC0xC1xC2xC3xC4xC5xC6xC7xC8';
  H_C9TOFF = 'xC9xCAxCBxCCxCDxCExCFxD0xD1xD2xD3xD4xD5xD6xD7xD8xD9xDAxDBxDCxDDxDExDFxE0xE1xE2xE3xE4xE5xE6xE7xE8xE9xEAxEBxECxEDxEExEFxF0xF1xF2xF3xF4xF5xF6xF7xF8xF9xFAxFBxFCxFDxFExFF';
  HE = 205; { CDh }
  HELP_TERM = '--help';
  HYPHEN = '-';
  INCLUDE_0DH = '-0';
  IDENTICAL_TIME = '-t';
  INTERVAL_TERM = '-i';
  JOINT_PIECE_TERM = '-a';
  JOINT_PIECE_TERM_SIZE = 2;
  JOINT_TERM = '-A';
  JOINT_TERM_SIZE = 2;
  LESS_THAN = 60; { 3Ch }
  LF = '-LF';
  LINE_FEED = '-L';
  LOWER_CASE_CSV = 'csv';
  LOWER_CASE_L = 108; { 6Ch, l }
  LOWER_CASE_S = 115; { 73h, s }
  LOWER_CASE_T = 116; { 74h, t }
  LOWER_CASE_V = 118; { 76h, v }
  LOWER_CASE_X = 120; { 78h, x }
  MAXIMUM_DIFFERENCE = 15.0;
  MAXIMUM_ROW_COUNT = 64;
  N_0TO1 = '01';
  N_0TO9 = '0123456789';
  NEAR_POINT = '-~';
  NEGATIVE_BIT_PATTERN = '-E';
  NON_SEPARATOR_ENDING = '-m';
  NOT_COUNTED = 2147483647; { 7FFFFFFFh }
  NOT_MEASURED = -1;
  PRODUCE_TEMPORARY_BINARY = '-b';
  SEPARATOR_PIECE_TERM = '-s';
  SEPARATOR_PIECE_TERM_SIZE = 2;
  SEPARATOR_TERM = '-S';
  SEPARATOR_TERM_SIZE = 2;
  SET_BIT = 49; { 31h, '1'}
  SETTING_TEXT = 'set_rufeol.txt';
  SIZE_OF_CHAR = 2;
  TAB = 9;
  TERM_1_FILE_NAME = '-o';
  TERM_2_FILE_NAME = '-_';
  TEXT_FILE = '.txt';
  UNALTERED = '_';
  UPPER_LIMIT_TO_FIELD_NUMBER = 20;
  UNNECESSARY_TERM = '-u';
  VERSION_WITH_COMMENTS = '-VC';
  VERSION_WITHOUT_COMMENTS = '-V';
  VERTICAL_LINE = 124; { 7Ch }
  YEN = '\';
  YEN_C = 92; { 5Ch }

implementation

destructor HyphenTerms.Destroy;
begin
  parametersList := nil;

  inherited Destroy;
end;

function HyphenTerms.afterPoint: Integer;
const
  F = 1;
begin
  if figure[F] = NOT_CONVERTED then
    begin
      after := 0;
    end
  else
    begin
      after := figure[F];
    end;

  Result := after;
end;

function HyphenTerms.atPoint: Integer;
const
  F = 3;
begin
  if figure[F] = NOT_CONVERTED then
    begin
      focal_point := FIRST_LINE;
    end
  else
    begin
      focal_point := figure[F];
    end;

  Result := focal_point;
end;

function HyphenTerms.beforePoint: Integer;
const
  F = 2;
begin
  if figure[F] = NOT_CONVERTED then
    begin
      before := 0;
    end
  else
    begin
      before := figure[F];
    end;

  Result := before;
end;

function HyphenTerms.binaryRepresented: Boolean;
begin
  Result := binary_represented; { 10111, for example }
end;

function HyphenTerms.bottomLine: Integer;
begin
  Result := bottom_line_number;
end;

function HyphenTerms.carriageReturn: Boolean;
begin
  Result := deniable[8];
end;

function HyphenTerms.copyingStringToArray(s: PChar; s_length: Integer): Integer;
var
  k: Integer;
  m: Integer;
  s_length_1: Integer;
begin
  m := 0;
  s_length_1 := s_length - 1;

  for k := 0 to s_length_1 do
  begin
    buffer[m] := s[k];
    Inc(m);
  end;

  Result := m;
end;

function HyphenTerms.count0Ah: Integer;
const
  F = 5;
begin
  if figure[F] = NOT_CONVERTED then
    begin
      full_level := 1;
    end
  else
    begin
      full_level := figure[F];
    end;

  Result := full_level;
end;

function HyphenTerms.displayData: Boolean;
begin
  Result := deniable[11]
end;

function HyphenTerms.doubleSpaced: Integer;
const
  F = 6;
begin
  if figure[F] = NOT_CONVERTED then
    begin
      formal := False;
      full_level := 1;
      carriage_return := False;
      unnecessary_characters := 0;
    end
  else
    begin
      formal := True;
      full_level := figure[F];
      carriage_return := True; { function include0Dh: Boolean; }
      unnecessary_characters := full_level * 2 - 1; { function unnecessaryWidth: Integer; }
    end;

  Result := full_level;
end;

function HyphenTerms.doubleSpacedFormal: Boolean;
begin
  Result := formal;
end;

function HyphenTerms.extensionEqualsCSV(file_name: String): Boolean;
var
  csvFile: Boolean;
  fileExtension: String;
  tailLength: Integer;
  q: Integer;
  nameLength: Integer;
begin
  csvFile := False;
  nameLength := Length(file_name);
  q := AnsiPos(DOT, file_name);

  if q >= 1 then
    begin
      tailLength := nameLength - q;
      q := q + DOT_SIZE;
      fileExtension := Copy(file_name, q, tailLength);
      if CompareStr(fileExtension, LOWER_CASE_CSV) = 0 then
        begin
          csvFile := True;
        end
      else
        begin
          if CompareStr(fileExtension, CAPITAL_CSV) = 0 then
            begin
              csvFile := True;
            end
          else
            begin
              csvFile := False;
            end;

        end;

    end;

  Result := csvFile;
end;

function HyphenTerms.fileName: String;
var
  nameLength: Integer;
begin
  nameLength := Length(first_file_name);
  if nameLength = 0 then
    first_file_name := EMPTY;

  Result := first_file_name;
end;

function HyphenTerms.forcibleEntry: Boolean;
begin
  Result := deniable[12];
end;

function HyphenTerms.guideFile: Boolean;
begin
  Result := deniable[3];
end;

function HyphenTerms.hexadecimal_to_integer(s: String): Integer;
var
  q: Integer;
begin
  q := AnsiPos(s, BYTES);
  Result := ( q - 1 ) div 3;
end;

function HyphenTerms.identicalTime: Boolean;
begin
  Result := deniable[9];
end;

function HyphenTerms.include0Dh: Boolean;
begin
  Result := carriage_return;
end;

function HyphenTerms.intervalFrom: Integer;
begin
  Result := interval_from;
end;

function HyphenTerms.intervalTo: Integer;
begin
  Result := interval_to;
end;

function HyphenTerms.jointByte: Byte;
begin
  Result := joint_byte;
end;

function HyphenTerms.jointCode: Byte;
begin
  Result := byteCode[1];
end;

function HyphenTerms.jointPiece: String;
begin
  Result := piece[1];
end;

function HyphenTerms.jointPieces(s: String): Boolean;
var
  b: Boolean;
  head: String;
  headLength: Integer;
  k: Integer;
  s_length: Integer;
  tail: String;
  tailLength: Integer;
begin
  b := False;
  s_length := Length(s);
  head := Copy(s, 1, JOINT_PIECE_TERM_SIZE);
  headLength := Length(head);
  tailLength := s_length - headLength;

  if CompareStr(head, JOINT_PIECE_TERM) = 0 then
    begin
      k := headLength + 1;
      tail := Copy(s, k, tailLength);
      b := True;
    end;

  joint_piece := tail;
  Result := b;
end;

function HyphenTerms.joints: String;
const
  F = 1;
var
  b: Byte;
  i: Longint;
  r: String;
begin
  b := byteCode[F];

  if CompareStr(piece[F], UNALTERED) = 0 then
    begin
      if b = 0 then
        begin
          r := piece[F];
        end
      else
        begin
          r := EMPTY + Chr(b);
        end;

    end
  else
    begin
      r := piece[F];
    end;

  i := Ord(r[1]);
  i := i and H_000000FF;
  joint_byte := Byte(i);
  Result := r;
end;

function HyphenTerms.lineFeed: Boolean;
begin
  Result := deniable[10];
end;

function HyphenTerms.listName(index: Integer): String;
var
  messages: String;
  r: String;
begin
  r := EMPTY;

  if index < 1 then
    begin
{ error }
      x := ErrorTexts.Create;
      messages := 'listName() error-409: ' + IntToStr(index);
      x.setting(C_ERR, TEXT_FILE, messages);
      x.Free;
    end
  else
    begin
      if index > 2 then
        begin
{ error }
          x := ErrorTexts.Create;
          messages := 'listName() error-410: ' + IntToStr(index);
          x.setting(C_ERR, TEXT_FILE, messages);
          x.Free;
        end
      else
        begin
          r := list_name[index];
        end;

    end;

  Result := r;
end;

function HyphenTerms.multipleBitPattern: String;
const
  F = 1;
var
  c: String;
  patternLength: Integer;
  q: Integer;
  tail: String;
  tailLength: Integer;
begin
  pattern := bit_pattern_array[F];
  patternLength := Length(pattern);

  c := EMPTY + Chr(CLEAR_BIT);
  q := AnsiPos(c, pattern);
  if q >= 1 then
    binary_represented := True { 10111, for example }
  else
    binary_represented := False;

  if patternLength > field_count then
    begin
      pattern := Copy(pattern, 1, field_count);
    end
  else
    begin
      tailLength := field_count - patternLength;
      tail := Copy(FLOOD, 1, tailLength);
      pattern := bit_pattern_array[F] + tail;
    end;

  Result := pattern;
end;

function HyphenTerms.nearPoint: Triplet;
const
  F = 2;
var
  d: Double;
  difference: Double;
  display_from: Double;
  display_to: Double;
  half: Double;
  messages: String;
begin
  if maximum[F] = NOT_CONVERTED then
    begin
      display_from := 1.0;
      display_to := display_from + MAXIMUM_DIFFERENCE;
    end
  else
    begin
      display_from := 0.0 + minimum[F];
      display_to := 0.0 + maximum[F];
    end;

  d := ( display_from + display_to )/2;

  difference := display_to - display_from;
  if difference < 0.0 then
    begin
{ error }
      display_from := 1.0;
      display_to := display_from + MAXIMUM_DIFFERENCE;

      x := ErrorTexts.Create;
      messages := 'nearPoint error-411: ' + FloatToStr(difference);
      x.setting(C_ERR, TEXT_FILE, messages);
      x.Free;
    end
  else
    begin
      if difference > MAXIMUM_DIFFERENCE then
        begin
          difference := MAXIMUM_DIFFERENCE;
          half := difference/2;
          display_from := d - half;
          display_to := display_from + MAXIMUM_DIFFERENCE;
        end;

    end;

  top_line_number := Floor(display_from);
  bottom_line_number := Floor(display_to);

  d := ( display_from + display_to )/2;
  d := d + 0.5;
  focal_point := Floor(d);

  triplet_in_context[0] := bottom_line_number - focal_point;
  triplet_in_context[1] := focal_point - top_line_number;
  triplet_in_context[2] := focal_point;
  Result := triplet_in_context;
end;

function HyphenTerms.negativePattern: String;
const
  F = 2;
var
  combo: String;
  k: Integer;
  s: String;
  s_length: Integer;
  s_length_1: Integer;
  tail: String;
  tailLength: Integer;
begin
  s := bit_pattern_array[F];
  s_length := copyingStringToArray( PChar(s), Length(s) ); { buffer: array of Char; }
  s_length_1 := s_length - 1;

  binary_represented := False;
  combo := EMPTY;

{ Converting 01000 into 10111 }
  for k := 0 to s_length_1 do
  begin
    if buffer[k] = CLEAR_BIT_C then
      begin
        combo := combo + SET_BIT_C;
      end
    else
      begin
        combo := combo + CLEAR_BIT_C;
        binary_represented := True { 10111, for example }
      end;

  end;

  if s_length > field_count then
    begin
      pattern := Copy(combo, 1, field_count);
    end
  else
    begin
      tailLength := field_count - s_length;
      tail := Copy(FLOOD, 1, tailLength);
      pattern := combo + tail;
    end;

  Result := pattern;
end;

function HyphenTerms.newName: String;
var
  nameLength: Integer;
begin
  nameLength := Length(second_file_name);
  if nameLength = 0 then
    second_file_name := COMMAND_NAME_FILE;

  Result := second_file_name;
end;

function HyphenTerms.nonSeparatorEnding: Boolean;
begin
  Result := deniable[7];
end;

function HyphenTerms.noRemoving: Boolean;
begin
  Result := deniable[5];
end;

function HyphenTerms.pathName: String;
begin
  Result := path_name;
end;

function HyphenTerms.produceTemporaryBinary: Boolean;
begin
  Result := deniable[6]
end;

function HyphenTerms.punctuation(s: String; q: Integer): Byte;
var
  c: array[0..255] of Char;
  h: String;
  i: Longint;
  k: Integer;
  n: Byte;
  r: Byte;
  s_length: Integer;
  trace: Integer;
begin
  if q < 1 then
    begin
      r := COMMA;
    end
  else
    begin
      s_length := Length(s);

      if q > s_length then
        begin
          r := COMMA;
        end
      else
        begin
          trace := q;

          q := trace + 1;
          h := Copy(s, q, 2); { \l, for example }
          StrPCopy(c, h);
          i := Ord(c[0]);
          i := i and H_000000FF;
          n := Byte(i);

          if n = YEN_C then
            begin
{*
1234567890
-S\l
*}
              i := Ord(c[1]);
              i := i and H_000000FF;
              n := Byte(i);

              case n of
                CAPITAL_A:    r := LESS_THAN;
                HE:           r := LESS_THAN;
                LOWER_CASE_L: r := VERTICAL_LINE;
                LOWER_CASE_S: r := GAP;
                LOWER_CASE_T: r := TAB;
                LOWER_CASE_V: r := GREATER_THAN;
                LOWER_CASE_X:
                  begin
{*
1234567890
-S\x7C
*}
                    h := Copy(s, 4, 3);

                    k := hexadecimal_to_integer(h);
                    k := k and H_000000FF;
                    r := Byte(k);
                  end;
              else
                r := COMMA;
              end; { case n of }

            end
          else
            begin
{*
1234567890
-s,
*}
              q := trace + 1;
              h := Copy(s, q, 1); { comma, for example }
              StrPCopy(c, h);
              i := Ord(c[0]);
              i := i and H_000000FF;
              n := Byte(i);

              r := n;
            end; { if n = YEN then, else }

        end; { if q > s_length then, else }

    end; { if q < 1 then, else }

  Result := r;
end;

function HyphenTerms.separatorByte: Byte;
begin
  Result := separator_byte;
end;

function HyphenTerms.separatorCode: Byte;
begin
  Result := byteCode[2];
end;

function HyphenTerms.separatorPieces(s: String): Boolean;
var
  b: Boolean;
  head: String;
  headLength: Integer;
  k: Integer;
  s_length: Integer;
  tail: String;
  tailLength: Integer;
begin
  b := False;
  s_length := Length(s);
  head := Copy(s, 1, SEPARATOR_PIECE_TERM_SIZE);
  headLength := Length(head);
  tailLength := s_length - headLength;

  if CompareStr(head, SEPARATOR_PIECE_TERM) = 0 then
    begin
      k := headLength + 1;
      tail := Copy(s, k, tailLength);
      b := True;
    end;

  separator_piece := tail;
  Result := b;
end;

function HyphenTerms.separatorPiece: String;
begin
  Result := piece[2];
end;

function HyphenTerms.separators: String;
const
  F = 2;
var
  b: Byte;
  i: Longint;
  r: String;
begin
  b := byteCode[F];

  if CompareStr(piece[F], UNALTERED) = 0 then
    begin
      if b = 0 then
        begin
          r := piece[F];
        end
      else
        begin
          r := EMPTY + Chr(b);
        end;

    end
  else
    begin
      r := piece[F];
    end;

  i := Ord(r[1]);
  i := i and H_000000FF;
  separator_byte := Byte(i);
  Result := r;
end;

function HyphenTerms.singleSurge: String;
const
  F = 1;
var
  difference: Integer;
  messages: String;
begin
  if (maximum[F] = NOT_CONVERTED) or (minimum[F] = NOT_CONVERTED) then
    begin
      interval_from := 1;
      interval_to := field_count;
    end
  else
    begin
      interval_from := minimum[F];
      interval_to := maximum[F];

      difference := interval_to - interval_from;
      if difference < 0 then
        begin
{ error }
          x := ErrorTexts.Create;
          messages := 'singleSurge error-412: ' + IntToStr(difference);
          x.setting(C_ERR, TEXT_FILE, messages);

          interval_from := 1;
          interval_to := field_count;

          x.Free;
        end;

    end;

  pattern := surge_of_set_bits(interval_from, interval_to);
  Result := pattern;
end;

function HyphenTerms.surge_of_set_bits(interval_0: Integer; interval_1: Integer): String;
var
  body: String;
  bodyLength: Integer;
  head: String;
  headLength: Integer;
  r: String;
  tail: String;
  tailLength: Integer;
begin
{*
  surge_of_set_bits
0011111000000000000, for example
Setting bits from interval_0 to interval_1
*}
  bodyLength := interval_1 - interval_0 + 1;

  if bodyLength > field_count then
    begin
      bodyLength := field_count;
      r := StringOfChar(SET_BIT_C, bodyLength);
    end
  else
    begin
      body := StringOfChar(SET_BIT_C, bodyLength);

      headLength := interval_0 - 1;
      head := StringOfChar(CLEAR_BIT_C, headLength);

      tailLength := field_count - interval_1;
      tail := StringOfChar(CLEAR_BIT_C, tailLength);

      r := Concat(head, body, tail);
    end;

  Result := r;
end;

function HyphenTerms.topLine: Integer;
begin
  Result := top_line_number;
end;

function HyphenTerms.tripletAfter: Integer;
begin
  if triplet_in_context[2] < 1 then
    begin
      pattern := singleSurge;
      if triplet_in_context[2] < 1 then
        begin
          nearPoint;
        end;

    end;

  Result := triplet_in_context[0];
end;

function HyphenTerms.tripletBefore: Integer;
begin
  if triplet_in_context[2] < 1 then
    begin
      pattern := singleSurge;
      if triplet_in_context[2] < 1 then
        begin
          nearPoint;
        end;

    end;

  Result := triplet_in_context[1];
end;

function HyphenTerms.tripletPoint: Integer;
begin
  if triplet_in_context[2] < 1 then
    begin
      pattern := singleSurge;
      if triplet_in_context[2] < 1 then
        begin
          nearPoint;
        end;

    end;

  Result := triplet_in_context[2];
end;

function HyphenTerms.unnecessaryCharacters: Integer;
const
  F = 4;
begin
  if figure[F] = NOT_CONVERTED then
    begin
      unnecessary_characters := 0;
    end
  else
    begin
      unnecessary_characters := figure[F];
    end;

  Result := unnecessary_characters;
end;

function HyphenTerms.unnecessaryWidth: Integer;
begin
  Result := unnecessary_characters;
end;

function HyphenTerms.versionNumber: Boolean;
begin
  Result := deniable[4];
end;

procedure HyphenTerms.acquireFromText;
var
  k: Integer;
  n: Integer;
  s: String;
  settingFile: TextFile;
begin

  for k := 1 to 64 do
  begin
    acquisition[k] := EMPTY;
  end;

  try
    Assign(settingFile, SETTING_TEXT);
    Reset(settingFile);

    if not Eof(settingFile) then
      begin
        ReadLn(settingFile, s);
        path_name := s;
      end;

    k := 1;
    p := CLEAR_PETTY_ARRAY;
    while not Eof(settingFile) do
    begin
      ReadLn(settingFile, s);
      s := Copy(s, 1, P_SIZE);
      StrPCopy(p, s);

      for n:= 0 to P_SIZE_1 do
      begin
        if p[n] = TAB_C then
          begin
            p[n] := #0; { Comments were removed. }
            break;
          end;

      end;

      acquisition[k] := EMPTY + p;
      Inc(k);
    end;

    CloseFile(settingFile);
  except
    s := 'Reset() error-413: ' + SETTING_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

end;

procedure HyphenTerms.presetting;
var
  i: Integer;
  s: String;
begin
  field_count := UPPER_LIMIT_TO_FIELD_NUMBER;
  GAP_C := Chr(H_20);
  NEW_LINE := EMPTY + Chr(H_0D);
  NEW_LINE := NEW_LINE + Chr(H_0A);

  s := ParamStr(1);
  if Length(s) = 0 then
    begin
      s := 'ParamStr() error-414: ParamStr(1) = EMPTY' + NEW_LINE;
      s := s + COMMAND_NAME;
      s := s + GAP_C;
      s := s + HELP_TERM;
      s := s + NEW_LINE;
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, s);
      x.Free;
      Halt(1);
    end;

  ASCII := ASCII_1 + Chr(H_27); { ' }
  BYTES := Concat(H_00TO42, H_43TO85, H_86TOC8, H_C9TOFF);
  CLEAR_BIT_C := Chr(CLEAR_BIT);
  EBB := StringOfChar('0', UPPER_LIMIT_TO_FIELD_NUMBER);
  FLOOD := StringOfChar('1', UPPER_LIMIT_TO_FIELD_NUMBER);
  P_SIZE := SizeOf(p) div SIZE_OF_CHAR;
  P_SIZE_1 := P_SIZE - 1;
  SET_BIT_C := Chr(SET_BIT);
  TAB_C := Chr(H_09);

  for i := 0 to P_SIZE_1 do
  begin
    CLEAR_PETTY_ARRAY[i] := #0;
  end;

end;

procedure HyphenTerms.setFieldCount(c: Integer);
begin
  field_count := c;
end;

procedure HyphenTerms.setting;
var
  body: String;
  c: String;
  firstFile: Boolean;
  head: String;
  headLength: Integer;
  hyphen_C: Boolean;
  hyphen_C_byte_code: Boolean;
  hyphen_C_figure: Boolean;
  hyphen_C_hyphen: Boolean;
  hyphen_C_interval: Boolean;
  hyphen_C_pattern: Boolean;
  hyphen_C_piece: Boolean;
  i: Integer;
  k: Integer;
  n: NumberedTerm;
  PARAMETER_COUNT: Integer;
  PARAMETER_COUNT_1: Integer;
  parameter_i: String;
  q: Integer;
  s: String;
  s_length: Integer;
  tail: String;
  tailLength: Integer;
  w: Integer;
begin
  bit_pattern_array[1] := FLOOD;
  bit_pattern_array[2] := EBB;
  byteCode[1] := H_20;
  byteCode[2] := H_20;
  deniable[1] := False;
  deniable[2] := False;
  deniable[3] := False;
  deniable[4] := False;
  deniable[5] := False;
  deniable[6] := False;
  deniable[7] := False;
  deniable[8] := False;
  deniable[9] := False;
  deniable[10] := False;
  deniable[11] := False;
  deniable[12] := False;
  figure[1] := NOT_CONVERTED;
  figure[2] := NOT_CONVERTED;
  figure[3] := NOT_CONVERTED;
  figure[4] := NOT_CONVERTED;
  figure[5] := NOT_CONVERTED;
  figure[6] := NOT_CONVERTED;
  maximum[1] := FINAL_INTEGER;
  maximum[2] := FINAL_INTEGER;
  minimum[1] := NOT_CONVERTED;
  minimum[2] := NOT_CONVERTED;
  piece[1] := UNALTERED;
  piece[2] := UNALTERED;

  list_name[1] := EMPTY;
  list_name[2] := EMPTY;
  firstFile := True;
  first_file_name := EMPTY;
  second_file_name := EMPTY;

  s := EMPTY;
  for i := 1 to ParamCount do
  begin
    s := s + ParamStr(i);
  end;

  PARAMETER_COUNT := EXPANSION_FACTOR * Length(s);
  PARAMETER_COUNT_1 := PARAMETER_COUNT - 1;
  SetLength(parametersList, PARAMETER_COUNT);

{ Three default parameters }
  parametersList[0] := '-w1';
  parametersList[1] := '-u1';
  parametersList[2] := '-L';
  k := 3;

  acquireFromText; { acquisition[] }
  for i := 1 to ACQUISITION_ITEM_COUNT do
  begin
    s := acquisition[i];
    if Length(s) > 0 then
      begin
        parametersList[k] := s;
        Inc(k);
      end
    else
      begin
        break;
      end;

  end;

  for i := 1 to ParamCount do
  begin
    s := ParamStr(i);
    s_length := Length(s);
    if s_length >= 3 then
      begin
        if CompareStr(s, CRLF) = 0 then
          begin
            parametersList[k] := LINE_FEED + DENIED;
            Inc(k);
            parametersList[k] := INCLUDE_0DH;
            Inc(k);
            continue;
          end;

        if CompareStr(s, HELP_TERM) = 0 then
          begin
            parametersList[k] := GUIDE_TERM;
            Inc(k);
            continue;
          end;

        if CompareStr(s, LF) = 0 then
          begin
            parametersList[k] := LINE_FEED;
            Inc(k);
            continue;
          end;

        if CompareStr(s, VERSION_WITH_COMMENTS) = 0 then
          begin
            parametersList[k] := GUIDE_TERM;
            Inc(k);
            continue;
          end;

        head := Copy(s, 1, 1);
        if CompareStr(head, HYPHEN) = 0 then
          begin
            tail := Copy(s, 3, 1);
            q := AnsiPos(tail, ASCII);
            if q >= 1 then
              begin
                if CompareStr(tail, HYPHEN) = 0 then
                  begin
                    parametersList[k] := s; { -L- }
                    Inc(k);
                    continue;
                  end
                else
                  begin
                    if CompareStr(tail, YEN) = 0 then
                      begin
                        parametersList[k] := s; { -S\x7C }
                        Inc(k);
                        continue;
                      end
                    else
                      begin
                        q := AnsiPos(tail, N_0TO9);
                        if q >= 1 then
                          begin
                            parametersList[k] := s;
                            Inc(k);
                            continue;
                          end
                        else
                          begin
                            if separatorPieces(s) then
                              begin
                                parametersList[k] := s;
                                Inc(k);
                                continue;
                              end
                            else
                              begin
                                if jointPieces(s) then
                                  begin
                                    parametersList[k] := s;
                                    Inc(k);
                                    continue;
                                  end
                                else
                                  begin
{*    
-bdLmrty 
-b -d -L -m -r -t y 
*}    
                                    for w := 2 to s_length do
                                    begin
                                      c := Copy(s, w, 1);
                                      parametersList[k] := HYPHEN + c;
                                      Inc(k);
                                    end;

                                  end;

                              end;

                          end;

                      end;

                  end;

              end
            else
              begin
                parametersList[k] := s; { -e }
                Inc(k);
                continue;
              end;

          end
        else
          begin
            parametersList[k] := s; { text.tmp }
            Inc(k);
            continue;
          end;

      end
    else
      begin
        parametersList[k] := s; { -f }
        Inc(k);
        continue;
      end;

  end;

  for k := 0 to PARAMETER_COUNT_1 do
  begin
    s := parametersList[k];
{*
  for i := 1 to ParamCount do
  begin
    s := ParamStr(i);
*}

    hyphen_C := False;
    hyphen_C_byte_code := False;
    hyphen_C_figure := False;
    hyphen_C_hyphen := False;
    hyphen_C_interval := False;
    hyphen_C_pattern := False;
    hyphen_C_piece := False;

    s_length := Length(s);
    if s_length < 1 then
      continue;

    head := Copy(s, 1, 1);
    if CompareStr(head, HYPHEN) = 0 then
      begin
{ -r }
        if CompareStr(s, TERM_1_FILE_NAME) = 0 then
          begin
            deniable[1] := True;
            continue;
          end
        else
          begin
            if CompareStr(s, TERM_2_FILE_NAME) = 0 then
              begin
                deniable[2] := True;
                continue;
              end
            else
              begin
                if CompareStr(s, GUIDE_TERM) = 0 then
                  begin
                    deniable[3] := True;
                    continue;
                  end
                else
                  begin
                    if CompareStr(s, VERSION_WITHOUT_COMMENTS) = 0 then
                      begin
                        deniable[4] := True;
                        continue;
                      end
                    else
                      begin
                        if CompareStr(s, DO_NOT_REMOVE_TEMPORARY_FILE) = 0 then
                          begin
                            deniable[5] := True;
                            continue;
                          end
                        else
                          begin
                            if CompareStr(s, PRODUCE_TEMPORARY_BINARY) = 0 then
                              begin
                                deniable[6] := True;
                                continue;
                              end
                            else
                              begin
                                if CompareStr(s, NON_SEPARATOR_ENDING) = 0 then
                                  begin
                                    deniable[7] := True;
                                    continue;
                                  end
                                else
                                  begin
                                    if CompareStr(s, INCLUDE_0DH) = 0 then
                                      begin
                                        deniable[8] := True;
                                        continue;
                                      end
                                    else
                                      begin
                                        if CompareStr(s, IDENTICAL_TIME) = 0 then
                                          begin
                                            deniable[9] := True;
                                            continue;
                                          end
                                        else
                                          begin
                                            if CompareStr(s, LINE_FEED) = 0 then
                                              begin
                                                deniable[10] := True;
                                                continue;
                                              end
                                            else
                                              begin
                                                if CompareStr(s, DISPLAY_DATA) = 0 then
                                                  begin
                                                    deniable[11] := True;
                                                    continue;
                                                  end
                                                else
                                                  begin
                                                    if CompareStr(s, FORCIBLE_ENTRY_OF_0DH) = 0 then
                                                      begin
                                                        deniable[12] := True;
                                                        continue;
                                                      end
                                                    else
                                                      begin
                                                        hyphen_C := True; { other parameters }
                                                      end;

                                                  end;

                                              end;

                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end;

              end;

          end; { if CompareStr(s, TERM_1_FILE_NAME) = 0 then, else }

      end;

    if hyphen_C then
      begin
{ -L- }
        if s_length >= 3 then
          begin
            tail := Copy(s, 3, 1);
            if CompareStr(tail, HYPHEN) = 0 then
              begin
                parameter_i := s; { setting will recover s. }
                s := Copy(s, 1, 2);
                if CompareStr(s, TERM_1_FILE_NAME) = 0 then
                  begin
                    deniable[1] := False;
                    list_name[1] := EMPTY;
                    continue;
                  end
                else
                  begin
                    if CompareStr(s, TERM_2_FILE_NAME) = 0 then
                      begin
                        deniable[2] := False;
                        list_name[2] := EMPTY;
                        continue;
                      end
                    else
                      begin
                        if CompareStr(s, GUIDE_TERM) = 0 then
                          begin
                            deniable[3] := False;
                            continue;
                          end
                        else
                          begin
                            if CompareStr(s, VERSION_WITHOUT_COMMENTS) = 0 then
                              begin
                                deniable[4] := False;
                                continue;
                              end
                            else
                              begin
                                if CompareStr(s, DO_NOT_REMOVE_TEMPORARY_FILE) = 0 then
                                  begin
                                    deniable[5] := False;
                                    continue;
                                  end
                                else
                                  begin
                                    if CompareStr(s, PRODUCE_TEMPORARY_BINARY) = 0 then
                                      begin
                                        deniable[6] := False;
                                        continue;
                                      end
                                    else
                                      begin
                                        if CompareStr(s, NON_SEPARATOR_ENDING) = 0 then
                                          begin
                                            deniable[7] := False;
                                            continue;
                                          end
                                        else
                                          begin
                                            if CompareStr(s, INCLUDE_0DH) = 0 then
                                              begin
                                                deniable[8] := False;
                                                continue;
                                              end
                                            else
                                              begin
                                                if CompareStr(s, IDENTICAL_TIME) = 0 then
                                                  begin
                                                    deniable[9] := False;
                                                    continue;
                                                  end
                                                else
                                                  begin
                                                    if CompareStr(s, LINE_FEED) = 0 then
                                                      begin
                                                        deniable[10] := False;
                                                        continue;
                                                      end
                                                    else
                                                      begin
                                                        if CompareStr(s, DISPLAY_DATA) = 0 then
                                                          begin
                                                            deniable[11] := False;
                                                            continue;
                                                          end
                                                        else
                                                          begin
                                                            if CompareStr(s, FORCIBLE_ENTRY_OF_0DH) = 0 then
                                                              begin
                                                                deniable[12] := False;
                                                                continue;
                                                              end
                                                            else
                                                              begin
                                                                hyphen_C_hyphen := True; { other parameters }
                                                              end;

                                                          end;

                                                      end;

                                                  end;

                                              end;

                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if CompareStr(s, TERM_1_FILE_NAME) = 0 then, else }

                s := parameter_i; { setting has recovered s. }
              end; { if CompareStr(tail, HYPHEN) = 0 then }

          end; { if s_length >= 3 then }

      end; { if hyphen_C then }

    if hyphen_C or hyphen_C_hyphen then
      begin
        if s_length >= 3 then
          begin
            tail := Copy(s, 3, 1);
            q := AnsiPos(tail, N_0TO9);
            if q >= 1 then
              begin
                n := NumberedTerm.Create;
                n.setting(s);
                if CompareStr(n.parameter, AFTER_POINT) = 0 then
                  begin
                    figure[1] := n.number;
                    continue;
                  end
                else
                  begin
                    if CompareStr(n.parameter, BEFORE_POINT) = 0 then
                      begin
                        figure[2] := n.number;
                        continue;
                      end
                    else
                      begin
                        if CompareStr(n.parameter, AT_POINT) = 0 then
                          begin
                            figure[3] := n.number;
                            continue;
                          end
                        else
                          begin
                            if CompareStr(n.parameter, UNNECESSARY_TERM) = 0 then
                              begin
                                figure[4] := n.number;
                                continue;
                              end
                            else
                              begin
                                if CompareStr(n.parameter, COUNT_0AH_BEFORE_TRIGGERING_LINE_FEED) = 0 then
                                  begin
                                    figure[5] := n.number;
                                    continue;
                                  end
                                else
                                  begin
                                    if CompareStr(n.parameter, DOUBLE_SPACED) = 0 then
                                      begin
                                        figure[6] := n.number;
                                        continue;
                                      end
                                    else
                                      begin
                                        hyphen_C_figure := True; { other parameters }
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end;

                n.Free;
              end;

          end;

      end;

    if hyphen_C or hyphen_C_figure or hyphen_C_hyphen then
      begin
{*
 -i2   
 -i2,3 
 -i2-3 
*}
        if s_length >= 3 then
          begin
            body := Copy(s, 3, 1);
            q := AnsiPos(body, N_0TO9);
            if q >= 1 then
              begin
                n := NumberedTerm.Create;
                n.setting(s, FINAL_1);
                if n.trimmedSequence then
                  begin
                    if CompareStr(n.parameter, INTERVAL_TERM) = 0 then
                      begin
                        maximum[1] := n.number;
                        minimum[1] := n.subnumber;
                        continue;
                      end
                    else
                      begin
                        if CompareStr(n.parameter, NEAR_POINT) = 0 then
                          begin
                            maximum[2] := n.number;
                            minimum[2] := n.subnumber;
                            continue;
                          end
                        else
                          begin
                            hyphen_C_interval := True; { other parameters }
                          end;

                      end;

                  end;

                n.Free;
              end;

          end;

      end;

    if hyphen_C or hyphen_C_figure or hyphen_C_hyphen or hyphen_C_interval then
      begin
{ -e10111 }
        if s_length >= 3 then
          begin
            body := Copy(s, 3, 1);
            q := AnsiPos(body, N_0TO1);
            if q >= 1 then
              begin
                headLength := 2;
                head := Copy(s, 1, headLength);
                tailLength := s_length - headLength;
                tail := Copy(s, 3, tailLength);

                if CompareStr(head, BITS) = 0 then
                  begin
                    bit_pattern_array[1] := tail;
                    continue;
                  end
                else
                  begin
                    if CompareStr(head, NEGATIVE_BIT_PATTERN) = 0 then
                      begin
                        bit_pattern_array[2] := tail;
                        continue;
                      end
                    else
                      begin
                        hyphen_C_pattern := True; { other parameters }
                      end;

                  end;

              end;

          end

      end;

    if hyphen_C or hyphen_C_figure or hyphen_C_hyphen or hyphen_C_interval or hyphen_C_pattern then
      begin
        head := Copy(s, 1, JOINT_TERM_SIZE);
        if CompareStr(head, JOINT_TERM) = 0 then
          begin
{*
 -S\l   
 -S\x7C 
*}
            byteCode[1] := punctuation(s, JOINT_TERM_SIZE);
            if byteCode[1] = 0 then
              begin
                hyphen_C_byte_code := True; { other parameters }
              end
            else
              begin
                continue;
              end;

          end
        else
          begin
            head := Copy(s, 1, SEPARATOR_TERM_SIZE);
            if CompareStr(head, SEPARATOR_TERM) = 0 then
              begin
                byteCode[2] := punctuation(s, SEPARATOR_TERM_SIZE);
                if byteCode[2] = 0 then
                  begin
                    hyphen_C_byte_code := True; { other parameters }
                  end
                else
                  begin
                    continue;
                  end;

              end
            else
              begin
                hyphen_C_byte_code := True; { other parameters }
              end;

          end;

      end;

    if hyphen_C or hyphen_C_byte_code or hyphen_C_figure or hyphen_C_hyphen or hyphen_C_interval or hyphen_C_pattern then
      begin
        if s_length >= 3 then
          begin
            headLength := 2;
            head := Copy(s, 1, headLength);
            tailLength := s_length - headLength;
            tail := Copy(s, 3, tailLength);

            if CompareStr(head, JOINT_PIECE_TERM) = 0 then
              begin
{ -j_ }
                piece[1] := tail;
                byteCode[1] := 0; { Regardless of -J, software recognizes that -j has priority. }
                continue;
              end
            else
              begin
                if CompareStr(head, SEPARATOR_PIECE_TERM) = 0 then
                  begin
{ -s, }
                    piece[2] := tail;
                    byteCode[2] := 0; { Regardless of -S, software recognizes that -s has priority. }
                    continue;
                  end
                else
                  begin
                    hyphen_C_piece := True; { other parameters }
                  end;

              end;

          end;

      end;

    if hyphen_C or hyphen_C_byte_code or hyphen_C_figure or hyphen_C_hyphen or hyphen_C_interval or hyphen_C_pattern or hyphen_C_piece then
      begin
        continue; { setting omitted s from parameters. }
      end;

    if deniable[1] then
      begin
{ TERM_1_FILE_NAME }
        list_name[1] := s;
        deniable[1] := False;
      end
    else
      begin
        if deniable[2] then
          begin
{ TERM_2_FILE_NAME }
            list_name[2] := s;
            deniable[2] := False;
          end
        else
          begin
            if firstFile then
              begin
                first_file_name := s;
                firstFile := False;
              end
            else
              begin
                second_file_name := s;
              end;

          end;

      end;

  end;

end;

end.
