unit LineEnd;

{$I+}

interface

uses
  Windows, SysUtils, ErrorText, FinalEffect, IntegerQueue;

type AnsiCharArray = array[0..127] of AnsiChar;
type BigAnsiCharArray = array[0..4095] of AnsiChar;
type ByteArray = array[0..223] of Byte;

type
  RemoveFromLine = class
  private
    ANSI_CHAR: array[0..255] of AnsiChar;
    beginning: Integer;
    buffer: array of AnsiChar;
    buffer_size: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_Y_ARRAY: BigAnsiCharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    count: Integer;
    EMPTY_ANSI: AnsiString;
    file_name: String;
    full_level: Integer;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_FE_C: AnsiChar;
    H_FF_C: AnsiChar;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    new_name: String;
    r: RingBuffer;
    ring_count: Integer;
    unnecessary_characters: Integer;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    x: ErrorTexts;
    y: BigAnsiCharArray;
    Y_COUNT: Integer;
    Y_COUNT_1: Integer;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function ANSI_CHAR_table: AnsiChar;
    function bufferingWriting(comboLength: Integer): Integer;
    function displayDataText: Boolean;
    function eight_bit_character(b: Byte): AnsiChar;
    function lineCount: Integer;
    function newLine: AnsiString;
    function produceBinaryFile(name: String; no_removing: Boolean): Integer;
    function reading_to_produce_MBCS_CRLF(nn: String): Integer;
    function reading_to_produce_MBCS_LF(nn: String): Integer;
    function reading_to_produce_MBCS_text(nn: String): Integer;
    function renameBinaryFile(name: String; no_removing: Boolean): Integer;
    procedure ifErrorOccurs(exit_code: Integer);
    procedure setting(f: FinalEffects; fn: String);
  end;

const
  BELL = 7;
  BOM_SIZE = 3; { EFh, BBh, BFh }
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  COMBO_ERROR = -3;
  COMMA_BYTE = 44; { 2C, ',' }
  DATA_TEXT = 'data.txt';
  EMPTY = '';
  FILE_ERROR = -2;
  H_000000FF = 255;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_3E = 62; { '>' }
  H_5F = 95; { '_' }
  H_7E = 126; { '~' }
  H_BB = 187;
  H_BF = 191;
  H_EF = 239;
  H_FE = 254;
  H_FF = 255;
  KANJI_SIZE = 2;
  LF_COUNT = 1;
  SIZE_OF_ANSICHAR = 1;
  TEMPORARY_BINARY = 'binary.tmp';
  TEXT_FILE = '.txt';

implementation

destructor RemoveFromLine.Destroy;
begin
  buffer := nil;
  combo := nil;

  inherited Destroy;
end;

function RemoveFromLine.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function RemoveFromLine.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
  s: String;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;

      for k := 0 to V_SIZE_1 do
      begin
        v[k] := combo[n];
        Inc(n);
        if n >= comboLength then
          begin
            V_SIZE := k + 1;
            V_SIZE_1 := V_SIZE - 1;
            break;
          end;

      end;

      BlockWrite(newFile, v, V_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    V_SIZE := SizeOf(v);
    V_SIZE_1 := V_SIZE - 1;
  except
    s := 'Rewrite() error-416: ' + new_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function RemoveFromLine.displayDataText: Boolean;
var
  b: Boolean;
  i: Longint;
  k: Longint;
  s: String;
  testFile: TextFile;
begin
  b := True;

  try
    AssignFile(testFile, DATA_TEXT);
    Rewrite(testFile);

{ Displaying lines of the file }
  if count = 1 then
    begin
      WriteLn( file_name + ': 1 line');
    end
  else
    begin
      WriteLn( file_name + ': ' + IntToStr(count) + ' lines' );
    end;


{ Trying to represent the end of the first line in hexadecimal as 0D 0A }
    if NEW_LINE_SIZE = 2 then
      begin
        i := Ord(NEW_LINE[1]);
        k := Ord(NEW_LINE[2]);
        if (i = H_0D) and (k = H_0A) then
          begin
            WriteLn('0D 0A ');
          end
        else
          begin
            WriteLn(NEW_LINE);
          end;

      end
    else
      begin
        i := Ord(NEW_LINE[1]);
        if i = H_0A then
          begin
            WriteLn('0A ');
          end
        else
          begin
            WriteLn(NEW_LINE);
          end;

      end;

    CloseFile(testFile);
  except
    s := 'Rewrite() error-417: ' + DATA_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    b := False;
  end;

  try
    AssignFile(testFile, DATA_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      WriteLn(s);
    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-418: ' + DATA_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    b := False;
  end;

  Result := b;
end;

function RemoveFromLine.eight_bit_character(b: Byte): AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
begin
  beginning_of_ASCII := 0;

  if b >= 1 then
    begin
      end_1 := b - 1;
      h := AnsiChar( Chr(beginning_of_ASCII) );

      for k := beginning_of_ASCII to end_1 do
      begin
        Inc(h);
      end;

    end
  else
    begin
      h := AnsiChar(#0);
    end;

  Result := h;
end;

function RemoveFromLine.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function RemoveFromLine.lineCount: Integer;
begin
  Result := count;
end;

function RemoveFromLine.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function RemoveFromLine.produceBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  s: String;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, TEMPORARY_BINARY);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;
      BlockRead(testFile, v, V_SIZE, block_has_been_read);
      BlockWrite(newFile, v, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, name);
        Erase(testFile);
      end;

  except
    s := 'Reset() error-419: ' + name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function RemoveFromLine.reading_to_produce_MBCS_CRLF(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  full: Boolean;
  level: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  full := False;
  level := 0;
  new_name := nn;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-420: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  ring_count := unnecessary_characters + LF_COUNT;

  r := RingBuffer.Create;
  r.setting(BELL_BYTE, ring_count);

{ Finding the first NEW_LINE }
  m := 0;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);
    if buffer[n] = H_0A_C then
      begin
        count := 1;
        level := 1;
        if level >= full_level then
          begin
            full := True;
          end;

      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end;

      end; { if buffer[n] = H_0A_C then, else }

    r.pop_front;
    r.push_back(w);

    if full then
      begin
        combo[m] := H_0D_C;
        Inc(m);

        combo[m] := H_0A_C;
        Inc(m);

        r.produce_ring_buffer(ring_count);
        level := 0;
        full := False;
      end
    else
      begin
        w := r.peek;
        if w = BELL_BYTE then
          begin
            continue;
          end
        else
          begin
            combo[m] := ANSI_CHAR[w];
            Inc(m);
          end;

      end;

    if buffer[n] = H_0A_C then
      begin
        break;
      end

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  for n := beginning to buffer_size_1 do
  begin
    w := Ord(buffer[n]);

    if buffer[n] = H_0A_C then
      begin
        Inc(count);
        Inc(level);
        if level >= full_level then
          begin
            full := True;
          end;

      end;

    r.pop_front;
    r.push_back(w);

    if full then
      begin
        combo[m] := H_0D_C;
        Inc(m);

        combo[m] := H_0A_C;
        Inc(m);

        r.produce_ring_buffer(ring_count);
        level := 0;
        full := False;
      end
    else
      begin
        w := r.peek;

        if w = BELL_BYTE then
          begin
            continue;
          end
        else
          begin
            combo[m] := ANSI_CHAR[w];
            Inc(m);
          end;

      end;

  end; { for n := beginning to buffer_size_1 do }

  r.Free;
  Result := bufferingWriting(m);
end;

function RemoveFromLine.reading_to_produce_MBCS_LF(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  full: Boolean;
  level: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  full := False;
  level := 0;
  new_name := nn;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-421: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  ring_count := unnecessary_characters + LF_COUNT;

  r := RingBuffer.Create;
  r.setting(BELL_BYTE, ring_count);

{ Finding the first NEW_LINE }
  m := 0;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);
    if buffer[n] = H_0A_C then
      begin
        count := 1;
        level := 1;
        if level >= full_level then
          begin
            full := True;
          end;

      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end;

      end; { if buffer[n] = H_0A_C then, else }

    r.pop_front;
    r.push_back(w);

    if full then
      begin
        combo[m] := H_0A_C;
        Inc(m);

        r.produce_ring_buffer(ring_count);
        level := 0;
        full := False;
      end
    else
      begin
        w := r.peek;
        if w = BELL_BYTE then
          begin
            continue;
          end
        else
          begin
            if w = H_0D then
              begin
                continue;
              end
            else
              begin
                combo[m] := ANSI_CHAR[w];
                Inc(m);
              end;

          end;

      end;

    if buffer[n] = H_0A_C then
      begin
        break;
      end

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  for n := beginning to buffer_size_1 do
  begin
    w := Ord(buffer[n]);

    if buffer[n] = H_0A_C then
      begin
        Inc(count);
        Inc(level);
        if level >= full_level then
          begin
            full := True;
          end;

      end;

    r.pop_front;
    r.push_back(w);

    if full then
      begin
        combo[m] := H_0A_C;
        Inc(m);

        r.produce_ring_buffer(ring_count);
        level := 0;
        full := False;
      end
    else
      begin
        w := r.peek;

        if w = BELL_BYTE then
          begin
            continue;
          end
        else
          begin
            if w = H_0D then
              begin
                continue;
              end
            else
              begin
                combo[m] := ANSI_CHAR[w];
                Inc(m);
              end;

          end;

      end;

  end; { for n := beginning to buffer_size_1 do }

  r.Free;
  Result := bufferingWriting(m);
end;

function RemoveFromLine.reading_to_produce_MBCS_text(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  full: Boolean;
  level: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  full := False;
  level := 0;
  new_name := nn;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-422: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  ring_count := unnecessary_characters + LF_COUNT;

  r := RingBuffer.Create;
  r.setting(BELL_BYTE, ring_count);

{ Finding the first NEW_LINE }
  m := 0;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);
    if buffer[n] = H_0A_C then
      begin
        count := 1;
        level := 1;
        if level >= full_level then
          begin
            full := True;
          end;

      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end;

      end; { if buffer[n] = H_0A_C then, else }

    r.pop_front;
    r.push_back(w);

    if full then
      begin
        combo[m] := H_0A_C;
        Inc(m);

        r.produce_ring_buffer(ring_count);
        level := 0;
        full := False;
      end
    else
      begin
        w := r.peek;
        if w = BELL_BYTE then
          begin
            continue;
          end
        else
          begin
            combo[m] := ANSI_CHAR[w];
            Inc(m);
          end;

      end;

    if buffer[n] = H_0A_C then
      begin
        break;
      end

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  for n := beginning to buffer_size_1 do
  begin
    w := Ord(buffer[n]);

    if buffer[n] = H_0A_C then
      begin
        Inc(count);
        Inc(level);
        if level >= full_level then
          begin
            full := True;
          end;

      end;

    r.pop_front;
    r.push_back(w);

    if full then
      begin
        combo[m] := H_0A_C;
        Inc(m);

        r.produce_ring_buffer(ring_count);
        level := 0;
        full := False;
      end
    else
      begin
        w := r.peek;

        if w = BELL_BYTE then
          begin
            continue;
          end
        else
          begin
            combo[m] := ANSI_CHAR[w];
            Inc(m);
          end;

      end;

  end; { for n := beginning to buffer_size_1 do }

  r.Free;
  Result := bufferingWriting(m);
end;

function RemoveFromLine.renameBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  s: String;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, TEMPORARY_BINARY);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;
      BlockRead(testFile, v, V_SIZE, block_has_been_read);
      BlockWrite(newFile, v, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, TEMPORARY_BINARY);
        Erase(testFile);
      end;

  except
    s := 'Reset() error-423: ' + name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function RemoveFromLine.setBufferSize: Integer;
var
  u: Integer;
  s: String;
begin
  u := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    s := 'SetLength() error-424: ' + IntToStr(buffer_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    u := BUFFER_ERROR;
  end;

  try
    SetLength(combo, combo_size);
  except
    s := 'SetLength() error-425: ' + IntToStr(combo_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    u := COMBO_ERROR;
  end;

  Result := u;
end;

procedure RemoveFromLine.ifErrorOccurs(exit_code: Integer);
begin
  if buffer_size < 0 then
    Halt(exit_code);

end;

procedure RemoveFromLine.setting(f: FinalEffects; fn: String);
var
  block_has_been_read: Integer;
  i: Integer;
  m: Integer;
  q: array[0..2] of Byte;
  s: String;
  testFile: File;
begin
  buffer_size := -1;
  file_name := fn;
  full_level := f.fullLevel;
  new_name := f.newName;
  unnecessary_characters := f.unnecessaryCharacters;

  count := -1;
  EMPTY_ANSI := '';

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y);
  Y_COUNT_1 := Y_COUNT - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, v, V_SIZE, block_has_been_read);
    q[0] := Ord(v[0]);
    q[1] := Ord(v[1]);
    q[2] := Ord(v[2]);
    if (q[0] = H_EF) and (q[1] = H_BB) and (q[2] = H_BF) then
      beginning := BOM_SIZE
    else
      beginning := 0;

    CloseFile(testFile);

    buffer_size := file_size(file_name);
    combo_size := buffer_size;
  except
    s := 'Reset() error-426: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];
  H_FE_C := ANSI_CHAR[H_FE];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0]. }
  EMPTY_ANSI := '';

  NEW_LINE := EMPTY_ANSI + H_0A_C;
  NEW_LINE_SIZE := 1;

  setBufferSize;
end;

end.
