program Rufeol;
{$APPTYPE CONSOLE}

{*
Rufeol Version 1.02
Removing unnecessary characters from every end of line
Copyright 2020 Sogaya

Delphi 2009

*}

uses
  SysUtils,
  ErrorText in 'ErrorText.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  IntegerQueue in 'IntegerQueue.pas',
  LineEnd in 'LineEnd.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  TimeName in 'TimeName.pas';

{$R *.res}

const
  BUFFER_ERROR = -1;
  COMMAND_NAME_FILE = 'draft.txt';
  DIFFERENT_TIME = False;
  DO_NOT_DISPLAY_DATA = False;
  DO_NOT_PRODUCE_ANOTHER_TEMPORARY_FILE = False;
  DO_NOT_REMOVE_ANCIENT_FILE = True;
  DO_NOT_REMOVE_0DH = False;
  DRAFT = False;
  EMPTY = '';
  EVERY_LINE = 1; { -w1 }
  INCLUDE_0DH = True; { -r }
  TWO_USELESS_SEPARATORS = 2; { -u2 }
  PATH_NAME_TEXT = 'unarfi.txt';
  REMOVE_ANCIENT_FILE = False;
  REMOVE_TEMPORARY_FILES = False;
  TEMPORARY_BINARY = 'binary.tmp';
  VERSION_NUMBER = '1.02';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;

var
  carriage_return: Boolean;
  e: RemoveFromLine;
  f: FinalEffects;
  file_name_length: Integer;
  full_level: Integer;
  g: Guiding;
  h: HyphenTerms;
  new_name: String;
  new_name_length: Integer;
  remove_0Dh: Boolean;
  s: String;
  temporaryFile: File;
  unnecessary_characters: Integer;
begin
  s := ParamStr(1);
  if Length(s) = 0 then
    begin
      f := FinalEffects.Create;
      f.setting(EMPTY, COMMAND_NAME_FILE, PATH_NAME_TEXT,
        EVERY_LINE, TWO_USELESS_SEPARATORS,
        INCLUDE_0DH, DO_NOT_DISPLAY_DATA, DRAFT, DIFFERENT_TIME, REMOVE_TEMPORARY_FILES, DO_NOT_PRODUCE_ANOTHER_TEMPORARY_FILE, DO_NOT_REMOVE_0DH);

{ Producing TEMPORARY_BINARY from PATH_NAME_TEXT }
      e := RemoveFromLine.Create;
      e.setting(f, PATH_NAME_TEXT);
      e.produceBinaryFile(PATH_NAME_TEXT, DO_NOT_REMOVE_ANCIENT_FILE);

{ Removing unnecessary characters from every end of line }
      e.reading_to_produce_MBCS_CRLF(COMMAND_NAME_FILE);

      AssignFile(temporaryFile, TEMPORARY_BINARY);
      Erase(temporaryFile);

      e.Free;
      Halt(0);
    end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          full_level := h.doubleSpaced; { formal, full_level, carriage_return, unnecessary_characters }
          if h.doubleSpacedFormal then
            begin
              carriage_return := h.include0Dh;
              remove_0Dh := h.lineFeed;
              unnecessary_characters := h.unnecessaryWidth;
            end
          else
            begin
              if h.nonSeparatorEnding then
                begin
                  carriage_return := True;
                  remove_0Dh := False;
                  full_level := 1;
                  unnecessary_characters := 2;
                end
              else
                begin
                  if h.forcibleEntry then
                    begin
                      carriage_return := True;
                      remove_0Dh := False;
                    end
                  else
                    begin
                      remove_0Dh := h.lineFeed;
                      if h.carriageReturn then
                        begin
                          if remove_0Dh then
                            begin
                              carriage_return := False;
                            end
                          else
                            begin
                              carriage_return := True;
                            end;

                        end
                      else
                        begin
                          carriage_return := False;
                        end;

                    end;

                  full_level := h.count0Ah; { COUNT_0AH_BEFORE_TRIGGERING_LINE_FEED }
                  unnecessary_characters := h.unnecessaryCharacters;
                end;

            end;

          new_name := h.listName(1);
          new_name_length := Length(new_name);
          if new_name_length = 0 then
            begin
              new_name := h.newName;
            end;

          f := FinalEffects.Create;
          f.setting(h.fileName, new_name, h.pathName,
            full_level, unnecessary_characters, 
            carriage_return, h.displayData, h.doubleSpacedFormal, h.identicalTime, h.noRemoving, h.produceTemporaryBinary, remove_0Dh);

{ main function }
  file_name_length := Length(f.fileName);
  if file_name_length = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end;

  new_name_length := Length(f.newName);
  if new_name_length = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end;

  e := RemoveFromLine.Create;
  e.setting(f, f.fileName);
  e.ifErrorOccurs(BUFFER_ERROR);

  if f.identicalTime then
    begin
      e.produceBinaryFile(f.fileName, REMOVE_ANCIENT_FILE);
      e.renameBinaryFile(f.fileName, f.noRemoving);
    end;

  if f.carriageReturn then
    begin
      if f.produceTemporaryBinary then
        begin
          e.reading_to_produce_MBCS_CRLF(TEMPORARY_BINARY);
          e.renameBinaryFile(f.newName, f.noRemoving);
        end
      else
        begin
          e.reading_to_produce_MBCS_CRLF(f.newName);
        end;

    end
  else
    begin
      if f.remove0Dh then
        begin
          if f.produceTemporaryBinary then
            begin
              e.reading_to_produce_MBCS_LF(TEMPORARY_BINARY);
              e.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              e.reading_to_produce_MBCS_LF(f.newName);
            end;

        end
      else
        begin
          if f.produceTemporaryBinary then
            begin
              e.reading_to_produce_MBCS_text(TEMPORARY_BINARY);
              e.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              e.reading_to_produce_MBCS_text(f.newName);
            end;

        end;

    end;

  if f.displayData then
    begin
      e.displayDataText;
    end;

  e.Free;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
