program Sjiseuc;
{$APPTYPE CONSOLE}

{*
Sjiseuc Delphi Version 1.04
Converting the text from Shift_JIS to EUC-JP
Copyright 2020 Sogaya

Delphi 2009

*}

uses
  SysUtils,
  AnsiQueue in 'AnsiQueue.pas',
  Combo in 'Combo.pas',
  Convert in 'Convert.pas',
  ErrorText in 'ErrorText.pas',
  FieldsArrayMany in 'FieldsArrayMany.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  PathEXE in 'PathEXE.pas',
  SearchBinary in 'SearchBinary.pas',
  SparseEUC in 'SparseEUC.pas',
  StandardIO in 'StandardIO.pas',
  TimeName in 'TimeName.pas',
  DivideSJIS in 'DivideSJIS.pas';

{$R *.res}

const
  BUFFER_ERROR = -1;
  DATA_TEXT = 'data.txt';
  DO_NOT_RECORD_PATH_NAME = False;
  PATH_NAME = 'C:\Narfi\MS932~EUC-JP.bin';
  REMOVE_ANCIENT_FILE = False;
  SEARCH_TABLE_IN_EACH_DIRECTORY = True;
  SELECTED_DIRECTORY_DOES_NOT_HAVE_PRIORITY = False;
  TEMPORARY_BINARY = 'binary.tmp';
  TEMPORARY_TEXT = 'text.tmp';
  VERSION_NUMBER = '1.04';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;

var
  c: ConvertRegulate;
  carriage_return: Boolean;
  dataFile: TextFile;
  e: SearchTable;
  f: FinalEffects;
  file_name_length: Integer;
  g: Guiding;
  h: HyphenTerms;
  i: StandardInput;
  new_name: String;
  new_name_length: Integer;
  s: String;
  temporaryFile: TextFile;

begin
  s := ParamStr(1);
  if Length(s) = 0 then
    begin
{ Producing TEMPORARY_TEXT from standard input }
      i := StandardInput.Create;
      i.produceTemporaryTextFile;

{ Converting TEMPORARY_TEXT from UTF-16 to Shift_JIS }
      e := SearchTable.Create;
      e.setting(PATH_NAME, SEARCH_TABLE_IN_EACH_DIRECTORY, SELECTED_DIRECTORY_DOES_NOT_HAVE_PRIORITY, DO_NOT_RECORD_PATH_NAME);
      i.setting(e);
      i.standard_input;

      AssignFile(temporaryFile, TEMPORARY_TEXT);
      Erase(temporaryFile);
      e.Free; i.Free;
      Halt(0);
    end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          new_name := h.listName(1);
          new_name_length := Length(new_name);
          if new_name_length = 0 then
            begin
              new_name := h.newName;
            end;

          if h.remove0Dh then
            begin
              if h.carriageReturn then
                begin
                  carriage_return := True;
                end
              else
                begin
                  carriage_return := False;
                end;

            end
          else
            begin
              carriage_return := True;
            end;

          f := FinalEffects.Create;
          f.setting(h.fileName, new_name, h.pathName, 
            h.atPoint, 
            carriage_return, h.identicalTime, h.noRemoving, h.noSearching, h.remove0Dh, h.removeControllers, h.priority, h.produceTemporaryBinary, h.whichTable);

{ main function }
  if f.whichTable then
    begin
      try
        Assignfile(dataFile, DATA_TEXT);
        Reset(dataFile);
        CloseFile(dataFile);

        Assignfile(dataFile, DATA_TEXT);
        Erase(dataFile);
      except
        ;
      end;

    end;

  file_name_length := Length(f.fileName);
  if file_name_length = 0 then
    begin
{ Producing TEMPORARY_TEXT from standard input }
      i := StandardInput.Create;
      i.produceTemporaryTextFile;

      i.setting(f);

      if f.whichTable then
        i.displayDataText;

      if f.removeControllers then
        begin
          i.presetting;
          i.standard_input_of_non_controllers;
        end
      else
        begin
          i.standard_input;
        end;

      i.Free; f.Free; h.Free;
      Halt(0);
    end;

  c := ConvertRegulate.Create;
  c.setting(f, f.fileName);
  c.ifErrorOccurs(BUFFER_ERROR);

  if f.whichTable then
    c.displayDataText;

  if f.identicalTime then
    begin
      c.produceBinaryFile(f.fileName, REMOVE_ANCIENT_FILE);
      c.renameBinaryFile(f.fileName, f.noRemoving);
    end;

  if f.carriageReturn then
    begin
      if f.produceTemporaryBinary then
        begin
          if f.removeControllers then
            begin
              c.presetting;
              c.skipping_control_to_produce_EUC_CRLF(TEMPORARY_BINARY);
            end
          else
            begin
              c.reading_to_produce_EUC_CRLF(TEMPORARY_BINARY);
            end;

          c.renameBinaryFile(f.newName, f.noRemoving);
        end
      else
        begin
          if f.removeControllers then
            begin
              c.presetting;
              c.skipping_control_to_produce_EUC_CRLF(f.newName);
            end
          else
            begin
              c.reading_to_produce_EUC_CRLF(f.newName);
            end;

        end;

    end
  else
    begin
      if f.remove0Dh then
        begin
          if f.produceTemporaryBinary then
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_EUC_LF(TEMPORARY_BINARY);
                end
              else
                begin
                  c.reading_to_produce_EUC_LF(TEMPORARY_BINARY);
                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_EUC_LF(f.newName);
                end
              else
                begin
                  c.reading_to_produce_EUC_LF(f.newName);
                end;

            end;

        end
      else
        begin
          if f.produceTemporaryBinary then
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_EUC_text(TEMPORARY_BINARY);
                end
              else
                begin
                  c.reading_to_produce_EUC_text(TEMPORARY_BINARY);
                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_EUC_text(f.newName);
                end
              else
                begin
                  c.reading_to_produce_EUC_text(f.newName);
                end;

            end;

        end;

    end;

  c.Free;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
