unit AnsiQueue;

{$I+}

interface

uses
  SysUtils, ErrorText;

type
  AnsiStringQueue = class
  protected
    ANSI_ASCII: AnsiString;
    BLANK: AnsiChar;
    dough: array[0..98303] of AnsiChar;
    EMPTY_ANSI: AnsiString;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    occupied: array[0..3071] of Boolean;
    offset: Integer;
    queue: array[0..3071] of AnsiString; { queue_size <= 3072 - 1 }
    queue_begin: Integer;
    queue_end: Integer;
    x: ErrorTexts;
  public
    constructor Create;
    destructor Destroy; override;
    function copyingAnsiStringToArray(s: PAnsiChar; s_length: Integer): Integer;
    function count: Integer;
    function dataText: AnsiString;
    function emptyRing: Boolean;
    function following(index: Integer): Integer;
    function pop_front: Integer;
    function produce_ring_buffer(size: Integer): Integer;
    function push_back(p: AnsiString): Integer;
    function q(index: Integer): AnsiString;
    function queue_size: Integer;
    procedure clearRing;
    procedure setting(ans: AnsiString; sequence: AnsiString; size: Integer);
  end;

const
  C_ERR = 'err';
  DATA_TEXT = 'data.txt';
  EMPTY_ERROR = 2;
  FULL_HOUSE_ERROR = 3;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_20 = 32; { SP }
  QUEUE_ARRAY_SIZE = 3071;
  QUEUE_ERROR_COUNT = 5;
  QUEUE_GROWN = 0;
  QUEUE_SHRUNK = 1;
  SET_LENGTH_ERROR = 4;
  SIZE_OF_ANSICHAR = 1;
  TEXT_FILE = '.txt';

implementation

constructor AnsiStringQueue.Create;
begin
  inherited Create;

end;

destructor AnsiStringQueue.Destroy;
begin

  inherited Destroy;
end;

function AnsiStringQueue.copyingAnsiStringToArray(s: PAnsiChar; s_length: Integer): Integer;
var
  k: Integer;
  m: Integer;
  s_length_1: Integer;
begin
  m := 0;
  s_length_1 := s_length - 1;

  for k := 0 to s_length_1 do
  begin
    dough[m] := s[k];
    Inc(m);
  end;

  Result := m;
end;

function AnsiStringQueue.count: Integer;
var
  r: Integer;
  s: String;
begin
  r := queue_size - offset;
  if r < 0 then
    begin
      s := 'queue_size - offset error-488: count < 0';
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, s);
      x.Free;
      Halt(1);
    end;

  Result := r;
end;

function AnsiStringQueue.dataText: AnsiString;
var
  ansiFile: File;
  block_has_been_written: Integer;
  combo: AnsiString;
  comboLength: Integer;
  index: Integer;
  k: Integer;
  size: Integer;
begin
  combo := EMPTY_ANSI;
  size := queue_size;
  index := queue_begin;

  for k := 1 to size do
  begin
    combo := combo + queue[index];
    combo := combo + NEW_LINE;
    index := following(index);
  end;

  AssignFile(ansiFile, DATA_TEXT);
  Rewrite(ansiFile, SIZE_OF_ANSICHAR);

  comboLength := Length(combo);
  copyingAnsiStringToArray( PAnsiChar(combo), comboLength);
  BlockWrite(ansiFile, dough, comboLength, block_has_been_written);

  CloseFile(ansiFile);
  Result := combo;
end;

function AnsiStringQueue.emptyRing: Boolean;
var
  b: Boolean;
begin
  b := False;
  if queue_begin = queue_end then
    b := True;

  Result := b;
end;

function AnsiStringQueue.following(index: Integer): Integer;
var
  k: Integer;
begin
  k := (index + 1) mod QUEUE_ARRAY_SIZE; 
  Result := k; 
end;

function AnsiStringQueue.pop_front: Integer;
var
  r: Integer;
begin
  if emptyRing then
    begin
      r := EMPTY_ERROR;
    end
  else
    begin
      queue_begin := following(queue_begin);
      r := QUEUE_SHRUNK;
    end;

  Result := r;
end;

function AnsiStringQueue.produce_ring_buffer(size: Integer): Integer;
var
  k: Integer;
  r: Integer;
begin
  clearRing;

  for k := 1 to size do
  begin
    push_back(BLANK);
  end;

  r := queue_size;
  Result := r;
end;

function AnsiStringQueue.push_back(p: AnsiString): Integer;
var
  index: Integer;
  r: Integer;
begin
  index := following(queue_end);
  if index = queue_begin then
    begin
      r := FULL_HOUSE_ERROR;
    end
  else
    begin
      queue[queue_end] := p;
      queue_end := index;
      r := QUEUE_GROWN;
      occupied[queue_end] := True;
    end;

  Result := r;
end;

function AnsiStringQueue.q(index: Integer): AnsiString;
var
  k: Integer;
  r: AnsiString;
  s: String;
begin
  r := EMPTY_ANSI;

  k := ( queue_begin + offset + index ) mod QUEUE_ARRAY_SIZE;
  if occupied[k] then
    begin
      r := queue[k];
    end
  else
    begin
      s := Format('q() error-489: occupied[ %d ] = False', [k]);
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, s);
      x.Free;
      Halt(1);
    end;

  Result := r;
end;

function AnsiStringQueue.queue_size: Integer;
var
  size: Integer;
begin
  if queue_begin < queue_end then
    begin
      size := queue_end - queue_begin;
    end
  else
    begin
      if queue_begin = queue_end then
        begin
          size := 0;
        end
      else
        begin
          size := queue_end + QUEUE_ARRAY_SIZE - queue_begin;
        end;

    end;

  Result := size;
end;

procedure AnsiStringQueue.clearRing;
var
  k: Integer;
  repetition: Integer;
begin
  repetition := queue_size;

  for k := 1 to repetition do
  begin
    pop_front;
  end;

end;

procedure AnsiStringQueue.setting(ans: AnsiString; sequence: AnsiString; size: Integer);
var
  k: Integer;
begin
  ANSI_ASCII := ans;

  BLANK := ANSI_ASCII[H_20];
  EMPTY_ANSI := '';
  NEW_LINE := EMPTY_ANSI + ANSI_ASCII[H_0D];
  NEW_LINE := NEW_LINE + ANSI_ASCII[H_0A];
  NEW_LINE_SIZE := 2;

  for k := 1 to size do
  begin
    push_back(sequence);
  end;

  offset := size;
end;

end.
