unit Guide;

interface

uses
  SysUtils;

type
  Guiding = class
  public
    procedure versionGuide(v: String; comments: Boolean);
  end;

const
  VERSION_TEXT = 'version.txt';

implementation

procedure Guiding.versionGuide(v: String; comments: Boolean);
var
  s: String;
  versionFile: TextFile;
begin
  AssignFile(versionFile, VERSION_TEXT);
  Rewrite(versionFile);
  WriteLn(versionFile, 'Sortk Delphi Version ' + v );
  WriteLn(versionFile, '');
  if comments then
    begin
      WriteLn(versionFile, 'sortk -G                     # Guide ');
      WriteLn(versionFile, 'sortk -m                     # version ');
      WriteLn(versionFile, 'sortk -t, -a4 -h1011 csv     # Compare in Field-1, Field-3 and Field-4. ');
      WriteLn(versionFile, 'sortk -t, -j2 csv            # Compare phrases in Field-2. ');
      WriteLn(versionFile, 'sortk -t, -k1 -$ csv         # Check if all lines share the field count. ');
      WriteLn(versionFile, 'sortk -t, -k1 -Z csv         # The number of fields may exceed 255. ');
      WriteLn(versionFile, 'sortk -t, -k2 csv            # Compare phrases from Field-2 to finish. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -# csv       # Add ID to each summary. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -b csv       # Skip space in the first part of Field-2. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -c- csv      # Skip combining characters. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -c- -p csv   # H and I are omitted from ASCII characters.');
      WriteLn(versionFile, 'sortk -t, -k2,2 -E csv       # Erase manager.txt ');
      WriteLn(versionFile, 'sortk -t, -k2,2 --e csv      # EUC-JP ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -f csv       # EIGHT equals eight. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -g csv       # Compare 0.12 with 0.78 in Field-2. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -g -R9 csv   # Raise the standard to 9.0001 ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -i csv       # Skip control characters. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -n csv       # Compare 3456 with 9012 in Field-2. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -q csv       # Commas disappear if placed in quotes. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -r csv       # Arrange in descending order. ');
      WriteLn(versionFile, 'sortk -t, -k2,2 -u csv       # Display a line for each unique summary. ');
      WriteLn(versionFile, 'sortk -t, -k2,5 -u -! csv    # Report ID if any summaries match each other.');
      WriteLn(versionFile, 'sortk -t, -k2,2 -% csv       # Produce manager.txt ');
      WriteLn(versionFile, 'sortk -t, -k2,2 csv          # Compare phrases in Field-2. ');
      WriteLn(versionFile, 'sortk -t, -k2,5 csv          # Compare phrases from Field-2 to Field-5. ');
      WriteLn(versionFile, 'sortk -t, -o new csv         # Read csv to produce a new file. ');
      WriteLn(versionFile, 'TYPE csv | sortk -t, -o new  # Read csv to produce a new file. ');
      WriteLn(versionFile, 'sortk -T\l -k2,2 -x- csv     # Do not check if the file name includes .csv');
      WriteLn(versionFile, 'sortk -T\l -k2,2 file        # Wiki table ');
      WriteLn(versionFile, 'sortk -T\x7C -k2,2 file      # Wiki table ');
      WriteLn(versionFile, 'sortk -U131072 -k2,2 csv     # The number of lines shall not exceed 131072.');
      WriteLn(versionFile, 'sortk -V\x1F -k2,5 csv       # 1Fh connects phrases from Field-2 to Field-5');
      WriteLn(versionFile, 'sortk -w~ -k2,5 csv          # ~ is used as quote. ');
    end
  else
    begin
      WriteLn(versionFile, 'sortk --help                 ');
    end;

  CloseFile(versionFile);

  AssignFile(versionFile, VERSION_TEXT);
  Reset(versionFile);
  while not Eof(versionFile) do
  begin
    ReadLn(versionFile, s);
    WriteLn(s);
  end;

  CloseFile(versionFile);
end;

end.
