unit MaxSpan;

interface

uses
  Windows, SysUtils, ErrorText, AnsiQueue, FinalEffect, ConvertUTF16, ConvertEUC, ConvertWideEUC;

type AnsiCharArray = array[0..4095] of AnsiChar;
type CharArray = array[0..4095] of Char;

type
  MeasureSpan = class
  private
    a: AnsiStringQueue;
    ANSI_ASCII: AnsiString;
    ANSI_CHAR: array[0..255] of AnsiChar;
    beginning: Integer;
    BLANK_C: AnsiChar;
    buffer: array of AnsiChar;
    buffer_size: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    compared: Boolean;
    count: Integer; { line count }
    counted: Boolean;
    dough: array[0..8191] of AnsiChar;
    doughLength: Integer;
    e: CharArray;
    E_COUNT: Integer;
    E_COUNT_1: Integer;
    EMPTY_ANSI: AnsiString;
    field_count: Integer;
    file_name: String;
    final_effects: FinalEffects;
    focal_point: Integer;
    focus_of_attention: String;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_FE_C: AnsiChar;
    H_FF_C: AnsiChar;
    maximum_field_span: Integer;
    maximum_item_width: Integer;
    maximum_item_width_line_number: Integer;
    maximum_span: Integer;
    maximum_span_field_number: Integer;
    maximum_span_line_number: Integer;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    quote_byte: Byte;
    remove_separators: Boolean;
    separator_byte: Byte;
    separators_disappear_if_placed_in_quotes: Boolean;
    shift_jis_text: Boolean;
    sum: Integer; { field count }
    summed_up: Boolean;
    UTF16_NEW_LINE: String;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    x: ErrorTexts;
    y: AnsiCharArray;
    Y_SIZE: Integer;
    Y_SIZE_1: Integer;
    function count_fields(dough: array of AnsiChar; doughLength: Integer): Integer;
    function file_size(name: String): Integer;
    function item_in_field(dough: array of AnsiChar; doughLength: Integer; f: Integer): Integer;
    function setBufferSize: Integer;
    function shift_jis_count_fields(dough: array of AnsiChar; doughLength: Integer): Integer;
    function shift_jis_item_in_field(dough: array of AnsiChar; doughLength: Integer; f: Integer): Integer;
  public
    destructor Destroy; override;
    function ANSI_CHAR_table: AnsiChar;
    function copyingAnsiStringToArray(s: PAnsiChar; s_length: Integer): Integer;
    function countFields: Integer;
    function countFieldsEUC: Integer;
    function countLines: Integer;
    function displayDataText: Integer;
    function displayDataTextEUC: Integer;
    function fieldCount: Integer;
    function fieldNumber: Integer;
    function focused: String;
    function focusedApproachToItem(f: Integer): String; overload;
    function focusedApproachToItem(p: array of Char; f: Integer): String; overload;
    function focusedApproachToLF: String; overload;
    function focusedApproachToLF(file_label: String): String; overload;
    function focusedApproachToLF(p: array of Char): String; overload;
    function focusedApproachToSeparators: String; overload;
    function focusedApproachToSeparators(file_label: String): String; overload;
    function focusedApproachToSeparators(p: array of Char): String; overload;
    function LF_to_LF_span: Integer;
    function lineNumber: Integer;
    function maximumFieldSpan: Integer;
    function maximumSpan: Integer;
    function newLine: AnsiString;
    function newLineSize: Integer;
    function produceTemporaryTextFile: Integer;
    function reading: Boolean;
    function readingEUC: Boolean;
    function separator_to_separator_span(dough: array of AnsiChar; doughLength: Integer): Integer;
    function separatorByte: Integer;
    function separatorSpan(s: String): Integer;
    function shift_jis_separator_to_separator_span(dough: array of AnsiChar; doughLength: Integer): Integer;
    function widestItem(f: Integer): Integer;
    function widestItemEUC(f: Integer): Integer;
    procedure ifErrorOccurs(exit_code: Integer);
    procedure setting(f: FinalEffects); overload;
    procedure setting(fn: String); overload;
    procedure setting(fn: String; t: Integer; q: Byte; s: Byte; j: Boolean; si: Boolean); overload;
    procedure setting_beginning(name: String);
    procedure singleLine;
  end;

const
  ASCII_SIZE = 1;
  BOM_SIZE = 3; { EFh, BBh, BFh }
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  COLON_GAP = ': ';
  COLON_BYTE = 58; { 3Ah, ':' }
  COMMA_BYTE = 44; { 2Ch, ',' }
  COMMAS_SHALL_DISAPPEAR_IF_PLACED_IN_QUOTES = True;
  DATA_TEXT = 'data.txt';
  EMPTY = '';
  FILE_ERROR = -1;
  FIRST_LINE = 1;
  GAP_BYTE = 32; { SP, ' ' }
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_5C = 92; { '\' }
  H_81 = 129;
  H_BB = 187;
  H_BF = 191;
  H_EF = 239;
  H_FC = 252;
  H_FE = 254;
  H_FF = 255;
  KANA_HANDAKUTEN = 223; { DFh }
  KANA_KUTEN = 161; { A1h }
  KANJI_SIZE = 2;
  KATAKANA_SIZE = 1;
  NOT_RECOGNIZED = -1;
  SEPARATOR_SIZE = 1;
  SHIFT_JIS_ENCODING = True;
  SIZE_OF_ANSICHAR = 1;
  STANDARD_QUOTE_BYTE = 34; { 22h, '"' }
  STANDARD_SEPARATOR_BYTE = 44; { 2Ch, ',' }
  TEMPORARY_BINARY = 'binary.tmp';
  TEMPORARY_TEXT = 'text.tmp';
  TEXT_FILE = '.txt';
  UPPER_LIMIT_TO_FIELD_NUMBER = 3071;

implementation

destructor MeasureSpan.Destroy;
begin
  a.Free;
  buffer := nil;

  inherited Destroy;
end;

function MeasureSpan.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function MeasureSpan.copyingAnsiStringToArray(s: PAnsiChar; s_length: Integer): Integer;
var
  k: Integer;
  m: Integer;
  s_length_1: Integer;
begin
  m := 0;
  s_length_1 := s_length - 1;

  for k := 0 to s_length_1 do
  begin
    dough[m] := s[k];
    Inc(m);
  end;

  Result := m;
end;

function MeasureSpan.countFields: Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  exit_code: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  shared_sum: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  a.clearRing;
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  exit_code := 0;
  m := 0;
  shared_sum := 0;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-616: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        count := 1;

{ Reading a line }
        shift_jis_count_fields(dough, b);
        b := 0;
        shared_sum := field_count;

        trace := n;
        beginning := trace + 1;
        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        Inc(count);

{ Reading a line }
        doughLength := b;
        shift_jis_count_fields(dough, doughLength);
        b := 0;

        if field_count = shared_sum then
          begin
            continue;
          end
        else
          begin
            pascal_string := dough;
            z := Copy(pascal_string, 1, doughLength);
            a.push_back(z);
          end;

      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            continue;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

  counted := True;
  a.dataText;
  Result := exit_code;
end;

function MeasureSpan.countFieldsEUC: Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  exit_code: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  shared_sum: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  a.clearRing;
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  exit_code := 0;
  m := 0;
  shared_sum := 0;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-617: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        count := 1;

{ Reading a line }
        count_fields(dough, b);
        b := 0;
        shared_sum := field_count;

        trace := n;
        beginning := trace + 1;
        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        Inc(count);

{ Reading a line }
        doughLength := b;
        count_fields(dough, doughLength);
        b := 0;

        if field_count = shared_sum then
          begin
            continue;
          end
        else
          begin
            pascal_string := dough;
            z := Copy(pascal_string, 1, doughLength);
            a.push_back(z);
          end;

      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            continue;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

  counted := True;
  a.dataText;
  Result := exit_code;
end;

function MeasureSpan.countLines: Integer;
var
  block_has_been_read: Integer;
  n: Integer;
  s: String;
  testFile: File;
begin
  count := 0;

  try
{ Reaeding a file and counting lines }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        if y[n] = H_0A_C then
          Inc(count);

      end;
    until block_has_been_read = 0;

    CloseFile(testFile);
  except
    s := 'Reset() error-618: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  counted := True;
  Result := count;
end;

function MeasureSpan.displayDataText: Integer;
var
  dataFile: TextFile;
  m: Integer;
  s: String;
begin
  m := 0;

  try
    AssignFile(dataFile, DATA_TEXT);
    Reset(dataFile);
    while not Eof(dataFile) do
    begin
      ReadLn(dataFile, s);
      Inc(m);
      WriteLn(s);
    end;

    CloseFile(dataFile);
  except
    s := 'Reset() error-619: ' + DATA_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    m := FILE_ERROR;
  end;

  Result := m;
end;

function MeasureSpan.displayDataTextEUC: Integer;
var
  c: Conversion;
  dataFile: TextFile;
  f: FinalEffects;
  m: Integer;
  s: String;
  temporaryFile: File;
begin
  m := 0;

  f := FinalEffects.Create;
  f.setting;
  c := Conversion.Create;
  c.setting(f, DATA_TEXT);
  c.ifErrorOccurs(BUFFER_ERROR);
  c.reading_to_produce_MS932_CRLF(TEMPORARY_BINARY);
  c.Free;

  try
    AssignFile(dataFile, TEMPORARY_BINARY);
    Reset(dataFile);
    while not Eof(dataFile) do
    begin
      ReadLn(dataFile, s);
      Inc(m);
      WriteLn(s);
    end;

    CloseFile(dataFile);
  except
    s := 'Reset() error-620: ' + TEMPORARY_BINARY;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    m := FILE_ERROR;
  end;

  if not f.noRemoving then
    begin
      AssignFile(temporaryFile, TEMPORARY_BINARY);
      Erase(temporaryFile);
    end;

  Result := m;
end;

function MeasureSpan.fieldCount: Integer;
begin
  Result := field_count;
end;

function MeasureSpan.fieldNumber: Integer;
begin
  Result := maximum_span_field_number;
end;

function MeasureSpan.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function MeasureSpan.focused: String;
begin
  Result := focus_of_attention;
end;

function MeasureSpan.focusedApproachToItem(f: Integer): String;
var
  ensemble: String;
begin
  ensemble := '';

  if compared and counted then
    begin
{ widest item in Field-f }
      ensemble := ensemble + file_name;
      ensemble := ensemble + COLON_GAP;
      ensemble := ensemble + IntToStr(count);
      if count = 1 then
        begin
          ensemble := ensemble + ' line';
          ensemble := ensemble + UTF16_NEW_LINE;
        end
      else
        begin
          ensemble := ensemble + ' lines';
          ensemble := ensemble + UTF16_NEW_LINE;
        end;

      ensemble := ensemble + 'widest item of ';
      ensemble := ensemble + IntToStr(maximum_item_width);
      if maximum_item_width = 1 then
        begin
          ensemble := ensemble + ' byte (in Field-';
        end
      else
        begin
          ensemble := ensemble + ' bytes (in Field-';
        end;

      ensemble := ensemble + IntToStr(f);
      ensemble := ensemble + ', Line-';
      ensemble := ensemble + IntToStr(maximum_item_width_line_number);
      ensemble := ensemble + ')';
      ensemble := ensemble + UTF16_NEW_LINE;
      ensemble := ensemble + 'Line-';
      ensemble := ensemble + IntToStr(focal_point);
      ensemble := ensemble + '|';
    end;

  ensemble := ensemble + focus_of_attention;
  ensemble := ensemble + UTF16_NEW_LINE;
  Result := ensemble;
end;

function MeasureSpan.focusedApproachToItem(p: array of Char; f: Integer): String;
var
  ensemble: String;
begin
  ensemble := '';

  if compared and counted then
    begin
{ widest item in Field-f }
      ensemble := ensemble + file_name;
      ensemble := ensemble + p[0];
      ensemble := ensemble + p[1];
      ensemble := ensemble + IntToStr(count);
      ensemble := ensemble + ' s';
      ensemble := ensemble + UTF16_NEW_LINE;

      ensemble := ensemble + ' ';
      ensemble := ensemble + IntToStr(maximum_item_width_line_number);
      ensemble := ensemble + ' sA ';
      ensemble := ensemble + IntToStr(f);
      ensemble := ensemble + ' ';
      ensemble := ensemble + p[0];
      ensemble := ensemble + p[1];
      ensemble := ensemble + IntToStr(maximum_item_width);
      ensemble := ensemble + ' ';
      ensemble := ensemble + UTF16_NEW_LINE;

      ensemble := ensemble + ' ';
      ensemble := ensemble + IntToStr(focal_point);
      ensemble := ensemble + ' s';
      ensemble := ensemble + p[0];
      ensemble := ensemble + p[1];
    end;

  ensemble := ensemble + focus_of_attention;
  ensemble := ensemble + UTF16_NEW_LINE;
  Result := ensemble;
end;

function MeasureSpan.focusedApproachToLF: String;
var
  ensemble: String;
begin
  ensemble := '';

  if counted then
    begin
{ maximum LF-to-LF span }
      ensemble := ensemble + file_name;
      ensemble := ensemble + COLON_GAP;
      ensemble := ensemble + IntToStr(count);
      if count = 1 then
        begin
          ensemble := ensemble + ' line';
          ensemble := ensemble + UTF16_NEW_LINE;
        end
      else
        begin
          ensemble := ensemble + ' lines';
          ensemble := ensemble + UTF16_NEW_LINE;
        end;

      ensemble := ensemble + 'maximum LF-to-LF span of ';
      ensemble := ensemble + IntToStr(maximum_span);
      if maximum_span = 1 then
        begin
          ensemble := ensemble + ' byte (Line-';
        end
      else
        begin
          ensemble := ensemble + ' bytes (Line-';
        end;

      ensemble := ensemble + IntToStr(maximum_span_line_number);
      ensemble := ensemble + ')';
      ensemble := ensemble + UTF16_NEW_LINE;
    end;

  ensemble := ensemble + focus_of_attention;
  ensemble := ensemble + UTF16_NEW_LINE;
  Result := ensemble;
end;

function MeasureSpan.focusedApproachToLF(file_label: String): String;
var
  ensemble: String;
begin
  ensemble := '';

  if counted then
    begin
{ maximum LF-to-LF span }
      ensemble := ensemble + file_label;
      ensemble := ensemble + COLON_GAP;
      ensemble := ensemble + IntToStr(count);
      if count = 1 then
        begin
          ensemble := ensemble + ' line';
          ensemble := ensemble + UTF16_NEW_LINE;
        end
      else
        begin
          ensemble := ensemble + ' lines';
          ensemble := ensemble + UTF16_NEW_LINE;
        end;

      ensemble := ensemble + 'maximum LF-to-LF span of ';
      ensemble := ensemble + IntToStr(maximum_span);
      if maximum_span = 1 then
        begin
          ensemble := ensemble + ' byte (Line-';
        end
      else
        begin
          ensemble := ensemble + ' bytes (Line-';
        end;

      ensemble := ensemble + IntToStr(maximum_span_line_number);
      ensemble := ensemble + ')';
      ensemble := ensemble + UTF16_NEW_LINE;
    end;

  ensemble := ensemble + focus_of_attention;
  ensemble := ensemble + UTF16_NEW_LINE;
  Result := ensemble;
end;

function MeasureSpan.focusedApproachToLF(p: array of Char): String;
var
  ensemble: String;
begin
  ensemble := '';

  if counted then
    begin
{ maximum LF-to-LF span }
      ensemble := ensemble + file_name;
      ensemble := ensemble + p[0];
      ensemble := ensemble + p[1];
      ensemble := ensemble + IntToStr(count);
      ensemble := ensemble + ' s';
      ensemble := ensemble + UTF16_NEW_LINE;

      ensemble := ensemble + ' ';
      ensemble := ensemble + IntToStr(maximum_span_line_number);
      ensemble := ensemble + ' s';
      ensemble := ensemble + p[0];
      ensemble := ensemble + p[1];
      ensemble := ensemble + IntToStr(maximum_span);
      ensemble := ensemble + ' ';
      ensemble := ensemble + UTF16_NEW_LINE;

      ensemble := ensemble + ' ';
      ensemble := ensemble + IntToStr(focal_point);
      ensemble := ensemble + ' s';
      ensemble := ensemble + p[0];
      ensemble := ensemble + p[1];
    end;

  ensemble := ensemble + focus_of_attention;
  ensemble := ensemble + UTF16_NEW_LINE;
  Result := ensemble;
end;

function MeasureSpan.focusedApproachToSeparators: String;
var
  ensemble: String;
begin
  ensemble := '';

  if counted and summed_up then
    begin
{ maximum separator-to-separator span }
      ensemble := ensemble + file_name;
      ensemble := ensemble + COLON_GAP;
      ensemble := ensemble + IntToStr(count);
      if count = 1 then
        begin
          ensemble := ensemble + ' line';
          ensemble := ensemble + UTF16_NEW_LINE;
        end
      else
        begin
          ensemble := ensemble + ' lines';
          ensemble := ensemble + UTF16_NEW_LINE;
        end;

      ensemble := ensemble + 'maximum separator-to-separator span of ';
      ensemble := ensemble + IntToStr(maximum_field_span);
      if maximum_field_span = 1 then
        begin
          ensemble := ensemble + ' byte (Field-';
        end
      else
        begin
          ensemble := ensemble + ' bytes (Field-';
        end;

      ensemble := ensemble + IntToStr(maximum_span_field_number);
      ensemble := ensemble + ')';
      ensemble := ensemble + UTF16_NEW_LINE;
      ensemble := ensemble + 'Line-';
      ensemble := ensemble + IntToStr(focal_point);
      ensemble := ensemble + '|';
    end;

  ensemble := ensemble + focus_of_attention;
  ensemble := ensemble + UTF16_NEW_LINE;
  Result := ensemble;
end;

function MeasureSpan.focusedApproachToSeparators(file_label: String): String;
var
  ensemble: String;
begin
  ensemble := '';

  if counted and summed_up then
    begin
{ maximum separator-to-separator span }
      ensemble := ensemble + file_label;
      ensemble := ensemble + COLON_GAP;
      ensemble := ensemble + IntToStr(count);
      if count = 1 then
        begin
          ensemble := ensemble + ' line';
          ensemble := ensemble + UTF16_NEW_LINE;
        end
      else
        begin
          ensemble := ensemble + ' lines';
          ensemble := ensemble + UTF16_NEW_LINE;
        end;

      ensemble := ensemble + 'maximum separator-to-separator span of ';
      ensemble := ensemble + IntToStr(maximum_field_span);
      if maximum_field_span = 1 then
        begin
          ensemble := ensemble + ' byte (Field-';
        end
      else
        begin
          ensemble := ensemble + ' bytes (Field-';
        end;

      ensemble := ensemble + IntToStr(maximum_span_field_number);
      ensemble := ensemble + ')';
      ensemble := ensemble + UTF16_NEW_LINE;
      ensemble := ensemble + 'Line-';
      ensemble := ensemble + IntToStr(focal_point);
      ensemble := ensemble + '|';
    end;

  ensemble := ensemble + focus_of_attention;
  ensemble := ensemble + UTF16_NEW_LINE;
  Result := ensemble;
end;

function MeasureSpan.focusedApproachToSeparators(p: array of Char): String;
var
  ensemble: String;
begin
  ensemble := '';

  if counted and summed_up then
    begin
{ maximum separator-to-separator span }
      ensemble := ensemble + file_name;
      ensemble := ensemble + p[0];
      ensemble := ensemble + p[1];
      ensemble := ensemble + IntToStr(count);
      ensemble := ensemble + ' s';
      ensemble := ensemble + UTF16_NEW_LINE;

      ensemble := ensemble + ' ';
      ensemble := ensemble + IntToStr(maximum_span_field_number);
      ensemble := ensemble + ' ';
      ensemble := ensemble + p[0];
      ensemble := ensemble + p[1];
      ensemble := ensemble + IntToStr(maximum_field_span);
      ensemble := ensemble + ' ';
      ensemble := ensemble + UTF16_NEW_LINE;

      ensemble := ensemble + ' ';
      ensemble := ensemble + IntToStr(focal_point);
      ensemble := ensemble + ' s';
      ensemble := ensemble + p[0];
      ensemble := ensemble + p[1];
    end;

  ensemble := ensemble + focus_of_attention;
  ensemble := ensemble + UTF16_NEW_LINE;
  Result := ensemble;
end;

function MeasureSpan.item_in_field(dough: array of AnsiChar; doughLength: Integer; f: Integer): Integer;
var
  field_number: Integer;
  i: Longint;
  k: Integer;
  m: Integer;
  n: Byte;
  no_splitting: Boolean;
  doughLength_1: Integer;
begin
  field_number := 1;
  m := 0;
  no_splitting := False;
  doughLength_1 := doughLength - 1;

  for k := 0 to doughLength_1 do
  begin
    i := Ord(dough[k]);
    n := Byte(i);

    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
            if field_number = f then
              begin
                Inc(m);
              end;

          end
        else
          begin
            if n = separator_byte then
              begin
                if separators_disappear_if_placed_in_quotes then
                  begin
                    continue; { Commas disappear if placed in quotes. }
                  end
                else
                  begin
                    if field_number = f then
                      begin
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if field_number = f then
                  begin
                    Inc(m);
                  end;

              end;

          end;

        continue;
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
        if field_number = f then
          begin
            Inc(m);
          end;

      end
    else
      begin
        if n = separator_byte then
          begin
            if field_number = f then
              begin
                break;
              end;

            m := 0;
            Inc(field_number);
          end
        else
          begin
            if field_number = f then
              begin
                Inc(m);
              end;

          end;

      end;

  end;

  Result := m;
end;

function MeasureSpan.LF_to_LF_span: Integer;
var
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  trace: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  k := 0;
  maximum_span := 0;
  trace := 0;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[k] := y[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-621: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        count := 1;

        if carriage_return_appears then
          maximum_span := n - 1 + NEW_LINE_SIZE { n exceeds zero because 0Dh precedes 0Ah. }
        else
          maximum_span := n + NEW_LINE_SIZE;

        maximum_span_line_number := 1;
        trace := n;
        beginning := trace + 1;
        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end;

      end;

  end;

{ Counting lines and measuring spans from LF to LF }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        Inc(count);
        m := n - trace;
        if m > maximum_span then
          begin
            maximum_span := m;
            maximum_span_line_number := count;
          end;

        trace := n;
      end;

  end;

{ Software shall provide a subtracted span regardless of NEW_LINE_SIZE. }
  maximum_span := maximum_span - NEW_LINE_SIZE;
  counted := True;
  Result := count;
end;

function MeasureSpan.lineNumber: Integer;
begin
  Result := maximum_span_line_number;
end;

function MeasureSpan.maximumFieldSpan: Integer;
begin
  Result := maximum_field_span;
end;

function MeasureSpan.maximumSpan: Integer;
begin
  Result := maximum_span;
end;

function MeasureSpan.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function MeasureSpan.newLineSize: Integer;
begin
  Result := NEW_LINE_SIZE;
end;

function MeasureSpan.produceTemporaryTextFile: Integer;
const
  NEW_LINE_SIZE = 2; { CR, LF }
var
  newFile: TextFile;
  s: String;
  sum: Integer;
begin
  sum := 0;

  try
    AssignFile(newFile, TEMPORARY_TEXT);
    Rewrite(newFile);

    while not Eof do
    begin
      ReadLn(s);
      WriteLn(newFile, s);
      sum := sum + Length(s);
      sum := sum + NEW_LINE_SIZE;
    end;

    CloseFile(newFile);
  except
    s := 'Reset() error-622: ' + TEMPORARY_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function MeasureSpan.reading: Boolean;
var
  found: Boolean;
  line_count: Integer;
  s: String;
  testFile: TextFile;
begin
  found := False;
  line_count := 0;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      Inc(line_count);
      if line_count = focal_point then
        begin
          focus_of_attention := s;
          found := True;
          break;
        end;

    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-623: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  Result := found;
end;

function MeasureSpan.readingEUC: Boolean;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  found: Boolean;
  k: Integer;
  line_count: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  pascal_unicode_string: String;
  s: String;
  t: FinalEffects;
  testFile: File;
  trace: Integer;
  u: ConvertEUCintoUTF16;
  z: AnsiString;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  line_count := 0;
  found := False;
  m := 0;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-624: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  t := FinalEffects.Create;
  t.setting;
  u := ConvertEUCintoUTF16.Create;
  u.setting(t);

{ Finding the first NEW_LINE }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        line_count := 1;
        doughLength := b;
        b := 0;

        if line_count = focal_point then
          begin
{ Reading a line }
            pascal_string := dough;
            z := Copy(pascal_string, 1, doughLength);

            u.produce_UTF16_phrase( PAnsiChar(z), Length(z) );
            E_COUNT := u.UTF16_phrase_size;
            E_COUNT_1 := E_COUNT - 1;
            for k := 0 to E_COUNT_1 do
            begin
              e[k] := u.copyingArrayItem(k);
            end;

            pascal_unicode_string := e;
            focus_of_attention := Copy(pascal_unicode_string, 1, E_COUNT);
            found := True;
            break;
          end;

        trace := n;
        beginning := trace + 1;
        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

if not found then
  begin
{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(line_count);
            doughLength := b;
            b := 0;

            if line_count = focal_point then
              begin
{ Reading a line }
                pascal_string := dough;
                z := Copy(pascal_string, 1, doughLength);

                u.produce_UTF16_phrase( PAnsiChar(z), Length(z) );
                E_COUNT := u.UTF16_phrase_size;
                E_COUNT_1 := E_COUNT - 1;
                for k := 0 to E_COUNT_1 do
                begin
                  e[k] := u.copyingArrayItem(k);
                end;

                pascal_unicode_string := e;
                focus_of_attention := Copy(pascal_unicode_string, 1, E_COUNT);
                found := True;
                break;
              end;

          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

  end;

  u.Free; t.Free;

  Result := found;
end;

function MeasureSpan.separatorByte: Integer;
begin
  Result := separator_byte;
end;

function MeasureSpan.separator_to_separator_span(dough: array of AnsiChar; doughLength: Integer): Integer;
var
  dough_length_1: Integer;
  n: Longint;
  k: Integer;
  m: Integer;
  no_splitting: Boolean;
  trace: Integer;
begin
  beginning := 0;
  sum := 0;
  maximum_field_span := 0;
  no_splitting := False;
  dough_length_1 := doughLength - 1;
  trace := 0;

{ Finding the first separator }
  for k := beginning to dough_length_1 do
  begin
    n := Ord(dough[k]);

    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
          end;

        continue; { Commas disappear if placed in quotes. }
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
      end
    else
      begin
        if n = separator_byte then
          begin
            sum := 1;
            maximum_field_span := k + SEPARATOR_SIZE;
            maximum_span_field_number := sum;
            trace := k;
            beginning := trace + 1;
            break;
          end;

      end;

  end;

{ Measuring spans from separator to separator }
  for k := beginning to dough_length_1 do
  begin
    n := Ord(dough[k]);

    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
          end;

        continue; { Commas disappear if placed in quotes. }
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
      end
    else
      begin
        if n = separator_byte then
          begin
            Inc(sum);
            m := k - trace;
            if m > maximum_field_span then
              begin
                maximum_field_span := m;
                maximum_span_field_number := sum;
              end;

            trace := k;
          end;

      end;

  end;

{ Software shall provide a subtracted span regardless of SEPARATOR_SIZE. }
  maximum_field_span := maximum_field_span - SEPARATOR_SIZE;
  summed_up := True;
  Result := sum;
end;

function MeasureSpan.separatorSpan(s: String): Integer;
var
  c: Converting;
  r: Integer;
  t: FinalEffects;
  z: AnsiString;
begin
  t := final_effects;

{ Converting the text from UTF-16 to Shift_JIS }
  c := Converting.Create;
  c.setting(t);

  z := c.produce_MS932_phrase( PChar(s), Length(s) );
  copyingAnsiStringToArray( PAnsiChar(z), Length(z) ); { dough }
  doughLength := Length(dough);

  r := shift_jis_separator_to_separator_span(dough, doughLength);
  c.Free;
  Result := r;
end;

function MeasureSpan.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    s := 'SetLength() error-625: ' + IntToStr(buffer_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  Result := r;
end;

function MeasureSpan.shift_jis_count_fields(dough: array of AnsiChar; doughLength: Integer): Integer;
var
  b: Integer;
  doughLength_1: Integer;
  c: AnsiChar;
  field_number: Integer;
  i: Longint;
  k: Integer;
  kanji: Boolean;
  n: Byte;
  no_splitting: Boolean;
begin
  b := 0;
  field_number := 1;
  kanji := False;
  no_splitting := False;
  doughLength_1 := doughLength - 1;

  for k := 0 to doughLength_1 do
  begin
    c := dough[k];
    i := Ord(c);
    n := Byte(i);

    if kanji then
      begin
        Inc(b, KANJI_SIZE);

        kanji := False;
        continue;
      end;

    if (H_81 <= n) and (n <= H_FC) then
      begin
        if (KANA_KUTEN <= n) and (n <= KANA_HANDAKUTEN) then
          begin
            kanji := False;
          end
        else
          begin
            kanji := True;
          end;

        continue;
      end;

{ ASCII }
    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
            Inc(b);
          end
        else
          begin
            if n = separator_byte then
              begin
                if remove_separators then
                  begin
                    continue; { Commas disappear if placed in quotes. }
                  end
                else
                  begin
                    Inc(b);
                  end;

              end
            else
              begin
                Inc(b);
              end;

          end;

        continue;
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
        Inc(b);
      end
    else
      begin
        if n = separator_byte then
          begin
            b := 0;
            Inc(field_number);
            if field_number > UPPER_LIMIT_TO_FIELD_NUMBER then
              begin
                break;
              end;

          end
        else
          begin
            Inc(b);
          end;

      end;

  end;

  if b > 0 then
    begin
      field_count := field_number;
    end
  else
    begin
      field_count := field_number - 1;
    end;

  Result := b;
end;

function MeasureSpan.count_fields(dough: array of AnsiChar; doughLength: Integer): Integer;
var
  b: Integer;
  doughLength_1: Integer;
  c: AnsiChar;
  field_number: Integer;
  i: Longint;
  k: Integer;
  n: Byte;
  no_splitting: Boolean;
begin
  b := 0;
  field_number := 1;
  no_splitting := False;
  doughLength_1 := doughLength - 1;

  for k := 0 to doughLength_1 do
  begin
    c := dough[k];
    i := Ord(c);
    n := Byte(i);

    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
            Inc(b);
          end
        else
          begin
            if n = separator_byte then
              begin
                if remove_separators then
                  begin
                    continue; { Commas disappear if placed in quotes. }
                  end
                else
                  begin
                    Inc(b);
                  end;

              end
            else
              begin
                Inc(b);
              end;

          end;

        continue;
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
        Inc(b);
      end
    else
      begin
        if n = separator_byte then
          begin
            b := 0;
            Inc(field_number);
            if field_number > UPPER_LIMIT_TO_FIELD_NUMBER then
              begin
                break;
              end;

          end
        else
          begin
            Inc(b);
          end;

      end;

  end;

  if b > 0 then
    begin
      field_count := field_number;
    end
  else
    begin
      field_count := field_number - 1;
    end;

  Result := b;
end;

function MeasureSpan.shift_jis_item_in_field(dough: array of AnsiChar; doughLength: Integer; f: Integer): Integer;
var
  dough_length_1: Integer;
  field_number: Integer;
  i: Longint;
  k: Integer;
  kanji: Boolean;
  m: Integer;
  n: Byte;
  no_splitting: Boolean;
begin
  field_number := 1;
  kanji := False;
  m := 0;
  no_splitting := False;
  dough_length_1 := doughLength - 1;

  for k := 0 to dough_length_1 do
  begin
    i := Ord(dough[k]);
    n := Byte(i);

    if kanji then
      begin
        if field_number = f then
          begin
            Inc(m, KANJI_SIZE);
          end;

        kanji := False;
        continue;
      end;

    if (H_81 <= n) and (n <= H_FC) then
      begin
        if (KANA_KUTEN <= n) and (n <= KANA_HANDAKUTEN) then
          begin
            if field_number = f then
              begin
                Inc(m);
              end;

            kanji := False;
          end
        else
          begin
            kanji := True;
          end;

        continue;
      end;

{ ASCII }
    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
            if field_number = f then
              begin
                Inc(m);
              end;

          end
        else
          begin
            if n = separator_byte then
              begin
                if separators_disappear_if_placed_in_quotes then
                  begin
                    continue; { Commas disappear if placed in quotes. }
                  end
                else
                  begin
                    if field_number = f then
                      begin
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if field_number = f then
                  begin
                    Inc(m);
                  end;

              end;

          end;

        continue;
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
        if field_number = f then
          begin
            Inc(m);
          end;

      end
    else
      begin
        if n = separator_byte then
          begin
            if field_number = f then
              begin
                break;
              end;

            m := 0;
            Inc(field_number);
          end
        else
          begin
            if field_number = f then
              begin
                Inc(m);
              end;

          end;

      end;

  end;

  Result := m;
end;

function MeasureSpan.shift_jis_separator_to_separator_span(dough: array of AnsiChar; doughLength: Integer): Integer;
var
  beginning: Integer;
  dough_length_1: Integer;
  i: Longint;
  k: Integer;
  kanji: Boolean;
  m: Integer;
  n: Byte;
  no_splitting: Boolean;
  trace: Integer;
begin
  beginning := 0;
  kanji := False;
  maximum_field_span := 0;
  no_splitting := False;
  dough_length_1 := doughLength - 1;
  sum := 0;
  trace := 0;

{ Finding the first separator }
  for k := beginning to dough_length_1 do
  begin
    i := Ord(dough[k]);
    n := Byte(i);

    if kanji then
      begin
        kanji := False;
        continue;
      end;

    if (H_81 <= n) and (n <= H_FC) then
      begin
        if (KANA_KUTEN <= n) and (n <= KANA_HANDAKUTEN) then
          begin
            kanji := False;
          end
        else
          begin
            kanji := True;
          end;

        continue;
      end;

{ ASCII }
    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
          end;

        continue; { Commas disappear if placed in quotes. }
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
      end
    else
      begin
        if n = separator_byte then
          begin
            sum := 1;
            maximum_field_span := k + SEPARATOR_SIZE;
            maximum_span_field_number := sum;
            trace := k;
            beginning := trace + 1;
            break;
          end;

      end;

  end;

{ Measuring spans from separator to separator }
  kanji := False;
  no_splitting := False;
  for k := beginning to dough_length_1 do
  begin
    i := Ord(dough[k]);
    n := Byte(i);

    if kanji then
      begin
        kanji := False;
        continue;
      end;

    if (H_81 <= n) and (n <= H_FC) then
      begin
        if (KANA_KUTEN <= n) and (n <= KANA_HANDAKUTEN) then
          begin
            kanji := False;
          end
        else
          begin
            kanji := True;
          end;

        continue;
      end;

{ ASCII }
    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
          end;

        continue; { Commas disappear if placed in quotes. }
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
      end
    else
      begin
        if n = separator_byte then
          begin
            Inc(sum);
            m := k - trace;
            if m > maximum_field_span then
              begin
                maximum_field_span := m;
                maximum_span_field_number := sum;
              end;

            trace := k;
          end;

      end;

  end;

{ Software shall provide a subtracted span regardless of SEPARATOR_SIZE. }
  maximum_field_span := maximum_field_span - SEPARATOR_SIZE;
  summed_up := True;
  Result := sum;
end;

function MeasureSpan.widestItem(f: Integer): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  trace: Integer;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  maximum_item_width := 0;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-626: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        count := 1;

{ Reading a line }
        maximum_item_width := shift_jis_item_in_field(dough, b, f);
        maximum_item_width_line_number := count;
        b := 0;

        trace := n;
        beginning := trace + 1;
        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        Inc(count);

{ Reading a line }
        doughLength := b;
        m := shift_jis_item_in_field(dough, doughLength, f);
        b := 0;

        if m > maximum_item_width then
          begin
            maximum_item_width := m;
            maximum_item_width_line_number := count;
          end;

      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            continue;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

  counted := True;
  compared := True;
  Result := maximum_item_width;
end;

function MeasureSpan.widestItemEUC(f: Integer): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  trace: Integer;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  maximum_item_width := 0;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-627: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        count := 1;

{ Reading a line }
        maximum_item_width := item_in_field(dough, b, f);
        maximum_item_width_line_number := count;
        b := 0;

        trace := n;
        beginning := trace + 1;
        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        Inc(count);

{ Reading a line }
        doughLength := b;
        m := item_in_field(dough, doughLength, f);
        b := 0;

        if m > maximum_item_width then
          begin
            maximum_item_width := m;
            maximum_item_width_line_number := count;
          end;

      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            continue;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

  counted := True;
  compared := True;
  Result := maximum_item_width;
end;

procedure MeasureSpan.ifErrorOccurs(exit_code: Integer);
begin
  if buffer_size < 0 then
    Halt(exit_code);

end;

procedure MeasureSpan.setting(f: FinalEffects); 
var
  i: Integer;
  z: AnsiString;
begin
  buffer_size := -1;
  count := -1;
  file_name := f.fileName;
  final_effects := f;
  focal_point := f.atPoint;
  quote_byte := f.quoteByte;
  separator_byte := f.separatorByte;
  separators_disappear_if_placed_in_quotes := f.removeSeparators;
  shift_jis_text := f.Shift_JIS_file;

  setting_beginning(file_name);

  EMPTY_ANSI := '';
  field_count := UPPER_LIMIT_TO_FIELD_NUMBER;
  Y_SIZE := SizeOf(y);
  Y_SIZE_1 := Y_SIZE - 1;

  for i := 0 to Y_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  BLANK_C := ANSI_CHAR[GAP_BYTE];
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];
  H_FE_C := ANSI_CHAR[H_FE];

  NEW_LINE_SIZE := 1;
  NEW_LINE := EMPTY_ANSI + H_0A_C;
  UTF16_NEW_LINE := '' + Chr(H_0D);
  UTF16_NEW_LINE := UTF16_NEW_LINE + Chr(H_0A);

  setBufferSize;

  ANSI_ASCII := EMPTY_ANSI;
  for i := 1 to H_FF do
  begin
    ANSI_ASCII := ANSI_ASCII + ANSI_CHAR[i];
  end;

  a := AnsiStringQueue.Create;
  z := EMPTY_ANSI + BLANK_C;
  a.setting(ANSI_ASCII, z, 1);
end;

procedure MeasureSpan.setting(fn: String);
var
  i: Integer;
  z: AnsiString;
begin
  buffer_size := -1;
  count := -1;
  file_name := fn;
  focal_point := FIRST_LINE;
  quote_byte := STANDARD_QUOTE_BYTE;
  separator_byte := STANDARD_SEPARATOR_BYTE;
  separators_disappear_if_placed_in_quotes := not COMMAS_SHALL_DISAPPEAR_IF_PLACED_IN_QUOTES;
  shift_jis_text := SHIFT_JIS_ENCODING;

  setting_beginning(file_name);

  EMPTY_ANSI := '';
  field_count := UPPER_LIMIT_TO_FIELD_NUMBER;
  Y_SIZE := SizeOf(y);
  Y_SIZE_1 := Y_SIZE - 1;

  for i := 0 to Y_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  BLANK_C := ANSI_CHAR[GAP_BYTE];
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];
  H_FE_C := ANSI_CHAR[H_FE];

  NEW_LINE_SIZE := 1;
  NEW_LINE := EMPTY_ANSI + H_0A_C;
  UTF16_NEW_LINE := '' + Chr(H_0D);
  UTF16_NEW_LINE := UTF16_NEW_LINE + Chr(H_0A);

  setBufferSize;

  ANSI_ASCII := EMPTY_ANSI;
  for i := 1 to H_FF do
  begin
    ANSI_ASCII := ANSI_ASCII + ANSI_CHAR[i];
  end;

  a := AnsiStringQueue.Create;
  z := EMPTY_ANSI + BLANK_C;
  a.setting(ANSI_ASCII, z, 1);
end;

procedure MeasureSpan.setting(fn: String; t: Integer; q: Byte; s: Byte; j: Boolean; si: Boolean);
var
  i: Integer;
  z: AnsiString;
begin
  buffer_size := -1;
  count := -1;
  file_name := fn;
  focal_point := t;
  quote_byte := q;
  separator_byte := s;
  separators_disappear_if_placed_in_quotes := si;
  shift_jis_text := j;

  setting_beginning(file_name);

  EMPTY_ANSI := '';
  field_count := UPPER_LIMIT_TO_FIELD_NUMBER;
  E_COUNT := SizeOf(e) div SIZE_OF_CHAR;
  E_COUNT_1 := E_COUNT - 1;
  Y_SIZE := SizeOf(y);
  Y_SIZE_1 := Y_SIZE - 1;

  for i := 0 to Y_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  BLANK_C := ANSI_CHAR[GAP_BYTE];
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];
  H_FE_C := ANSI_CHAR[H_FE];

  NEW_LINE_SIZE := 1;
  NEW_LINE := EMPTY_ANSI + H_0A_C;
  UTF16_NEW_LINE := '' + Chr(H_0D);
  UTF16_NEW_LINE := UTF16_NEW_LINE + Chr(H_0A);

  setBufferSize;

  ANSI_ASCII := EMPTY_ANSI;
  for i := 1 to H_FF do
  begin
    ANSI_ASCII := ANSI_ASCII + ANSI_CHAR[i];
  end;

  a := AnsiStringQueue.Create;
  z := EMPTY_ANSI + BLANK_C;
  a.setting(ANSI_ASCII, z, 1);
end;

procedure MeasureSpan.setting_beginning(name: String);
var
  block_has_been_read: Integer;
  messages: String;
  q: array[0..2] of Byte;
  testFile: File;
begin
  V_SIZE := Sizeof(v);
  V_SIZE_1 := V_SIZE - 1;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, v, V_SIZE, block_has_been_read);
    q[0] := Ord(v[0]);
    q[1] := Ord(v[1]);
    q[2] := Ord(v[2]);
    if (q[0] = H_EF) and (q[1] = H_BB) and (q[2] = H_BF) then
      beginning := BOM_SIZE
    else
      beginning := 0;

    CloseFile(testFile);

    buffer_size := file_size(file_name);
  except
    messages := 'Reset() error-628: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
  end;

end;

procedure MeasureSpan.singleLine;
begin
  NEW_LINE := EMPTY_ANSI;
  NEW_LINE_SIZE := 0;
end;

end.
