unit NondakutenSJIS;

interface

uses
  SysUtils, SegmentedSJIS, FieldsArrayMany;

type ByteArray = array[0..223] of Byte;

type
  NondakutenSummarySJIS = class(SegmentedSummarySJIS)
  private
  public
    function editing_field(buffer: array of AnsiChar; bufferLength: Integer): Integer; override;
    procedure presetting(ans: AnsiString; j: Byte; q: Byte; s: Byte; e: Boolean); override;
    procedure setting(fields_list: FieldsListMany; c: Integer; substitute_for_dakuten: Byte; substitute_for_handakuten: Byte); overload;
    procedure setting(fields_list: FieldsListMany; c: Integer; n: array of AnsiChar; substitute_for_dakuten: Byte; substitute_for_handakuten: Byte); overload;
  end;

const
  BELL = 7;
  COMMA_BYTE = 44; { 2C, ',' }
  H_80 = 128;
  H_81 = 129;
  H_FC = 252;
  H_FF = 255;
  KANA_DAKUTEN = 222; { DEh }
  KANA_HANDAKUTEN = 223; { DFh }
  KANA_KUTEN = 161; { A1h }
  KANJI_SIZE = 2;

var
  ANSI_ASCII_S: AnsiString;
  BELL_ASCII_KATAKANA_S: ByteArray;
  ensemble: array[0..8191] of AnsiChar;
  ensembleLength: Integer;

implementation

function non_dakuten_field(s: AnsiString): AnsiString;
var
  b: Byte;
  c: AnsiChar;
  firstByte: AnsiChar;
  i: Longint;
  k: Integer;
  kanji: Boolean;
  m: Integer;
  n: Byte;
  pascal_string: AnsiString;
  s_length: Integer;
  z: AnsiString;
begin
  firstByte := ANSI_ASCII_S[COMMA_BYTE];
  kanji := False;
  m := 0;
  s_length := Length(s);

{ Removing control characters and substitutes from each field }
  for k := 1 to s_length do
  begin
    c := s[k];
    i := Ord(c);
    n := Byte(i);

    if kanji then
      begin
        ensemble[m] := firstByte;
        ensemble[ m + 1 ] := c;
        Inc(m, KANJI_SIZE);
        kanji := False;
        continue;
      end;

    if (H_81 <= n) and (n <= H_FC) then
      begin
        if (KANA_KUTEN <= n) and (n < KANA_DAKUTEN) then
          begin
            ensemble[m] := c;
            Inc(m);
            kanji := False;
            continue;
          end
        else
          begin
            if n = KANA_DAKUTEN then
              begin
                kanji := False;
                continue; { skipping hankaku-katakana dakuten }
              end
            else
              begin
                if n = KANA_HANDAKUTEN then
                  begin
                    kanji := False;
                    continue; { skipping hankaku-katakana handakuten }
                  end
                else
                  begin
                    firstByte := c;
                    kanji := True;
                    continue;
                  end;

              end;

          end;

      end;

{ ASCII }
    b := BELL_ASCII_KATAKANA_S[n];
    if b = BELL then
      begin
        continue;
      end
    else
      begin
        ensemble[m] := c;
        Inc(m);
        kanji := False;
      end;

  end;

  pascal_string := ensemble;
  ensembleLength := m;
  z := Copy(pascal_string, 1, ensembleLength);
  Result := z;
end;

function NondakutenSummarySJIS.editing_field(buffer: array of AnsiChar; bufferLength: Integer): Integer;
begin
    smashed_to_pieces(buffer, bufferLength, manyFields);
    vector_to_array_editing(non_dakuten_field);

  Result := field_count;
end;

procedure NondakutenSummarySJIS.presetting(ans: AnsiString; j: Byte; q: Byte; s: Byte; e: Boolean);
begin
  inherited presetting(ans, j, q, s, e);

  ANSI_ASCII_S := ans;
  BELL_ASCII_KATAKANA_S[0] := 7; BELL_ASCII_KATAKANA_S[1] := 7; BELL_ASCII_KATAKANA_S[2] := 7; BELL_ASCII_KATAKANA_S[3] := 7; BELL_ASCII_KATAKANA_S[4] := 7; BELL_ASCII_KATAKANA_S[5] := 7;
  BELL_ASCII_KATAKANA_S[6] := 7; BELL_ASCII_KATAKANA_S[7] := 7; BELL_ASCII_KATAKANA_S[8] := 7; BELL_ASCII_KATAKANA_S[9] := 7; BELL_ASCII_KATAKANA_S[10] := 7; BELL_ASCII_KATAKANA_S[11] := 7;
  BELL_ASCII_KATAKANA_S[12] := 7; BELL_ASCII_KATAKANA_S[13] := 7; BELL_ASCII_KATAKANA_S[14] := 7; BELL_ASCII_KATAKANA_S[15] := 7; BELL_ASCII_KATAKANA_S[16] := 7; BELL_ASCII_KATAKANA_S[17] := 7;
  BELL_ASCII_KATAKANA_S[18] := 7; BELL_ASCII_KATAKANA_S[19] := 7; BELL_ASCII_KATAKANA_S[20] := 7; BELL_ASCII_KATAKANA_S[21] := 7; BELL_ASCII_KATAKANA_S[22] := 7; BELL_ASCII_KATAKANA_S[23] := 7;
  BELL_ASCII_KATAKANA_S[24] := 7; BELL_ASCII_KATAKANA_S[25] := 7; BELL_ASCII_KATAKANA_S[26] := 7; BELL_ASCII_KATAKANA_S[27] := 7; BELL_ASCII_KATAKANA_S[28] := 7; BELL_ASCII_KATAKANA_S[29] := 7;
  BELL_ASCII_KATAKANA_S[30] := 7; BELL_ASCII_KATAKANA_S[31] := 7; BELL_ASCII_KATAKANA_S[32] := 32; BELL_ASCII_KATAKANA_S[33] := 33; BELL_ASCII_KATAKANA_S[34] := 34; BELL_ASCII_KATAKANA_S[35] := 35;
  BELL_ASCII_KATAKANA_S[36] := 36; BELL_ASCII_KATAKANA_S[37] := 37; BELL_ASCII_KATAKANA_S[38] := 38; BELL_ASCII_KATAKANA_S[39] := 39; BELL_ASCII_KATAKANA_S[40] := 40; BELL_ASCII_KATAKANA_S[41] := 41;
  BELL_ASCII_KATAKANA_S[42] := 42; BELL_ASCII_KATAKANA_S[43] := 43; BELL_ASCII_KATAKANA_S[44] := 44; BELL_ASCII_KATAKANA_S[45] := 45; BELL_ASCII_KATAKANA_S[46] := 46; BELL_ASCII_KATAKANA_S[47] := 47;
  BELL_ASCII_KATAKANA_S[48] := 48; BELL_ASCII_KATAKANA_S[49] := 49; BELL_ASCII_KATAKANA_S[50] := 50; BELL_ASCII_KATAKANA_S[51] := 51; BELL_ASCII_KATAKANA_S[52] := 52; BELL_ASCII_KATAKANA_S[53] := 53;
  BELL_ASCII_KATAKANA_S[54] := 54; BELL_ASCII_KATAKANA_S[55] := 55; BELL_ASCII_KATAKANA_S[56] := 56; BELL_ASCII_KATAKANA_S[57] := 57; BELL_ASCII_KATAKANA_S[58] := 58; BELL_ASCII_KATAKANA_S[59] := 59;
  BELL_ASCII_KATAKANA_S[60] := 60; BELL_ASCII_KATAKANA_S[61] := 61; BELL_ASCII_KATAKANA_S[62] := 62; BELL_ASCII_KATAKANA_S[63] := 63; BELL_ASCII_KATAKANA_S[64] := 64; BELL_ASCII_KATAKANA_S[65] := 65;
  BELL_ASCII_KATAKANA_S[66] := 66; BELL_ASCII_KATAKANA_S[67] := 67; BELL_ASCII_KATAKANA_S[68] := 68; BELL_ASCII_KATAKANA_S[69] := 69; BELL_ASCII_KATAKANA_S[70] := 70; BELL_ASCII_KATAKANA_S[71] := 71;
  BELL_ASCII_KATAKANA_S[72] := 72; BELL_ASCII_KATAKANA_S[73] := 73; BELL_ASCII_KATAKANA_S[74] := 74; BELL_ASCII_KATAKANA_S[75] := 75; BELL_ASCII_KATAKANA_S[76] := 76; BELL_ASCII_KATAKANA_S[77] := 77;
  BELL_ASCII_KATAKANA_S[78] := 78; BELL_ASCII_KATAKANA_S[79] := 79; BELL_ASCII_KATAKANA_S[80] := 80; BELL_ASCII_KATAKANA_S[81] := 81; BELL_ASCII_KATAKANA_S[82] := 82; BELL_ASCII_KATAKANA_S[83] := 83;
  BELL_ASCII_KATAKANA_S[84] := 84; BELL_ASCII_KATAKANA_S[85] := 85; BELL_ASCII_KATAKANA_S[86] := 86; BELL_ASCII_KATAKANA_S[87] := 87; BELL_ASCII_KATAKANA_S[88] := 88; BELL_ASCII_KATAKANA_S[89] := 89;
  BELL_ASCII_KATAKANA_S[90] := 90; BELL_ASCII_KATAKANA_S[91] := 91; BELL_ASCII_KATAKANA_S[92] := 92; BELL_ASCII_KATAKANA_S[93] := 93; BELL_ASCII_KATAKANA_S[94] := 94; BELL_ASCII_KATAKANA_S[95] := 95;
  BELL_ASCII_KATAKANA_S[96] := 96; BELL_ASCII_KATAKANA_S[97] := 97; BELL_ASCII_KATAKANA_S[98] := 98; BELL_ASCII_KATAKANA_S[99] := 99; BELL_ASCII_KATAKANA_S[100] := 100; BELL_ASCII_KATAKANA_S[101] := 101;
  BELL_ASCII_KATAKANA_S[102] := 102; BELL_ASCII_KATAKANA_S[103] := 103; BELL_ASCII_KATAKANA_S[104] := 104; BELL_ASCII_KATAKANA_S[105] := 105; BELL_ASCII_KATAKANA_S[106] := 106; BELL_ASCII_KATAKANA_S[107] := 107;
  BELL_ASCII_KATAKANA_S[108] := 108; BELL_ASCII_KATAKANA_S[109] := 109; BELL_ASCII_KATAKANA_S[110] := 110; BELL_ASCII_KATAKANA_S[111] := 111; BELL_ASCII_KATAKANA_S[112] := 112; BELL_ASCII_KATAKANA_S[113] := 113;
  BELL_ASCII_KATAKANA_S[114] := 114; BELL_ASCII_KATAKANA_S[115] := 115; BELL_ASCII_KATAKANA_S[116] := 116; BELL_ASCII_KATAKANA_S[117] := 117; BELL_ASCII_KATAKANA_S[118] := 118; BELL_ASCII_KATAKANA_S[119] := 119;
  BELL_ASCII_KATAKANA_S[120] := 120; BELL_ASCII_KATAKANA_S[121] := 121; BELL_ASCII_KATAKANA_S[122] := 122; BELL_ASCII_KATAKANA_S[123] := 123; BELL_ASCII_KATAKANA_S[124] := 124; BELL_ASCII_KATAKANA_S[125] := 125;
  BELL_ASCII_KATAKANA_S[126] := 126; BELL_ASCII_KATAKANA_S[127] := 7; BELL_ASCII_KATAKANA_S[128] := 7; BELL_ASCII_KATAKANA_S[129] := 7; BELL_ASCII_KATAKANA_S[130] := 7; BELL_ASCII_KATAKANA_S[131] := 7;
  BELL_ASCII_KATAKANA_S[132] := 7; BELL_ASCII_KATAKANA_S[133] := 7; BELL_ASCII_KATAKANA_S[134] := 7; BELL_ASCII_KATAKANA_S[135] := 7; BELL_ASCII_KATAKANA_S[136] := 7; BELL_ASCII_KATAKANA_S[137] := 7;
  BELL_ASCII_KATAKANA_S[138] := 7; BELL_ASCII_KATAKANA_S[139] := 7; BELL_ASCII_KATAKANA_S[140] := 7; BELL_ASCII_KATAKANA_S[141] := 7; BELL_ASCII_KATAKANA_S[142] := 7; BELL_ASCII_KATAKANA_S[143] := 7;
  BELL_ASCII_KATAKANA_S[144] := 7; BELL_ASCII_KATAKANA_S[145] := 7; BELL_ASCII_KATAKANA_S[146] := 7; BELL_ASCII_KATAKANA_S[147] := 7; BELL_ASCII_KATAKANA_S[148] := 7; BELL_ASCII_KATAKANA_S[149] := 7;
  BELL_ASCII_KATAKANA_S[150] := 7; BELL_ASCII_KATAKANA_S[151] := 7; BELL_ASCII_KATAKANA_S[152] := 7; BELL_ASCII_KATAKANA_S[153] := 7; BELL_ASCII_KATAKANA_S[154] := 7; BELL_ASCII_KATAKANA_S[155] := 7;
  BELL_ASCII_KATAKANA_S[156] := 7; BELL_ASCII_KATAKANA_S[157] := 7; BELL_ASCII_KATAKANA_S[158] := 7; BELL_ASCII_KATAKANA_S[159] := 7; BELL_ASCII_KATAKANA_S[160] := 7; BELL_ASCII_KATAKANA_S[161] := 161;
  BELL_ASCII_KATAKANA_S[162] := 162; BELL_ASCII_KATAKANA_S[163] := 163; BELL_ASCII_KATAKANA_S[164] := 164; BELL_ASCII_KATAKANA_S[165] := 165; BELL_ASCII_KATAKANA_S[166] := 166; BELL_ASCII_KATAKANA_S[167] := 167;
  BELL_ASCII_KATAKANA_S[168] := 168; BELL_ASCII_KATAKANA_S[169] := 169; BELL_ASCII_KATAKANA_S[170] := 170; BELL_ASCII_KATAKANA_S[171] := 171; BELL_ASCII_KATAKANA_S[172] := 172; BELL_ASCII_KATAKANA_S[173] := 173;
  BELL_ASCII_KATAKANA_S[174] := 174; BELL_ASCII_KATAKANA_S[175] := 175; BELL_ASCII_KATAKANA_S[176] := 176; BELL_ASCII_KATAKANA_S[177] := 177; BELL_ASCII_KATAKANA_S[178] := 178; BELL_ASCII_KATAKANA_S[179] := 179;
  BELL_ASCII_KATAKANA_S[180] := 180; BELL_ASCII_KATAKANA_S[181] := 181; BELL_ASCII_KATAKANA_S[182] := 182; BELL_ASCII_KATAKANA_S[183] := 183; BELL_ASCII_KATAKANA_S[184] := 184; BELL_ASCII_KATAKANA_S[185] := 185;
  BELL_ASCII_KATAKANA_S[186] := 186; BELL_ASCII_KATAKANA_S[187] := 187; BELL_ASCII_KATAKANA_S[188] := 188; BELL_ASCII_KATAKANA_S[189] := 189; BELL_ASCII_KATAKANA_S[190] := 190; BELL_ASCII_KATAKANA_S[191] := 191;
  BELL_ASCII_KATAKANA_S[192] := 192; BELL_ASCII_KATAKANA_S[193] := 193; BELL_ASCII_KATAKANA_S[194] := 194; BELL_ASCII_KATAKANA_S[195] := 195; BELL_ASCII_KATAKANA_S[196] := 196; BELL_ASCII_KATAKANA_S[197] := 197;
  BELL_ASCII_KATAKANA_S[198] := 198; BELL_ASCII_KATAKANA_S[199] := 199; BELL_ASCII_KATAKANA_S[200] := 200; BELL_ASCII_KATAKANA_S[201] := 201; BELL_ASCII_KATAKANA_S[202] := 202; BELL_ASCII_KATAKANA_S[203] := 203;
  BELL_ASCII_KATAKANA_S[204] := 204; BELL_ASCII_KATAKANA_S[205] := 205; BELL_ASCII_KATAKANA_S[206] := 206; BELL_ASCII_KATAKANA_S[207] := 207; BELL_ASCII_KATAKANA_S[208] := 208; BELL_ASCII_KATAKANA_S[209] := 209;
  BELL_ASCII_KATAKANA_S[210] := 210; BELL_ASCII_KATAKANA_S[211] := 211; BELL_ASCII_KATAKANA_S[212] := 212; BELL_ASCII_KATAKANA_S[213] := 213; BELL_ASCII_KATAKANA_S[214] := 214; BELL_ASCII_KATAKANA_S[215] := 215;
  BELL_ASCII_KATAKANA_S[216] := 216; BELL_ASCII_KATAKANA_S[217] := 217; BELL_ASCII_KATAKANA_S[218] := 218; BELL_ASCII_KATAKANA_S[219] := 219; BELL_ASCII_KATAKANA_S[220] := 220; BELL_ASCII_KATAKANA_S[221] := 221;
  BELL_ASCII_KATAKANA_S[222] := 222; BELL_ASCII_KATAKANA_S[223] := 223;
end;

procedure NondakutenSummarySJIS.setting(fields_list: FieldsListMany; c: Integer; substitute_for_dakuten: Byte; substitute_for_handakuten: Byte);
begin
  inherited setting(fields_list, c);

  BELL_ASCII_KATAKANA_S[substitute_for_dakuten] := BELL;
  BELL_ASCII_KATAKANA_S[substitute_for_handakuten] := BELL;

end;

procedure NondakutenSummarySJIS.setting(fields_list: FieldsListMany; c: Integer; n: array of AnsiChar; substitute_for_dakuten: Byte; substitute_for_handakuten: Byte);
begin
  inherited setting(fields_list, c, n);

  BELL_ASCII_KATAKANA_S[substitute_for_dakuten] := BELL;
  BELL_ASCII_KATAKANA_S[substitute_for_handakuten] := BELL;
end;

end.
