unit SegmentedEUC;

interface

uses
  SysUtils, FieldsArrayMany, Combo, AnsiQueue;

type EditFunction = function(s: AnsiString): AnsiString;
type PettyArray = array[0..255] of AnsiChar;

type
  SegmentedSummaryEUC = class
  protected
    a: AnsiStringQueue;
    ANSI_ASCII: AnsiString;
    ascii: AnsiString;
    ascii_byte: Byte;
    ASCII_C: AnsiChar;
    CLEAR_BIT_C: AnsiChar;
    combo: AnsiString;
    DIFFERS_FROM_JOINT: AnsiChar;
    dough: array[0..98303] of AnsiChar;
    doughLength: Integer;
    EMPTY_ANSI: AnsiString;
    f: FieldsListMany;
    field_count: Integer;
    GAP_C: AnsiChar;
    joint: AnsiString;
    joint_byte: Byte;
    JOINT_C: AnsiChar;
    manyFields: array[0..3071] of Boolean;
    pattern: array[0..3071] of AnsiChar;
    quote: AnsiString;
    quote_byte: Byte;
    QUOTE_C: AnsiChar;
    remove_separators: Boolean;
    separator: AnsiString;
    separator_byte: Byte;
    SEPARATOR_C: AnsiChar;
    SET_BIT_C: AnsiChar;
    size_of_united_items: Integer;
    procedure vector_to_array_complementing(extra: AnsiString; bond: Byte);
    procedure vector_to_array_editing(editing: EditFunction);
    procedure vector_to_array_mapping;
  public
    constructor Create;
    destructor Destroy; override;
    function compactCombo(buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
    function complementing_fields_list(buffer: array of AnsiChar; bufferLength: Integer; extra: AnsiString; bond: Byte): Integer;
    function count_characters(s: PAnsiChar; s_length: Integer): Integer;
    function countCharacters(s: PAnsiChar; s_length: Integer): Integer;
    function doughyCombo(buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
    function editing_field(buffer: array of AnsiChar; bufferLength: Integer): Integer; virtual;
    function fieldCount: Integer;
    function extracting_decimal_from_field(buffer: array of AnsiChar; bufferLength: Integer): Integer;
    function extracting_integer_from_field(buffer: array of AnsiChar; bufferLength: Integer): Integer;
    function leadingCharacters(buffer: array of AnsiChar; bufferLength: Integer; character: AnsiChar): Integer;
    function mapping_fields_list(buffer: array of AnsiChar; bufferLength: Integer): Integer;
    function quoteCode: Byte;
    function search_separator(buffer: array of AnsiChar; bufferLength: Integer; beginning: Integer): Integer;
    function segmented_summary(buffer: array of AnsiChar; bufferLength: Integer; selected: array of Boolean): Integer;
    function separatorCode: Byte;
    function smashed_to_pieces(buffer: array of AnsiChar; bufferLength: Integer; selected: array of Boolean): Integer;
    function supplying_bits(c: Integer): AnsiString;
    function surge_of_set_bits(c: Integer; interval_0: Integer; interval_1: Integer): AnsiString;
    function unite(a: AnsiStringQueue; c: ComboAppend; buffer: array of AnsiChar; bufferLength: Integer): AnsiString; overload;
    function unite(buffer: array of AnsiChar; bufferLength: Integer): AnsiString; overload;
    function unite(buffer: array of AnsiChar; bufferLength: Integer; listed: Boolean): AnsiString; overload;
    function unitedSize: Integer;
    procedure presetting(ans: AnsiString; j: Byte; s: Byte); overload;
    procedure presetting(ans: AnsiString; j: Byte; q: Byte; s: Byte; e: Boolean); overload; virtual;
    procedure presetting(ans: AnsiString; j: Byte; q: Byte; s: Byte; e: Boolean; a: Byte); overload; virtual;
    procedure setting(fields_list: FieldsListMany; c: Integer); overload;
    procedure setting(fields_list: FieldsListMany; c: Integer; pn: array of AnsiChar); overload;
    procedure setting_for_extracting(v: Currency);
  end;

const
  ANSI_N_0TO9_MINUS_SIZE = 11;
  ANSI_N_0TO9_SIZE = 10;
  CLEAR_BIT = 48; { 30h, '0' }
  COMMA_BYTE = 44; { 2Ch, ',' }
  FRACTION = 4;
  GAP_BYTE = 32; { SP, ' ' }
  H_2E = 46; { dot '.'  }
  H_30 = 48; { '0' }
  H_39 = 57; { '9' }
  H_81 = 129;
  HEAD_OF_ZERO_CURRENCY = '               0';
  MINUS_BYTE = 45;
  NOT_RECOGNIZED = -1;
  QUAD_ZERO_FRACTION = '0000';
  SET_BIT = 49; { 31h, '1'}
  SIZE_OF_HEAD = 16;
  SIZE_OF_HEAD_1 = 17; { SIZE_OF_HEAD + 1 }
  SIZE_OF_ZERO_CURRENCY = 21;
  STANDARD_QUOTE_BYTE = 34; { 22h, '"' }
  UNDERLINE_BYTE = 95; { 5Fh, '_' }
  UPPER_LIMIT_TO_FIELD_NUMBER = 3071;
  ZERO_CURRENCY = '               0.0000';

var
  ANSI_ASCII_S: AnsiString;
  ANSI_N_0TO9: AnsiString;
  ANSI_N_0TO9_MINUS: AnsiString;
  COMMA_C_S: AnsiChar;
  DOT_C_S: AnsiChar;
  EMPTY_ANSI_S: AnsiString;
  HEAD_OF_ZERO_CURRENCY_ANSI: AnsiString;
  LARGE_VALUE: Currency;
  MINUS_C_S: AnsiString;
  QUAD_ZERO_FRACTION_ANSI: AnsiString;
  ZERO_CURRENCY_ANSI: AnsiString;

implementation

function leadingNumbers(full: AnsiString): Integer;
var
  c: AnsiChar;
  count: Integer;
  found: Boolean;
  fullLength: Integer;
  k: Integer;
  m: Integer;
begin
  count := 0;
  fullLength := Length(full);

  for k := 1 to fullLength do
  begin
    c := full[k];

{ Checking if a number appears. }
    found := False;
    for m := 1 to ANSI_N_0TO9_SIZE do
    begin
      if ANSI_N_0TO9[m] = c then
        begin
          found := True;
          break;
        end
      else
        begin
          continue
        end;

    end;

    if found then
      begin
{ Counting numbers }
        Inc(count);
        continue
      end
    else
      begin
        break;
      end;

  end;

  Result := count;
end;

function positiveOrNegativeNumbers(full: AnsiString): Integer;
var
  c: AnsiChar;
  count: Integer;
  found: Boolean;
  fullLength: Integer;
  k: Integer;
  m: Integer;
begin
  count := 0;
  fullLength := Length(full);

  k := 1;
  c := full[k];

{ Checking if a number or minus appears. }
  found := False;
  for m := 1 to ANSI_N_0TO9_MINUS_SIZE do
  begin
    if ANSI_N_0TO9_MINUS[m] = c then
      begin
        found := True;
        break;
      end
    else
      begin
        continue
      end;

  end;

  if found then
    begin
{ Counting numbers }
      Inc(count);

      for k := 2 to fullLength do
      begin
        c := full[k];

{ Checking if a number appears. }
        found := False;
        for m := 1 to ANSI_N_0TO9_SIZE do
        begin
          if ANSI_N_0TO9[m] = c then
            begin
              found := True;
              break;
            end
          else
            begin
              continue
            end;

        end;

        if found then
          begin
{ Counting numbers }
            Inc(count);
            continue
          end
        else
          begin
            break;
          end; { if found then, else }

      end; { for k := 2 to fullLength do }

    end; { if found then }

  Result := count;
end;

function countingNonnumbers(full: AnsiString): Integer;
var
  c: AnsiChar;
  count: Integer;
  found: Boolean;
  fullLength: Integer;
  k: Integer;
  m: Integer;
  trace: AnsiChar;
begin
  c := COMMA_C_S;
  count := 0;
  fullLength := Length(full);
  trace := c;

  for k := 1 to fullLength do
  begin
    trace := c;
    c := full[k];

{ Checking if a number appears. }
    found := False;
    for m := 1 to ANSI_N_0TO9_SIZE do
    begin
      if ANSI_N_0TO9[m] = c then
        begin
          found := True;
          break;
        end
      else
        begin
          continue
        end;

    end;

    if found then
      begin
        break;
      end
    else
      begin
{ Counting nonnumbers }
        Inc(count);
        continue
      end;

  end;

  if trace = MINUS_C_S then
    begin
      count := count - 1;
    end;

  Result := count;
end;

function trailingNumbers(full: AnsiString): Integer;
var
  c: AnsiString;
  found: Boolean;
  full_length: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
begin
  full_length := Length(full);
  n := 0;
  for k := full_length downto 1 do
  begin
    c := full[k];

{ Checking if a number appears. }
    found := False;
    for m := 1 to ANSI_N_0TO9_SIZE do
    begin
      if ANSI_N_0TO9[m] = c then
        begin
          found := True;
          break;
        end
      else
        begin
          continue
        end;

    end;

    if found then
      begin
        Inc(n);
        continue;
      end
    else
      begin
        break;
      end;

  end;

{ Checking if a number or minus appears. }
  if ANSI_N_0TO9_MINUS[ANSI_N_0TO9_MINUS_SIZE] = c then
    begin
      n := n + 1;
    end;

  Result := n;
end;

function extract_decimal(field: AnsiString): AnsiString;
var
  b: Byte;
  count: Integer;
  ensemble: array[0..21] of AnsiChar;
  fieldLength: Integer;
  found: Boolean;
  head: AnsiString;
  i: Longint;
  k: Integer;
  p: Integer;
  pascal_string: AnsiString;
  r: AnsiString;
  s: String;
  s_length: Integer;
  tail: AnsiString;
  tailLength: Integer;
  trace: Integer;
  value: Currency;
  w: PAnsiChar;
  z: AnsiString;
begin
  fieldLength := Length(field);

{ Checking if a dot appears. }
  found := False;
  for p := 1 to fieldLength do
  begin
    if field[p] = DOT_C_S then
      begin
        found := True;
        break;
      end
    else
      begin
        continue
      end;

  end;

  if found then
    begin
      trace := p;
      p := p - 1;
      head := Copy(field, 1, p);

      tailLength := fieldLength - trace;
      p := trace + 1;
      tail := Copy(field, p, tailLength);
      count := leadingNumbers(tail);
      if count = 0 then
        begin
          tail := QUAD_ZERO_FRACTION;
        end
      else
        begin
          if count < FRACTION then
            begin
              z := Copy(tail, 1, count);
              z := z + QUAD_ZERO_FRACTION; { Adding zeros }
              tail := Copy(z, 1, FRACTION);
            end
          else
            begin
              tail := Copy(tail, 1, FRACTION);
            end;

        end;

      count := trailingNumbers(head);
      if count = 0 then
        begin
          head := HEAD_OF_ZERO_CURRENCY_ANSI;
        end
      else
        begin
          p := trace - count;
          head := Copy(head, p, count);

          k := SIZE_OF_HEAD - count;
          z := Copy(HEAD_OF_ZERO_CURRENCY_ANSI, 1, k);
          head := z + head; { Adding space }
        end;

      r := head;
      r := r + DOT_C_S;
      r := r + tail;
    end
  else
    begin
      r := ZERO_CURRENCY_ANSI;
    end;

  w := StrPCopy(ensemble, r);
  TextToFloat(w, value, fvCurrency);
  value := value + LARGE_VALUE;
  s := CurrToStr(value);

  s_length := Length(s);
  k := 0;
  for p := 1 to s_length do
  begin
    i := Ord(s[p]);
    b := Byte(i);
    ensemble[k] := ANSI_ASCII_S[b];
    Inc(k);
  end;

  pascal_string := ensemble;
  fieldLength := k;
  field := Copy(pascal_string, 1, fieldLength);

  found := False;
  for p := 1 to fieldLength do
  begin
    if field[p] = DOT_C_S then
      begin
        found := True;
        break;
      end
    else
      begin
        continue
      end;

  end;

  if found then
    begin
      trace := p;
      p := p - 1;
      head := Copy(field, 1, p);

      tailLength := fieldLength - trace;
      p := trace + 1;
      tail := Copy(field, p, tailLength);
      count := leadingNumbers(tail);
      if count = 0 then
        begin
          tail := QUAD_ZERO_FRACTION;
        end
      else
        begin
          if count < FRACTION then
            begin
              z := Copy(tail, 1, count);
              z := z + QUAD_ZERO_FRACTION;
              tail := Copy(z, 1, FRACTION);
            end
          else
            begin
              tail := Copy(tail, 1, FRACTION);
            end;

        end;

      count := trailingNumbers(head);
      if count = 0 then
        begin
          head := HEAD_OF_ZERO_CURRENCY_ANSI;
        end
      else
        begin
          p := trace - count;
          head := Copy(head, p, count);

          k := SIZE_OF_HEAD - count;
          z := Copy(HEAD_OF_ZERO_CURRENCY_ANSI, 1, k);
          head := z + head;
        end;

      r := head;
      r := r + DOT_C_S;
      r := r + tail;
    end
  else
    begin
      r := field + DOT_C_S;
      r := field + QUAD_ZERO_FRACTION;
    end;

  Result := r;
end;

function extract_integer(field: AnsiString): AnsiString;
var
  b: Byte;
  count: Integer;
  ensemble: array[0..21] of AnsiChar;
  fieldLength: Integer;
  found: Boolean;
  from: Integer;
  head: AnsiString;
  i: Longint;
  k: Integer;
  p: Integer;
  pascal_string: AnsiString;
  r: AnsiString;
  s: String;
  s_length: Integer;
  tail: AnsiString;
  tailLength: Integer;
  trace: Integer;
  value: Currency;
  w: PAnsiChar;
  z: AnsiString;
begin
  r := EMPTY_ANSI_S;
  fieldLength := Length(field);
  p := countingNonnumbers(field);
  if p = fieldLength then { countingNonnumbers() cannot find any number. }
    begin
      r := ZERO_CURRENCY;
    end
  else
    begin
      from := p + 1;
      tailLength := fieldLength - p;
      tail := Copy(field, from, tailLength);
      count := positiveOrNegativeNumbers(tail);
      r := Copy(tail, 1, count);
      if count = 1 then
        begin
          i := Ord(r[1]);
          if i = MINUS_BYTE then
            begin
              r := ZERO_CURRENCY;
            end
          else
            begin
              k := SIZE_OF_HEAD - count;
              z := Copy(HEAD_OF_ZERO_CURRENCY_ANSI, 1, k);
              head := z + r; { Adding space }
              r := head;
              r := r + DOT_C_S;
              r := r + QUAD_ZERO_FRACTION;
            end;

        end
      else
        begin
          k := SIZE_OF_HEAD - count;
          z := Copy(HEAD_OF_ZERO_CURRENCY_ANSI, 1, k);
          head := z + r; { Adding space }
          r := head;
          r := r + DOT_C_S;
          r := r + QUAD_ZERO_FRACTION;
        end;

    end;

  w := StrPCopy(ensemble, r);
  TextToFloat(w, value, fvCurrency);
  value := value + LARGE_VALUE;
  s := CurrToStr(value);

  s_length := Length(s);
  k := 0;
  for p := 1 to s_length do
  begin
    i := Ord(s[p]);
    b := Byte(i);
    ensemble[k] := ANSI_ASCII_S[b];
    Inc(k);
  end;

  pascal_string := ensemble;
  fieldLength := k;
  field := Copy(pascal_string, 1, fieldLength);

{ Checking if a dot appears. }
  found := False;
  for p := 1 to fieldLength do
  begin
    if field[p] = DOT_C_S then
      begin
        found := True;
        break;
      end
    else
      begin
        continue
      end;

  end;

  if found then
    begin
      trace := p;
      p := p - 1;
      head := Copy(field, 1, p);

      tailLength := fieldLength - trace;
      p := trace + 1;
      tail := Copy(field, p, tailLength);
      count := leadingNumbers(tail);
      if count = 0 then
        begin
          tail := QUAD_ZERO_FRACTION;
        end
      else
        begin
          if count < FRACTION then
            begin
              z := Copy(tail, 1, count);
              z := z + QUAD_ZERO_FRACTION; { Adding zeros }
              tail := Copy(z, 1, FRACTION);
            end
          else
            begin
              tail := Copy(tail, 1, FRACTION);
            end;

        end;

      count := trailingNumbers(head);
      if count = 0 then
        begin
          head := HEAD_OF_ZERO_CURRENCY_ANSI;
        end
      else
        begin
          p := trace - count;
          head := Copy(head, p, count);

          k := SIZE_OF_HEAD - count;
          z := Copy(HEAD_OF_ZERO_CURRENCY_ANSI, 1, k);
          head := z + head; { Adding space }
        end;

      r := head;
      r := r + DOT_C_S;
      r := r + tail;
    end
  else
    begin
      r := field + DOT_C_S;
      r := field + QUAD_ZERO_FRACTION;
    end;

  Result := r;
end;

function transparent(field: AnsiString): AnsiString;
begin
  Result := field;
end;

constructor SegmentedSummaryEUC.Create;
begin
  inherited Create;

  quote_byte := 34; { 22h }
  separator_byte := 44; { 2Ch }
end;

destructor SegmentedSummaryEUC.Destroy;
begin
  a.Free;

  inherited Destroy;
end;

function SegmentedSummaryEUC.compactCombo(buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
var
  from: Integer;
  pascal_string: AnsiString;
  space_width: Integer;
begin
  segmented_summary(buffer, bufferLength, manyFields);

  space_width := leadingCharacters(dough, doughLength, GAP_C);
  size_of_united_items := doughLength - space_width;
  pascal_string := dough;
  from := space_width + 1;
  combo := Copy(pascal_string, from, size_of_united_items);
  Result := combo;
end;

function SegmentedSummaryEUC.complementing_fields_list(buffer: array of AnsiChar; bufferLength: Integer; extra: AnsiString; bond: Byte): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_complementing(extra, bond);
  Result := field_count;
end;

function SegmentedSummaryEUC.count_characters(s: PAnsiChar; s_length: Integer): Integer;
var
  s_length_1: Integer;
  c: AnsiChar;
  i: Longint;
  k: Integer;
  kanji: Boolean;
  n: Byte;
  r: Integer;
begin
  kanji := False;
  r := 0;
  s_length_1 := s_length - 1;

  for k := 0 to s_length_1 do
  begin
    c := s[k];
    i := Ord(c);
    n := Byte(i);

    if kanji then
      begin
        Inc(r);
        kanji := False;
        continue;
      end;

    if H_81 <= n then
      begin
        kanji := True;
        continue;
      end;

{ ASCII }
    Inc(r);
  end;

  Result := r;
end;

function SegmentedSummaryEUC.countCharacters(s: PAnsiChar; s_length: Integer): Integer;
var
  r: Integer;
begin
  r := count_characters(s, s_length);

  Result := r;
end;

function SegmentedSummaryEUC.doughyCombo(buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
var
  pascal_string: AnsiString;
begin
  segmented_summary(buffer, bufferLength, manyFields);

  pascal_string := dough;
  combo := Copy(pascal_string, 1, doughLength);
  size_of_united_items := doughLength;
  Result := combo;
end;

function SegmentedSummaryEUC.editing_field(buffer: array of AnsiChar; bufferLength: Integer): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_editing(transparent);
  Result := field_count;
end;

function SegmentedSummaryEUC.fieldCount: Integer;
begin
  Result := field_count;
end;

function SegmentedSummaryEUC.extracting_decimal_from_field(buffer: array of AnsiChar; bufferLength: Integer): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_editing(extract_decimal);
  Result := field_count;
end;

function SegmentedSummaryEUC.extracting_integer_from_field(buffer: array of AnsiChar; bufferLength: Integer): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_editing(extract_integer);
  Result := field_count;
end;

function SegmentedSummaryEUC.leadingCharacters(buffer: array of AnsiChar; bufferLength: Integer; character: AnsiChar): Integer;
var
  c: AnsiChar;
  k: Integer;
  bufferLength_1: Integer;
begin
  bufferLength_1 := bufferLength - 1;
  for k := 0 to bufferLength_1 do
  begin
    c := buffer[k];
    if c = character then
      begin
        continue;
      end
    else
      begin
        break;
      end;

  end;

  Result := k;
end;

function SegmentedSummaryEUC.mapping_fields_list(buffer: array of AnsiChar; bufferLength: Integer): Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  vector_to_array_mapping;
  Result := field_count;
end;

function SegmentedSummaryEUC.quoteCode: Byte;
begin
  Result := quote_byte;
end;

function SegmentedSummaryEUC.search_separator(buffer: array of AnsiChar; bufferLength: Integer; beginning: Integer): Integer;
var
  i: Longint;
  k: Integer;
  n: Byte;
  no_splitting: Boolean;
  bufferLength_1: Integer;
  r: Integer;
begin
  no_splitting := False;
  bufferLength_1 := bufferLength - 1;
  r := NOT_RECOGNIZED;

  for k := 0 to bufferLength_1 do
  begin
    i := Ord(buffer[k]);
    n := Byte(i);

    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
          end;

        continue; { Commas disappear if placed in quotes. }
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
      end
    else
      begin
        if n = separator_byte then
          begin
            r := k;
            break;
          end;

      end;

  end;

  Result := r;
end;

function SegmentedSummaryEUC.segmented_summary(buffer: array of AnsiChar; bufferLength: Integer; selected: array of Boolean): Integer;
var
  appended: AnsiChar;
  b: Integer;
  bufferLength_1: Integer;
  c: AnsiChar;
  field_number: Integer;
  i: Longint;
  k: Integer;
  n: Byte;
  no_splitting: Boolean;
begin
  appended := DIFFERS_FROM_JOINT;
  b := 0;
  field_number := 1;
  no_splitting := False;
  bufferLength_1 := bufferLength - 1;

  for k := 0 to bufferLength_1 do
  begin
    c := buffer[k];
    i := Ord(c);
    n := Byte(i);

    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
            if selected[field_number] then
              begin
                dough[b] := c;
                Inc(b);
                appended := c;
              end;

          end
        else
          begin
            if n = separator_byte then
              begin
                if remove_separators then
                  begin
                    continue;
                  end
                else
                  begin
                    if selected[field_number] then
                      begin
                        dough[b] := c;
                        Inc(b);
                        appended := c;
                      end;

                  end;

              end
            else
              begin
                if selected[field_number] then
                  begin
                    dough[b] := c;
                    Inc(b);
                    appended := c;
                  end;

              end;

          end;

        continue; { Commas disappear if placed in quotes. }
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
        if selected[field_number] then
          begin
            dough[b] := c;
            Inc(b);
            appended := c;
          end;

      end
    else
      begin
        if n = separator_byte then
          begin
            if selected[field_number] then
              begin
                dough[b] := JOINT_C;
                Inc(b);
                appended := JOINT_C;
              end;

            field_number := field_number + 1;
            if field_number > field_count then
              begin
                break;
              end;

          end
        else
          begin
            if selected[field_number] then
              begin
                dough[b] := c;
                Inc(b);
                appended := c;
              end;

          end;

      end;

  end;

  if appended = JOINT_C then
    begin
      field_number := field_number - 1;
    end;

  field_count := field_number;
  doughLength := b;
  combo := Copy(dough, 1, doughLength); { doughyCombo }
  Result := doughLength;
end;

function SegmentedSummaryEUC.separatorCode: Byte;
begin
  Result := separator_byte;
end;

function SegmentedSummaryEUC.smashed_to_pieces(buffer: array of AnsiChar; bufferLength: Integer; selected: array of Boolean): Integer;
var
  b: Integer;
  bufferLength_1: Integer;
  c: AnsiChar;
  field_number: Integer;
  i: Longint;
  k: Integer;
  n: Byte;
  no_splitting: Boolean;
  pascal_string: AnsiString;
  z: AnsiString;
begin
  b := 0;
  field_number := 1;
  no_splitting := False;
  bufferLength_1 := bufferLength - 1;

  for k := 0 to bufferLength_1 do
  begin
    c := buffer[k];
    i := Ord(c);
    n := Byte(i);

    if no_splitting then
      begin
        if n = quote_byte then
          begin
            no_splitting := False;
            if selected[field_number] then
              begin
                dough[b] := c;
                Inc(b);
              end;

          end
        else
          begin
            if n = separator_byte then
              begin
                if remove_separators then
                  begin
                    continue; { Commas disappear if placed in quotes. }
                  end
                else
                  begin
                    if selected[field_number] then
                      begin
                        dough[b] := c;
                        Inc(b);
                      end;

                  end;

              end
            else
              begin
                if selected[field_number] then
                  begin
                    dough[b] := c;
                    Inc(b);
                  end;

              end;

          end;

        continue;
      end;

{ splitting }
    if n = quote_byte then
      begin
        no_splitting := True;
        if selected[field_number] then
          begin
            dough[b] := c;
            Inc(b);
          end;

      end
    else
      begin
        if n = separator_byte then
          begin
            if selected[field_number] then
              begin
                pascal_string := dough;
                doughLength := b;
                z := Copy(pascal_string, 1, doughLength);
                a.push_back(z);
              end;

            b := 0;
            field_number := field_number + 1;
            if field_number > field_count then
              begin
                break;
              end;

          end
        else
          begin
            if selected[field_number] then
              begin
                dough[b] := c;
                Inc(b);
              end;

          end;

      end;

  end;

  if b > 0 then
    begin
      pascal_string := dough;
      doughLength := b;
      z := Copy(pascal_string, 1, doughLength);
      a.push_back(z);
    end
  else
    begin
      field_number := field_number - 1;
    end;

  field_count := field_number;
  Result := b;
end;

function SegmentedSummaryEUC.supplying_bits(c: Integer): AnsiString;
begin
  field_count := c;
  if field_count > UPPER_LIMIT_TO_FIELD_NUMBER then
    field_count := UPPER_LIMIT_TO_FIELD_NUMBER;

  Result := StringOfChar(SET_BIT_C, field_count);
end; 

function SegmentedSummaryEUC.surge_of_set_bits(c: Integer; interval_0: Integer; interval_1: Integer): AnsiString;
var
  body: AnsiString;
  bodyLength: Integer;
  head: AnsiString;
  headLength: Integer;
  tail: AnsiString;
  tailLength: Integer;
  z: AnsiString;
begin
{*
  surge_of_set_bits
0011111000000000000, for example
Setting bits from interval_0 to interval_1
*}

  field_count := c;
  if interval_1 > field_count then
    interval_1 := field_count;

  bodyLength := interval_1 - interval_0 + 1;
  body := StringOfChar(SET_BIT_C, bodyLength);

  headLength := interval_0 - 1;
  head := StringOfChar(CLEAR_BIT_C, headLength);

  tailLength := field_count - interval_1;
  tail := StringOfChar(CLEAR_BIT_C, tailLength);

  z := head;
  z := z + body;
  z := z + tail;
  Result := z;
end;

function SegmentedSummaryEUC.unite(a: AnsiStringQueue; c: ComboAppend; buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
var
  item_count_1: Integer;
  n: Integer;
  g: AnsiString;
begin
  if a.count = 0 then
    mapping_fields_list(buffer, bufferLength);

  item_count_1 := f.itemCount - 1;
  for n := 0 to item_count_1 do
  begin
    g := f.item(n);
    c.append(g);
    c.append(joint);
  end;

  size_of_united_items := Length(c.built);
  Result := c.built;
end;

function SegmentedSummaryEUC.unite(buffer: array of AnsiChar; bufferLength: Integer): AnsiString;
var
  a_count_1: Integer;
  i: Integer;
begin
  smashed_to_pieces(buffer, bufferLength, manyFields);

  a_count_1 := a.count - 1;
  combo := EMPTY_ANSI;
  for i := 0 to a_count_1 do
  begin
    combo := combo + a.q(i);
    combo := combo + joint;
  end;

  size_of_united_items := Length(combo);
  Result := combo;
end;

function SegmentedSummaryEUC.unite(buffer: array of AnsiChar; bufferLength: Integer; listed: Boolean): AnsiString;
var
  item_count_1: Integer;
  n: Integer;
begin
  combo := EMPTY_ANSI;

  if not listed then
    mapping_fields_list(buffer, bufferLength);

  item_count_1 := f.itemCount - 1;
  for n := 0 to item_count_1 do
  begin
    combo := combo + f.item(n);
    combo := combo + joint;
  end;

  size_of_united_items := Length(combo);
  Result := combo;
end;

function SegmentedSummaryEUC.unitedSize: Integer;
begin
  Result := size_of_united_items;
end;

procedure SegmentedSummaryEUC.presetting(ans: AnsiString; j: Byte; s: Byte);
var
  n: Integer;
begin
  ANSI_ASCII := ans;
  ANSI_ASCII_S := ans;
  ascii_byte := UNDERLINE_BYTE;
  joint_byte := j;
  quote_byte := STANDARD_QUOTE_BYTE;
  separator_byte := s;
  remove_separators := False;

  EMPTY_ANSI := '';
  EMPTY_ANSI_S := '';

  ASCII_C := ANSI_ASCII[ascii_byte];
  ascii := EMPTY_ANSI + ASCII_C;
  JOINT_C := ANSI_ASCII[joint_byte];
  joint := EMPTY_ANSI + JOINT_C;
  QUOTE_C := ANSI_ASCII[quote_byte];
  quote := EMPTY_ANSI + QUOTE_C;
  SEPARATOR_C := ANSI_ASCII[separator_byte];
  separator := EMPTY_ANSI + SEPARATOR_C;

  n := Integer(joint_byte);
  n := n + 1;
  DIFFERS_FROM_JOINT := ANSI_ASCII[n];

  CLEAR_BIT_C := ANSI_ASCII[CLEAR_BIT];
  COMMA_C_S := ANSI_ASCII[COMMA_BYTE];
  DOT_C_S := ANSI_ASCII[H_2E];
  GAP_C := ANSI_ASCII[GAP_BYTE];
  MINUS_C_S := ANSI_ASCII[MINUS_BYTE];
  SET_BIT_C := ANSI_ASCII[SET_BIT];
end;

procedure SegmentedSummaryEUC.presetting(ans: AnsiString; j: Byte; q: Byte; s: Byte; e: Boolean);
var
  n: Integer;
begin
  ANSI_ASCII := ans;
  ANSI_ASCII_S := ans;
  ascii_byte := UNDERLINE_BYTE;
  joint_byte := j;
  quote_byte := q;
  separator_byte := s;
  remove_separators := e;

  EMPTY_ANSI := '';
  EMPTY_ANSI_S := '';

  ASCII_C := ANSI_ASCII[ascii_byte];
  ascii := EMPTY_ANSI + ASCII_C;
  JOINT_C := ANSI_ASCII[joint_byte];
  joint := EMPTY_ANSI + JOINT_C;
  QUOTE_C := ANSI_ASCII[quote_byte];
  quote := EMPTY_ANSI + QUOTE_C;
  SEPARATOR_C := ANSI_ASCII[separator_byte];
  separator := EMPTY_ANSI + SEPARATOR_C;

  n := Integer(joint_byte);
  n := n + 1;
  DIFFERS_FROM_JOINT := ANSI_ASCII[n];

  CLEAR_BIT_C := ANSI_ASCII[CLEAR_BIT];
  COMMA_C_S := ANSI_ASCII[COMMA_BYTE];
  DOT_C_S := ANSI_ASCII[H_2E];
  GAP_C := ANSI_ASCII[GAP_BYTE];
  MINUS_C_S := ANSI_ASCII[MINUS_BYTE];
  SET_BIT_C := ANSI_ASCII[SET_BIT];
end;

procedure SegmentedSummaryEUC.presetting(ans: AnsiString; j: Byte; q: Byte; s: Byte; e: Boolean; a: Byte);
var
  n: Integer;
begin
  ANSI_ASCII := ans;
  ANSI_ASCII_S := ans;
  ascii_byte := a;
  joint_byte := j;
  quote_byte := q;
  separator_byte := s;
  remove_separators := e;

  EMPTY_ANSI := '';
  EMPTY_ANSI_S := '';

  ASCII_C := ANSI_ASCII[ascii_byte];
  ascii := EMPTY_ANSI + ASCII_C;
  JOINT_C := ANSI_ASCII[joint_byte];
  joint := EMPTY_ANSI + JOINT_C;
  QUOTE_C := ANSI_ASCII[quote_byte];
  quote := EMPTY_ANSI + QUOTE_C;
  SEPARATOR_C := ANSI_ASCII[separator_byte];
  separator := EMPTY_ANSI + SEPARATOR_C;

  n := Integer(joint_byte);
  n := n + 1;
  DIFFERS_FROM_JOINT := ANSI_ASCII[n];

  CLEAR_BIT_C := ANSI_ASCII[CLEAR_BIT];
  COMMA_C_S := ANSI_ASCII[COMMA_BYTE];
  DOT_C_S := ANSI_ASCII[H_2E];
  GAP_C := ANSI_ASCII[GAP_BYTE];
  MINUS_C_S := ANSI_ASCII[MINUS_BYTE];
  SET_BIT_C := ANSI_ASCII[SET_BIT];
end;

procedure SegmentedSummaryEUC.vector_to_array_complementing(extra: AnsiString; bond: Byte);
var
  a_count_1: Integer;
  s: AnsiString;
  i: Integer;
begin
  a_count_1 := a.count - 1;
  for i := 0 to a_count_1 do
  begin
    s := a.q(i);
    s := s + ANSI_ASCII[bond];
    s := s + extra;
    f.setting(s, i);
  end;

end;

procedure SegmentedSummaryEUC.vector_to_array_editing(editing: EditFunction);
var
  a_count_1: Integer;
  i: Integer;
  s: AnsiString;
begin
  a_count_1 := a.count - 1;
  for i := 0 to a_count_1 do
  begin
    s := editing( a.q(i) );
    f.setting(s, i);
  end;

end;

procedure SegmentedSummaryEUC.vector_to_array_mapping;
var
  a_count_1: Integer;
  i: Integer;
begin
  a_count_1 := a.count - 1;
  for i := 0 to a_count_1 do
  begin
    f.setting( a.q(i), i);
  end;

end;

procedure SegmentedSummaryEUC.setting(fields_list: FieldsListMany; c: Integer);
var
  m: Integer;
  z: AnsiString;
begin
  f := fields_list;
  field_count := c;

  manyFields[0] := True;

  for m:= 1 to field_count do
  begin
    manyFields[m] := True;
  end;

  a := AnsiStringQueue.Create;
  z := EMPTY_ANSI + GAP_C;
  a.setting(ANSI_ASCII, z, 1);
end;

procedure SegmentedSummaryEUC.setting(fields_list: FieldsListMany; c: Integer; pn: array of AnsiChar);
var
  b: Byte;
  k: Integer;
  m: Integer;
  i: Longint;
  z: AnsiString;
begin
  f := fields_list;
  field_count := c;

  manyFields[0] := True;

  for m:= 1 to field_count do
  begin
    k := m - 1;
    pattern[k] := pn[k];
    i := Ord( pattern[k] );
    b := Byte(i);

    if b = SET_BIT then
      manyFields[m] := True
    else
      manyFields[m] := False;

  end;

  a := AnsiStringQueue.Create;
  z := EMPTY_ANSI + GAP_C;
  a.setting(ANSI_ASCII, z, 1);
end;

procedure SegmentedSummaryEUC.setting_for_extracting(v: Currency);
var
  CLEAR_PETTY_ARRAY: PettyArray;
  i: Longint;
  k: Integer;
  m: Integer;
  p: PettyArray;
  pascal_string: AnsiString;
begin
  LARGE_VALUE := v;

  for m := 0 to 255 do
  begin
    CLEAR_PETTY_ARRAY[m] := #0;
    p[m] := #0;
  end;

  m := 0;
  for k := 1 to FRACTION do
  begin
    i := Ord(QUAD_ZERO_FRACTION[k]);
    p[m] := ANSI_ASCII[i];
    Inc(m);
  end;

  pascal_string := p;
  QUAD_ZERO_FRACTION_ANSI := Copy(pascal_string, 1, FRACTION);

  p := CLEAR_PETTY_ARRAY;
  m := 0;
  for k := H_30 to H_39 do
  begin
    p[m] := ANSI_ASCII[k];
    Inc(m);
  end;

  pascal_string := p;
  ANSI_N_0TO9 := Copy(pascal_string, 1, ANSI_N_0TO9_SIZE);

  p[m] := ANSI_ASCII[MINUS_BYTE];
  pascal_string := p;
  ANSI_N_0TO9_MINUS := Copy(pascal_string, 1, ANSI_N_0TO9_MINUS_SIZE);

  p := CLEAR_PETTY_ARRAY;
  m := 0;
  for k := 1 to SIZE_OF_HEAD do
  begin
    i := Ord(HEAD_OF_ZERO_CURRENCY[k]);
    p[m] := ANSI_ASCII[i];
    Inc(m);
  end;

  pascal_string := p;
  HEAD_OF_ZERO_CURRENCY_ANSI := Copy(pascal_string, 1, SIZE_OF_HEAD);
{*
  HEAD_OF_ZERO_CURRENCY = '               0';
  ZERO_CURRENCY         = '               0.0000';
*}
  for k := SIZE_OF_HEAD_1 to SIZE_OF_ZERO_CURRENCY do
  begin
    i := Ord(ZERO_CURRENCY[k]);
    p[m] := ANSI_ASCII[i];
    Inc(m);
  end;

  pascal_string := p;
  ZERO_CURRENCY_ANSI := Copy(pascal_string, 1, SIZE_OF_ZERO_CURRENCY);
end;

end.
