program Sortk;
{$APPTYPE CONSOLE}

{*
Sortk Delphi Version 1.08
Comparing phrases from Field-2 to Field-3, for example, to arrange lines in descending order
Copyright 2020 Sogaya

Delphi 2009

*}

uses
  Windows,
  SysUtils,
  AnsiQueue in 'AnsiQueue.pas',
  Block in 'Block.pas',
  CaseSensitiveDescendingUnique in 'CaseSensitiveDescendingUnique.pas',
  CaseSensitiveUnique in 'CaseSensitiveUnique.pas',
  Combo in 'Combo.pas',
  ConvertUTF16 in 'ConvertUTF16.pas',
  Descending in 'Descending.pas',
  DescendingUnique in 'DescendingUnique.pas',
  Divide in 'Divide.pas',
  ErrorText in 'ErrorText.pas',
  FieldsArray in 'FieldsArray.pas',
  FieldsArrayMany in 'FieldsArrayMany.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  PathEXE in 'PathEXE.pas',
  ReportID in 'ReportID.pas',
  SearchBinary in 'SearchBinary.pas',
  SurvivorUTF16 in 'SurvivorUTF16.pas',
  TimeName in 'TimeName.pas',
  Ucombo in 'Ucombo.pas',
  Unique in 'Unique.pas',
  MaxSpan in 'MaxSpan.pas',
  StandardIO in 'StandardIO.pas',
  SegmentedEUC in 'SegmentedEUC.pas',
  NoncontrollerEUC in 'NoncontrollerEUC.pas',
  NondakutenEUC in 'NondakutenEUC.pas',
  SummaryEUC in 'SummaryEUC.pas',
  NoncontrollerSJIS in 'NoncontrollerSJIS.pas',
  NondakutenSJIS in 'NondakutenSJIS.pas',
  SegmentedSJIS in 'SegmentedSJIS.pas',
  SummarySJIS in 'SummarySJIS.pas',
  DivideSJIS in 'DivideSJIS.pas',
  PutCSV in 'PutCSV.pas',
  ConvertWide in 'ConvertWide.pas',
  ConvertWideEUC in 'ConvertWideEUC.pas',
  SparseUTF16 in 'SparseUTF16.pas',
  RuralUTF16 in 'RuralUTF16.pas',
  Utf16intoEUC in 'Utf16intoEUC.pas',
  PastoralUTF16 in 'PastoralUTF16.pas',
  ConvertEUC in 'ConvertEUC.pas',
  SurvivorEUC in 'SurvivorEUC.pas';

{$R *.res}

const
  ARRANGE_LINES_IN_ASCENDING_ORDER = False;
  BUFFER_ERROR = -1;
  CASE_SENSITIVE_SEARCH = '-f';
  COMBINING_CONTROLLING = 0;
  COMMA_BYTE = 44; { ',' }
  CRLF = True;
  DATA_TEXT = 'data.txt';
  DENIED = '-';
  DESCENDING = '-r';
  DO_NOT_REMOVE_SEPARATORS = False;
  DO_NOT_REMOVE_TEMPORARY_FILE = '-y';
  EMPTY = '';
  EUC_JP_TERM = '-e';
  FINAL_INTEGER = 2147483647; { 7FFFFFFFh }
  H_1F = 31; { US }
  H_20 = 32; { SP, GAP }
  H_22 = 34; { " }
  H_2C = 44; { , }
  H_3C = 60; { < }
  H_5C = 92; { '\' }
  INCLUDE_0DH = '-0';
  JOINT_TERM = '-V';
  LINE_FEED = '-L';
  LINE_FOR_EACH_UNIQUE_SUMMARY = '-u';
  MAXIMUM_LINE_COUNT = '-U';
  NEVER_READ_UTF16_FILE = False;
  NEW_LINE_OF_CR_LF = True;
  NOT_SUBSTITUTED = False;
  SET_BIT = 49; { 31h, '1'}
  SETTING_TEXT = 'manager.txt';
  SHIFT_JIS_TERM = '-s';
  SKIP_COMBINING = 1;
  SKIP_CONTROLLING = 2;
  STANDARD_OUTPUT = '';
  STANDARD_VALUE = 562949953421312.0001; { 2^49 + 0.0001 }
  SURRO_CSV = 'surro.csv';
  UNDERLINES_REPLACE_SURROGATE_PAIRS = True;
  SUTAI_TEXT = 'sutai.txt';
  TEMPORARY_TEXT = 'text.tmp';
  TERM_1_FILE_NAME = '-o';
  UPPER_LIMIT_TO_FIELD_NUMBER = 3071;
  VERSION_NUMBER = '1.08';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;
  YEN = '\';
  YEN_X = '\x';

var
  b: CurrencyBlock;
  buffer: array[0..4095] of Char;
  c: DescendingUniqueBlock;
  carriage_return: Boolean;
  e: DescendingBlock;
  EUC_text: Boolean;
  deleteFile: File;
  f: FinalEffects;
  field_count: Integer;
  file_name: String;
  file_name_length: Integer;
  final_separator: Byte;
  flood_ebb: String;
  g: Guiding;
  h: HyphenTerms;
  i: CaseSensitiveUniqueBlock;
  j: SummaryDetailsSJIS;
  k: Integer;
  line_count: Integer;
  list_name: String;
  list_name_length: Integer;
  m: MeasureSpan;
  maximumFieldCount: Integer;
  multipattern: String;
  n: UniqueBlock;
  p: PutSurroCSV;
  pascal_string: String;
  pattern: String;
  q: CaseSensitiveDescendingUniqueBlock;
  s: String;
  SET_BIT_C: Char;
  setFile: TextFile;
  shared_sum: Integer;
  shift_jis_text: Boolean;
  t: CommandPromptIO;
  temporaryFile: File;
  testFile: File;
  x: ErrorTexts;
  y: SummaryDetailsEUC;
  yen_byte: Longint;

begin
{ Checking if software can open SURRO_CSV in the current directory. }
  try
    AssignFile(testFile, SURRO_CSV);
    Reset(testFile);
    CloseFile(testFile);
  except
{ Producing SURRO_CSV }
    p := PutSurroCSV.Create;
    p.setting(SURRO_CSV);
  end;

  s := ParamStr(1);
  if Length(s) = 0 then
    begin
{ Producing TEMPORARY_TEXT from standard input }
      t := CommandPromptIO.Create;
      t.produceTemporaryTextFile;

{ pattern }
      SET_BIT_C := Chr(SET_BIT);
      s := StringOfChar(SET_BIT_C, UPPER_LIMIT_TO_FIELD_NUMBER); { 111111111... }

{ 20h is used to separate fields. }
      j := SummaryDetailsSJIS.Create;
      j.copyingPatternFromStringToArray( PChar(s), Length(s), UPPER_LIMIT_TO_FIELD_NUMBER);
      j.setting(TEMPORARY_TEXT, STANDARD_VALUE, H_1F, H_22, H_20, UPPER_LIMIT_TO_FIELD_NUMBER, H_3C, NEW_LINE_OF_CR_LF, DO_NOT_REMOVE_SEPARATORS, NOT_SUBSTITUTED, ARRANGE_LINES_IN_ASCENDING_ORDER);

      j.selecting(SUTAI_TEXT);

      t.setting(SUTAI_TEXT);
      t.arranging_lines_in_ascending_order;

{ Removing TEMPORARY_TEXT }
      AssignFile(temporaryFile, TEMPORARY_TEXT);
      Erase(temporaryFile);

      j.Free; t.Free;
      Halt(0);
    end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;
  maximumFieldCount := h.upperLimit;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          if h.deleteSettingText then
            begin
              try
                AssignFile(testFile, SETTING_TEXT);
                Reset(testFile);
                CloseFile(testFile);

                AssignFile(deleteFile, SETTING_TEXT);
                Erase(deleteFile);

                h.Free;
                h := HyphenTerms.Create;
                h.presetting;
                h.setting;
                maximumFieldCount := h.upperLimit;
              except
                ;
              end;

            end;

          if h.EUC_JP_file then
            begin
              if h.Shift_JIS_file then
                begin
                  EUC_text := False;
                  shift_jis_text := True;
                end
              else
                begin
                  EUC_text := True;
                  shift_jis_text := False;
                end;

            end
          else
            begin
              EUC_text := False;
              shift_jis_text := True;
            end;

          if h.lineFeed then
            begin
              if h.carriageReturn then
                begin
                  carriage_return := True;
                end
              else
                begin
                  carriage_return := False;
                end;

            end
          else
            begin
              carriage_return := True;
            end;

          field_count := h.fieldCount;
          if field_count = maximumFieldCount then
            begin
              pattern := h.singleField;
              if not h.binaryRepresented then
                begin
                  pattern := h.multipleBitPattern;

                  if not h.binaryRepresented then
                    begin
                      pattern := h.negativePattern;
                      if not h.binaryRepresented then
                        begin
                          pattern := h.singleSurge;
                        end;

                    end;

                end;

            end
          else
            begin
{ -a5, for example }
              flood_ebb := h.fieldCountLessThanMax;

              multipattern := h.multipleBitPattern;
              if not h.binaryRepresented then
                begin
                  multipattern := h.negativePattern;
                  if not h.binaryRepresented then
                    begin
                      pattern := flood_ebb;
                    end
                  else
                    begin
                      pattern := h.gating(flood_ebb, multipattern);
                    end;

                end
              else
                begin
                  pattern := h.gating(flood_ebb, multipattern);
                end;

            end;

          h.joints; { joint_byte }
          h.labels; { label_byte }
          h.quotes; { quote_byte }
          h.separators; { separator_byte }
          if h.noChecking then
            begin
              final_separator := h.separatorByte;
            end
          else
            begin
              if h.extensionEqualsCSV(h.fileName) then
                begin
                  final_separator := COMMA_BYTE;
                end
              else
                begin
                  final_separator := h.separatorByte;
                end;

            end;

          file_name_length := Length(h.fileName);
          if file_name_length = 0 then
            begin
{ Producing TEMPORARY_TEXT from standard input }
              t := CommandPromptIO.Create;
              t.produceTemporaryTextFile;
              t.Free;
              file_name := TEMPORARY_TEXT;
            end
          else
            begin
              file_name := h.fileName;
            end;

          f := FinalEffects.Create;
          f.setting(file_name, h.listName(1), h.pathName, pattern, 
            h.largeValue, 
            field_count, h.atPoint, h.intervalFrom, h.intervalTo, h.manager, maximumFieldCount, h.maximumLineCount, 
            h.jointByte, h.labelByte, h.quoteByte, final_separator, 
            h.addID, h.binaryRepresented, carriage_return, h.notCaseSensitive, h.descendingOrder, h.extractDecimal, h.extractInteger, EUC_text, h.noChecking, h.noCombining, h.noRemoving, h.noSearching, h.reportLineNumbers, h.priority, shift_jis_text, UNDERLINES_REPLACE_SURROGATE_PAIRS, h.produceSettingText, h.sharedSum, h.skipSeparators, h.skipSpace, h.substituteForDakuten, h.uniqueSummary, NEVER_READ_UTF16_FILE, h.whichTable);

{ main function }
  m := MeasureSpan.Create;
  m.setting(f.fileName, f.atPoint, f.quoteByte, f.separatorByte, f.Shift_JIS_file, f.removeSeparators);
  m.ifErrorOccurs(BUFFER_ERROR);
  line_count := m.LF_to_LF_span;
  if line_count < 2 then
    begin
      s := 'LF_to_LF_span error-486: line_count = ' + IntToStr(line_count);
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, s);
      x.Free; f.Free; h.Free;
      Halt(1);
    end;

  m.setting_beginning(f.fileName);

  if f.Shift_JIS_file then
    begin
      m.countFields;
      shared_sum := m.fieldCount;

      if f.sharedSum then
        begin
          m.displayDataText;
          m.Free; f.Free; h.Free;
          Halt(0);
        end;

      j := SummaryDetailsSJIS.Create;
      s := f.multipleBitPattern;
      j.copyingPatternFromStringToArray( PChar(s), Length(s), shared_sum);
      j.setting(f, shared_sum);

      if f.addingID then
        begin
          if f.extractDecimal then
            begin
              j.selecting_extracting_decimal_adding_ID(SUTAI_TEXT);
            end
          else
            begin
              if f.extractInteger then
                begin
                  j.selecting_extracting_integer_adding_ID(SUTAI_TEXT);
                end
              else
                begin
                  if f.skipWideSpace then
                    begin
                      case f.manager of
                        0: j.skipping_adding_ID(SUTAI_TEXT);
                        1: j.non_dakuten_skipping_adding_ID(SUTAI_TEXT);
                        2: j.non_controller_skipping_adding_ID(SUTAI_TEXT);
                      else
                        j.skipping(SUTAI_TEXT);
                      end;

                    end
                  else
                    begin
                      case f.manager of
                        0: j.selecting_adding_ID(SUTAI_TEXT);
                        1: j.non_dakuten_selecting_adding_ID(SUTAI_TEXT);
                        2: j.non_controller_selecting_adding_ID(SUTAI_TEXT);
                      else
                        j.selecting(SUTAI_TEXT);
                      end;

                    end;

                end;

            end;

        end
      else
        begin
          if f.extractDecimal then
            begin
              j.selecting_extracting_decimal(SUTAI_TEXT);
            end
          else
            begin
              if f.extractInteger then
                begin
                  j.selecting_extracting_integer(SUTAI_TEXT);
                end
              else
                begin
                  if f.skipWideSpace then
                    begin
                      case f.manager of
                        0: j.skipping(SUTAI_TEXT);
                        1: j.non_dakuten_skipping(SUTAI_TEXT);
                        2: j.non_controller_skipping(SUTAI_TEXT);
                      else
                        j.skipping(SUTAI_TEXT);
                      end;

                    end
                  else
                    begin
                      case f.manager of
                        0: j.selecting(SUTAI_TEXT);
                        1: j.non_dakuten_selecting(SUTAI_TEXT);
                        2: j.non_controller_selecting(SUTAI_TEXT);
                      else
                        j.selecting(SUTAI_TEXT);
                      end;

                    end;

                end;

            end;

        end;

      j.Free;
    end
  else
    begin
      m.countFieldsEUC;
      shared_sum := m.fieldCount;

      if f.sharedSum then
        begin
          m.displayDataTextEUC;
          m.Free; f.Free; h.Free;
          Halt(0);
        end;

      y := SummaryDetailsEUC.Create;
      s := f.multipleBitPattern;
      y.copyingPatternFromStringToArray( PChar(s), Length(s), shared_sum);
      y.setting(f, shared_sum);

      if f.addingID then
        begin
          if f.extractDecimal then
            begin
              y.selecting_extracting_decimal_adding_ID(SUTAI_TEXT);
            end
          else
            begin
              if f.extractInteger then
                begin
                  y.selecting_extracting_integer_adding_ID(SUTAI_TEXT);
                end
              else
                begin
                  if f.skipWideSpace then
                    begin
                      case f.manager of
                        0: y.skipping_adding_ID(SUTAI_TEXT);
                        1: y.non_dakuten_skipping_adding_ID(SUTAI_TEXT);
                        2: y.non_controller_skipping_adding_ID(SUTAI_TEXT);
                      else
                        y.skipping(SUTAI_TEXT);
                      end;

                    end
                  else
                    begin
                      case f.manager of
                        0: y.selecting_adding_ID(SUTAI_TEXT);
                        1: y.non_dakuten_selecting_adding_ID(SUTAI_TEXT);
                        2: y.non_controller_selecting_adding_ID(SUTAI_TEXT);
                      else
                        y.selecting(SUTAI_TEXT);
                      end;

                    end;

                end;

            end;

        end
      else
        begin
          if f.extractDecimal then
            begin
              y.selecting_extracting_decimal(SUTAI_TEXT);
            end
          else
            begin
              if f.extractInteger then
                begin
                  y.selecting_extracting_integer(SUTAI_TEXT);
                end
              else
                begin
                  if f.skipWideSpace then
                    begin
                      case f.manager of
                        0: y.skipping(SUTAI_TEXT);
                        1: y.non_dakuten_skipping(SUTAI_TEXT);
                        2: y.non_controller_skipping(SUTAI_TEXT);
                      else
                        y.skipping(SUTAI_TEXT);
                      end;

                    end
                  else
                    begin
                      case f.manager of
                        0: y.selecting(SUTAI_TEXT);
                        1: y.non_dakuten_selecting(SUTAI_TEXT);
                        2: y.non_controller_selecting(SUTAI_TEXT);
                      else
                        y.selecting(SUTAI_TEXT);
                      end;

                    end;

                end;

            end;

        end;

      y.Free;
    end;

  m.Free;

  if f.descendingOrder then
    begin
      if f.uniqueSummary then
        begin
          if f.notCaseSensitive then
            begin
              c := DescendingUniqueBlock.Create;
              c.setting(f);
              c.arranging_lines_in_descending_order;
              c.Free;
            end
          else
            begin
              q := CaseSensitiveDescendingUniqueBlock.Create;
              q.setting(f);
              q.arranging_lines_in_descending_order;
              q.Free;
            end;

        end
      else
        begin
          e := DescendingBlock.Create;
          e.setting(f);
          e.arranging_lines_in_descending_order;
          e.Free;
        end;

    end
  else
    begin
      if f.uniqueSummary then
        begin
          if f.notCaseSensitive then
            begin
              n := UniqueBlock.Create;
              n.setting(f);
              n.arranging_lines_in_ascending_order;
              n.Free;
            end
          else
            begin
              i := CaseSensitiveUniqueBlock.Create;
              i.setting(f);
              i.arranging_lines_in_ascending_order;
              i.Free;
            end;

        end
      else
        begin
          b := CurrencyBlock.Create;
          b.setting(f);
          b.arranging_lines_in_ascending_order;
          b.Free;
        end;

    end;

  if f.produceSettingText then
    begin
      try
        AssignFile(setFile, SETTING_TEXT);
        Rewrite(setFile);

        GetCurrentDirectory( SizeOf(buffer), @buffer);
        pascal_string := buffer;
        k := Length(pascal_string);
        yen_byte := Ord(pascal_string[k]);
        if yen_byte = H_5C then { E:\ for example }
          begin
            s := pascal_string + SUTAI_TEXT;
          end
        else { C:\Narfi for example }
          begin
            s := pascal_string + YEN + SUTAI_TEXT;
          end;

        WriteLn(setFile, s);

        s := JOINT_TERM;
        s := s + YEN_X;
        s := s + Format('%2x', [f.jointByte]);
        WriteLn(setFile, s);

        if f.EUC_JP_file then
          begin
            WriteLn(setFile, SHIFT_JIS_TERM + DENIED );
            WriteLn(setFile, EUC_JP_TERM);
          end;

        if f.Shift_JIS_file then
          begin
            WriteLn(setFile, EUC_JP_TERM + DENIED );
            WriteLn(setFile, SHIFT_JIS_TERM);
          end;

        if f.carriageReturn then
          begin
            WriteLn(setFile, LINE_FEED + DENIED );
            WriteLn(setFile, INCLUDE_0DH);
          end
        else
          begin
            WriteLn(setFile, INCLUDE_0DH + DENIED );
            WriteLn(setFile, LINE_FEED);
          end;

        if f.notCaseSensitive then
          begin
            WriteLn(setFile, CASE_SENSITIVE_SEARCH);
          end;

        if f.descendingOrder then
          begin
            WriteLn(setFile, DESCENDING);
          end;

        if f.uniqueSummary then
          begin
            WriteLn(setFile, LINE_FOR_EACH_UNIQUE_SUMMARY);
          end;

        if f.maximumLineCount < FINAL_INTEGER then
          begin
            s := MAXIMUM_LINE_COUNT + IntToStr(f.maximumLineCount);
            WriteLn(setFile, s);
          end;

        list_name := f.newName;
        list_name_length := Length(list_name);
        if list_name_length > 0 then
          begin
            WriteLn(setFile, TERM_1_FILE_NAME);
            WriteLn(setFile, list_name);
          end;

        CloseFile(setFile);
      except
        s := 'Rewrite() = 0, error-487: ' + SETTING_TEXT;
        x := ErrorTexts.Create;
        x.setting(C_ERR, TEXT_FILE, s);
        x.Free;
        Halt(1);
      end;

    end;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
