unit StandardIO;

{$I+}

interface

uses
  Windows, SysUtils, Classes, ErrorText, FinalEffect, ConvertWide;

type AnsiCharArray = array[0..4095] of AnsiChar;
type CharArray = array[0..4095] of Char;

type
  CommandPromptIO = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    ANSI_CHAR: array[0..255] of AnsiChar;
    beginning: Integer;
    carriage_return: Boolean;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_Y_ARRAY: CharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    count: Integer;
    file_name: String;
    GAP_C: Char;
    GAP_S: String;
    joint_byte: Byte;
    H_0A_C: Char;
    H_0D_C: Char;
    H_1F_C: Char;
    H_20_ANSI: AnsiChar;
    H_20_C: Char;
    H_21_ANSI: AnsiChar;
    H_21_C: Char;
    H_FE_C: AnsiChar;
    H_FF_C: AnsiChar;
    new_name: String;
    NEW_LINE: String;
    NEW_LINE_LENGTH: Integer;
    shift_jis_text: Boolean;
    summary_size: Integer;
    upper_limit: Integer;
    UTF16_text: Boolean;
    v: TStringList;
    x: ErrorTexts;
    y: CharArray;
    Y_COUNT: Integer;
    Y_COUNT_1: Integer;
    function ANSI_CHAR_table: AnsiChar;
    function file_size(name: String): Integer;
    function read_lines_to_produce_list: Integer;
    function setBufferSize: Integer;
    function summarySize(s: String): Integer; overload;
  public
    destructor Destroy; override;
    function arranging_lines_in_ascending_order: Integer;
    function eight_bit_character(b: Byte): AnsiChar;
    function newLine: String;
    function produceTemporaryTextFile: Integer;
    function sequence_to_integer(s: String): Integer;
    function wideSpace(size: Integer): String;
    procedure setting(nn: String);
  end;

{*
123456789012345678901
-922337203685477.5808
*}
const
  BOM_COUNT = 1;
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  CHARACTERS_BYTES = 16; { size of 16 characters }
  COMBO_ERROR = -3;
  CURRENCY_COUNT = 21;
  EMPTY = '';
  FINAL_INTEGER = 2147483647; { 7FFFFFFFh }
  GAP_SIZE = 1;
  H_000000FF = 255;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_1F = 31; { US }
  H_20 = 32;
  H_21 = 33;
  H_FE = 254;
  H_FF = 255;
  JOINT_SIZE = 1;
  NOT_CONVERTED = -1;
  NOT_FOUND = -1;
  OFFSET = 930; { SEGMENT_SIZE * ( 20 + 1 ) * 2 + TAG_AREA_SIZE + 1 }
  OFFSET_1 = 929; { OFFSET - 1 }
  OFFSET_2 = 928; { OFFSET - 2 }
  SEGMENT_SIZE = 22; { CURRENCY_COUNT + JOINT_SIZE }
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 2;
  SUTAI_TEXT = 'sutai.txt';
  TAG_AREA_SIZE = 5; { Length( ' 462 '), Length( ' 483 '), for example }
  TAG_AREA_SIZE_2 = 3; { TAG_AREA_SIZE - 2 }
  TEMPORARY_TEXT = 'text.tmp';
  TEXT_FILE = '.txt';
  UPPER_LIMIT_TO_FIELD_NUMBER = 3071;

implementation

destructor CommandPromptIO.Destroy;
begin
  combo := nil;
  v.Free;

  inherited Destroy;
end;

function CommandPromptIO.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function CommandPromptIO.arranging_lines_in_ascending_order: Integer;
var
  d: Char;
  details: String;
  details_size: Integer;
  exit_code: Integer;
  i: Longint;
  k: Integer;
  m: Integer;
  n: Integer;
  pascal_string: String;
  s: String;
  s_length: Integer;
  v_count_1: Integer;
begin
  exit_code := 0;

  read_lines_to_produce_list;

  v.Sort;
  v_count_1 := v.Count - 1;

  Y_COUNT := SizeOf(y) div SIZE_OF_CHAR;
  Y_COUNT_1 := Y_COUNT - 1;
  for n := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[n] := #0;
  end;

  for k := 0 to v_count_1 do
  begin
    s := v[k];
    s_length := Length(s); { summary_size + TAG_AREA_SIZE + details_size = s_length }

    y := CLEAR_Y_ARRAY;
    m := 0;
    for n := OFFSET to s_length do
    begin
      if m > Y_COUNT_1 then
        begin
          break;
        end;

      d := s[n];
      i := Ord(d);

      if i = H_20 then
        begin
          y[m] := H_21_C;
          Inc(m);
        end
      else
        begin
          if i = H_21 then
            begin
              y[m] := H_20_C;
              Inc(m);
            end
          else
            begin
              y[m] := d;
              Inc(m);
            end;

        end;

    end;

    details_size := m;
    pascal_string := y;
    details := Copy(pascal_string, 1, details_size);
    WriteLn(details);
  end;

  Result := exit_code;
end;

function CommandPromptIO.eight_bit_character(b: Byte): AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
begin
  beginning_of_ASCII := 0;

  if b >= 1 then
    begin
      end_1 := b - 1;
      h := AnsiChar( Chr(beginning_of_ASCII) );

      for k := beginning_of_ASCII to end_1 do
      begin
        Inc(h);
      end;

    end
  else
    begin
      h := AnsiChar(#0);
    end;

  Result := h;
end;

function CommandPromptIO.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function CommandPromptIO.newLine: String;
begin
  Result := NEW_LINE;
end;

function CommandPromptIO.read_lines_to_produce_list: Integer;
var
  block_has_been_read: Integer;
  combo_size_1: Integer;
  c: AnsiChar;
  characters_bytes_summary_size: Integer;
  details: String;
  details_size: Integer;
  from: Integer;
  i: Longint;
  j: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  pascal_unicode_string: String;
  s: String;
  summary: String;
  t: FinalEffects;
  testFile: File;
  u: ConvertWiden;
  z: String;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  carriage_return_appears := False;
  combo_size_1 := combo_size - 1;
  k := 0;
  m := 0;

  for n := 0 to combo_size_1 do
  begin
    combo[n] := #0;
  end;

  for n := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n := 0 to A_SIZE_1 do
      begin
        combo[k] := a[n];
        Inc(k);
        if k > combo_size_1 then
          break;

      end;
    until k > combo_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-1779: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  t := FinalEffects.Create;
  t.setting;
  u := ConvertWiden.Create;
  u.setting(t);

{ Finding the first NEW_LINE }
  for n := beginning to combo_size_1 do
  begin
    if m >= A_SIZE_1 then
      begin
        break;
      end;

    c := combo[n];
    i := Ord(c);

    if i = H_0A then
      begin
        count := 1;

        pascal_string := a;
        u.produce_UTF16_phrase( PAnsiChar(pascal_string), m);
        Y_COUNT := u.UTF16_phrase_size;
        Y_COUNT_1 := Y_COUNT - 1;
        for k := 0 to Y_COUNT_1 do
        begin
          y[k] := u.copyingArrayItem(k);
        end;

        pascal_unicode_string := y;
        s := Copy(pascal_unicode_string, 1, Y_COUNT);

        summary_size := summarySize(s);
        characters_bytes_summary_size := CHARACTERS_BYTES + summary_size;
        details_size := Y_COUNT - characters_bytes_summary_size;
        from := characters_bytes_summary_size + 1;
        details := Copy(s, from, details_size);

        from := CHARACTERS_BYTES + 1;
        summary := Copy(s, from, summary_size);
        z := summary;

        j := OFFSET_1 - summary_size - TAG_AREA_SIZE;
        z := z + wideSpace(j);

        z := z + GAP_C;
        z := z + IntToStr(summary_size);
        z := z + GAP_C;

        z := z + details;

        v.Add(z);
        m := 0;

        beginning := n + 1;
        break;
      end
    else
      begin
        if i = H_0D then
          begin
            carriage_return_appears := True;
            NEW_LINE_LENGTH := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            if i = H_20 then
              begin
                GAP_C := H_21_C;
                GAP_S := '!';
                a[m] := H_21_ANSI;
                Inc(m);
              end
            else
              begin
                if i = H_21 then
                  begin
                    a[m] := H_20_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if i = joint_byte then
                      begin
                        a[m] := H_20_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        a[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end;

      end;

  end; { for n := beginning to combo_size_1 do }

{ Counting lines }
  for n := beginning to combo_size_1 do
  begin
    if m >= A_SIZE_1 then
      begin
        break;
      end;

    c := combo[n];
    i := Ord(c);

    if i = H_0A then
      begin
        Inc(count);
        if count > upper_limit then
          begin
            break;
          end;

        pascal_string := a;
        u.produce_UTF16_phrase( PAnsiChar(pascal_string), m);
        Y_COUNT := u.UTF16_phrase_size;
        Y_COUNT_1 := Y_COUNT - 1;
        for k := 0 to Y_COUNT_1 do
        begin
          y[k] := u.copyingArrayItem(k);
        end;

        pascal_unicode_string := y;
        s := Copy(pascal_unicode_string, 1, Y_COUNT);

        summary_size := summarySize(s);
        characters_bytes_summary_size := CHARACTERS_BYTES + summary_size;
        details_size := Y_COUNT - characters_bytes_summary_size;
        from := characters_bytes_summary_size + 1;
        details := Copy(s, from, details_size);

        from := CHARACTERS_BYTES + 1;
        summary := Copy(s, from, summary_size);
        z := summary;

        j := OFFSET_1 - summary_size - TAG_AREA_SIZE;
        z := z + wideSpace(j);

        z := z + GAP_C;
        z := z + IntToStr(summary_size);
        z := z + GAP_C;

        z := z + details;

        v.Add(z);
        m := 0;
      end
    else
      begin
        if i = H_0D then
          begin
            continue;
          end
        else
          begin
            if i = H_20 then
              begin
                a[m] := H_21_ANSI;
                Inc(m);
              end
            else
              begin
                if i = H_21 then
                  begin
                    a[m] := H_20_ANSI;
                    Inc(m);
                  end
                else
                  begin
                    if i = joint_byte then
                      begin
                        a[m] := H_20_ANSI;
                        Inc(m);
                      end
                    else
                      begin
                        a[m] := c;
                        Inc(m);
                      end;

                  end;

              end;

          end;

      end;

  end; { for n := beginning to combo_size_1 do }

  u.Free; t.Free;
  Result := count;
end;

function CommandPromptIO.produceTemporaryTextFile: Integer;
const
  NEW_LINE_SIZE = 2; { CR, LF }
var
  newFile: TextFile;
  s: String;
  sum: Integer;
begin
  sum := 0;

  try
    AssignFile(newFile, TEMPORARY_TEXT);
    Rewrite(newFile);

    while not Eof do
    begin
      ReadLn(s);
      WriteLn(newFile, s);
      sum := sum + Length(s);
      sum := sum + NEW_LINE_SIZE;
    end;

    CloseFile(newFile);
  except
    s := 'Reset() error-1780: ' + TEMPORARY_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function CommandPromptIO.sequence_to_integer(s: String): Integer;
var
  r: Integer;
  messages: String;
begin
  try
    r := StrToInt(s);
  except
    messages := 'StrToInt() error-1781: s = ' + s;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := NOT_CONVERTED;
    Halt(1);
  end;

  Result := r;
end;

function CommandPromptIO.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := combo_size;

  try
    SetLength(combo, combo_size);
  except
    s := 'SetLength() error-1782: ' + IntToStr(combo_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := COMBO_ERROR;
  end;

  Result := r;
end;

function CommandPromptIO.summarySize(s: String): Integer;
var
  c: Char;
  head: String;
  n: Integer;
  r: Integer;
begin
  r := 0;

  for n := 1 to CHARACTERS_BYTES do
  begin
    c := s[n];
    if c = GAP_C then
      begin
        continue;
      end
    else
      begin
        r := n;
        break;
      end;
  
  end;

  if r = 1 then
    begin
      for n := r to CHARACTERS_BYTES do
      begin
        c := s[n];
        if c = GAP_C then
          begin
            r := n - 1;
            break;
          end
        else
          begin
            continue;
          end;

      end;

      head := Copy(s, 1, r);
    end
  else
    begin
      s := 'summarySize() error-1783: r = ' + IntToStr(r);
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, s);
      x.Free;
      Halt(1);
    end;

  Result := sequence_to_integer(head);
end;

function CommandPromptIO.wideSpace(size: Integer): String;
var
  z: String;
begin
  z := StringOfChar(GAP_C, size);
  Result := z;
end;

procedure CommandPromptIO.setting(nn: String);
var
  b: array[0..1] of Byte;
  block_has_been_read: Integer;
  i: Integer;
  s: String;
  testFile: File;
begin
  carriage_return := True;
  joint_byte := H_1F;
  new_name := nn;
  shift_jis_text := True;
  upper_limit := UPPER_LIMIT_TO_FIELD_NUMBER;
  UTF16_text := False;

  file_name := SUTAI_TEXT;
  GAP_C := Chr(H_20);
  GAP_S := ' ';
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  H_1F_C := Chr(H_1F);
  H_20_C := Chr(H_20);
  H_21_C := Chr(H_21);
  H_FF_C := ANSI_CHAR_table;
  H_20_ANSI := ANSI_CHAR[H_20];
  H_21_ANSI := ANSI_CHAR[H_21];
  H_FE_C := ANSI_CHAR[H_FE];

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, a, A_SIZE, block_has_been_read);
    b[0] := Ord(a[0]);
    b[1] := Ord(a[1]);
    if (b[0] = H_FF) and (b[1] = H_FE) then
      beginning := BOM_COUNT
    else
      beginning := 0;

    CloseFile(testFile);

    combo_size := file_size(file_name);
  except
    s := 'Reset() error-1784: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  setBufferSize;
  v := TStringList.Create;
  v.CaseSensitive := True;
  v.Sorted := False;
end;

end.
