unit SummaryEUC;

{$I+}

interface

uses
  Windows, SysUtils, ErrorText, FinalEffect, SegmentedEUC, FieldsArrayMany, NoncontrollerEUC, NondakutenEUC;

type AnsiCharArray = array[0..4095] of AnsiChar;

type
  SummaryDetailsEUC = class
  private
    ANSI_ASCII: AnsiString;
    ANSI_CHAR: array[0..255] of AnsiChar;
    ASCII_BYTE: Byte;
    buffer: array of AnsiChar;
    buffer_size: Integer;
    carriage_return: Boolean;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_BIT_C: AnsiChar;
    count: Integer;
    counted: Boolean;
    d: SegmentedSummaryEUC;
    dakuten_byte: Byte;
    dough: array[0..98303] of AnsiChar;
    doughLength: Integer;
    EMPTY_ANSI: AnsiString;
    f: FieldsListMany;
    field_count: Integer;
    file_name: String;
    full_house: Boolean;
    GAP_C: AnsiChar;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_FF_C: AnsiChar;
    handakuten_byte: Byte;
    joint_byte: Byte;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    new_name: String;
    pattern: array[0..3071] of AnsiChar;
    quote_byte: Byte;
    r: NoncontrollerSummaryEUC;
    remove_separators: Boolean;
    separator_byte: Byte;
    SET_BIT_C: AnsiChar;
    substituted: Boolean;
    u: NondakutenSummaryEUC;
    way_of_arranging: Boolean;
    x: ErrorTexts;
    y: AnsiCharArray;
    Y_SIZE: Integer;
    Y_SIZE_1: Integer;
    function ANSI_CHAR_table: AnsiChar;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    function ANSI_format(L: Integer; r: Integer): AnsiString;
    function bufferingWriting(s: PAnsiChar; s_length: Integer): Integer;
    function copyingPatternFromStringToArray(s: PChar; s_length: Integer; number_of_fields: Integer): Integer;
    function count_of_currency_type(count: Integer): AnsiString;
    function eight_bit_character(b: Byte): AnsiChar;
    function fieldCount: Integer;
    function newLine: AnsiString;
    function leadingCharacters(s: PAnsiChar; s_length: Integer; character: AnsiChar): Integer;
    function non_controller_selecting(nn: String): Integer;
    function non_controller_selecting_adding_ID(nn: String): Integer;
    function non_controller_skipping(nn: String): Integer;
    function non_controller_skipping_adding_ID(nn: String): Integer;
    function non_dakuten_selecting(nn: String): Integer;
    function non_dakuten_selecting_adding_ID(nn: String): Integer;
    function non_dakuten_skipping(nn: String): Integer;
    function non_dakuten_skipping_adding_ID(nn: String): Integer;
    function selecting(nn: String): Integer;
    function selecting_adding_ID(nn: String): Integer;
    function selecting_extracting_decimal(nn: String): Integer;
    function selecting_extracting_decimal_adding_ID(nn: String): Integer;
    function selecting_extracting_integer(nn: String): Integer;
    function selecting_extracting_integer_adding_ID(nn: String): Integer;
    function skipping(nn: String): Integer;
    function skipping_adding_ID(nn: String): Integer;
    function wideSpace(size: Integer): AnsiString;
    procedure setting(f: FinalEffects; fc: Integer); overload;
    procedure setting(fn: String; r: Currency; jb: Byte; qb: Byte; sb: Byte; fc: Integer; ab: Byte; cr: Boolean; rs: Boolean; SUB: Boolean; wo: Boolean); overload;
  end;

{*
SEGMENT_SIZE = 22;
1234567890123456789012
-922337203685477.5808 
*}

const
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  CLEAR_BIT = 48; { 30h, '0' }
  CURRENCY_SIZE = 21;
  EMPTY = '';
  ERROR_CODE = -1;
  FILE_ERROR = -2;
  FRACTION = 4;
  GAP_BYTE = 32; { SP }
  H_000000FF = 255;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_3E = 62; { '>' }
  H_5C = 92; { '\' }
  H_7E = 126; { '~' }
  H_FF = 255;
  HEAD_OF_ZERO_CURRENCY = '               0';
  JOINT_SIZE = 1;
  KANA_DAKUTEN = 222; { DEh }
  KANA_HANDAKUTEN = 223; { DFh }
  LISTED = True;
  OFFSET = 463; { OFFSET_1 + 1 }
  OFFSET_1 = 462; { SEGMENT_SIZE * ( 20 + 1 ) }
  OFFSET_EXCEPT_ID = 440; { SEGMENT_SIZE * 20 }
  SEGMENT_SIZE = 22; { CURRENCY_SIZE + JOINT_SIZE }
  SET_BIT = 49; { 31h, '1'}
  SIZE_OF_ANSICHAR = 1;
  SUTAI_TEXT = 'sutai.txt';
  TEXT_FILE = '.txt';

var
  RAISED: Currency;

implementation

constructor SummaryDetailsEUC.Create;
begin
  inherited Create;

end;

destructor SummaryDetailsEUC.Destroy;
begin
{ NoncontrollerSummaryEUC }
  r.Free;
  r := nil;

{ SummaryDetailsEUC }
  d.Free;
  d := nil;

{ FieldsListMany }
  f.Free;
  f := nil;

{ NondakutenSummaryEUC }
  u.Free;
  u := nil;

{ array of AnsiChar }
  buffer := nil;

  inherited Destroy;
end;

function SummaryDetailsEUC.ANSI_format(L: Integer; r: Integer): AnsiString;
var
  i: Integer;
  s: String;
  s_length: Integer;
  w: Integer;
  z: AnsiString;
begin
  z := '';

  s := Format('%0:-7d %1:7d ', [L, r]);
  s_length := Length(s);

  for i := 1 to s_length do
  begin
    w := Ord(s[i]);
    z := z + ANSI_CHAR[w];
  end;

  Result := z;
end;

function SummaryDetailsEUC.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function SummaryDetailsEUC.bufferingWriting(s: PAnsiChar; s_length: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
  messasges: String;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;

      for k := 0 to Y_SIZE_1 do
      begin
        y[k] := s[n];
        Inc(n);
        if n >= s_length then
          begin
            Y_SIZE := k + 1;
            Y_SIZE_1 := Y_SIZE - 1;
            break;
          end;

      end;

      BlockWrite(newFile, y, Y_SIZE, block_has_been_written);
    until n >= s_length;

    CloseFile(newFile);
    Y_SIZE := SizeOf(y);
    Y_SIZE_1 := Y_SIZE - 1;
  except
    messasges := 'Rewrite() error-670: ' + new_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messasges);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function SummaryDetailsEUC.copyingPatternFromStringToArray(s: PChar; s_length: Integer; number_of_fields: Integer): Integer;
var
  c: Char;
  k: Integer;
  m: Integer;
  s_length_1: Integer;
begin
  CLEAR_BIT_C := eight_bit_character(CLEAR_BIT);
  m := 0;
  s_length_1 := s_length - 1;
  SET_BIT_C := eight_bit_character(SET_BIT);

  for k := 0 to s_length_1 do
  begin
    c := s[k];

    if c = '1' then
      begin
        pattern[m] := SET_BIT_C;
        Inc(m);
      end
    else
      begin
        if c = '0' then
          begin
            pattern[m] := CLEAR_BIT_C;
            Inc(m);
          end
        else
          begin
            continue;
          end;

      end;

  end;

  while m < number_of_fields do
  begin
    pattern[m] := SET_BIT_C;
    Inc(m);
  end;

  Result := m;
end;

function SummaryDetailsEUC.count_of_currency_type(count: Integer): AnsiString;
const
  GAP = ' ';
  ZERO_CURRENCY = 0.0000;
var
  c: Currency;
  i: Integer;
  k: Integer;
  s: String;
  s_length: Integer;
  w: Integer;
  z: AnsiString;
begin
  c := ZERO_CURRENCY + count;

  s := CurrToStrF(c, ffFixed, FRACTION);
  s := s + ' ';

  k := SEGMENT_SIZE - Length(s);
  s := StringOfChar(GAP, k) + s;
  s_length := Length(s);

  z := '';
  for i := 1 to s_length do
  begin
    w := Ord(s[i]);
    z := z + ANSI_CHAR[w];
  end;

  Result := z;
end;

function SummaryDetailsEUC.eight_bit_character(b: Byte): AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
begin
  beginning_of_ASCII := 0;

  if b >= 1 then
    begin
      end_1 := b - 1;
      h := AnsiChar( Chr(beginning_of_ASCII) );

      for k := beginning_of_ASCII to end_1 do
      begin
        Inc(h);
      end;

    end
  else
    begin
      h := AnsiChar(#0);
    end;

  Result := h;
end;

function SummaryDetailsEUC.fieldCount: Integer;
begin
  Result := field_count;
end;

function SummaryDetailsEUC.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function SummaryDetailsEUC.leadingCharacters(s: PAnsiChar; s_length: Integer; character: AnsiChar): Integer;
var
  c: AnsiChar;
  k: Integer;
  s_length_1: Integer;
begin
  s_length_1 := s_length - 1;
  for k := 0 to s_length_1 do
  begin
    c := s[k];
    if c = character then
      begin
        continue;
      end
    else
      begin
        break;
      end;

  end;

  Result := k;
end;

function SummaryDetailsEUC.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function SummaryDetailsEUC.non_controller_selecting(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-671: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            g := r.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - r.unitedSize;
            g := g + wideSpace(space_width);

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            g := r.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - r.unitedSize;
            g := g + wideSpace(space_width);

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            g := r.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - r.unitedSize;
            g := g + wideSpace(space_width);

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            g := r.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - r.unitedSize;
            g := g + wideSpace(space_width);

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.non_controller_selecting_adding_ID(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-672: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            g := r.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - r.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            g := r.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - r.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            g := r.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - r.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            g := r.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - r.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.non_controller_skipping(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  from: Integer;
  g: AnsiString;
  g_length: Integer;
  h: AnsiString;
  h_length: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-673: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            h := r.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_1 - g_length;
            g := g + wideSpace(space_width);

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            h := r.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_1 - g_length;
            g := g + wideSpace(space_width);

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            h := r.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_1 - g_length;
            g := g + wideSpace(space_width);

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            h := r.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_1 - g_length;
            g := g + wideSpace(space_width);

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.non_controller_skipping_adding_ID(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  from: Integer;
  g: AnsiString;
  g_length: Integer;
  h: AnsiString;
  h_length: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-674: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            h := r.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_EXCEPT_ID - g_length;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            h := r.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_EXCEPT_ID - g_length;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            h := r.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_EXCEPT_ID - g_length;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            r.Free;
            f.Free;
            f := FieldsListMany.Create;
            r := NoncontrollerSummaryEUC.Create;
            r.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            r.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := r.editing_field(dough, doughLength);
            h := r.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_EXCEPT_ID - g_length;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := r.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.non_dakuten_selecting(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  dakuten_byte := KANA_DAKUTEN;
  handakuten_byte := KANA_HANDAKUTEN;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  if substituted then
    begin { Please see sortk.txt }
      dakuten_byte := Ord('H');
      handakuten_byte := Ord('I');
    end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-675: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            g := u.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - u.unitedSize;
            g := g + wideSpace(space_width);

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            g := u.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - u.unitedSize;
            g := g + wideSpace(space_width);

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            g := u.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - u.unitedSize;
            g := g + wideSpace(space_width);

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            g := u.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - u.unitedSize;
            g := g + wideSpace(space_width);

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.non_dakuten_selecting_adding_ID(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  dakuten_byte := KANA_DAKUTEN;
  handakuten_byte := KANA_HANDAKUTEN;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  if substituted then
    begin { Please see sortk.txt }
      dakuten_byte := Ord('H');
      handakuten_byte := Ord('I');
    end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-676: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            g := u.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - u.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            g := u.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - u.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            g := u.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - u.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            g := u.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - u.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.non_dakuten_skipping(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  from: Integer;
  g: AnsiString;
  g_length: Integer;
  h: AnsiString;
  h_length: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  dakuten_byte := KANA_DAKUTEN;
  handakuten_byte := KANA_HANDAKUTEN;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  if substituted then
    begin { Please see sortk.txt }
      dakuten_byte := Ord('H');
      handakuten_byte := Ord('I');
    end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-677: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            h := u.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_1 - g_length;
            g := g + wideSpace(space_width);

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            h := u.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_1 - g_length;
            g := g + wideSpace(space_width);

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            h := u.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_1 - g_length;
            g := g + wideSpace(space_width);

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            h := u.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_1 - g_length;
            g := g + wideSpace(space_width);

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.non_dakuten_skipping_adding_ID(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  from: Integer;
  g: AnsiString;
  g_length: Integer;
  h: AnsiString;
  h_length: Integer;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  dakuten_byte := KANA_DAKUTEN;
  handakuten_byte := KANA_HANDAKUTEN;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  if substituted then
    begin { Please see sortk.txt }
      dakuten_byte := Ord('H');
      handakuten_byte := Ord('I');
    end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-678: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            h := u.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_EXCEPT_ID - g_length;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            h := u.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_EXCEPT_ID - g_length;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            h := u.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_EXCEPT_ID - g_length;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            u.Free;
            f.Free;
            f := FieldsListMany.Create;
            u := NondakutenSummaryEUC.Create;
            u.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            u.setting(f, field_count, pattern, dakuten_byte, handakuten_byte);

            doughLength := b;
            b := 0;
            field_count := u.editing_field(dough, doughLength);
            h := u.unite(dough, doughLength, LISTED);
            h_length := Length(h);
            space_width := leadingCharacters( PAnsiChar(h), h_length, GAP_C);

            g_length := h_length - space_width;
            from := space_width + 1;
            g := Copy(h, from, g_length); { Removing a wide space }

            space_width := OFFSET_EXCEPT_ID - g_length;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := u.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.selecting(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-679: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.selecting_adding_ID(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := '';

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-680: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.unite(dough, doughLength);
            field_count := d.fieldCount;

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.selecting_extracting_decimal(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-681: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.selecting_extracting_decimal_adding_ID(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-682: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_decimal_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.selecting_extracting_integer(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-683: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.selecting_extracting_integer_adding_ID(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-684: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting_for_extracting(RAISED);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            field_count := d.extracting_integer_from_field(dough, doughLength);
            g := d.unite(dough, doughLength, LISTED);

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    s := 'SetLength() error-685: ' + IntToStr(buffer_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  Result := r;
end;

function SummaryDetailsEUC.skipping(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-686: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.compactCombo(dough, doughLength); { Removing a wide space }
            field_count := d.fieldCount;

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.compactCombo(dough, doughLength); { Removing a wide space }
            field_count := d.fieldCount;

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.compactCombo(dough, doughLength); { Removing a wide space }
            field_count := d.fieldCount;

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.compactCombo(dough, doughLength); { Removing a wide space }
            field_count := d.fieldCount;

            space_width := OFFSET_1 - d.unitedSize;
            g := g + wideSpace(space_width);

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.skipping_adding_ID(nn: String): Integer;
var
  b: Integer;
  beginning: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  e: Integer;
  exit_code: Integer;
  g: AnsiString;
  m: Integer;
  n: Integer;
  pascal_string: AnsiString;
  s: String;
  space_width: Integer;
  testFile: File;
  trace: Integer;
  z: AnsiString;
begin
  b := 0;
  beginning := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  m := 0;
  new_name := nn;
  z := EMPTY_ANSI;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-687: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  if full_house then
    begin
{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.compactCombo(dough, doughLength); { Removing a wide space }
            field_count := d.fieldCount;

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count);

            doughLength := b;
            b := 0;
            g := d.compactCombo(dough, doughLength); { Removing a wide space }
            field_count := d.fieldCount;

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end
  else
    begin
{ full_house = False }

{ Finding the first NEW_LINE }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            count := 1;

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.compactCombo(dough, doughLength); { Removing a wide space }
            field_count := d.fieldCount;

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;

            trace := n;
            beginning := trace + 1;
            break;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI+ H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
      for n := beginning to buffer_size_1 do
      begin
        if buffer[n] = H_0A_C then
          begin
            Inc(count);

{ Reading a line }
            d.Free;
            f.Free;
            f := FieldsListMany.Create;
            d := SegmentedSummaryEUC.Create;
            d.presetting(ANSI_ASCII, joint_byte, quote_byte, separator_byte, remove_separators);
            d.setting(f, field_count, pattern);

            doughLength := b;
            b := 0;
            g := d.compactCombo(dough, doughLength); { Removing a wide space }
            field_count := d.fieldCount;

            space_width := OFFSET_EXCEPT_ID - d.unitedSize;
            g := g + wideSpace(space_width);
            g := g + count_of_currency_type(count); { ID of summary }

            e := d.countCharacters( PAnsiChar(g), Length(g) );
            z := z + ANSI_format(e, OFFSET_1); { 16 bytes }

            z := z + g; { summary }

            pascal_string := dough;
            z := z + Copy(pascal_string, 1, doughLength); { details }

            z := z + NEW_LINE;
          end
        else
          begin
            if buffer[n] = H_0D_C then
              begin
                continue;
              end
            else
              begin
                dough[b] := buffer[n];
                Inc(b);
              end; { if buffer[n] = H_0D_C then, else }

          end; { if buffer[n] = H_0A_C then, else }

      end; { for n := beginning to buffer_size_1 do }

    end; { if full_house then, else }

  exit_code := bufferingWriting( PAnsiChar(z), Length(z) );
  counted := True;
  Result := exit_code;
end;

function SummaryDetailsEUC.wideSpace(size: Integer): AnsiString;
var
  z: AnsiString;
begin
  z := StringOfChar(GAP_C, size);
  Result := z;
end;

procedure SummaryDetailsEUC.setting(f: FinalEffects; fc: Integer);
var
  b: Byte;
  m: Integer;
  messages: String;
  o: Longint;
  pascal_string: AnsiString;
  testFile: File;
begin
{ String }
  file_name := f.fileName;

{ Currency }
  RAISED := f.largeValue;

{ Integer }
  field_count := fc;

{ Byte }
  ascii_byte := f.labelByte;
  joint_byte := f.jointByte;
  quote_byte := f.quoteByte;
  separator_byte := f.separatorByte;

{ Boolean }
  carriage_return := f.carriageReturn;
  remove_separators := f.removeSeparators;
  substituted := f.substituteForDakuten;
  way_of_arranging := f.descendingOrder;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);
    CloseFile(testFile);

    buffer_size := file_size(file_name);
  except
    messages := 'Reset() error-688: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
  end;

  Y_SIZE := SizeOf(y);
  Y_SIZE_1 := Y_SIZE - 1;

  full_house := True;
  for m := 1 to field_count do
  begin
    o := Ord( pattern[ m - 1 ] );
    b := Byte(o);
    if b = CLEAR_BIT then
      full_house := False;

  end;

  for m := 0 to Y_SIZE_1 do
  begin
    CLEAR_ARRAY[m] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  GAP_C := ANSI_CHAR[GAP_BYTE];
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  if carriage_return then
    begin
      NEW_LINE_SIZE := 2;
      NEW_LINE := EMPTY_ANSI + H_0D_C;
      NEW_LINE := NEW_LINE + H_0A_C;
    end
  else
    begin
      NEW_LINE_SIZE := 1;
      NEW_LINE := EMPTY_ANSI + H_0A_C;
    end;

  setBufferSize;
end;

procedure SummaryDetailsEUC.setting(fn: String; r: Currency; jb: Byte; qb: Byte; sb: Byte; fc: Integer; ab: Byte; cr: Boolean; rs: Boolean; SUB: Boolean; wo: Boolean);
var
  b: Byte;
  m: Integer;
  messages: String;
  o: Longint;
  pascal_string: AnsiString;
  testFile: File;
begin
{ String }
  file_name := fn;

{ Currency }
  RAISED := r;

{ Integer }
  field_count := fc;

{ Byte }
  ascii_byte := ab;
  joint_byte := jb;
  quote_byte := qb;
  separator_byte := sb;

{ Boolean }
  carriage_return := cr;
  remove_separators := rs;
  substituted := SUB;
  way_of_arranging := wo;

  try
    AssignFile(testFile, file_name);
    Reset(testFile);
    CloseFile(testFile);

    buffer_size := file_size(file_name);
  except
    messages := 'Reset() error-689: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
  end;

  Y_SIZE := SizeOf(y);
  Y_SIZE_1 := Y_SIZE - 1;

  full_house := True;
  for m := 1 to field_count do
  begin
    o := Ord( pattern[ m - 1 ] );
    b := Byte(o);

    if b = CLEAR_BIT then
      full_house := False;

  end;

  for m := 0 to Y_SIZE_1 do
  begin
    CLEAR_ARRAY[m] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;
  GAP_C := ANSI_CHAR[GAP_BYTE];
  H_0A_C := ANSI_CHAR[H_0A];
  H_0D_C := ANSI_CHAR[H_0D];

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  if carriage_return then
    begin
      NEW_LINE_SIZE := 2;
      NEW_LINE := EMPTY_ANSI + H_0D_C;
      NEW_LINE := NEW_LINE + H_0A_C;
    end
  else
    begin
      NEW_LINE_SIZE := 1;
      NEW_LINE := EMPTY_ANSI + H_0A_C;
    end;

  setBufferSize;
end;

end.
