unit Combo;

interface

uses
  SysUtils, ErrorText;

type
  ComboAppend = class
  private
    ANSI_ASCII: AnsiString;
    buffer: array[0..4096] of AnsiChar;
    combo: AnsiString;
    dough: array[0..4096] of Char;
    EMPTY_ANSI: AnsiString;
    separator: AnsiChar;
    x: ErrorTexts;
  public
    function built: AnsiString;
    function comboSize: Integer;
    function narrow(s: String): AnsiString;
    function widen(z: AnsiString): String;
    procedure append(b: Byte); overload;
    procedure append(c: AnsiChar); overload;
    procedure append(s: AnsiString); overload;
    procedure separatorFollows(b: Byte); overload;
    procedure separatorFollows(c: AnsiChar); overload;
    procedure separatorFollows(s: AnsiString); overload;
    procedure setting(ans: AnsiString); overload;
    procedure setting(ans: AnsiString; c: AnsiChar); overload;
    procedure setting(ans: AnsiString; s: AnsiString); overload;
    procedure append(d: Double); overload;
    procedure append(i: Integer); overload;
    procedure separatorFollows(d: Double); overload;
    procedure separatorFollows(i: Integer); overload;
  end;

const
  C_ERR = 'err';
  EMPTY = '';
  H_2C = 44; { ',' }
  H_2D = 45; { '-' }
  H_2E = 46; { '.' }
  H_30 = 48; { '0' }
  H_39 = 57; { '9' }
  TAB = 9; { 09h }
  TEXT_FILE = '.txt';

implementation

function ComboAppend.built: AnsiString;
begin
  Result := combo;
end;

function ComboAppend.comboSize: Integer;
begin
  Result := Length(combo);
end;

function ComboAppend.narrow(s: String): AnsiString;
var
  i: Longint;
  k: Integer;
  m: Integer;
  pascal_string: AnsiString;
  s_length: Integer;
  z: AnsiString;
begin
  i := 0;
  m := 0;
  z := EMPTY_ANSI;
  s_length := Length(s);

  if s_length > 0 then
    begin
{ Converting numerical data from UTF16 to Shift_JIS }
      try
        for k := 1 to s_length do
        begin
          i := Ord(s[k]);

          if i = 0 then
            begin
              break;
            end
          else
            begin
              if i = H_2D then
                begin
                  buffer[m] := ANSI_ASCII[i];
                  Inc(m);
                end
              else
                begin
                  if i = H_2E then
                    begin
                      buffer[m] := ANSI_ASCII[i];
                      Inc(m);
                    end
                  else
                    begin
                      if i < H_30 then
                        begin
                          raise Exception.Create('narrow() error');
                        end
                      else
                        begin
                          if H_39 < i then
                            begin
                              raise Exception.Create('narrow() error');
                            end
                          else
                            begin
                              buffer[m] := ANSI_ASCII[i];
                              Inc(m);
                            end;

                        end;

                    end; { if i = H_2E then, else }

                end; { if i = H_2D then, else }

            end; { if i = 0 then, else }

        end; { for k := 1 to z_length do }

      except
        s := 'narrow() error-767: i = ' + IntToStr(i);
        x := ErrorTexts.Create;
        x.setting(C_ERR, TEXT_FILE, s);
        x.Free;
        s := EMPTY;
      end; { try, except }

    end; { if s_length > 0 then }

  pascal_string := buffer;
  z := Copy(pascal_string, 1, m);
  Result := z;
end;

function ComboAppend.widen(z: AnsiString): String;
var
  i: Longint;
  k: Integer;
  m: Integer;
  pascal_string: String;
  s: String;
  z_length: Integer;
begin
  i := 0;
  m := 0;
  s := EMPTY;
  z_length := Length(z);

  if z_length > 0 then
    begin
{ Converting numerical data and separators from Shift_JIS to UTF16 }
      try
        for k := 1 to z_length do
        begin
          i := Ord(z[k]);

          if i = 0 then
            begin
              break;
            end
          else
            begin
              if i = H_2C then
                begin
                  dough[m] := Chr(i);
                  Inc(m);
                end
              else
                begin
                  if i = H_2D then
                    begin
                      dough[m] := Chr(i);
                      Inc(m);
                    end
                  else
                    begin
                      if i = H_2E then
                        begin
                          dough[m] := Chr(i);
                          Inc(m);
                        end
                      else
                        begin
                          if i < H_30 then
                            begin
                              raise Exception.Create('widen() error');
                            end
                          else
                            begin
                              if H_39 < i then
                                begin
                                  raise Exception.Create('widen() error');
                                end
                              else
                                begin
                                  dough[m] := Chr(i);
                                  Inc(m);
                                end;

                            end;

                        end; { if i = H_2E then, else }

                    end; { if i = H_2D then, else }

                end; { if i = H_2C then, else }

            end; { if i = 0 then, else }

        end; { for k := 1 to z_length do }

      except
        s := 'widen() error-768: i = ' + IntToStr(i);
        x := ErrorTexts.Create;
        x.setting(C_ERR, TEXT_FILE, s);
        x.Free;
        s := EMPTY;
      end; { try, except }

    end; { if z_length > 0 then }

  pascal_string := dough;
  s := Copy(pascal_string, 1, m);
  Result := s;
end;

procedure ComboAppend.append(b: Byte);
begin
  combo := combo + ANSI_ASCII[b];
end;

procedure ComboAppend.append(c: AnsiChar);
begin
  combo := combo + c;
end;

procedure ComboAppend.append(s: AnsiString);
begin
  combo := combo + s;
end;

procedure ComboAppend.separatorFollows(b: Byte);
begin
  combo := combo + ANSI_ASCII[b];
  combo := combo + separator;
end;

procedure ComboAppend.separatorFollows(c: AnsiChar);
begin
  combo := combo + c;
  combo := combo + separator;
end;

procedure ComboAppend.separatorFollows(s: AnsiString);
begin
  combo := combo + s;
  combo := combo + separator;
end;

procedure ComboAppend.setting(ans: AnsiString);
begin
  ANSI_ASCII := ans;

  EMPTY_ANSI := '';
  combo := EMPTY_ANSI;
  separator := ANSI_ASCII[TAB];
end;

procedure ComboAppend.setting(ans: AnsiString; c: AnsiChar);
begin
  ANSI_ASCII := ans;
  separator := ANSI_ASCII[ Ord(c) ];

  EMPTY_ANSI := '';
  combo := EMPTY_ANSI;
end;

procedure ComboAppend.setting(ans: AnsiString; s: AnsiString);
var
  c: AnsiChar;
begin
  ANSI_ASCII := ans;
  c := s[1];
  separator := ANSI_ASCII[ Ord(c) ];

  EMPTY_ANSI := '';
  combo := EMPTY_ANSI;
end;

procedure ComboAppend.append(d: Double);
begin
  combo := combo + narrow( FloatToStr(d) );
end;

procedure ComboAppend.append(i: Integer);
begin
  combo := combo + narrow( IntToStr(i) );
end;

procedure ComboAppend.separatorFollows(d: Double);
begin
  combo := combo + narrow( FloatToStr(d) );
  combo := combo + separator;
end;

procedure ComboAppend.separatorFollows(i: Integer);
begin
  combo := combo + narrow( IntToStr(i) );
  combo := combo + separator;
end;

end.
