unit DeriveUTF16Euc;

{$I+}

interface

uses
  SysUtils, ErrorText, SparseEUC, SearchBinary;

type
  DeriveUTF16EucTable = class
  private
    r: array[1..4] of AnsiChar;
    w: array[1..5] of AnsiChar;
  public
    table: SparsePopulationEUC;
    destructor Destroy; override;
    procedure setting(search_table: SearchTable);
  end;

const
  C_ERR = 'err';
  H_80 = 128;
  MS932_UTF16 = 'MS932~UTF16.bin';
  NEW_RECORD_SIZE = 5;
  RECORD_SIZE = 4;
  TEXT_FILE = '.txt';
  UTF16_EUC = 'UTF16~EUC-JP.bin';

implementation

destructor DeriveUTF16EucTable.Destroy;
begin
  table.Free;

  inherited Destroy;
end;

procedure DeriveUTF16EucTable.setting(search_table: SearchTable);
var
  ASCII_C: AnsiChar;
  b: Longint;
  binaryFile: File;
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  c: AnsiChar;
  first_byte_of_EUC: AnsiChar;
  first_SJIS_byte: Integer;
  k: Longint;
  KANJI_C: AnsiChar;
  newFile: File;
  NON_KANJI_C: AnsiChar;
  s: String;
  second_byte_of_EUC: AnsiChar;
  SJIS_code: Integer;
  x: ErrorTexts;
begin
  ASCII_C := AnsiChar(#0);
  KANJI_C := AnsiChar(#1);
  NON_KANJI_C := AnsiChar(#0);

{ Producing a conversion table in the main memory }
  table := SparsePopulationEUC.Create;
  table.setting(search_table);

  AssignFile(newFile, UTF16_EUC);
  Rewrite(newFile, NEW_RECORD_SIZE);

  try
    AssignFile(binaryFile, MS932_UTF16);
    Reset(binaryFile, RECORD_SIZE);

    repeat
      BlockRead(binaryFile, r, 1, block_has_been_read);

      if r[2] = NON_KANJI_C then
        begin
          b := Ord(r[1]);
          if b < H_80 then
            begin
              w[1] := r[3];
              w[2] := r[4];
              w[3] := ASCII_C;
              w[4] := r[1];
              w[5] := NON_KANJI_C;
            end
          else
            begin
              continue; { Skipping hankaku-katakana }
            end;

        end
      else
        begin
          c := r[1];
          k := Ord(c);
          first_SJIS_byte := k shl 8;

          c := r[2];
          k := Ord(c);

          SJIS_code := first_SJIS_byte + k;

          second_byte_of_EUC := table.working(SJIS_code, 1);
          first_byte_of_EUC := table.working(SJIS_code, 0);

          w[1] := r[3];
          w[2] := r[4];
          w[3] := KANJI_C;
          w[4] := first_byte_of_EUC;
          w[5] := second_byte_of_EUC;
        end;

      BlockWrite(newFile, w, block_has_been_read, block_has_been_written);
    until block_has_been_read = 0;

    CloseFile(binaryFile);
  except
    s := 'Reset() error-769: ' + MS932_UTF16;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  CloseFile(newFile);
end;

end.
