unit PathEXE;

interface

uses
  Windows, SysUtils, FieldsArrayMany, DivideSJIS, ErrorText;

type PettyArray = array[0..255] of Char;

type
  CommandPath = class
  private
    ANSI_ASCII: AnsiString;
    ANSI_CHAR: array[0..255] of AnsiChar;
    buffer: array[0..4095] of Char;
    CLEAR_PETTY_ARRAY: PettyArray;
    combo: array[0..4095] of AnsiChar;
    comboLength: Integer;
    EMPTY_ANSI: AnsiString;
    extra: AnsiString;
    extra_buffer: array[0..255] of AnsiChar;
    full_path: String;
    H_FF_C: AnsiChar;
    p: PettyArray;
    P_SIZE: Integer;
    P_SIZE_1: Integer;
    manyErrorTexts: Boolean;
    x: ErrorTexts;
    function ANSI_CHAR_table: AnsiChar;
    function exeDirectories: Integer;
    function openClose(path_name: String): Boolean;
    function trimming(s: PChar; s_length: Integer): Integer;
  public
    function pathName(g: String): String;
    procedure setting(m: Boolean);
  end;

const
  C_ERR = 'err';
  EMPTY = '';
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_3E = 62; { '>' }
  H_5C = 92; { '\' }
  H_7E = 126; { '~' }
  H_FF = 255;
  MANY = 256;
  PATH_TEXT = 'path.txt'; { Enter 'PATH > path.txt' }
  SEMICOLON_BYTE = 59; { 3B, ';' }
  SIZE_OF_CHAR = 2;
  TEXT_FILE = '.txt';

implementation

function CommandPath.ANSI_CHAR_table: AnsiChar;
var
  beginning_of_ASCII: Integer;
  end_1: Integer;
  h: AnsiChar;
  k: Integer;
  m: Integer;
begin
  beginning_of_ASCII := 0;

  end_1 := H_FF - 1;
  ANSI_CHAR[0] := #0;
  h := ANSI_CHAR[0];
  m := 1;

  for k := beginning_of_ASCII to end_1 do
  begin
    Inc(h);
    ANSI_CHAR[m] := h;
    Inc(m);
  end;

  Result := h; { H_FF_C }
end;

function CommandPath.exeDirectories: Integer;
var
  flag: TReplaceFlags;
  s: String;
begin
  GetEnvironmentVariable('Path', @buffer, SizeOf(buffer) );
  flag := [rfReplaceAll];
  s := StringReplace(buffer, '\;', ';', flag);
  Result := trimming( PChar(s), Length(s) );
end;

function CommandPath.trimming(s: PChar; s_length: Integer): Integer;
var
  c: Char;
  i: Longint;
  k: Integer;
  m: Integer;
  s_length_1: Integer;
begin
  m := 0;
  s_length_1 := s_length - 1;

  for k := 0 to s_length_1 do
  begin
    c := s[k];
    i := Ord(c);

    if i = 0 then
      begin
        break;
      end
    else
      begin
        combo[m] := ANSI_ASCII[i];
        Inc(m);
      end;

  end;

  comboLength := m;
  Result := comboLength;
end;

function CommandPath.openClose(path_name: String): Boolean;
var
  b: Boolean;
  s: String;
  testFile: File;
begin
  b := False;

  try
    Assign(testFile, path_name);
    Reset(testFile);
    CloseFile(testFile);
    b := True;
  except
    if manyErrorTexts then
      begin
        s := 'Reset() error-801: ' + path_name;
        x := ErrorTexts.Create;
        x.setting(C_ERR, TEXT_FILE, s);
        x.Free;
      end;

  end;

  Result := b;
end;

function CommandPath.pathName(g: String): String;
var
  d: DividingSJIS;
  f: FieldsListMany;
  many_fields: Integer;
  field_count: Integer;
  i: Integer;
  r: String;
  z: AnsiString;
begin
  WideCharToMultiByte(CP_OEMCP, 0, LPCWSTR(g), Length(g), @extra_buffer, SizeOf(extra_buffer), nil, nil);
  extra := extra_buffer;

  many_fields:= MANY;
  r := EMPTY;

  f := FieldsListMany.Create;
  d := DividingSJIS.Create;
  d.presetting(ANSI_ASCII, SEMICOLON_BYTE);
  d.setting(f, many_fields);

  field_count := d.complementing_fields_list(combo, comboLength, extra, H_5C);
  full_path := EMPTY;

  for i := 0 to field_count do
  begin
    p := CLEAR_PETTY_ARRAY;

    z :=  f.item(i);
    MultiByteToWideChar(CP_OEMCP, 0, LPCSTR(z), Length(z), @p, SizeOf(p) );
    full_path := p;
    if openClose(full_path) then
      begin
        r := full_path;
        break;
      end;

  end;

  f.Free; d.Free;
  Result := r;
end;

procedure CommandPath.setting(m: Boolean);
var
  i: Integer;
  pascal_string: AnsiString;
begin
  manyErrorTexts := m;

  P_SIZE := SizeOf(p) div SIZE_OF_CHAR;
  P_SIZE_1 := P_SIZE - 1;

  for i := 0 to P_SIZE_1 do
  begin
    CLEAR_PETTY_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := ANSI_CHAR_table;

{ No pascal string starts with zero. Please use EMPTY_ANSI, not ANSI_CHAR[0].}
  EMPTY_ANSI := '';
  ANSI_CHAR[0] := ANSI_CHAR[H_3E];
  pascal_string := ANSI_CHAR;
  ANSI_ASCII := Copy(pascal_string, 2, H_7E);

  exeDirectories;
end;

end.
