program Utf16euc;
{$APPTYPE CONSOLE}

{*
Utf16euc Version 1.06
Converting the text from UTF-16 to EUC-JP
Copyright 2020 Sogaya

Delphi 2009

*}

uses
  SysUtils,
  AnsiQueue in 'AnsiQueue.pas',
  Combo in 'Combo.pas',
  DeriveUTF16Euc in 'DeriveUTF16Euc.pas',
  Divide in 'Divide.pas',
  DivideSJIS in 'DivideSJIS.pas',
  ErrorText in 'ErrorText.pas',
  FieldsArray in 'FieldsArray.pas',
  FieldsArrayMany in 'FieldsArrayMany.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  PastoralUTF16 in 'PastoralUTF16.pas',
  PathEXE in 'PathEXE.pas',
  PutCSV in 'PutCSV.pas',
  SearchBinary in 'SearchBinary.pas',
  SparseEUC in 'SparseEUC.pas',
  TimeName in 'TimeName.pas',
  Ucombo in 'Ucombo.pas',
  Utf16intoEUC in 'Utf16intoEUC.pas',
  SelectField in 'SelectField.pas';

{$R *.res}

const
  BUFFER_ERROR = -1;
  DATA_TEXT = 'data.txt';
  DO_NOT_RECORD_PATH_NAME = False;
  DO_NOT_REMOVE_THE_FILE = True;
  FIELD_4 = 4;
  MAXIMUM_FIELD_COUNT = 3071;
  NOT_COUNTED = 2147483647; { 7FFFFFFFh }
  NOT_MEASURED = -1;
  PATH_NAME = 'C:\Narfi\UTF16~EUC-JP.bin';
  REMOVE_ANCIENT_FILE = false;
  SEARCH_TABLE_IN_EACH_DIRECTORY = True;
  SELECTED_DIRECTORY_DOES_NOT_HAVE_PRIORITY = False;
  SURRO_CSV = 'surro.csv';
  TEMPORARY_BINARY = 'binary.tmp';
  TEMPORARY_TEXT = 'text.tmp';
  UNDERLINES_REPLACE_SURROGATE_PAIRS = True;
  VERSION_NUMBER = '1.06';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;

var
  c: ConvertUtf16intoEUC;
  carriage_return: Boolean;
  d: DeriveUTF16EucTable;
  dataFile: TextFile;
  e: SearchTable;
  f: FinalEffects;
  file_name_length: Integer;
  g: Guiding;
  h: HyphenTerms;
  maximumFieldCount: Integer;
  new_name: String;
  new_name_length: Integer;
  p: PutSurroCSV;
  s: String;
  t: FinalEffects;
  testFile: TextFile;
  u: Selecting;

begin
  s := ParamStr(1);
  if Length(s) = 0 then
    begin
      e := SearchTable.Create;
      e.setting(PATH_NAME, SEARCH_TABLE_IN_EACH_DIRECTORY, SELECTED_DIRECTORY_DOES_NOT_HAVE_PRIORITY, DO_NOT_RECORD_PATH_NAME);

      d := DeriveUTF16EucTable.Create;
      d.setting(e);

      d.Free; e.Free;
      Halt(0);
    end;

{ Checking if software can open SURRO_CSV in the current directory. }
  try
    AssignFile(testFile, SURRO_CSV);
    Reset(testFile);
    CloseFile(testFile);
  except
{ Producing SURRO_CSV }
    p := PutSurroCSV.Create;
    p.setting(SURRO_CSV);
  end;

{ Checking if software can open SURRO_TEXT in the current directory. }
  try
    AssignFile(testFile, SURRO_TEXT);
    Reset(testFile);
    CloseFile(testFile);
  except
{ Producing SURRO_TEXT from SURRO_CSV }
    t := FinalEffects.Create;
    t.setting(SURRO_CSV, SURRO_TEXT, FIELD_4, UNDERLINES_REPLACE_SURROGATE_PAIRS);
    u := Selecting.Create;
    u.presetting(t);
    u.setting(t, SURRO_CSV);
    u.ifErrorOccurs(BUFFER_ERROR);

    u.reading_to_produce_EUC_CRLF(SURRO_TEXT);
    u.Free; t.Free;
  end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;
  maximumFieldCount := h.upperLimit;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          new_name := h.listName(1);
          new_name_length := Length(new_name);
          if new_name_length = 0 then
            begin
              new_name := h.newName;
            end;

          if h.remove0Dh then
            begin
              if h.carriageReturn then
                begin
                  carriage_return := True;
                end
              else
                begin
                  carriage_return := False;
                end;

            end
          else
            begin
              carriage_return := True;
            end;

          f := FinalEffects.Create;
          f.setting(h.fileName, new_name, h.pathName, 
            h.atPoint, h.fieldSelected, 
            carriage_return, h.identicalTime, h.noRemoving, h.noSearching, h.remove0Dh, h.removeControllers, h.priority, h.surrogatePairToUnderline, h.produceTemporaryBinary, h.whichTable);

{ main function }
  if f.whichTable then
    begin
      try
        Assignfile(dataFile, DATA_TEXT);
        Reset(dataFile);
        CloseFile(dataFile);

        Assignfile(dataFile, DATA_TEXT);
        Erase(dataFile);
      except
        ;
      end;

    end;

  if f.fieldSelected < maximumFieldCount then
    begin
{ Selecting a field from the UTF-16 file to produce an EUC-JP text }
      u := Selecting.Create;
      u.presetting(f);
      u.setting(f, f.fileName);
      u.ifErrorOccurs(BUFFER_ERROR);

      u.reading_to_produce_EUC_CRLF(f.newName);
      u.Free; f.Free;
      Halt(0);
    end;

  file_name_length := Length(f.fileName);
  if file_name_length = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end;

  c := ConvertUtf16intoEUC.Create;
  c.setting(f, f.fileName);
  c.ifErrorOccurs(BUFFER_ERROR);

  if f.whichTable then
    c.displayDataText;

  new_name_length := Length(f.newName);
  if new_name_length = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end;

  if f.identicalTime then
    begin
      c.produceBinaryFile(f.fileName, REMOVE_ANCIENT_FILE);
      c.renameBinaryFile(f.fileName, f.noRemoving);
    end;

  if f.carriageReturn then
    begin
      if f.produceTemporaryBinary then
        begin
          if f.removeControllers then
            begin
              c.presetting;
              c.skipping_control_to_produce_EUC_CRLF(TEMPORARY_BINARY);
            end
          else
            begin
              c.reading_to_produce_EUC_CRLF(TEMPORARY_BINARY);
            end;

          c.renameBinaryFile(f.newName, f.noRemoving);
        end
      else
        begin
          if f.removeControllers then
            begin
              c.presetting;
              c.skipping_control_to_produce_EUC_CRLF(f.newName);
            end
          else
            begin
              c.reading_to_produce_EUC_CRLF(f.newName);
            end;

        end;

    end
  else
    begin
      if f.remove0Dh then
        begin
          if f.produceTemporaryBinary then
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_EUC_LF(TEMPORARY_BINARY);
                end
              else
                begin
                  c.reading_to_produce_EUC_LF(TEMPORARY_BINARY);
                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_EUC_LF(f.newName);
                end
              else
                begin
                  c.reading_to_produce_EUC_LF(f.newName);
                end;

            end;

        end
      else
        begin
          if f.produceTemporaryBinary then
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_EUC_text(TEMPORARY_BINARY);
                end
              else
                begin
                  c.reading_to_produce_EUC_text(TEMPORARY_BINARY);
                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_EUC_text(f.newName);
                end
              else
                begin
                  c.reading_to_produce_EUC_text(f.newName);
                end;

            end;

        end;

    end;

  c.Free;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
